
/****************************************************************
*  File : linux/arch/arm/kernel/arm-stub.c
*
*  This is the main implementation file for arm stub. 
*
*****************************************************************/

#include "gdb-main.h"
#include "arm-stub.h"
#include <linux/ptrace.h>


#if !defined TRUE
#define TRUE (1==1)
#endif 

#if !defined FALSE
#define FALSE (!TRUE)
#endif 

//#define GDB_CLI_PRINTF(s) kprintf(s)
#define MAX_BPOINTS 15
#define SUBMASK(x) ((1L << ((x) + 1)) - 1)
#define BIT(obj,st) (((obj) >> (st)) & 1)
#define BITS(obj,st,fn) (((obj) >> (st)) & SUBMASK ((fn) - (st)))
#define sbits(obj,st,fn) \
		((long) (BITS(obj,st,fn) | ((long) BIT(obj,fn) * ~ SUBMASK (fn - st))))
#define BranchDest(addr,instr) \
		((unsigned) (((long) (addr)) + 8 + (sbits (instr, 0, 23) << 2)))
#define addr_bits_remove(x) (x & 0xfffffffc)

#define read_register(id) gdb_register_file.r[id]  /* returns the value of the register - id.  */
#define GDB_MAXREGS	(16 + 8 * 3 + 1 + 1)  /* Added by sandeep : 16 GPR + 8 Floating point registers + 1 FPS + 1 CPSR */	

/**********Structure/Enum*****************/
typedef struct  
{
	unsigned long addr;
	unsigned long orgdata;
}breakpoint_info;

/* Added floating point registers. This is done because gdb was not sending correct 
 * cpsr to host. after pc(r15) there are floating point registers.with this GDB stub 
 * sends correct register values to host e.g register 25 is CPSR 
 */
typedef enum
{
  R0, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, R11, IP, SP, LR, PC, 
  F0_1, F0_2, F0_3, F1_1, F1_2, F1_3, F2_1, F2_2, F2_3, F3_1, F3_2, 
  F3_3, F4_1, F4_2, F4_3, F5_1, F5_2, F5_3, F6_1, F6_2, F6_3, F7_1, 
  F7_2, F7_3, SPSR, CPSR
} register_id_E;
/*********************************************/


/*********** Globals *************************/
long gdb_connected = FALSE;				   /* gives the current connection state of GDB */
breakpoint_info breakpoints[MAX_BPOINTS];  /* Store all breakpoints.  */
int bpoint_num=0;
arm32_instr_t arm32_stepped_addr = 0;      /* single step information.  */
arm32_instr_t arm32_stepped_instr = 0;
register_file_S gdb_register_file;		   /* variable to save processor context */
/********************************************/



/************ Function prototype ************/

/* Extern Functions */
#ifdef CONFIG_GDB_THREAD
long int  get_task_registers (struct task_struct*);
#endif

/* References to the routines that communicate directly with the uart (cnxtuart.c)*/
int (*debugger_getchar) (void);
int (*debugger_putchar) (int);

/* Routines for memory fault handling */
int gdb_setjmp(int *machine_context);
int gdb_longjmp(int *machine_context, int flag);
void gdb_handle_bus_error(void);
int fault_jmp_buf[32]; /* Jump buffer for gdb_setjmp/longjmp */
int gdb_fault_expected;	/* Boolean to ignore bus errs (i.e. in GDB) */

/* static functions declarations.  */
arm32_instr_t get_stepi_dest (void);
static int check_cond_code (unsigned long, arm32_instr_t);
static unsigned long shifted_reg_val (unsigned long, int, unsigned long, unsigned long);
static void clear_breakpoints(void);
/**********************************************/



/******************************************************************************
*  FUNCTION:    bitcount                                  
*                                                                            
*  DESCRIPTION:                                                              
*     Return number of 1-bits(set) in VAL                                         
*                                                                            
*  INPUT PARAMETERS:                                                         
*     val : value whose bits are to be checked                                                                   
*                                                                            
*  OUTPUT PARAMETERS:                                                        
*     None                                                        
*                                                                            
*  RETURN VALUES:                                                            
*     number of bits set    
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
static int
bitcount (unsigned long val)
{
  int nbits;
  for (nbits = 0; val != 0; nbits++)
    val &= val - 1;		/* delete rightmost 1-bit in val */
  return nbits;
}


/******************************************************************************
*  FUNCTION:    check_cond_code                                  
*                                                                            
*  DESCRIPTION:                                                              
*     Checks the conditional codes the the instruction set.
*                                                                            
*  INPUT PARAMETERS:                                                         
*     cond : opcode of the instrution to check  
*     cpsr : value of the status register(cpsr)                                                             
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     TRUE - if condition is statisfied.
*     FALSE - otherwise.
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
static int
check_cond_code (unsigned long cond, arm32_instr_t cpsr)
{
	/* enum for all the conditional codes.  */
	enum conditions
	{ m_eq, m_ne, m_cs, m_cc, m_mi, m_pl, m_vs, m_vc, m_hi, m_ls, m_ge, m_lt,
	  m_gt, m_le, m_always, m_never,
	};
  
	if (cond == m_always || cond == m_never)
	return 1;

	switch( cond )
	  {
	case m_eq:
		return  ((cpsr & ARM32_PSR_FLAGS_Z)  != 0);
	case m_ne:
		return  ((cpsr & ARM32_PSR_FLAGS_Z)  == 0);
	case m_cs:
		return  ((cpsr & ARM32_PSR_FLAGS_C)  != 0);
	 case m_cc:
			return  ((cpsr & ARM32_PSR_FLAGS_C)  == 0);
	case m_mi:
		return  ((cpsr & ARM32_PSR_FLAGS_N)  != 0);
	case  m_pl:
		return  ((cpsr & ARM32_PSR_FLAGS_N)  == 0);
	case  m_vs:
		return  ((cpsr & ARM32_PSR_FLAGS_V)  != 0);
	case  m_vc:
			return  ((cpsr & ARM32_PSR_FLAGS_V)  == 0);
	case  m_hi:
		return  ((cpsr & (ARM32_PSR_FLAGS_C | ARM32_PSR_FLAGS_Z )) == 0);
	case  m_ls:
 		return  ((cpsr & (ARM32_PSR_FLAGS_C | ARM32_PSR_FLAGS_Z )) != 0);
	case  m_ge:
		return  ( ((cpsr & ARM32_PSR_FLAGS_N)  == 0) 
				== ((cpsr & ARM32_PSR_FLAGS_V)  == 0));
	case  m_lt:
		return  ( ((cpsr & ARM32_PSR_FLAGS_N)  == 0) 
				!= ((cpsr & ARM32_PSR_FLAGS_V)  == 0));
	case  m_gt:
		return  ( ((cpsr & ARM32_PSR_FLAGS_Z)  == 0) &&
				(((cpsr & ARM32_PSR_FLAGS_N)  == 0) 
					== ((cpsr & ARM32_PSR_FLAGS_V)  == 0)));
	case  m_le:
		return  ( ((cpsr & ARM32_PSR_FLAGS_Z)  != 0) ||
							(((cpsr & ARM32_PSR_FLAGS_N)  == 0)
						 != ((cpsr & ARM32_PSR_FLAGS_V)  == 0)));
	default:
		break;	
	  }

  return 1;
}


/******************************************************************************
*  FUNCTION:    shifted_reg_val                                  
*                                                                            
*  DESCRIPTION:                                                              
*	   Does processing for shifted register in when calculation next instruction 
*      address	     
*                                                                            
*  INPUT PARAMETERS:                                                         
*     inst   : opcode of the instrution  
*     carry  : carry flag status                                                             
*     pc_val : value of pc 
*	  cpsr   : status register(cpsr)
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     shift register value
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
static unsigned long 
shifted_reg_val (unsigned long inst, int carry, unsigned long pc_val,
				 unsigned long status_reg)
{
	unsigned long res = 0, shift = 0;
	int rm = BITS (inst, 0, 3);
	unsigned long shifttype = BITS (inst, 5, 6);
	if (BIT (inst, 4))
	{
		int rs = BITS (inst, 8, 11);
		shift = (rs == 15 ? pc_val + 8 : read_register (rs)) & 0xFF;
	}
	else
		shift = BITS (inst, 7, 11);
	
	res = (rm == 15
		? ((pc_val | (1 ? 0 : status_reg))
		+ (BIT (inst, 4) ? 12 : 8)) : read_register (rm));
	
	switch (shifttype)
	{
	case 0:			/* LSL */
		res = shift >= 32 ? 0 : res << shift;
		break;
	case 1:		/* LSR */
		res = shift >= 32 ? 0 : res >> shift;
		break;
	case 2:			/* ASR */
		if (shift >= 32)
		shift = 31;
		res = ((res & 0x80000000L) ? ~((~res) >> shift) : res >> shift);
		break;
	case 3:			/* ROR/RRX */
		shift &= 31;
		if (shift == 0)
		res = (res >> 1) | (carry ? 0x80000000L : 0);
		else
		res = (res >> shift) | (res << (32 - shift));
		break;
	}
	return res & 0xffffffff;
}


/******************************************************************************
*  FUNCTION:    get_stepi_dest                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  Analyzes the next instruction, to see where the program
*     will go to when it runs.  Returns the destination address.	     
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     next instruction address
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
arm32_instr_t  
get_stepi_dest (void)
{
	arm32_instr_t pc_val, this_instr, status, nextpc;

	pc_val = gdb_register_file.r[PC];
	nextpc = pc_val + 4;
	this_instr = *(arm32_instr_t *) pc_val;
	status = gdb_register_file.cpsr;
	if (check_cond_code ( BITS(this_instr, 28, 31), status))
    {
		unsigned long operand1, operand2, result = 0;
		unsigned long rn =0;
		int c=0;
		switch (BITS (this_instr, 24, 27))
		{
		case 0x0:
		case 0x1:		/* data processing */
		case 0x2:
		case 0x3:
		  if (BITS (this_instr, 12, 15) != 15)
			break;
		  if (BITS (this_instr, 22, 25) == 0 && BITS (this_instr, 4, 7) == 9)

			/* Multiply into PC */
			c = (status & ARM32_PSR_FLAGS_C) ? 1 : 0;
		  rn = BITS (this_instr, 16, 19);
		  operand1 = (rn == 15) ? pc_val + 8 : gdb_register_file.r[rn];
		  if (BIT (this_instr, 25))

			{
			  unsigned long immval = BITS (this_instr, 0, 7);
			  unsigned long rotate = 2 * BITS (this_instr, 8, 11);
			  operand2 = ((immval >> rotate) | (immval << (32 - rotate)))
			& 0xffffffff;
			}

		  else			/* operand 2 is a shifted register */
			operand2 = shifted_reg_val (this_instr, c, pc_val, status);
		  switch (BITS (this_instr, 21, 24))

			{
			case 0x0:		/*and */
			  result = operand1 & operand2;
			  break;
			case 0x1:		/*eor */
			  result = operand1 ^ operand2;
			  break;
			case 0x2:		/*sub */
			  result = operand1 - operand2;
			  break;
			case 0x3:		/*rsb */
			  result = operand2 - operand1;
			  break;
			case 0x4:		/*add */
			  result = operand1 + operand2;
			  break;
			case 0x5:		/*adc */
			  result = operand1 + operand2 + c;
			  break;
			case 0x6:		/*sbc */
			  result = operand1 - operand2 + c;
			  break;
			case 0x7:		/*rsc */
			  result = operand2 - operand1 + c;
			  break;
			case 0x8:
			case 0x9:
			case 0xa:
			case 0xb:		/* tst, teq, cmp, cmn */
			  result = (unsigned long) nextpc;
			  break;
			case 0xc:		/* orr */
			  result = operand1 | operand2;
			  break;
			case 0xd:		/* mov */

			  /* Always step into a function.  */
			  result = operand2;
			  break;
			case 0xe:		/* bic */
			  result = operand1 & ~operand2;
			  break;
			case 0xf:		/* mvn */
			  result = ~operand2;
			  break;
			}
		  nextpc = addr_bits_remove (result);
		  break;
		case 0x4:
		case 0x5:		/* data transfer */
		case 0x6:
		case 0x7:
		  if (BIT (this_instr, 20))

			{
			  /* load */
			  if (BITS (this_instr, 12, 15) == 15)

			{
			  /* rd == pc */
			  /* unsigned long rn; */
			  unsigned long base;
			  if (BIT (this_instr, 22))
				//GDB_CLI_PRINTF ("Illegal update to pc in instruction");

			  /* byte write to PC */
			  rn = BITS (this_instr, 16, 19);
			  base = (rn == 15) ? pc_val + 8 : read_register (rn);
			  if (BIT (this_instr, 24))

				{
				  /* pre-indexed */
				  int c1 = (status & ARM32_PSR_FLAGS_C) ? 1 : 0;
				  unsigned long offset =
				(BIT (this_instr, 25)
				 ? shifted_reg_val (this_instr, c1, pc_val, status)
				 : BITS (this_instr, 0, 11));
				  if (BIT (this_instr, 23))
				base += offset;

				  else
				base -= offset;
				}
			  nextpc = *((unsigned long *) base);
			  nextpc = addr_bits_remove (nextpc);
			  //if (nextpc == read_register (PC))
			   // GDB_CLI_PRINTF ("Infinite loop detected");
			}
			}
		  break;
		case 0x8:
		case 0x9:		/* block transfer */
		  if (BIT (this_instr, 20))

			{
			  /* LDM */
			  if (BIT (this_instr, 15))

			{
			  /* loading pc */
			  int offset = 0;
			  if (BIT (this_instr, 23))

				{
				  /* up */
				  unsigned long reglist = BITS (this_instr, 0, 14);
				  offset = bitcount (reglist) * 4;
				  if (BIT (this_instr, 24))	/* pre */
				offset += 4;
				}

			  else if (BIT (this_instr, 24))
				offset = -4;

			  {
				unsigned long rn_val =
				  read_register (BITS (this_instr, 16, 19));
				nextpc = *((unsigned int *) (rn_val + offset));
			  } nextpc = addr_bits_remove (nextpc);
			  //if (nextpc == read_register (PC))
				//GDB_CLI_PRINTF ("Infinite loop detected");
			}
		  }
		  break;
		case 0xb:		/* branch & link */
		case 0xa:		/* branch */
			{
				nextpc = BranchDest (read_register (PC), this_instr);
				nextpc = addr_bits_remove (nextpc);
				//if (nextpc == read_register (PC))
				//GDB_CLI_PRINTF ("Infinite loop detected");
				break;
			}
		case 0xc:
		case 0xd:
		case 0xe:		/* coproc ops */
		case 0xf:		/* SWI */
			break;
		default:
		  //GDB_CLI_PRINTF ("Bad bit-field extraction");
		  return (read_register (PC));
		}
	}
	return nextpc;
}



/******************************************************************************
*  FUNCTION:    gdb_peek_register_file                                  
*                                                                            
*  DESCRIPTION:                                                              
*	   Retrieves a register value from gdb_register_file.  Returns the size
*	   of the register, in bytes, or zero if an invalid id is specified
*                                                                            
*  INPUT PARAMETERS:                                                         
*     id  : register number
*     val : variable in which the value is saved
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     size of register(4) if valid register, 0 otherwise
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
int
gdb_peek_register_file (int id, long *val)
{
  /* we will only support 32-bit arm.  */
  int retval = sizeof (long);
  
#ifdef CONFIG_GDB_THREAD
  struct pt_regs *tregs =NULL;
  if(current_thread)
  {
	  tregs = (struct pt_regs *)get_task_registers(current_thread);
  }
#endif

  switch (id)
    {
  case R0:  case R1:  case R2:    case R3:
  case R4:  case R5:  case R6:    case R7:
  case R8:  case R9:  case R10:   case R11:
  case IP:  case SP:  case LR:
  case PC:
		#ifdef CONFIG_GDB_THREAD
		if(current_thread)
		{
			*val =tregs->uregs[id];
			break;
		}
		#endif
		
		*val = gdb_register_file.r[id];
		break;

	/* Added by Sandeep */
  case F0_1: case F0_2: case F0_3: case F1_1: case F1_2: case F1_3:
  case F2_1: case F2_2: case F2_3: case F3_1: case F3_2: case F3_3: 
  case F4_1: case F4_2: case F4_3: case F5_1: case F5_2: case F5_3:
  case F6_1: case F6_2: case F6_3: case F7_1: case F7_2: case F7_3:
		*val = 0;
		break;
  
  case SPSR:
		#ifdef CONFIG_GDB_THREAD
		if(current_thread)
		{
			*val =tregs->uregs[GDB_MAXREGS-2];
			break;
		}
		#endif

		*val = gdb_register_file.spsr;
		break;

  case CPSR:
		#ifdef CONFIG_GDB_THREAD
		if(current_thread)
		{
			*val = tregs->ARM_cpsr;
			break;
		}
		#endif
		
		*val = gdb_register_file.cpsr;
		break;

  default:
      retval = 0;
  }

  return retval;
}


/******************************************************************************
*  FUNCTION:    gdb_poke_register_file                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  Stuffs a register value into gdb_register_file.  
*                                                                            
*  INPUT PARAMETERS:                                                         
*     id  : register number
*     val : register contents
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     Returns the size of
*     the register, in bytes, or zero if an invalid id is specified.
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
int
gdb_poke_register_file (int id, long val)
{

  /* we will support only 32-bit mode.  */
  int retval = sizeof (long);

  switch (id)
  {
  case R0:    case R1:   case R2:    case R3:
  case R4:    case R5:   case R6:    case R7:
  case R8:    case R9:   case R10:   case R11:
  case IP:    case SP:   case LR:
  case PC:
      gdb_register_file.r[id] = val;
      break;
   /* Added by Sandeep */
  case F0_1: case F0_2: case F0_3: case F1_1: case F1_2: case F1_3:
  case F2_1: case F2_2: case F2_3: case F3_1: case F3_2: case F3_3: 
  case F4_1: case F4_2: case F4_3: case F5_1: case F5_2: case F5_3:
  case F6_1: case F6_2: case F6_3: case F7_1: case F7_2: case F7_3:
		break;

  case SPSR:
      gdb_register_file.spsr = val;
      break;
  case CPSR:
      gdb_register_file.cpsr = val;
      break;
  default:
      retval = 0;
  }

  return retval;
}


/******************************************************************************
*  FUNCTION:    gdb_step                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  processes the step packet from gdb host. replaces the next instruction 
*     with BREAKPOINT_INSTRUCTION.
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     None
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void
gdb_step (long __unused addr)
{
  /*
  if (addr>0)
     gdb_register_file.r[PC] = addr; */
	addr = addr;
	
	/* determine where the step will take us.  */
	arm32_stepped_addr = get_stepi_dest ();

	/* save the target opcode, replace with BREAKPOINT INSN.  */
	arm32_stepped_instr = *(unsigned long *) arm32_stepped_addr;
	*(unsigned long *) arm32_stepped_addr = BREAKPOINT_INSTRUCTION;
	  // gdb_return_from_exception ();
	return;
}


/******************************************************************************
*  FUNCTION:    gdb_continue                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  processes the continue packet from gdb host. Releases the application to run. 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     addr : addres of instruction at which to resume.
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     None
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void
gdb_continue (long addr)
{
	if (addr>0)
     gdb_register_file.r[PC] = addr;

    gdb_connected = GDB_CONTINUE;

    /*  show_breakpoints();  */
    //  gdb_return_from_exception ();
}


/******************************************************************************
*  FUNCTION:    insert_breakpoint                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  Inserts breakpoint at the address 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     addr : addres of instruction at which to put the breakpoint.
*     len  : length of address
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     zero is breakpoint is successfully placed, -1 otherwise
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
int insert_breakpoint(long addr, long len)
{
	long i=0; 
    len = len;
    	
	if (gdb_setjmp(fault_jmp_buf) == 0) 
	{		
		gdb_fault_expected =1;

		 /* check if breakpoint already at this address.  */
		 for(i=0;i<bpoint_num;i++)
		 {
			 if(breakpoints[i].addr == (unsigned long)addr)
			 {
				 gdb_fault_expected = 0;
				 return 0;
			 }
		 }

		 breakpoints[bpoint_num].orgdata = *(unsigned long*)addr;
		 breakpoints[bpoint_num].addr = addr;
		 bpoint_num++;  

		 *(unsigned long *) addr = BREAKPOINT_INSTRUCTION;
		 gdb_fault_expected = 0;
		 return 0;
	}
	
	gdb_fault_expected = 0;
	return -1;
}


/******************************************************************************
*  FUNCTION:    remove_breakpoint                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  Removes breakpoint at the address 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     addr : addres of instruction from which breakpoint is to be removed
*     len  : length of address
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     zero is breakpoint is successfully removed, -1 otherwise
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
int remove_breakpoint(long addr, long len)
{
	int i;
	len = len;

	if (gdb_setjmp(fault_jmp_buf) == 0) 
	{
		gdb_fault_expected = 1;

		/* find the location of the breakpoint. */
		for(i=0;i<bpoint_num;i++)
		{
			/* found the location.  */
			if(breakpoints[i].addr == (unsigned long)addr)
			{
				*(unsigned long*)addr = breakpoints[i].orgdata;
				
				/* update the array of breakpoints.  */	
				while(i!=bpoint_num) 
				{
					breakpoints[i].orgdata = breakpoints[i+1].orgdata;
					breakpoints[i].addr = breakpoints[i+1].addr;
					i++;
				}
				
				bpoint_num--;
				gdb_fault_expected = 0;
 				return 0;
			}
		}
	}

	gdb_fault_expected = 0;
	return -1; 
}

/******************************************************************************
*  FUNCTION:    clear_breakpoints                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  clear all the breakpoints.
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*     zero is breakpoint is successfully removed, -1 otherwise
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
static void clear_breakpoints(void)
{
	int i ;
	for(i=0;i<bpoint_num;i++)
	{
		*(unsigned long*)breakpoints[i].addr = breakpoints[i].orgdata;
	}
	bpoint_num = 0;  
}

#if 0
static void show_breakpoints()
{
 int i=0;
 for(i=0;i<bpoint_num;i++)
  { 
    //kprintf("Num: %d, *PC=%x\t baddr=%x\n", i, *(unsigned long*)breakpoints[i].addr, breakpoints[i].addr);
  }
}
#endif


/******************************************************************************
*  FUNCTION:    gdb_kill                                  
*                                                                            
*  DESCRIPTION:                                                              
*	  Releases gdb and stops execution 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void
gdb_kill (void)
{
	clear_breakpoints();
	//gdb_return_from_exception ();
	return;
}


/******************************************************************************
*  FUNCTION:    gdb_monitor_onentry                                  
*                                                                            
*  DESCRIPTION:                                                              
*	 The stub calls this before dropping into the monitor, to give us a
*    chance to clean things like software stepping up.
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:                                                            
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void
gdb_monitor_onentry (void)
{
	/* check if we are steping, undo the step.  */
	if (arm32_stepped_instr)
    {
		*(arm32_instr_t *) arm32_stepped_addr = arm32_stepped_instr;
		arm32_stepped_instr = 0;
		arm32_stepped_addr = 0;
    }
	
	return;
}


/********** Not required *************/
#if 0
/* Not required in case of linux : handling is done by linux kernel */
void __attribute__((naked)) gdb_return_from_exception (void)
{
  asm volatile ("ldr r0, .REGFILE\n"
		"ldr r3, [r0, #17*4]\n"
		"msr spsr, r3\n"
		"mov r0, r0\n"
		"ldr r3, [r0, #17*4]\n"
		"msr cpsr, r3\n"
		"ldmia r0, {r0-pc}^\n"
		".REGFILE: .word gdb_register_file\n");
	return;
}
#endif


#if 0
static int
my_strcmp (const char *s1, const char *s2)
{
	unsigned char uc1, uc2;
	/* Move s1 and s2 to the first differing characters 
     in each string, or the ends of the strings if they
     are identical.  */
	while (*s1 != '\0' && *s1 == *s2)
    {
		s1++;
		s2++;
    }
	/* Compare the characters as unsigned char and
     return the difference.  */
	uc1 = (*(unsigned char *) s1);
	uc2 = (*(unsigned char *) s2);
	return ((uc1 < uc2) ? -1 : (uc1 > uc2));
}
#endif

/******************************************/


/******************************************************************************
*  FUNCTION:    gdb_setjmp                                  
*                                                                            
*  DESCRIPTION:                                                              
*	This function saves the processor context in case there is memory fault. 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     machine_context : processor context
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:
*     None                                                            
*                                                                            
*  NOTES:                                                                    
*   gcc will generate warning for this function as there is no C statement 
*   returning integer value. We can safely ignore this as we are returning 
*   the value in assembly directly through registers.
*                                                                            
******************************************************************************/
int __attribute__((naked)) gdb_setjmp (int *machine_context)
{
  asm volatile ("stmia	r0, {r0-r14}\n"
				"str	lr,[r0, #60]\n"
				"mrs	r1,cpsr\n"
				"str	r1,[r0,#64]\n"
				"ldr	r1,[r0,#4]\n"
				"mov	r0, #0\n"
				"mov	pc,lr\n"
			   );	
	return 0;
}

/******************************************************************************
*  FUNCTION:    gdb_longjmp                                  
*                                                                            
*  DESCRIPTION:                                                              
*	This function restores the processor context when the fault is handled
*   (called from fault handler). 
*                                                                            
*  INPUT PARAMETERS:                                                         
*     machine_context : processor context
*     flag : flag to set the return value as 1
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:
*     None                                                            
*                                                                            
*  NOTES:                                                                    
*   gcc will generate warning for this function as there is no C statement 
*   returning integer value. We can safely ignore this as we are returning 
*   the value in assembly directly through registers.
*                                                                            
******************************************************************************/
int __attribute__((naked)) gdb_longjmp (int *machine_context, int flag)
{
  asm volatile ("str r1,[r0]\n"
				"ldmia	r0,{r0-pc}^\n"
			   );
  return 0;	
}


/******************************************************************************
*  FUNCTION:    gdb_handle_bus_error                                  
*                                                                            
*  DESCRIPTION:                                                              
*	The following provides support for recovery from bus errors during stub 
*   memory read/write operations.
*                                                                            
*  INPUT PARAMETERS:                                                         
*     None
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:
*     None                                                            
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void gdb_handle_bus_error(void)
{	
	gdb_longjmp(fault_jmp_buf, 1);
}


/******************************************************************************
*  FUNCTION:    breakpoint_handler                                  
*                                                                            
*  DESCRIPTION:                                                              
*	Main function for handling breakpoint traps. This function is called from 
*   kernel trap handler(do_undefinstr() in /linux/arch/arm/traps.c).
*                                                                            
*  INPUT PARAMETERS:                                                         
*     unused : unused parameter
*                                                                            
*  OUTPUT PARAMETERS:   
*     None	                                                     
*                                                                            
*  RETURN VALUES:
*     None                                                            
*                                                                            
*  NOTES:                                                                    
*     None                                                                   
*                                                                            
******************************************************************************/
void
breakpoint_handler (const char *pszReason __unused)
{
  unsigned long* c_pc;
  int signal = GDB_SIGTRAP;

  if(!gdb_connected)
  {
	  printk("Hard breakpoint hit, Waiting for GDB to connect...\n");
	  debugger_getchar = debug_getc ;
	  debugger_putchar = debug_putc ;
  }

  /* get the pc that caused the exception.  */
  c_pc = (unsigned long *)(gdb_register_file.r[PC] -4 );
 
  /* adjust the PC value, for GDB breakpoint instructions.   */
  if ((*c_pc)== BREAKPOINT_INSTRUCTION)
  {
	  gdb_register_file.r[PC] = gdb_register_file.r[PC] - 4;
  }
 
  gdb_handle_exception (signal);
  return;
}
