/**********************************************************************************
*  File : linux/arch/arm/kernel/arm-stub.c
***********************************************************************************/

#if !defined(ARM_H_INCLUDED)
#define ARM_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

/* tells if GDB is continuing. :*/
#define GDB_CONTINUE 0x10

/* This is an invalid instruction used to trap the
   processor. For ARM processors >5e there is 
   a BKPT insn to do the same.  */
#define BREAKPOINT_INSTRUCTION 0xe7ffdefe
#define SPECIAL_BREAKPOINT 0xe7ffdeff
#define BREAKPOINT() asm volatile ("\n.word 0xe7ffdeff\n")

/* PSR register bit masks and constants.  */
#define ARM32_PSR_MODE        (0x0000001fu)	/* Mask to get processor mode */
#define ARM32_PSR_MODE_USER   (0x00000010u)	/* User mode */
#define ARM32_PSR_MODE_FIQ    (0x00000011u)	/* FIQ mode */
#define ARM32_PSR_MODE_IRQ    (0x00000012u)	/* IRQ mode */
#define ARM32_PSR_MODE_SVC    (0x00000013u)	/* Supervisor mode */
#define ARM32_PSR_MODE_ABORT  (0x00000017u)	/* Abort mode */
#define ARM32_PSR_MODE_UNDEF  (0x0000001Bu)	/* Undefined mode */
#define ARM32_PSR_MODE_SYSTEM (0x0000001Fu)	/* System mode */

#define ARM32_PSR_THUMB_STATE (0x00000020u)	/* Set if in Thumb mode */

#define ARM32_PSR_DISABLE_FIQ (0x00000040u)	/* Set to disable FIQ */
#define ARM32_PSR_DISABLE_IRQ (0x00000080u)	/* Set to disable IRQ */

#define ARM32_PSR_FLAGS_N     (0x80000000u)	/* Negative flag */
#define ARM32_PSR_FLAGS_Z     (0x40000000u)	/* Zero flag */
#define ARM32_PSR_FLAGS_C     (0x20000000u)	/* Carry flag */
#define ARM32_PSR_FLAGS_V     (0x10000000u)	/* Overflow flag */

/* Register names as defined in the ARM Procedure Call Standard  */
#define ARM_A1     (0)
#define ARM_A2     (1)
#define ARM_A3     (2)
#define ARM_A4     (3)

#define ARM_V1     (4)
#define ARM_V2     (5)
#define ARM_V3     (6)
#define ARM_V4     (7)
#define ARM_V5     (8)
extern int (*debugger_getchar) (void);
extern int (*debugger_putchar) (int);


typedef unsigned int arm32_instr_t;
typedef char arm32_bool;
typedef void arm32_pcb;


typedef struct
{
  arm32_instr_t r[16];
  arm32_instr_t spsr;		/* status registers.  */
  arm32_instr_t cpsr;
  unsigned long abort_type;
} register_file_S;


#ifdef __cplusplus
 }
#endif 

#endif /* ARM_H_INCLUDED */
