
/**********************************************************************************
*  File : linux/arch/arm/kernel/gdb-main.h
***********************************************************************************/

#if !defined(GDB_H_INCLUDED)
#define GDB_H_INCLUDED

/* Added for thread debugging support */
#define CONFIG_GDB_THREAD

/* Added by Sandeep For GDB remote debugging : Start */
extern int (*debugger_getchar) (void);
extern int (*debugger_putchar) (int);
int debug_getc(void);
int debug_putc(int );
/* Added by Sandeep : End */

/* Function to read/write from uart */
#define gdb_putc(c) debugger_putchar(c)
#define gdb_getc() debugger_getchar()

/* tells if GDB is connected or not.  */
extern long gdb_connected;
/* tell if gdb is working in continue mode.  */
#define IS_GDB_CONTINUE() (gdb_connected==GDB_CONTINUE)
/* tell if gdb is connected or not.  */
#define IS_GDB_CONNECTED() (gdb_connected!=0)


#define __unused __attribute__((unused))

/* platform-specific stuff, in <target>[-<platform>].c */
void  breakpoint_handler (const char *pszReason);
void gdb_monitor_onentry (void);
int gdb_peek_register_file (int id, long *val);
int gdb_poke_register_file (int id, long val);
void gdb_step (long addr);
void gdb_continue (long addr);
void gdb_kill (void);
void gdb_return_from_exception (void);

#define gdb_flush_cache(start, len)  
#define gdb_detach()  

#define gdb_monitor_onexit() 
#define gdb_startup() 

/* insert breakpoint at address (addr) with lenght (len).  */
int insert_breakpoint(long addr, long len);
int remove_breakpoint(long addr, long len);

/* platform-neutral stuff, in gdb-main.c */
void gdb_console_output (int len, const char *buf);
void gdb_monitor (int sigval);
void gdb_handle_exception (int sigval);
int gdb_get_signal_number (char *pszReason);

/* Routines for memory fault handling */
extern int gdb_setjmp(int *machine_context);
extern int gdb_longjmp(int *machine_context, int flag);
extern void gdb_handle_bus_error(void);
extern int fault_jmp_buf[32]; /* Jump buffer for kgdb_setjmp/longjmp */
extern int gdb_fault_expected;	/* Boolean to ignore bus errs (i.e. in GDB) */

#ifdef CONFIG_GDB_THREAD
#include <linux/sched.h> 
extern struct task_struct *current_thread;
#endif

/* gdb signal values.  */
#define GDB_SIGHUP           1
#define GDB_SIGINT           2
#define GDB_SIGQUIT          3
#define GDB_SIGILL           4
#define GDB_SIGTRAP          5
#define GDB_SIGABRT          6
#define GDB_SIGIOT           6
#define GDB_SIGBUS           7
#define GDB_SIGFPE           8
#define GDB_SIGKILL          9
#define GDB_SIGUSR1         10
#define GDB_SIGSEGV         11
#define GDB_SIGUSR2         12
#define GDB_SIGPIPE         13
#define GDB_SIGALRM         14
#define GDB_SIGTERM         15
#define GDB_SIGSTKFLT       16
#define GDB_SIGCHLD         17
#define GDB_SIGCONT         18
#define GDB_SIGSTOP         19
#define GDB_SIGTSTP         20
#define GDB_SIGTTIN         21
#define GDB_SIGTTOU         22
#define GDB_SIGURG          23
#define GDB_SIGXCPU         24
#define GDB_SIGXFSZ         25
#define GDB_SIGVTALRM       26
#define GDB_SIGPROF         27
#define GDB_SIGWINCH        28
#define GDB_SIGIO           29



#endif /* GDB_H_INCLUDED */
