/*
 *  linux/arch/arm/mach-pecos/irq.c
 *
 *  Copyright (C) 2003 Conexant Systems Inc. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**************************************************/
/*   PREPROCESSOR                                 */
/**************************************************/
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/stddef.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/time.h>
#include <linux/ioport.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>

#include <asm/mach/irq.h>

#include <asm/mach/irq.h>
#include <asm/arch/irq.h>
#include <asm/arch/platform.h>
#include <asm/arch/intid.h>

#define DEBUG 0

#if DEBUG
#   define DBG(x...) printk(x)
#else
#   define DBG(x...) while(0);
#endif

typedef volatile unsigned int reg;
typedef reg *lpreg;

struct s_latency latency = { 
    count: 0
};


#define IRQ_PIC0_START  0
#define IRQ_PIC0_END    (IRQ_PIC0_START + 31)
#define IRQ_PIC1_START  32
#define IRQ_PIC1_END    (IRQ_PIC1_START + 31)
#define IRQ_TIM_START  64
#define IRQ_TIM_END    (IRQ_TIM_START + 15) // Rest is unused so far

#define IRQ_SM2_UNUSED  (IRQ_PIC0_START + 5)

#define GPIO_IRQSTATUS_FROMBANK(x)  ASX_TO_VIRT( (GPI_INT_STAT_REG + ((x) * GPI_BANK_SIZE)) )
#define GPIO_IRQENABLE_FROMBANK(x)  ASX_TO_VIRT( (GPI_INT_ENABLE0_REG + ((x) * GPI_BANK_SIZE)) )

/* Resource descriptor for the PECOS default interrupt controller */

int get_latency_list( char *buf ) {
	/** blocked by noida **/
#if 1
    int min=1000000, avg=0, max=0;
    int mindur=1000000, avgdur=0, maxdur=0;
    int i;
    char *p = buf;
    
    p += sprintf(p, "Sample size = %d.  freq(sample) = freq(tick) (%d mS)\n\n", LATENCY_COUNT, 1000/HZ);

    for (i = 0; i < LATENCY_COUNT; i++) {
        if (latency.duration[i] < mindur) mindur = latency.duration[i];
        if (latency.duration[i] > maxdur) maxdur = latency.duration[i];
        if (latency.delta[i] < min) min = latency.delta[i];
        if (latency.delta[i] > max) max = latency.delta[i];
        avg += latency.delta[i];
        avgdur += latency.duration[i];
    }
    p += sprintf(p, "min lat : %d uS\n", min/54 );
    p += sprintf(p, "avg lat : %d uS\n", avg / (LATENCY_COUNT * 54) );
    p += sprintf(p, "max lat : %d uS\n", max/54 );
    
    p += sprintf(p, "min dur : %d uS\n", mindur );
    p += sprintf(p, "avg dur : %d uS\n", avgdur / LATENCY_COUNT );
    p += sprintf(p, "max dur : %d uS\n", maxdur );
    return p - buf;
	#endif

}


static void pic0_ack_irq(unsigned int irq)
{
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_0)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic0_mask_irq(unsigned irq)
{
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic0_unmask_irq(unsigned irq)
{
//  __raw_writel(1 << irq ,ITC_INTRIRQ_REG(PIC_0)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) | (1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic1_ack_irq(unsigned int irq)
{
  irq -= IRQ_PIC1_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_1)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
  if ( irq == ITC_PCI_POS) {
    __raw_writel((__raw_readl(PCI_INTR_STATUS_REG) | PCI_INTR_INTA_ENABLE_MASK) ,PCI_INTR_STATUS_REG); 
  }
}

static void pic1_mask_irq(unsigned irq)
{
  irq -= IRQ_PIC1_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
  if ( irq == ITC_PCI_POS) {
    __raw_writel((__raw_readl(PCI_INTR_STATUS_REG) | PCI_INTR_INTA_ENABLE_MASK) ,PCI_INTR_STATUS_REG); 
  }
}

static void pic1_unmask_irq(unsigned irq)
{
  irq -= IRQ_PIC1_START;
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) | (1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
}

static void mask_irq_pic_tim( unsigned int irq ) {
    lpreg mode;
    /* With the timers, masking a particular timer interrupt
     * also has the effect of acknowledging it
     */
    irq -= CALC_INT_ID(PIC_TIMERS, 0);
    mode = (lpreg)ASX_TO_VIRT( TIMER_NUM_MODE_REG(irq) );
    *mode &= ~TIM_INT_ENABLE_MASK;
}


static void unmask_irq_pic_tim( unsigned int irq ) {
    lpreg mode;

    irq -= CALC_INT_ID(PIC_TIMERS, 0);
    mode = (lpreg)ASX_TO_VIRT( TIMER_NUM_MODE_REG(irq) );
    *mode |= TIM_INT_ENABLE_MASK;
}

static void mask_and_ack_irq_pic_gpio (unsigned int irq) { 
    int bank;
    int bit;
    lpreg sta, ena;

    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;

    sta = (lpreg)(GPIO_IRQSTATUS_FROMBANK(bank)); 
    ena = (lpreg)(GPIO_IRQENABLE_FROMBANK(bank));
    *ena &= ~(1 << bit); 
    *sta |= (1 << bit); 
}

static void mask_irq_pic_gpio (unsigned int irq) {
    int bank;
    int bit;
    lpreg ena;
 
    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;

    ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
    *ena &= ~(1 << bit); 
} 

static void unmask_irq_pic_gpio (unsigned int irq) {
    int bank;
    int bit;
    lpreg ena;

    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;

    ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
    *ena |= (1 << bit); 
} 


static struct irqchip pecos_pic0 = {
	.ack	= pic0_ack_irq,
	.mask	= pic0_mask_irq,
	.unmask = pic0_unmask_irq,
};

static struct irqchip pecos_pic1 = {
	.ack	= pic1_ack_irq,
	.mask	= pic1_mask_irq,
	.unmask = pic1_unmask_irq,
};

static struct irqchip cactus_chip_tim = {
	.ack	= mask_irq_pic_tim,
	.mask	= mask_irq_pic_tim,
	.unmask = unmask_irq_pic_tim,
};

static struct irqchip cactus_chip_gpio = {
	.ack	= mask_and_ack_irq_pic_gpio,
	.mask	= mask_irq_pic_gpio,
	.unmask = unmask_irq_pic_gpio,
};


static void timer_irq_handler(unsigned int irq, struct irqdesc *desc,
            struct pt_regs *regs)
{
    int bit;
    register int timer_r;
    struct irqdesc *d;

    lpreg timer_int_sta = (lpreg)ASX_TO_VIRT(TIM_INT_STATUS_REG);
    desc->chip->ack(irq);

    for (bit = 0; bit < NUM_TIM_BANKS; bit++) {
        if (*timer_int_sta & (1 << bit)) {
			timer_r= CALC_INT_ID(PIC_TIMERS, bit);
			d = irq_desc +timer_r;
			desc_handle_irq(timer_r,d,regs);
        }
    }
    desc->chip->unmask(irq);
}

static void gpio_irq_handler(unsigned int irq, struct irqdesc *desc,
            struct pt_regs *regs)
{
   register unsigned int shift;
   register int irqbit;
   register int r;
   register int gpio_r;
   
   struct irqdesc *d;  

    int bank;

    desc->chip->ack(irq);
    for (bank = 0; bank < NUM_GPI_BANKS; bank++)
    {
        while (1)
        {
           lpreg sta = (lpreg)GPIO_IRQSTATUS_FROMBANK(bank);
           lpreg ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
        
           irqbit = *ena & *sta;
           if (irqbit)
           {
              r = 0;
              shift = ( ( irqbit & 0xFFFF0000 ) != 0 ) << 4; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xFF00     ) != 0 ) << 3; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xF0       ) != 0 ) << 2; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xC        ) != 0 ) << 1; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0x2        ) != 0 ) << 0; irqbit >>= shift; r |= shift;
              gpio_r= CALC_INT_ID(PIC_GPIO, r + (32*bank));
	      d = irq_desc + gpio_r; 
              desc_handle_irq(gpio_r,d,regs);
			  
           } else {
              break;
           }

        }
    }
    desc->chip->unmask(irq);
}


void __init cnxt_init_irq( void )
{
	unsigned int irq;

       /* Disable Interrupt Generation */ 
        __raw_writel(0x0,ITC_INTENABLE_REG(0,PIC_0)); 
        __raw_writel(0x0,ITC_INTENABLE_REG(0,PIC_1)); 
	
        for (irq = 0; irq < NR_IRQS  ; irq++)
        {
           if ( (irq == IRQ_SM2_UNUSED) )
		continue;
           switch(irq) {
               case IRQ_PIC0_START ... IRQ_PIC0_END:
                   set_irq_chip(irq, &pecos_pic0);
                   set_irq_flags(irq, IRQF_VALID);
                   set_irq_handler(irq, do_level_IRQ);
                   break;

               case IRQ_PIC1_START ... IRQ_PIC1_END:
                   set_irq_chip(irq, &pecos_pic1);
                   set_irq_flags(irq, IRQF_VALID);
                   set_irq_handler(irq, do_level_IRQ);
                   break;

               case IRQ_TIM_START ... IRQ_TIM_END:
                  set_irq_chip(irq, &cactus_chip_tim);
                  set_irq_flags(irq, IRQF_VALID);
                  set_irq_handler(irq, do_level_IRQ);
                  break;

               default:
                  set_irq_chip(irq, &cactus_chip_gpio);
                  set_irq_flags(irq, IRQF_VALID);
                  set_irq_handler(irq, do_level_IRQ);
           }
        }
        set_irq_chained_handler(INT_TIMERS ,timer_irq_handler);
        set_irq_chained_handler(INT_GPIOS ,gpio_irq_handler);

	/* Enable IRQ mode for all PIC, No FIQ till now */
        __raw_writel(0xffffffff,ITC_INTDEST_REG(0,PIC_0)); 
        __raw_writel(0xffffffff,ITC_INTDEST_REG(0,PIC_1));
}

