/*
 * linux/arch/arm/mach-cactus/arch.c
 *
 * Copyright (C) 2004 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/****************************************************************************
 * $Header: mach.c, 7, 9/7/07 5:52:30 AM IST, Miles Bintz$
 ****************************************************************************/
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/cpufreq.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/mtd/physmap.h>

#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/arch/irq.h>

#include <asm/mach/time.h>
#include <asm/arch/time.h>

#include <linux/serial_core.h>

#include <asm/arch/startup.h>
#include <asm/hardware/cache-l2x0.h>

#include "board.h"

#define FALSE 0
#define TRUE 1

static unsigned int ArmClkFreq32, ArmClkPeriod32, MemClkFreq32, MemClkPeriod32;
extern void __init cnxt_init_irq( void );

/*
 * Here we setup the defualt sizes and locatinos of various
 * partitions in the flash.
 *
 * Adding the string CNXT_PARTN_TAG: <tag> to the size
 * field of a partition enables the cnxt_mk_jjfs2.sh
 * script to locate the size of the partition
 *
 */

#ifdef CONFIG_MTD_PARTITIONS
static struct mtd_partition cnxt_map_partitions[] = {
        {
                .name =          "codeldr",
                .size =          0x80000,
                .offset =        0x0000,
                .mask_flags =    MTD_WRITEABLE,  /* force read-only */
        },
        {
                .name =          "tlv kernel",
                .size =          0x400000,   
                .offset =        0x100000,
        },
        {
                .name =          "rootfs",
                .size =          0x300000,       /* CNXT_PARTN_TAG: ROOTFS */
                .offset =        0x500000,
        },
        {
                .name =          "appfs",
                .size =          0x800000,       /* CNXT_PARTN_TAG: APPFS */
                .offset =        0x800000,
        }

};

static int num_cnxt_map_partitions = (sizeof(cnxt_map_partitions)/sizeof(struct mtd_partition));
#endif

static struct map_desc cnxt_io_desc[] __initdata = 
{
	{
		.virtual	= ASX_VADDR_BASE, 
		 .pfn		= __phys_to_pfn(ASX_PHYS_BASE),
		.length		= ASX_IO_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual 	= EXT_IO_VADDR_BASE,
		.pfn		= __phys_to_pfn(EXT_IO_PHYS_BASE),
		.length 	= EXT_IO_SIZE,
		.type 		=  MT_DEVICE,
	}, {
		.virtual 	= TEMPEST_VADDR_BASE,
		.pfn		= __phys_to_pfn(TEMPEST_PHYS_BASE),
		.length 	= TEMPEST_IO_SIZE, 
		.type		= MT_DEVICE,
	}, {
		.virtual	= PCI_MEM_VADDR_BASE,
		.pfn 		= __phys_to_pfn(PCI_MEM_PHYS_BASE),
		.length 	=  PCI_MEM_SIZE,
		.type		=    MT_DEVICE,
	}, {
		.virtual 	= FLASH_VADDR_BASE,
		.pfn		= __phys_to_pfn(FLASH_PHYS_BASE),
		.length 	= FLASH_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual	= FLASHNC_VADDR_BASE,
		.pfn		= __phys_to_pfn(FLASH_PHYS_BASE),
		.length		= FLASH_SIZE,
		.type		=  MT_DEVICE,
	},
 };

unsigned long ChipID;
EXPORT_SYMBOL(ChipID);

unsigned long ChipRevID;
EXPORT_SYMBOL(ChipRevID);
    
static void __init cnxt_map_io(void)
{
    iotable_init( cnxt_io_desc, ARRAY_SIZE(cnxt_io_desc));
}

/* This function is used to get the chipid and chipRevId */

static void cnxt_get_chip_rev_id(struct machine_desc *pmach,
                               struct tag *ptag, char **pptr,
                                struct meminfo *pmem)
{
    LPREG pPCICfgAddr;
    LPREG pPCICfgData; 

    iotable_init(cnxt_io_desc , 1);
    iotable_init(cnxt_io_desc + 3 ,1);

    pPCICfgAddr = (LPREG) PCI_CFG_ADDR_REG;
    pPCICfgData = (LPREG) PCI_CFG_DATA_REG;

     /* Read vendor/device ID */
    *pPCICfgAddr = 0x0;
    ChipID = *pPCICfgData;

    /* Read Rev Id */
    *pPCICfgAddr = 0x8;
    ChipRevID = *pPCICfgData  & 0xFF;

    printk("ChipID=0x%lx ChipRevID=0x%lx\n",ChipID,ChipRevID);
}


unsigned int GetChipRev (void)
{
    return ChipRevID;
}

unsigned int GetChipID (void)
{
    return ChipID;
}

 void  GetCalcClkFreqAndPeriod( unsigned int *pMemClkFreq32 ,  unsigned int *pMemClkPeriod32)
{

  *pMemClkFreq32 = MemClkFreq32;
  *pMemClkPeriod32 = MemClkPeriod32;
  
   return;
}
EXPORT_SYMBOL(GetChipRev);
EXPORT_SYMBOL(GetChipID);

/*
static void __init fixup_cactus( struct machine_desc *desc,
            struct param_struct *params, char **cmdline, struct meminfo *mi )
{
  return;
}
*/

/* MFB: We're cheating... all edwards-based devices will have the same machine
 * type.  We could use the loader to pass machine-type and run-time support all
 * platforms, but this is an embedded systems and broader support means bigger 
 * kernels
 */


static void __init program_descriptor( ISA_DEVICE *Device )
{
   LPREG  pReg;
   LPREG  pPrimary;
   LPREG  pSecondary;

   pPrimary   = (LPREG)PCI_ISA_PRIMARY_DESC_ADDR(Device->DescNum);
   pSecondary = (LPREG)PCI_ISA_SECONDARY_DESC_ADDR(Device->DescNum);

   /*
    * Setup the descriptor designated in the ISADevice structure with the
    * parameters from the ISADevice structure.
    */

   *pPrimary = (PCI_ISA_DESC_TYPE_ISA                  |
      (Device->ExtWait << PCI_ISA_DESC_EXT_WAIT_SHIFT) |
      (Device->IOMemory << PCI_ISA_DESC_IO_TYPE_SHIFT) |
      (Device->ChipSelect << PCI_ISA_DESC_CS_SHIFT));

   switch (Device->IORegMode)
   {
      case 1:
         *pPrimary |= PCI_NEEDS_ISA_REG;
         *pPrimary &= ~PCI_ISA_DESC_REG_IS_PROG;
         break;
      case 2:
         *pPrimary |= PCI_NEEDS_ISA_REG;
         *pPrimary |= PCI_ISA_DESC_REG_IS_PROG;
         break;
      case 0:
      default:
         *pPrimary &= ~PCI_NEEDS_ISA_REG;
         *pPrimary &= ~PCI_ISA_DESC_REG_IS_PROG;
   }

   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_WIDTH_MASK) | 
                (Device->Width << PCI_ISA_DESC_WIDTH_SHIFT));

   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_INCR_MASK) |
                (Device->ISAIncr << PCI_ISA_DESC_INCR_SHIFT));

   /* Set syncIO mode if required */
   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_SYNC_IO_ENABLE_MASK) |
      (Device->bClocked << PCI_ISA_DESC_SYNC_IO_ENABLE_SHIFT));

   /*
    * The timings designated in the ISADevice structure specify absolute time
    * in ns. However, the descriptor itself interprets timings in number of
    * memclk cycles. Therefore, we have to convert the absolute time required
    * by the card into memclk cycles.
    */

   *pPrimary = ( (*pPrimary & ~PCI_ISA_DESC_WRITE_WAIT_MASK) |
      ( ( (unsigned int)( ((unsigned int)Device->WriteAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_ISA_DESC_WRITE_WAIT_SHIFT));
   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_READ_WAIT_MASK) |
      ( ( (unsigned int)( ((unsigned int)Device->ReadAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_ISA_DESC_READ_WAIT_SHIFT));

   *pPrimary &= ~(PCI_ISA_DESC_SETUP_TIME_MASK | PCI_ISA_DESC_HOLD_TIME_MASK);


   *pSecondary = (
      (( (unsigned int)( ((unsigned int)Device->IORegSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_ADDR_HOLD_TIME_SHIFT)  |
      (( (unsigned int)( ((unsigned int)Device->IORegAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_REG_ACCESS_TIME_SHIFT) |
      (( (unsigned int)( ((unsigned int)Device->CSSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CS_SETUP_TIME_SHIFT)   |
      (( (unsigned int)( ((unsigned int)Device->CSHoldTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CS_HOLD_TIME_SHIFT)    |
      (( (unsigned int)( ((unsigned int)Device->CtrlSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CTRL_SETUP_TIME_SHIFT) |
      (( (unsigned int)( ((unsigned int)Device->CtrlHoldTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_ADDR_HOLD_TIME_SHIFT));


   pReg = (LPREG)PCI_ROM_XOE_MASK_REG;
   *pReg = *pReg | (Device->XOEMask << Device->ChipSelect);

   return;
} /* program_descriptor */


static void __init config_isa_descriptors(void)
{
    LPREG pReg;
    int UsingSyncIO = FALSE;
    ISA_DEVICE Device;

    /*
     * Run-time select the Xtal frequency from the EEPROM data
     * <<<< Needs to be fixed ! >>>>
     */
#define xtal_frequency 74250000UL


    /**********************************/
    /* Approximate the MemClk period. */
    /**********************************/

    CalcClkFreqAndPeriod (&ArmClkFreq32, &ArmClkPeriod32, ARM_PLL_SOURCE, xtal_frequency, 0);
    CalcClkFreqAndPeriod (&MemClkFreq32, &MemClkPeriod32, MEM_PLL_SOURCE, xtal_frequency, 0);
    //printk( KERN_INFO "ArmClkFreq32 = 0x%x\n",ArmClkFreq32 );
    //printk( KERN_INFO "ArmClkPeriod32 = 0x%x\n",ArmClkPeriod32 );
    //printk( KERN_INFO "MemClkFreq32 = 0x%x\n",MemClkFreq32 );
    //printk( KERN_INFO "MemClkPeriod32 = 0x%x\n",MemClkPeriod32 );
   #ifdef CONFIG_PROGRAM_DESC_2
      Device.DescNum         = 2;
      Device.ChipSelect      = CONFIG_CNXT_DESC_2_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_2_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_2_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_2_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_2_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_2_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_2_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_2_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_2_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_2_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_2_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_2_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_2_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_2_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_2_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_2_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif
   #ifdef CONFIG_PROGRAM_DESC_3
      Device.DescNum         = 3;
      Device.ChipSelect      = CONFIG_CNXT_DESC_3_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_3_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_3_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_3_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_3_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_3_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_3_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_3_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_3_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_3_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_3_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_3_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_3_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_3_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_3_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_3_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_4
      Device.DescNum         = 4;
      Device.ChipSelect      = CONFIG_CNXT_DESC_4_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_4_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_4_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_4_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_4_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_4_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_4_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_4_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_4_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_4_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_4_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_4_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_4_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_4_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_4_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_4_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif
   #ifdef CONFIG_PROGRAM_DESC_5
      Device.DescNum         = 5;
      Device.ChipSelect      = CONFIG_CNXT_DESC_5_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_5_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_5_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_5_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_5_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_5_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_5_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_5_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_5_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_5_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_5_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_5_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_5_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_5_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_5_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_5_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_6
      Device.DescNum         = 6;
      Device.ChipSelect      = CONFIG_CNXT_DESC_6_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_6_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_6_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_6_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_6_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_6_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_6_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_6_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_6_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_6_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_6_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_6_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_6_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_6_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_6_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_6_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_7
      Device.DescNum         = 7;
      Device.ChipSelect      = CONFIG_CNXT_DESC_7_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_7_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_7_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_7_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_7_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_7_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_7_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_7_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_7_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_7_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_7_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_7_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_7_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_7_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_7_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_7_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

    /* If anything is using SyncIO, enable it */
    if(UsingSyncIO)
    {
        /*****************************************
         * Enable Program synchronous I/O clock  *
         *****************************************/
         pReg = (LPREG)ASX_TO_VIRT(PCI_ROM_MODE_REG);
         *pReg = *pReg | 0x400;
    }

}


typedef struct {
 LPREG preg;
 unsigned long val;
} cnxt_pll_config ;

static cnxt_pll_config conf_pll[] __initdata = {
  /*Initialize the GPIO pin mux and alt func registers for unique*/
#if CONFIG_PLL_CONFIG0_REG_DEFAULT
  { (LPREG)PLL_CONFIG0_REG, CONFIG_PLL_CONFIG0_REG_DEFAULT, },
#endif
   /*MUX settings*/
  { (LPREG)PLL_PIN_GPIO_MUX0_REG, CONFIG_PLL_PIN_GPIO_MUX0_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX1_REG, CONFIG_PLL_PIN_GPIO_MUX1_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX2_REG, CONFIG_PLL_PIN_GPIO_MUX2_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX3_REG, CONFIG_PLL_PIN_GPIO_MUX3_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX4_REG, CONFIG_PLL_PIN_GPIO_MUX4_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX5_REG, CONFIG_PLL_PIN_GPIO_MUX5_REG_DEFAULT, },
   /*Initialize the secondary pin mux registers*/  
  { (LPREG)SREG_2ND_PIN_MUX0_REG, CONFIG_SREG_2ND_PIN_MUX0_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX1_REG, CONFIG_SREG_2ND_PIN_MUX1_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX2_REG, CONFIG_SREG_2ND_PIN_MUX2_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX3_REG, CONFIG_SREG_2ND_PIN_MUX3_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX4_REG, CONFIG_SREG_2ND_PIN_MUX4_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX5_REG, CONFIG_SREG_2ND_PIN_MUX5_REG_DEFAULT, },
  { (LPREG)PLL_PIN_ALT_FUNC_REG, CONFIG_PLL_PIN_ALT_FUNC_REG_DEFAULT, },
 };
 

static int __init cnxt_init( void )
{
    LPREG pReg;
    int ret = 0;
    int i=0;

#ifdef CONFIG_MTD_PARTITIONS
    physmap_set_partitions(cnxt_map_partitions, num_cnxt_map_partitions);
#endif

    while ( i < ARRAY_SIZE(conf_pll) ) {
	pReg = conf_pll[i].preg;
	*pReg = conf_pll[i].val ; 
	i++;
    }

   /* Init all GPIO interrupt sources to known inactive state */
   pReg = (LPREG)GPI_POS_EDGE_REG;
   *pReg = 0;
   pReg = (LPREG)GPI_NEG_EDGE_REG;
   *pReg = 0;

   /* Make sure all GPIOs are inactive and in high impedance state */
   pReg = (LPREG)GPI_DRIVE_HIGH_REG;
   *pReg = 0;
   pReg = (LPREG)GPI_DRIVE_LOW_REG;
   *pReg = 0;
   pReg = (LPREG)GPI_DRIVE_OFF_REG;
   *pReg = 0xffffffff;

     config_isa_descriptors();

     return ret;
}

__initcall( cnxt_init );



/********************************************************************/
/*  FUNCTION:    hwlib_program_descriptor                           */
/*                                                                  */
/*  PARAMETERS:  device information for programming descriptor      */
/*                                                                  */
/*  DESCRIPTION: given a device description, program the requested  */
/*               descriptor to access that device                   */
/*                                                                  */
/*  RETURNS:     CNXT_STATUS_OK - success                           */
/*                                                                  */
/********************************************************************/

extern void idleAddCount(void);

static unsigned long cnxt_gettimeoffset(void)
{
  unsigned long ticks;

  ticks =  __raw_readl(TIMER_NUM_VALUE_REG(SYS_TIMER << 4));
  if (__raw_readl(0xE0430100) & 0x01)
       ticks += TIMER_INTERVAL;

  return (ticks) / (uSEC_1);
}

extern struct s_latency latency;

static irqreturn_t  cnxt_sys_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
   volatile unsigned int *timer_count_reg;

   timer_count_reg = (volatile unsigned int *)ASX_TO_VIRT( TIM_VALUE_REG );
   write_seqlock(&xtime_lock);
     __raw_writel( (__raw_readl(TIMER_NUM_MODE_REG(SYS_TIMER << 4)) & ~(TIM_INT_ENABLE_MASK)),
                         TIMER_NUM_MODE_REG(SYS_TIMER << 4)) ; /* Disable IRQ */

   latency.delta[latency.count++] =  *timer_count_reg;
   if (latency.count > LATENCY_COUNT) latency.count = 0;

    timer_tick(regs);
     __raw_writel( (__raw_readl(TIMER_NUM_MODE_REG(SYS_TIMER << 4)) | TIM_INT_ENABLE_MASK),
                         TIMER_NUM_MODE_REG(SYS_TIMER << 4)) ; /* Enable IRQ */
    write_sequnlock(&xtime_lock);
    return IRQ_HANDLED;
}

static struct irqaction cnxt_sys_timer = {
        .name = "TRINITY System Timer",
        .flags = IRQF_DISABLED | IRQF_TIMER,
        .handler = cnxt_sys_timer_interrupt
};

/*
 * Set up timer system timer interrupt
 */
static inline void cnxt_setup_timer( void )
{
    int i;

    for (i=0; i < NUM_TIM_BANKS; i++) {
      __raw_writel(0x0, TIMER_NUM_MODE_REG(i << 4)); /* Disable  timers */
      __raw_writel(0x0, TIMER_NUM_VALUE_REG(i << 4)); /* Clear Counter Value  */
    }
    /* System Timer -- Progam to ~ 10ms */
    /* Program Limit to 10ms Controlled by timer Base register 54 Mhz (Not sure
     *  how to program it), as  read/write resets to 0's */
    __raw_writel(TIMER_INTERVAL, TIMER_NUM_LIMIT_REG(SYS_TIMER << 4));

   setup_irq(INT_SYSTIMER, &cnxt_sys_timer);

    /* Enable irq generation by timer 0 */
     __raw_writel(TIM_INT_ENABLE_MASK | TIM_ENABLE_MASK, TIMER_NUM_MODE_REG(SYS_TIMER << 4));
}



static struct sys_timer cnxt_timer ={
	.init = &cnxt_setup_timer,
	.offset = &cnxt_gettimeoffset, 
 };

MACHINE_START( CACTUS, "Conexant Edwards-Based IRD" )
       /* maintainer:  "Conexant Systems Noida Design Center" */
	.phys_io	= 0xE0000000,
	.io_pg_offst	= ((0xfd000000) >> 18) & 0xfffc,
	.boot_params	= 0x00001000,
        .map_io 	= cnxt_map_io,
        .fixup          = cnxt_get_chip_rev_id,
	.init_irq 	= cnxt_init_irq, 	
	.timer 		= &cnxt_timer
MACHINE_END

