/****************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  */
/*                       SOFTWARE FILE/MODULE HEADER                        */
/*                    Conexant Systems Inc. (c) 2004-2005                   */
/*                               Austin, TX                                 */
/*                            All Rights Reserved                           */
/****************************************************************************/
/*
 * Filename:       mipidle.c
 *
 *
 * Description:    Main driver file for measuring MIPs left over
 *
 *
 * Author:         Miles Bintz (linux version), Glennon, White
 *
 ****************************************************************************/
/* $Id$
 ****************************************************************************/

#include <linux/kernel.h>

#include <asm/param.h> /* for HZ */



/**********
* Globals *
**********/
unsigned long gIdleCount = 0;

#define TICKS_PER_MIN  ((HZ) * 60)

static unsigned long idleSample[TICKS_PER_MIN];
static int index = 0;

void idleAddCount(void);
void cnxt_mip_idle(void)  __attribute__ ((pure));
   
void idleAddCount()
{
   idleSample[index++] = gIdleCount;
   if (index == TICKS_PER_MIN) index = 0;
   gIdleCount = 0;
}

int get_idle_list( char *buf ) {
    int i;
    int j; /* Cycles per second */
    int k; /* seconds per minute */
    int sum;
    
    char *p = buf;
    
    p += sprintf(p, "Idle MIPS for last minute:\n\n");

    if ( (index + 1) == TICKS_PER_MIN ) i = 0; else i = index;
    
    k = 60;
    while (k--)
    {
       sum = 0;
       j = HZ;
       while (j--)
       {
          sum += idleSample[i++];
          if (i == TICKS_PER_MIN) i = 0;
       }
       p += sprintf(p, "%2d: %10d\n", k, sum);
    }

    p += sprintf(p, "\n");

    return p - buf;
}

void cnxt_mip_idle() 
{
   /* TODO:  Fix these hardcoded register addresses.  Be more 
    * dyanmic with the use of which hwtimer to use.  Be more
    * dyanmic with detecticing CPU speed.
    */

   static volatile unsigned int *ptimVal = 0xe04300f0;
   static volatile unsigned int *ptimLim = 0xe04300f4;
   static volatile unsigned int *ptimMode = 0xe04300f8;
   static int bInited = 0;
   int timein, timeout;
   
   if (!bInited)
   {
      bInited = 1;
      *ptimVal = 0;
      *ptimLim = -1;
      *ptimMode = 0x1;
   }
      
   timein = *ptimVal;
   cpu_arm920_do_idle();
   timeout = *ptimVal;

   /* hw timer has 54 MHz clock input.  dT / 54 is number of elapsed
    * uSecs while idle.  Arm is running at 225 MHz, so you should, in 
    * theory, have about 225 instructions per microsecond
    */
   
   gIdleCount += ((timeout - timein) * 225) / 54;
}

