/****************************************************************************
 *
 *  arch/arm/mach-cactus/trinitypci.c
 *
 *  Copyright (C) 2002 Conexant Systems Inc, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/
/* $Header $
 ****************************************************************************/

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/init.h>
#include <linux/delay.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/pci.h>
#include <asm/arch/dma.h>
#include <asm/arch/trinity.h>
#include <asm/arch/intid.h>

#include <asm/mach-types.h>

#ifdef CONFIG_PCI

#define WORD_SIZE               4   /* bytes */

#undef  DEBUG              

static u8 cactus_swizzle(struct pci_dev *dev, u8 *pin);
static int cactus_setup(int nr, struct pci_sys_data *sys);
static int cactus_read_config(struct pci_bus *bus, unsigned int devfn, int where, int size, u32 *val);
static int cactus_write_config(struct pci_bus *bus, unsigned int devfn, int where,  int size, u32 val);
static void pci_arch_init( void  );
static void pci_arch_reset ( void );

/***********************************************************************
 *  dev->devfn:
 *	     7:3 = slot
 *	     2:0 = function
 ***********************************************************************/
static int cactus_read_config(struct pci_bus *bus, unsigned int devfn, int where, int size, u32 *val)
{
	switch (size)
	{
		case 1:
		{
		__raw_writel((where&0xFC) | (devfn<<PCI_CFG_ADDR_FUNCTION_SHIFT) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		*val=__raw_readb(ASX_TO_VIRT( PCI_CFG_DATA_REG | (where&3) ) );
	
#ifdef DEBUG
               	printk( KERN_INFO "  CNXT: read_config_byte : config addr:%08x idsel:%02x func:%02x data:%02x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), *val );
#endif
		}
		break;
		
		case 2:
		{
		__raw_writel((where&0xFC) | (devfn<<8) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		*val=__raw_readw(ASX_TO_VIRT( PCI_CFG_DATA_REG | (where&3) ) );
#ifdef DEBUG
               	printk( KERN_INFO "  CNXT: read_config_word : config addr:%08x idsel:%02x func:%02x data:%04x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), *val );
#endif
		}
                break;

		default:
		{
		__raw_writel(where | (devfn<<PCI_CFG_ADDR_FUNCTION_SHIFT) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		*val=__raw_readl(ASX_TO_VIRT( PCI_CFG_DATA_REG ));
#ifdef DEBUG
              	printk( KERN_INFO "  CNXT: read_config_dword : config addr:%08x idsel:%02x func:%02x data:%08x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), *val );
#endif
		}
		break;
	}
	return PCIBIOS_SUCCESSFUL;
}


static int cactus_write_config(struct pci_bus *bus, unsigned int devfn, int where,  int size, u32 val)

{
	switch(size)
	{
		case 1:
		{
#ifdef DEBUG
		printk( KERN_INFO "  CNXT: write_config_byte : config addr:%08x idsel:%02x func:%02x data:%02x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), val );
#endif
		__raw_writel((where&0xFC) | (devfn<<PCI_CFG_ADDR_FUNCTION_SHIFT) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		__raw_writeb(val, ASX_TO_VIRT( PCI_CFG_DATA_REG | (where&3) ) );
		}
		break;

		case 2:
		{
#ifdef DEBUG
		printk( KERN_INFO "  CNXT: write_config_word : config addr:%08x idsel:%02x func:%02x data:%04x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), val);
#endif
		__raw_writel((where&0xFC) | (devfn<<PCI_CFG_ADDR_FUNCTION_SHIFT) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		__raw_writew(val, ASX_TO_VIRT( PCI_CFG_DATA_REG | (where&3) ) );
		}
		break;

		default:
		{
#ifdef DEBUG
		printk( KERN_INFO "  CNXT: write_config_dword : config addr:%08x idsel:%02x func:%02x data:%08x\n", where, PCI_SLOT(devfn), PCI_FUNC(devfn), val );
#endif
		__raw_writel(where | (devfn<<PCI_CFG_ADDR_FUNCTION_SHIFT) , ASX_TO_VIRT( PCI_CFG_ADDR_REG ));
		__raw_writel(val, ASX_TO_VIRT( PCI_CFG_DATA_REG ));
		}
		break;
	}

	return PCIBIOS_SUCCESSFUL;
}

static struct pci_ops edwards_ops = {
	.read	=	cactus_read_config ,  
	.write	= 	cactus_write_config  
};  

static int __init cactus_setup(int nr, struct pci_sys_data *sys)
{
	struct resource *res;

#ifdef DEBUG
    	printk( KERN_INFO "CNXT: cactus_setup called\n" );
#endif

	if(nr != 0)
		return 0;

	res = kmalloc(sizeof(struct resource) * 2, GFP_KERNEL);
	if (!res)
		panic("PCI: unable to alloc resources");

	memset(res, 0, sizeof(struct resource) * 2);

	res[0].name   =  "CX241[6|7]X PCI I/O Bridge",
	res[0].start    =  EXT_IO_PHYS_BASE,
	res[0].end     =   EXT_IO_PHYS_BASE + PCI_IO_SIZE + - 1,
	res[0].flags    =   IORESOURCE_IO,

	res[1].name  ="CX241[6|7]X PCI MEM Bridge",
	res[1].start  =   PCI_MEM_PHYS_BASE,
	res[1].end    =  PCI_MEM_PHYS_BASE + PCI_MEM_SIZE - 1,
	res[1].flags  =   IORESOURCE_MEM,

	request_resource(&ioport_resource, &res[0]);
	request_resource(&iomem_resource, &res[1]);

	sys->resource[0] = &res[0];
	sys->resource[1] = &res[1];
	sys->resource[2] = NULL;

	return 1;
}

struct pci_bus *__init cactus_scan_bus(int nr, struct pci_sys_data *sys)
{
#ifdef DEBUG
    	printk( KERN_INFO "CNXT: cactus_scan_bus called\n" );
#endif
  
    	return pci_scan_bus(sys->busnr, &edwards_ops, sys);
}

/***********************************************************************/

static void __init pci_arch_init(void)     /* struct pci_sys_data sysdata  */
{
    	volatile unsigned int v32;

#ifdef DEBUG
    	printk( KERN_INFO "CNXT: pci_arch_init:\n" );
#endif

    	/* Initialize Trinity PCI controller  */

    	/*
       	NOTE -- NOTE -- NOTE: we rely on some power-up defaults:

        The bridge window at 0x40 comes out of reset at 64mb. We accept
	this default. Also, the remap address at 0x40 comes up with bit 28
	non-cacheable aperture) set. We accept this (kinda' got to since you
	can't turn off bit 28 no matter how hard you write 0x40 :-).

       	NOTE!!!
        I changed the default latency timer from 32 to 64 in bios32.c in the
		  function pcibios_fixup_bus(). You may wish something different!
    	*/

    	/* On trinity, we need to set the PCI Clock delay registers.  The value
     	* below induces about a 1ns delay */
    	__raw_writel( (1 << PCI_CLK_DELAY_BIG_SHIFT) | (0 << PCI_CLK_DELAY_SMALL_SHIFT),
                 ASX_TO_VIRT( PCI_CLK_DELAY_REG ) );

    	/* Set alt func register so PCI_REQGNT1 is on PIO006-7 instead of normal PCI_REQ pins */
    	v32 = __raw_readl(ASX_TO_VIRT(PLL_PIN_ALT_FUNC_REG));
    	__raw_writel(v32 
                  & ~((PLL_PIN_ALT_FUNC_PCI_REQGNT1_MASK | PLL_PIN_ALT_FUNC_IO_RW_MASK)
                  | PLL_PIN_ALT_FUNC_PCI_REQGNT1_PIO_7_6
                  | PLL_PIN_ALT_FUNC_IO_RW_NORMAL),
                  ASX_TO_VIRT(PLL_PIN_ALT_FUNC_REG));

    	/* On newer chips (Wabash and beyond), need to set the PCI sync bits */
    	v32 = __raw_readl(ASX_TO_VIRT( PCI_ROM_MODE_REG));
    	__raw_writel( (v32 & 0x3ff00)
                      | PCI_ROM_PREFETCH_ENABLE
                      | (1UL << PCI_ROM_REQ0_SYNC_SHIFT)
                      | (1UL << PCI_ROM_REQ1_SYNC_SHIFT)
                      | (1UL << PCI_ROM_REQ2_SYNC_SHIFT),
                  ASX_TO_VIRT(PCI_ROM_MODE_REG) );

    	/* Remap register: 31:29 Memory remapping bits, 15:4 IO remap bits  */
	/*  This gives us a straight-thru mapping i.e. CPU addr == PCI addr */
    	__raw_writel( 0x8000E100, ASX_TO_VIRT(PCI_REMAP_REG) );

    	/* Take PCI out of reset */
    	__raw_writel( 0, ASX_TO_VIRT(PCI_RESET_REG) );

    	/* Enable the bridge mode to forward interrupts */
    	__raw_writel( PCI_INTR_INTA_ENABLE_MASK, ASX_TO_VIRT(PCI_INTR_ENABLE_REG) );

    	/* set Memory Space / Bus Master bits */
    	__raw_writel( PCI_CMD_STAT_OFF << PCI_CFG_ADDR_REGISTER_SHIFT, ASX_TO_VIRT(PCI_CFG_ADDR_REG) );
    
    	v32 = __raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
    	__raw_writel( v32 
                   | PCI_CMD_MASTER_ENABLE_MASK
                   | PCI_CMD_MEM_ACCESS_ENABLE_MASK,
                  ASX_TO_VIRT(PCI_CFG_DATA_REG) );

#ifdef DEBUG
    	v32=__raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
    	printk( KERN_INFO "  CNXT: wrote 0x%x to bridge cmd_status\n",v32);
#endif

    	/************************************************************************/
    	/******** NOTE: further treatment of Trinity Bridge in bios32.c **********/
    	/************************************************************************/
	/*
	* Clear any error conditions
	*/
    	__raw_writel(PCI_CMD_STAT_OFF << PCI_CFG_ADDR_REGISTER_SHIFT,  ASX_TO_VIRT( PCI_CFG_ADDR_REG) );
    	v32=__raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
    	__raw_writel( v32 | 0xF8000000 , ASX_TO_VIRT(PCI_CFG_DATA_REG) );

#if 1
    	//Eden HACK start
    	__raw_writel( PCI_CLS_LT_HT_BIST_OFF << PCI_CFG_ADDR_REGISTER_SHIFT, ASX_TO_VIRT(PCI_CFG_ADDR_REG) );
    	__raw_writel( 0x4000 , ASX_TO_VIRT(PCI_CFG_DATA_REG) );

    	__raw_writel( PCI_BASE_ADDR_0_OFF << PCI_CFG_ADDR_REGISTER_SHIFT, ASX_TO_VIRT(PCI_CFG_ADDR_REG) );
    	__raw_writel( 0x05000000 , ASX_TO_VIRT(PCI_CFG_DATA_REG) );

    	__raw_writel( 0x10 << PCI_CFG_ADDR_REGISTER_SHIFT, ASX_TO_VIRT(PCI_CFG_ADDR_REG) );
    	__raw_writel( 0x00000094 , ASX_TO_VIRT(PCI_CFG_DATA_REG) );
    	//HACK end
#endif

#ifdef DEBUG
    	v32=__raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
	printk( KERN_INFO "  CNXT: cmd_status - 0x%x\n",v32 );
    	__raw_writel( 0x10 << PCI_CFG_ADDR_REGISTER_SHIFT,  ASX_TO_VIRT( PCI_CFG_ADDR_REG) );
    	v32=__raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
	printk( KERN_INFO "  CNXT: base0 remap / size - 0x%x\n",v32 );
    	__raw_writel( 0x11 << PCI_CFG_ADDR_REGISTER_SHIFT,  ASX_TO_VIRT( PCI_CFG_ADDR_REG) );
    	v32=__raw_readl( ASX_TO_VIRT(PCI_CFG_DATA_REG) );
	printk( KERN_INFO "  CNXT: base1 remap / size - 0x%x\n",v32 );
#endif

}

static u8 cactus_swizzle(struct pci_dev *dev, u8 *pin)
{
#ifdef DEBUG
    	printk( KERN_INFO "CNXT: cactus_swizzle called\n" );
#endif

	return 0;
}

static int __init pci_arch_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
#ifdef DEBUG
    	printk( KERN_INFO "CNXT: pci_arch_map_irq called\n" );
#endif

	return INT_PCI;
}

struct hw_pci trinity_pci __initdata = {
	.nr_controllers	=  	1,	
	.setup          =	cactus_setup,
	.scan           =  	cactus_scan_bus,
	.preinit        =  	pci_arch_init,
	.swizzle	=   	cactus_swizzle, 
	.map_irq	=   	pci_arch_map_irq
};


/***********************************************************************

static struct resource pci_arch_ioport = {
	name:	"CX241[6|7]X PCI I/O Bridge",
	start:	EXT_IO_PHYS_BASE,
	end:	EXT_IO_PHYS_BASE + PCI_IO_SIZE + - 1,
	flags:	IORESOURCE_IO,
};

static struct resource pci_arch_iomem = {
	name:	"CX241[6|7]X PCI MEM Bridge",
	start:	PCI_MEM_PHYS_BASE,
	end:	PCI_MEM_PHYS_BASE + PCI_MEM_SIZE - 1,
	flags:	IORESOURCE_MEM,
};

***********************************************************************/

static void pci_arch_reset ( void )
{

#ifdef DEBUG
    	printk( KERN_INFO "CNXT: pci_arch_reset called\n" );
#endif

    	__raw_writel(1 , ASX_TO_VIRT( PCI_RESET_REG ));
    	__raw_writel(0 , ASX_TO_VIRT( PCI_RESET_REG ));

	return;
}

int __init cactus_pci_init(void)
{

#ifdef DEBUG
    	printk( KERN_INFO "CNXT: cactus_pci_init called\n" );
#endif
    	pci_common_init(&trinity_pci);
	return 0;
}

subsys_initcall(cactus_pci_init);

#endif

/****************************************************************************
 * Modifications:
 * $Log$
 *
 ****************************************************************************/

