/****************************************************************************
 *
 * arch/arm/mach-bronco/pllc.c
 *
 * Copyright (C) 2004 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/
/* $Id $
 ****************************************************************************/

/******************/
/* Include Files  */
/******************/
#include <asm/arch/hardware.h>
#include <asm/arch/startup.h>



/* FIXME: TODO:  Need to use  a global definition of CNXT_GET */
#define CNXT_GET(a, b)  (*((volatile unsigned int*)ASX_TO_VIRT(a)) & b)

/********************************************************************/
/*  CalcFreqFromFracPart                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Calculate ( freq * pll_frac ) / ( 2 ^ frac_len )            */
/*      Note that ( freq * pll_frac ) is much bigger                */
/*      than unsigned 0xffffffff.  The algorithm used may result    */
/*      up to 3 less than what should be                            */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      See description                                             */
/*                                                                  */
/*  RETURNS:                                                        */
/*      result described above                                      */
/********************************************************************/
static unsigned int CalcFreqFromFracPart ( unsigned int freq,
                                      unsigned int pll_frac,
                                      unsigned int frac_len )
{
   unsigned int k, l, freq0, frac0, retval;

   if ( frac_len == 25 )
   {
      k = 13;
      l = 12;
   }
   else if ( frac_len == 16 )
   {
      k = 8;
      l = 8;
   }
   else
   {
      return 0xffffffff;
   }

   freq0    = freq >> k;
   freq     = freq - ( freq0 << k );
   frac0    = pll_frac >> l;
   pll_frac = pll_frac - ( frac0 << l );

   retval = freq0 * frac0 + ( ( freq0 * pll_frac ) >> l )
                          + ( ( frac0 * freq ) >> k )
                          + ( ( freq * pll_frac ) >> frac_len );

   return retval;
}

/********************************************************************/
/*  CalkClkFreqAndPeriod                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Calculate clock frequency and period for a specific PLL.    */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      pll_source - One of:  ARM_PLL_SOURCE, MEM_PLL_SOURCE        */
/*      xtal_freq  - Xtal frequency in Hz                           */
/*      raw        - Do not include the post divider (i.e. VCO)     */
/*                                                                  */
/*  RETURNS:                                                        */
/*      frequency  - Returned clock frequency in Hz                 */
/*      period     - Returned clock period in 100ns increments      */
/********************************************************************/
void CalcClkFreqAndPeriod (unsigned int *frequency,
                           unsigned int *period,
                           unsigned int pll_source,
                           unsigned int xtal_freq,
                           int raw )
{
    unsigned int pll_int, pll_frac, pll_div, pll_prescale, clk_freq, remainder, 
                 frac_shift, clk_period;

    *frequency = 0;
    *period    = 0;

    /*
     * Calculate the clk_freq and clk_period
     */

    /* Calculation below depends on the PLL details of the chip.  */
    /* Colorado/Hondo use a pre-divide of 1 and an XTAL frequency */
    /* of 14.31818MHz. Wabash uses a pre-divide of 3, another     */
    /* un-documented pre-divide of 2, and either a XTAL frequency */
    /* of 36.4089MHz (USA) or 46.8114MHz (Europe).                */

    /* In the following calculations, the frac value read from the*/
    /* PLL multiplier must be first converted to a floating       */
    /* point value, then divided by the number of bits used to    */
    /* represent the frac value (2^16) represented as a floating  */
    /* point value. This will cause the fixed-point integer       */
    /* representation of a fractional value to be converted to a  */
    /* floating-point fractional value.                           */

    /*
     * Read the PLL values from the chip
     */
    switch(pll_source)
    {
        case ARM_PLL_SOURCE:
            pll_int  = CNXT_GET(PLL_ARM_INTFRAC_REG, PLL_ARM_INTFRAC_INT_MASK) >>
                                                    PLL_ARM_INTFRAC_INT_SHIFT;
            pll_frac = CNXT_GET(PLL_ARM_INTFRAC_REG, PLL_ARM_INTFRAC_FRAC_MASK) >>
                                                    PLL_ARM_INTFRAC_FRAC_SHIFT;
            pll_div  = CNXT_GET(PLL_DIV0_REG,       PLL_DIV0_ARMCLK_POSTDIV_MASK) >>
                                                    PLL_DIV0_ARMCLK_POSTDIV_SHIFT;
            pll_prescale = PLL_PRESCALE ( 
            ((*(volatile unsigned int *)ASX_TO_VIRT(PLL_PRESCALE_REG))&PLL_PRESCALE_ARM_MASK)>>PLL_PRESCALE_ARM_SHIFT);

            frac_shift = PLL_ARM_INTFRAC_FRAC_SHIFT;

            break;

        case MEM_PLL_SOURCE:

            pll_int  = CNXT_GET(PLL_MEM_INTFRAC_REG, PLL_MEM_INTFRAC_INT_MASK) >>
                                            PLL_MEM_INTFRAC_INT_SHIFT;
            pll_frac = CNXT_GET(PLL_MEM_INTFRAC_REG, PLL_MEM_INTFRAC_FRAC_MASK) >>
                                            PLL_MEM_INTFRAC_FRAC_SHIFT;
            pll_div  = CNXT_GET(PLL_DIV0_REG, PLL_DIV0_MEMCLK_POSTDIV_MASK) >>
                                            PLL_DIV0_MEMCLK_POSTDIV_SHIFT;
            pll_prescale = PLL_PRESCALE (
            ((*(volatile unsigned int*)ASX_TO_VIRT(PLL_PRESCALE_REG))&PLL_PRESCALE_MEM_MASK)>>PLL_PRESCALE_MEM_SHIFT);

            frac_shift = PLL_MEM_INTFRAC_FRAC_SHIFT;

            break;

       default:
            return;
    }

  #ifdef DLOAD

    /*
     * If called from the Download (DLOAD) Utility, run-time select the
     * appropriate calculation function.
     */

    /* BRAZOS and beyond */

     xtal_freq    = 16317333;
     pll_prescale = 1;

  #endif /* DLOAD */

    /*
     * If raw, do not factor in the post divide value, just use pre-scale.
     * Otherwise, multiply the pre-scale with the post divide value.
     */
    if(raw)
    {
        pll_div = pll_prescale;
    }
    else
    {
        pll_div = pll_div * pll_prescale;
    }

    /* 
     * calculate clk_freq = xtal_freq*(pll_int+pll_frac/2^n)/pll_div
     * note that algorithm used here may result up to 4Hz less than what it should be
     */
    clk_freq  = xtal_freq / pll_div;
    remainder = xtal_freq % pll_div;

    clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (25-frac_shift) )
                      + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div);

    /*
     * Calculate clk_period = (10^11)/clk_freq
     * note that the algorithm will be broken if clk_freq is higher
     * than 429.5MHz.  it should not happen in the predictable future.
     */
    clk_period  = ( 1000000000 / clk_freq ) * 100;
    remainder   = ( 1000000000 % clk_freq ) * 10;

    clk_period += ( remainder / clk_freq ) * 10;
    remainder   = ( remainder % clk_freq ) * 10;

    clk_period += remainder / clk_freq;

    /*
     * Return the values
     */
    *frequency = clk_freq;
    *period    = clk_period;
}

/****************************************************************************
 * Modifications:
 * $Log$
 ****************************************************************************/
