/**************************************************************************
 *
 * arch/arm/mach-bronco/startupc.c
 *
 *
 * Copyright (C) 2004 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 ****************************************************************************/
/* $Id $
 ****************************************************************************/

/******************/
/* Include Files  */
/******************/
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/list.h>

#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <asm/arch/startup.h>
#include "board.h"

 
/********************************/
/* External Function Prototypes */
/********************************/
extern BOARD_TYPES BoardTypes[];

#define TRUE 1
#define FALSE 0

/**********************/
/* Local Definitions  */
/**********************/

/***************/
/* Global Data */
/***************/
unsigned int ArmClkFreq32, ArmClkPeriod32, MemClkFreq32, MemClkPeriod32;


/********************************************************************/
/*                                                                  */
/********************************************************************/
void CactusStartup(void)
{
    LPREG               pReg, pPrimary, pSecondary;
    int                 Card;
    unsigned int        UsingSyncIO = FALSE;
    unsigned int        MemClkPeriod;
	volatile unsigned int tmp;



    /*
     * Run-time select the Xtal frequency from the EEPROM data
     * <<<< Needs to be fixed ! >>>>
     */
#define xtal_frequency 74250000UL


    /**********************************/
    /* Approximate the MemClk period. */
    /**********************************/

    CalcClkFreqAndPeriod (&ArmClkFreq32, &ArmClkPeriod32, ARM_PLL_SOURCE, xtal_frequency, 0);
    CalcClkFreqAndPeriod (&MemClkFreq32, &MemClkPeriod32, MEM_PLL_SOURCE, xtal_frequency, 0);
    //MemClkPeriod = (float)MemClkPeriod32 / 100.0;
    // MemClkPeriod32 is in 100ns units, use this fact below...
    MemClkPeriod = MemClkPeriod32;
    printk( KERN_DEBUG "ArmClkFreq32 = 0x%x\n",ArmClkFreq32 );
    printk( KERN_DEBUG "ArmClkPeriod32 = 0x%x\n",ArmClkPeriod32 );
    printk( KERN_DEBUG "MemClkFreq32 = 0x%x\n",MemClkFreq32 );
    printk( KERN_DEBUG "MemClkPeriod32 = 0x%x\n",MemClkPeriod32 );

    /* If anything is using SyncIO, enable it */
    if(UsingSyncIO)
    {
        /*****************************************
         * Enable Program synchronous I/O clock  *
         *****************************************/
         pReg = (LPREG)ASX_TO_VIRT(PCI_ROM_MODE_REG);
         *pReg = *pReg | 0x400;
    }


    /*****************************/
    /* Clear out GPIO registers. */
    /*****************************/

    /*
     * Init all GPIO interrupt sources to known inactive state
     */
    pReg = (LPREG)ASX_TO_VIRT(GPI_POS_EDGE_REG);
    *pReg = 0;
    pReg = (LPREG)ASX_TO_VIRT(GPI_NEG_EDGE_REG);
    *pReg = 0;

    /*
     * Make sure all GPIOs are inactive and in high impedance state
     */
    pReg = (LPREG)ASX_TO_VIRT(GPI_DRIVE_HIGH_REG);
    *pReg = 0;
    pReg = (LPREG)ASX_TO_VIRT(GPI_DRIVE_LOW_REG);
    *pReg = 0;
    pReg = (LPREG)ASX_TO_VIRT(GPI_DRIVE_OFF_REG);
    *pReg = 0xffffffff;


#define PLL_PIN_ALT_FUNC_REG_DEFAULT  0x00100020
#define PLL_PIN_GPIO_MUX0_REG_DEFAULT 0xE0000C07
#define PLL_PIN_GPIO_MUX1_REG_DEFAULT 0x43FFFAFF
#define PLL_PIN_GPIO_MUX2_REG_DEFAULT 0x00000200
#define PLL_PIN_GPIO_MUX3_REG_DEFAULT 0x00000300

    pReg = (LPREG)ASX_TO_VIRT(PLL_PIN_ALT_FUNC_REG);
    *pReg = PLL_PIN_ALT_FUNC_REG_DEFAULT;

    pReg = (LPREG)ASX_TO_VIRT(PLL_PIN_GPIO_MUX0_REG);
    *pReg = PLL_PIN_GPIO_MUX0_REG_DEFAULT;

    pReg = (LPREG)ASX_TO_VIRT(PLL_PIN_GPIO_MUX1_REG);
    *pReg = PLL_PIN_GPIO_MUX1_REG_DEFAULT;

    pReg = (LPREG)ASX_TO_VIRT(PLL_PIN_GPIO_MUX2_REG);
    *pReg = PLL_PIN_GPIO_MUX2_REG_DEFAULT;

    pReg = (LPREG)ASX_TO_VIRT(PLL_PIN_GPIO_MUX3_REG);
    *pReg = PLL_PIN_GPIO_MUX3_REG_DEFAULT;
        
}

/****************************************************************************
 * Modifications:
 * $log[8] $
*****************************************************************************/

