/*********************************************************************************
 *  linux/arch/arm/mach-pecos/irq.c
 *
 *  Copyright (C) 2007 Conexant Systems Inc. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 *********************************************************************************/
/*$Id: irq.c,v 1.4, 2007-03-27 11:46:01Z, Satpal Parmar$
 *********************************************************************************/



/**************************************************/
/*   PREPROCESSOR                                 */
/**************************************************/
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/stddef.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/time.h>

#include <asm/irq.h>
#include <asm/io.h>

#include <asm/mach/irq.h>
#include <asm/arch/hardware.h>
#include <asm/arch/intid.h>
#include <asm/arch/irq.h>

#define GPIO_IRQSTATUS_FROMBANK(x)  ASX_TO_VIRT( (GPI_INT_STAT_REG + ((x) * GPI_BANK_SIZE)) )
#define GPIO_IRQENABLE_FROMBANK(x)  ASX_TO_VIRT( (GPI_INT_ENABLE0_REG + ((x) * GPI_BANK_SIZE)) )

#define IRQ_PIC0_START  0
#define IRQ_PIC0_END    (IRQ_PIC0_START + 31)
#define IRQ_PIC1_START  IRQ_PIC0_END + 1   
#define IRQ_PIC1_END    (IRQ_PIC1_START + 31)
#define IRQ_PIC2_START IRQ_PIC1_END + 1
#define IRQ_PIC2_END   IRQ_PIC2_START + 31 
#define IRQ_TIM_START  64
#define IRQ_TIM_END    (IRQ_TIM_START + 15) // Rest is unused so far

#define IRQ_SM2_UNUSED  (IRQ_PIC0_START + 5)
#define IRQ_TIMER1_UNUSED  (IRQ_PIC1_START + 7)


/**************************************************/
/*    TYPEDEFs                                    */
/**************************************************/
typedef volatile unsigned int reg;
typedef reg *lpreg;
struct s_latency latency = { 
    count: 0
};

/**************************************************/
/*    STATIC GLOBALS                              */
/**************************************************/

int get_latency_list( char *buf ) 
{
    int min=1000000, avg=0, max=0;
    int mindur=1000000, avgdur=0, maxdur=0;
    int i;
    char *p = buf;
    
    p += sprintf(p, "Sample size = %d.  freq(sample) = freq(tick) (%d mS)\n\n", LATENCY_COUNT, 1000/HZ);

    for (i = 0; i < LATENCY_COUNT; i++) {
        if (latency.duration[i] < mindur) mindur = latency.duration[i];
        if (latency.duration[i] > maxdur) maxdur = latency.duration[i];
        if (latency.delta[i] < min) min = latency.delta[i];
        if (latency.delta[i] > max) max = latency.delta[i];
        avg += latency.delta[i];
        avgdur += latency.duration[i];
    }
    p += sprintf(p, "min lat : %d uS\n", min/54 );
    p += sprintf(p, "avg lat : %d uS\n", avg / (LATENCY_COUNT * 54) );
    p += sprintf(p, "max lat : %d uS\n", max/54 );
    
    p += sprintf(p, "min dur : %d uS\n", mindur );
    p += sprintf(p, "avg dur : %d uS\n", avgdur / LATENCY_COUNT );
    p += sprintf(p, "max dur : %d uS\n", maxdur );

    return p - buf;
}

static void pic0_ack_irq(unsigned int irq)
{
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_0)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic0_mask_irq(unsigned irq)
{
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic0_unmask_irq(unsigned irq)
{
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_0)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_0)) | (1 << irq)) ,ITC_INTENABLE_REG(0,PIC_0)); 
}

static void pic1_ack_irq(unsigned int irq)
{
  irq -= IRQ_PIC1_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_1)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
  if ( irq == ITC_PCI_POS) {
    __raw_writel((__raw_readl(PCI_INTR_STATUS_REG) | PCI_INTR_INTA_ENABLE_MASK) ,PCI_INTR_STATUS_REG); 
  }
}

static void pic1_mask_irq(unsigned irq)
{
  irq -= IRQ_PIC1_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
  if ( irq == ITC_PCI_POS) {
    __raw_writel((__raw_readl(PCI_INTR_STATUS_REG) | PCI_INTR_INTA_ENABLE_MASK) ,PCI_INTR_STATUS_REG); 
  }
}

static void pic1_unmask_irq(unsigned irq)
{
  irq -= IRQ_PIC1_START;
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_1)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_1)) | (1 << irq)) ,ITC_INTENABLE_REG(0,PIC_1)); 
}

static void pic2_ack_irq(unsigned int irq)
{
  irq -= IRQ_PIC2_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_2)); 
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_2)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_2)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_2)); 
}

static void pic2_mask_irq(unsigned irq)
{
  irq -= IRQ_PIC2_START;
  __raw_readl(ITC_INTRIRQ_REG(0,PIC_2)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_2)) & ~(1 << irq)) ,ITC_INTENABLE_REG(0,PIC_2)); 
}

static void pic2_unmask_irq(unsigned irq)
{
  irq -= IRQ_PIC2_START;
  __raw_writel( 1 << irq ,ITC_INTSTATCLR_REG(0,PIC_2)); 
  __raw_writel((__raw_readl(ITC_INTENABLE_REG(0,PIC_2)) | (1 << irq)) ,ITC_INTENABLE_REG(0,PIC_2)); 
}


/**************************************************/
/*   FUNCTION DEFINITIONS - gpio pic              */
/**************************************************/

static void mask_and_ack_irq_pic_gpio (unsigned int irq) { 
    int bank;
    int bit;
    lpreg sta, ena;

    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;

    sta = (lpreg)(GPIO_IRQSTATUS_FROMBANK(bank)); 
    ena = (lpreg)(GPIO_IRQENABLE_FROMBANK(bank));
    *ena &= ~(1 << bit); 
    *sta |= (1 << bit); 
}

static void mask_irq_pic_gpio (unsigned int irq) {
    int bank;
    int bit;
    lpreg ena;
 
    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;
    
    ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
    *ena &= ~(1 << bit);
} 

static void unmask_irq_pic_gpio (unsigned int irq) {
    int bank;
    int bit;
    lpreg ena;

    /* which gpio number are we? */
    irq -= CALC_INT_ID(PIC_GPIO, 0);
    
    bank = irq / 32;
    bit  = irq % 32;

    ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
    *ena |= (1 << bit); 
} 

static struct irqchip cnxt_pic0 = {
	.ack	= pic0_ack_irq,
	.mask	= pic0_mask_irq,
	.unmask = pic0_unmask_irq,
};

static struct irqchip cnxt_pic1 = {
	.ack	= pic1_ack_irq,
	.mask	= pic1_mask_irq,
	.unmask = pic1_unmask_irq,
};

static struct irqchip cnxt_pic2 = {
	.ack	= pic2_ack_irq,
	.mask	= pic2_mask_irq,
	.unmask = pic2_unmask_irq,
};

static struct irqchip cnxt_chip_gpio = {
	.ack	= mask_and_ack_irq_pic_gpio,
	.mask	= mask_irq_pic_gpio,
	.unmask = unmask_irq_pic_gpio,
};
/**************************************************/
/*                                                */
/**************************************************/
static void gpio_irq_handler(unsigned int irq, struct irqdesc *desc,
            struct pt_regs *regs)
{
   register unsigned int shift;
   register int irqbit;
   register int r;
   register int gpio_r;
   struct irqdesc *d; 
   int bank;

    desc->chip->ack(irq);
    for (bank = 0; bank < NUM_GPI_BANKS; bank++)
    {
           lpreg sta = (lpreg)GPIO_IRQSTATUS_FROMBANK(bank);
           lpreg ena = (lpreg)GPIO_IRQENABLE_FROMBANK(bank);
        
           irqbit = *ena & *sta;
           if (irqbit)
           {
              r = 0;
              shift = ( ( irqbit & 0xFFFF0000 ) != 0 ) << 4; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xFF00     ) != 0 ) << 3; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xF0       ) != 0 ) << 2; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0xC        ) != 0 ) << 1; irqbit >>= shift; r |= shift;
              shift = ( ( irqbit & 0x2        ) != 0 ) << 0; irqbit >>= shift; r |= shift;
              gpio_r= CALC_INT_ID(PIC_GPIO, r + (32*bank));
	      d = irq_desc + gpio_r; 
              desc_handle_irq(gpio_r,d,regs);
              break;
            } else {
              continue;
           }
    }
    desc->chip->unmask(irq);
}


void __init cnxt_init_irq( void )
{
  unsigned int irq;

  /* Disable Interrupt Generation */ 
  __raw_writel(0x0,ITC_INTENABLE_REG(0,PIC_0)); 
  __raw_writel(0x0,ITC_INTENABLE_REG(0,PIC_1)); 
  __raw_writel(0x0,ITC_INTENABLE_REG(0,PIC_2)); 

  for (irq = 0; irq < NR_IRQS  ; irq++)
    {
       if ( (irq == IRQ_SM2_UNUSED) || (irq == IRQ_TIMER1_UNUSED))
		continue;
       switch(irq) {
          case IRQ_PIC0_START ... IRQ_PIC0_END:
            set_irq_chip(irq, &cnxt_pic0);
            set_irq_flags(irq, IRQF_VALID);
            set_irq_handler(irq, do_level_IRQ);
            break;

          case IRQ_PIC1_START ... IRQ_PIC1_END:
             set_irq_chip(irq, &cnxt_pic1);
             set_irq_flags(irq, IRQF_VALID);
             set_irq_handler(irq, do_level_IRQ);
             break;

          case IRQ_PIC2_START ... IRQ_PIC2_END:
             set_irq_chip(irq, &cnxt_pic2);
             set_irq_flags(irq, IRQF_VALID);
             set_irq_handler(irq, do_level_IRQ);
             break;

          default:
             set_irq_chip(irq, &cnxt_chip_gpio);
             set_irq_flags(irq, IRQF_VALID);
             set_irq_handler(irq, do_level_IRQ);
           }
        }
//        set_irq_chained_handler(INT_TIMERS ,timer_irq_handler);
        set_irq_chained_handler(INT_GPIO ,gpio_irq_handler);

	/* Enable IRQ mode for all PIC, No FIQ till now */
        __raw_writel(0xffffffff,ITC_INTDEST_REG(0,PIC_0)); 
        __raw_writel(0xffffffff,ITC_INTDEST_REG(0,PIC_1));
        __raw_writel(0xffffffff,ITC_INTDEST_REG(0,PIC_2));
}

/**************************************************************************************
* Modifications:
* $Log:
*  5    Linux_SDK 1.4         3/27/2007 5:16:01 PM   Satpal Parmar   GPL header
*       and starteam footer addition.
*  4    Linux_SDK 1.3         3/9/2007 8:37:47 PM    Vineet Seth     Updated
*       PCI according to 2.6.11.12 code..and added PIC mask flag for PCI
*  3    Linux_SDK 1.2         3/7/2007 6:46:08 PM    Vineet Seth     GPIO
*       interrupt handling corrected...
*  2    Linux_SDK 1.1         3/3/2007 5:28:31 PM    Vineet Seth     Enabling
*       of GPIO interrupt processing...
*  1    Linux_SDK 1.0         3/1/2007 10:52:57 PM   Vineet Seth     
* $
*
***************************************************************************************/












