/****************************************************************************
 *
 *  arch/arm/mach-pecos/pll.c
 *
 *  Copyright (C) 2007 Conexant Systems Inc, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/
/*$Id: pllc.c,v 1.1, 2007-03-27 11:46:08Z, Satpal Parmar$
 ****************************************************************************/

/******************/
/* Include Files  */
/******************/
#include <asm/arch/hardware.h>
#include <asm/arch/startup.h>

/* FIXME: TODO:  Need to use  a global definition of CNXT_GET */
#define CNXT_GET(a, b)  (*((volatile unsigned int*)ASX_TO_VIRT(a)) & b)
#define CNXT_GET_VAL(reg,mask)           ((*((volatile unsigned int*)ASX_TO_VIRT(reg)) & (mask)) >> RMO(mask))

/********************************************************************/
/*  CalcFreqFromFracPart                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Calculate ( freq * pll_frac ) / ( 2 ^ frac_len )            */
/*      Note that ( freq * pll_frac ) is much bigger                */
/*      than unsigned 0xffffffff.  The algorithm used may result    */
/*      up to 3 less than what should be                            */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      See description                                             */
/*                                                                  */
/*  RETURNS:                                                        */
/*      result described above                                      */
/********************************************************************/
static unsigned int CalcFreqFromFracPart(
                    unsigned int uFreq,
                    unsigned int uPLLDivider,
                    unsigned int uDividerWidth)
{
    unsigned long int uCalc;

    uCalc = ((unsigned long int)uFreq * (unsigned long int)(uPLLDivider & ((1 << uDividerWidth)-1)))/(unsigned long int)(1 << uDividerWidth);
 
    return((unsigned int)uCalc);
}

/******************************************************************************/
/*  CalkClkFreqAndPeriod                                                      */
/*                                                                            */
/*  DESCRIPTION:                                                              */
/*      Calculate clock frequency and period for a specific PLL.              */
/*                                                                            */
/*  PARAMETERS:                                                               */
/*      pll_source - One of:  ARM_PLL_SOURCE, MEM_PLL_SOURCE, MPG0_PLL_SOURCE */
/*      xtal_freq  - Xtal frequency in Hz                                     */
/*                                                                            */
/*  RETURNS:                                                                  */
/*      frequency  - Returned clock frequency in Hz                           */
/*      period     - Returned clock period in 100ns increments                */
/******************************************************************************/
void CalcClkFreqAndPeriod (unsigned int *frequency,
                           unsigned int *period,
                           unsigned int pll_source,
                           unsigned int xtal_freq )
{
    unsigned int pll_int, pll_frac, pll_div, pll_prescale, clk_freq, remainder, frac_shift=0, clk_period;
    unsigned int pll_sel;
    
    *frequency = 0;
    *period    = 0;

    /*
     * Calculate the clk_freq and clk_period
     */

    /* Calculation below depends on the PLL details of the chip.  */
    /* According to Eric Deal (1/29/04) Trinity uses a XTAL       */
    /* frequency of 74.25 MHz, but there is a divide by 2         */
    /* before the clock ever reaches the PLL logic                */
   
   /* We need to do a XTAL /2 for MEM and FENRUS PLLs only, the other Verve PLLs take the 60 Mhz crystal directly */
    if (pll_source == MEM_PLL_SOURCE)
    {
    xtal_freq >>= 1;
    }

    /* In the following calculations, the frac value read from the*/
    /* PLL multiplier must be first converted to a floating       */
    /* point value, then divided by the number of bits used to    */
    /* represent the frac value (2^16) represented as a floating  */
    /* point value. This will cause the fixed-point integer       */
    /* representation of a fractional value to be converted to a  */
    /* floating-point fractional value.                           */

    /*
     * Read the PLL values from the chip
     */
    switch(pll_source)
    {
       case ARM_PLL_SOURCE:
	  /* For Pecos, there is no dedicated ARM PLL, any of the 9 PLLs can source it.
	   * We need to derive this from the PLL_SEL bits for the ARM clock 
	   * We are handling only the 7 Verve PLLs here.*/

          pll_sel = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_SEL_MASK); 

          switch(pll_sel)
          {
              /* Handle the Verve PLLs in a common fashion */
              /* All the INTFRAC and CTRL regs for the Verve PLLs start at offset 0 of 
	            * PLL_BASE with 0x4 increments. We derive the corresponding INTFRAC and
	            * CTRL reg offsets from the corresponding PLL_SEL value
               *
               * INT_MASK = 0x7E000000 
	            * FRAC_MASK = 0x01FFFFFF 
	            * PREDIV_MASK = 0x000F0000 */
          case  MPG0PLL:
          case  MPG1PLL:
          case  HDPLL:
          case  AUDPLL:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), 0x7E000000);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), 0x01FFFFFF);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), 0x000F0000);
              break;
          case  PLL0:
          case  PLL1:
          case  PLL2:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), 0x7E000000);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), 0x01FFFFFF);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), 0x000F0000);
              break;
          default:
              return;
          }
          
          break;
          
       case MEM_PLL_SOURCE:
       /* For Pecos, use the INT/FRAC masks from the MEM_PLL_CTRL2 register, and the
       * MEM_PLL_CTRL1 register for the POSTDIV/PREDIV masks */
         pll_int  = CNXT_GET_VAL(PLL_MEM_CTRL2_REG, PLL_MEM_CTRL2_PLL_N_VALUE_MASK);
         pll_frac = CNXT_GET_VAL(PLL_MEM_CTRL2_REG, PLL_MEM_CTRL2_PLL_FRAC_VALUE_MASK);
         pll_div  = CNXT_GET_VAL(PLL_MEM_CTRL1_REG, PLL_MEM_CTRL1_PLL_POST_DIV_MASK);
         pll_prescale = CNXT_GET_VAL(PLL_MEM_CTRL1_REG, PLL_MEM_CTRL1_PLL_PRE_DIV_MASK);
         frac_shift = PLL_MEM_CTRL2_PLL_FRAC_VALUE_SHIFT;
         break;

       case MPG0_PLL_SOURCE:
         pll_int  = CNXT_GET_VAL(PLL_MPG0_INTFRAC_REG, PLL_MPG0_INTFRAC_INT_MASK);
         pll_frac = CNXT_GET_VAL(PLL_MPG0_INTFRAC_REG, PLL_MPG0_INTFRAC_FRAC_MASK);
         pll_div  = 1; /* We want the basic PLL rate here, not the MPG0 clock rate */
         pll_prescale = CNXT_GET_VAL(PLL_MPG0_CTRL_REG, PLL_MPG0_CTRL_PRE_DIV_MASK);
         frac_shift = PLL_MPG0_INTFRAC_FRAC_SHIFT;
         break;

       case FENRUS_PLL_SOURCE:
         pll_int  = CNXT_GET_VAL(PLL_FENRUS_CTRL_REG, PLL_FENRUS_CTRL_LOOP_DIV_MASK);
         pll_frac = 0;
         pll_div  = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
         pll_prescale = XTAL_PRESCALE_FACTOR;
         break;

       default:
            return;
    }

  #ifdef DLOAD

    /*
     * If called from the Download (DLOAD) Utility, run-time select the
     * appropriate calculation function.
     */

    xtal_freq    = CHIP_CRYSTAL_FREQUENCY;
    pll_prescale = XTAL_PRESCALE_FACTOR; 

  #endif /* DLOAD */

    pll_div = pll_div * pll_prescale;

    /* 
     * calculate clk_freq = xtal_freq*(pll_int+pll_frac/2^n)/pll_div
     * note that algorithm used here may result up to 4Hz less than what it should be
     */
    clk_freq  = xtal_freq / pll_div;
    remainder = xtal_freq % pll_div;

    /* 
     * The no. of fractional bits have not changed across Pecos and Trinity, except
     * for the MEM PLL in Pecos which is 20 fractional bits, instead of the usual 25 bits
     */

	   if (pll_source == MEM_PLL_SOURCE)
	   {
	    clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (32-frac_shift))
                      + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div);	
           }
           else if (pll_source == FENRUS_PLL_SOURCE && PLL_TYPE == PECOS)
           {
            clk_freq = (clk_freq /pll_div) * pll_int;
           }
	   else
	   {
	    clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (25-frac_shift))
                      + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div);	
	   }  	

    /*
     * Calculate clk_period = (10^11)/clk_freq
     * note that the algorithm will be broken if clk_freq is higher
     * than 429.5MHz.  it should not happen in the predictable future.
     */
    clk_period  = ( 1000000000 / clk_freq ) * 100;
    remainder   = ( 1000000000 % clk_freq ) * 10;

    clk_period += ( remainder / clk_freq ) * 10;
    remainder   = ( remainder % clk_freq ) * 10;

    clk_period += remainder / clk_freq;

    /*
     * Return the values
     */
    *frequency = clk_freq;
    *period    = clk_period;
}

/****************************************************************************
 * Modifications:
 * $Log:
 *  2    Linux_SDK 1.1         3/27/2007 5:16:08 PM   Satpal Parmar   GPL
 *       header and starteam footer addition.
 *  1    Linux_SDK 1.0         3/1/2007 10:53:16 PM   Vineet Seth     
 * $
 ****************************************************************************/
