/****************************************************************************
 * linux/arch/arm/mach-pecos/mach.c
 *
 * Copyright (C) 2007 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/

/****************************************************************************
 * $Header$
 ****************************************************************************/


#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/cpufreq.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/bootmem.h>
#ifdef CONFIG_MTD_PARTITIONS
#include <linux/mtd/physmap.h>
#endif
#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/arch/irq.h>

#include <asm/mach/time.h>
#include <asm/arch/time.h>

#include <linux/serial_core.h>

#include <asm/arch/startup.h>
#include <asm/hardware/cache-l2x0.h>

#include "board.h"

#define FALSE 0
#define TRUE 1


/* ToCheck : Should this be in gpi_pecos.h */
#define DRIVE_GPIO_LOW_BANK_CLEAR(b, x)      *(LPREG)(GPI_DRIVE_LOW_REG +  \
                                                                    ((b)*GPI_BANK_SIZE)) &= (~(1<<(x)))

#define DRIVE_GPIO_HIGH_BANK_SET(b, x)         *(LPREG)(GPI_DRIVE_HIGH_REG +  \
                                                                      ((b)*GPI_BANK_SIZE)) |= (1<<(x))

/* from arch/arm/kernel/setup.c  */
extern unsigned int decarm_shared_start;
extern unsigned int decarm_shared_size;
extern unsigned int decarm_code_start;
extern unsigned int decarm_code_size;

static unsigned int ArmClkFreq32, ArmClkPeriod32, MemClkFreq32, MemClkPeriod32;

extern void __init cnxt_init_irq( void );
/*
 * Here we setup the defualt sizes and locatinos of various
 * partitions in the flash.
 *
 * Adding the string CNXT_PARTN_TAG: <tag> to the size
 * field of a partition enables the cnxt_mk_jjfs2.sh
 * script to locate the size of the partition
 *
 */

//#ifdef CONFIG_MTD_PARTITIONS
#if 0
static struct mtd_partition cnxt_map_partitions[] = {
        {
                .name =          "codeldr",
                .size =          0x80000,
                .offset =        0x0000,
                .mask_flags =    MTD_WRITEABLE,  /* force read-only */
        },
        {
                .name =          "Arm 920 code",
                .size =          0x80000,
                .offset =        0x80000,
        },
        {
                .name =          "tlv kernel",
                .size =          0x400000,   
                .offset =        0x100000,
        },
        {
                .name =          "rootfs",
                .size =          0x400000,       /* CNXT_PARTN_TAG: ROOTFS */
                .offset =        0x500000,
        },
        {
                .name =          "appfs",
                .size =          0x1000000,       /* CNXT_PARTN_TAG: APPFS */
                .offset =        0x1000000,
        }
};

static int num_cnxt_map_partitions = (sizeof(cnxt_map_partitions)/sizeof(struct mtd_partition));
#endif


/*
    Setup Mapping for Register Space and Flash needed by BSP
*/
static struct map_desc cnxt_io_desc[] __initdata = 
{

	{
		.virtual	= ASX_VADDR_BASE,
		.pfn		= __phys_to_pfn(ASX_PHYS_BASE),
		.length		= ASX_IO_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual 	= EXT_IO_VADDR_BASE,
		.pfn		= __phys_to_pfn(EXT_IO_PHYS_BASE),
		.length 	= EXT_IO_SIZE,
		.type 		=  MT_DEVICE,
	}, {
		.virtual 	= TEMPEST_VADDR_BASE,
		.pfn		= __phys_to_pfn(TEMPEST_PHYS_BASE),
		.length 	= TEMPEST_IO_SIZE, 
		.type		= MT_DEVICE,
        } , 
  	// {   Not needed because PCI driver should ioremap this
	// 	.virtual	= PCI_MEM_VADDR_BASE,
	//	.pfn 		= __phys_to_pfn(PCI_MEM_PHYS_BASE),
	//	.length 	=  PCI_MEM_SIZE,
	//      .type		=    MT_DEVICE, 
	// } , 
           {
		.virtual 	= APP_VADDR_BASE,
		.pfn		= __phys_to_pfn(APP_PHYS_BASE),
		.length 	= APP_IO_SIZE,
		.type		= MT_DEVICE,
	}, {
		.virtual 	= FLASH_VADDR_BASE,
		.pfn		= __phys_to_pfn(FLASH_PHYS_BASE),
		.length 	= FLASH_SIZE,
		.type		= MT_DEVICE,
    }
 };


unsigned long ChipID;
EXPORT_SYMBOL(ChipID);

unsigned long ChipRevID;
EXPORT_SYMBOL(ChipRevID);

unsigned long KAL_mem_phys_base;
EXPORT_SYMBOL(KAL_mem_phys_base);

unsigned long KAL_mem_size;
EXPORT_SYMBOL(KAL_mem_size);

#define ATAGLIST_MAX_WORDS 192
unsigned long uKernelAtaglist[ATAGLIST_MAX_WORDS];
EXPORT_SYMBOL(uKernelAtaglist);

void __init cnxt_reserve_KAL_memory(pg_data_t *pgdat)
{
    /* Reserve Space for arm9 interrupts */
    /* TODO: Fixme:  the ARM9 vecs are not at PHYS_OFFSET + 0x10000
    they are at "just 0x10000" (PHYS_OFFSET must be 0 for this to be true).
    This is benign because the kernel isn't aware of RAM below PHYS_OFFSET.
    */
    reserve_bootmem_node(pgdat, PHYS_OFFSET + 0x10000, 0x10000);

    if ( (decarm_shared_start) && (decarm_code_start) && 
         (decarm_code_size)    && (decarm_shared_size) )
    {         
        /* Reserve Space for decarm shared Mem */
        reserve_bootmem_node(pgdat, decarm_shared_start, decarm_shared_size );
        reserve_bootmem_node(pgdat, decarm_code_start, decarm_code_size );
    }
    else
    {
        printk(KERN_INFO "INFO: Not reserving decarm memory (will use runtime allocation)\n");
    }
}
EXPORT_SYMBOL(cnxt_reserve_KAL_memory);

static void __init cnxt_map_io(void)
{
    iotable_init( cnxt_io_desc,ARRAY_SIZE(cnxt_io_desc)); 
}

/* This function is used to get the chipid and chipRevId */

static void cnxt_get_chip_rev_id(struct machine_desc *pmach,
                               struct tag *ptag, char **pptr,
                                struct meminfo *pmem)
{
    LPREG pPCICfgAddr;
    LPREG pPCICfgData; 
    LPREG pFuse = (LPREG)0xe0440148;

    iotable_init(cnxt_io_desc , 1);
//    iotable_init(cnxt_io_desc + 3 ,1);


    /* Take care of some special cases first. Pecos and Nevis need fixing   */
    /* up for A0 and B0, Nevis needs fixup for C0.                          */
    /* PCI Chip ID | PCI Chip Rev | PBIN Fuse | Build Target | "Real" Chip  */
    /* ===================================================================  */
    /*      X      |       0      |     X     |     Pecos    |   Pecos A0   */
    /*      X      |       0      |     X     |     Nevis    |   Nevis A0   */
    /*   <Pecos>   |      0x10    |     0     |       X      |   Pecos B0   */
    /*   <Pecos>   |      0x10    |     1     |       X      |   Nevis B0   */
    /*   <Nevis>   |      0x10    |     X     |       X      |   Nevis C0   */

    pPCICfgAddr = (LPREG) PCI_CFG_ADDR_REG;
    pPCICfgData = (LPREG) PCI_CFG_DATA_REG;

     /* Read vendor/device ID */
    *pPCICfgAddr = 0x0;
    ChipID = *pPCICfgData;

    /* Read Rev Id */
    *pPCICfgAddr = 0x8;
    ChipRevID = *pPCICfgData  & 0xFF;

    /* paraphrased from cnxt_base/drivers/startup/api.s */
    if (ChipRevID < 0x10) {
        /* TODO:  When mach-nevis and mach-pecos get merged, we'll need to 
        use a KCONFIG switch to decide which chip we are. For Rev A chips
        we're always going to read "Pecos", but obviously we want to report
        nevis when we're running on a nevis.  For now, I KNOW I'm a pecos 
        because I am the file mach-pecos/mach.c */
        ChipID = PECOS;
    } else if (ChipRevID == 0x10) {
        if (ChipID == PECOS) {
            /* We could be a Pecos OR a nevis.  Look at the fuses to tell */
            if (*pFuse == 0x0) {
                /* fuses have not been set, follow the same logic for RevID < 0x10 */
                ChipID = PECOS;
            }
            else
            {
                /* Fuses are set so we can differentiate between Pecos and Nevis. */
                if (*pFuse & 0x01000000)
                {
                    ChipID = PECOS;
                }
                else
                {
                    ChipID = NEVIS;
                }
            }
        } else if (ChipID == NEVIS) {
            /* If we read a nevis and we thought we were a B0, we're really a C0 */
            ChipRevID = 0x20;
        }
    } else /* our chiprev > 0x10 */ {
        /* do nothing, our ChipID is already correct. What needs to be done
        with the rev is undefined at the time of this writing. */
    }

    printk("ChipID=0x%lx ChipRevID=0x%lx\n",ChipID,ChipRevID);
}


/* fixup function */
static void cnxt_fixup( struct machine_desc *pmach,
                   struct tag *ptag, 
                   char **pptr,
                   struct meminfo *pmem)
{

   /* copy the ataglist to the global variable uKernelAtaglist */
   memcpy( uKernelAtaglist, ptag, ATAGLIST_MAX_WORDS*4 );

   cnxt_get_chip_rev_id(pmach, ptag, pptr, pmem);
   return;
}

unsigned int GetChipRev (void)
{
    return ChipRevID;
}

unsigned int GetChipID (void)
{
    return ChipID;
}

 void  GetCalcClkFreqAndPeriod( unsigned int *pMemClkFreq32 ,  unsigned int *pMemClkPeriod32)
{

  *pMemClkFreq32 = MemClkFreq32;
  *pMemClkPeriod32 = MemClkPeriod32;
  
   return;
}

EXPORT_SYMBOL(GetChipRev);
EXPORT_SYMBOL(GetChipID);

typedef struct {
 LPREG preg;
 unsigned long val;
} cnxt_pll_config ;

static cnxt_pll_config conf_pll[] __initdata = {
  /*Initialize the GPIO pin mux and alt func registers for unique*/
#if CONFIG_PLL_CONFIG0_REG_DEFAULT
  { (LPREG)PLL_CONFIG0_REG, CONFIG_PLL_CONFIG0_REG_DEFAULT, },
#endif
   /*MUX settings*/
  { (LPREG)PLL_PIN_GPIO_MUX0_REG, CONFIG_PLL_PIN_GPIO_MUX0_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX1_REG, CONFIG_PLL_PIN_GPIO_MUX1_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX2_REG, CONFIG_PLL_PIN_GPIO_MUX2_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX3_REG, CONFIG_PLL_PIN_GPIO_MUX3_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX4_REG, CONFIG_PLL_PIN_GPIO_MUX4_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX5_REG, CONFIG_PLL_PIN_GPIO_MUX5_REG_DEFAULT, },
  { (LPREG)PLL_PIN_GPIO_MUX6_REG, CONFIG_PLL_PIN_GPIO_MUX6_REG_DEFAULT, },
   /*Initialize the secondary pin mux registers*/  
  { (LPREG)SREG_2ND_PIN_MUX0_REG, CONFIG_SREG_2ND_PIN_MUX0_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX1_REG, CONFIG_SREG_2ND_PIN_MUX1_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX2_REG, CONFIG_SREG_2ND_PIN_MUX2_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX3_REG, CONFIG_SREG_2ND_PIN_MUX3_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX4_REG, CONFIG_SREG_2ND_PIN_MUX4_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX5_REG, CONFIG_SREG_2ND_PIN_MUX5_REG_DEFAULT, },
  { (LPREG)SREG_2ND_PIN_MUX6_REG, CONFIG_SREG_2ND_PIN_MUX6_REG_DEFAULT, },
  { (LPREG)PLL_PIN_ALT_FUNC_REG, CONFIG_PLL_PIN_ALT_FUNC_REG_DEFAULT, },
 };
 
/********************************************************************/
/*  FUNCTION:    hwlib_program_descriptor                           */
/*                                                                  */
/*  PARAMETERS:  device information for programming descriptor      */
/*                                                                  */
/*  DESCRIPTION: given a device description, program the requested  */
/*               descriptor to access that device                   */
/*                                                                  */
/*  RETURNS:     CNXT_STATUS_OK - success                           */
/*                                                                  */
/********************************************************************/
static void __init program_descriptor( ISA_DEVICE *Device )
{
   LPREG  pReg;
   LPREG  pPrimary;
   LPREG  pSecondary;
   
   pPrimary   = (LPREG)PCI_ISA_PRIMARY_DESC_ADDR(Device->DescNum);
   pSecondary = (LPREG)PCI_ISA_SECONDARY_DESC_ADDR(Device->DescNum);

   /*
    * Setup the descriptor designated in the ISADevice structure with the
    * parameters from the ISADevice structure.
    */

   *pPrimary = (PCI_ISA_DESC_TYPE_ISA                  |
      (Device->ExtWait << PCI_ISA_DESC_EXT_WAIT_SHIFT) |
      (Device->IOMemory << PCI_ISA_DESC_IO_TYPE_SHIFT) |
      (Device->ChipSelect << PCI_ISA_DESC_CS_SHIFT));

   switch (Device->IORegMode)
   {
      case IO_REG_PCCARD:
         *pPrimary |= PCI_NEEDS_ISA_REG;
         break;

      case IO_REG_PROGRAMMABLE:
         *pPrimary |= PCI_NEEDS_ISA_REG;
         break;

      case IO_REG_OFF:
      default:
         *pPrimary &= ~PCI_NEEDS_ISA_REG;
   }


   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_WIDTH_MASK) | 
                (Device->Width << PCI_ISA_DESC_WIDTH_SHIFT));

   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_INCR_MASK) |
                (Device->ISAIncr << PCI_ISA_DESC_INCR_SHIFT));
   /* Set syncIO mode if required */
   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_SYNC_IO_ENABLE_MASK) |
      (Device->bClocked << PCI_ISA_DESC_SYNC_IO_ENABLE_SHIFT));

   /*
    * The timings designated in the ISADevice structure specify absolute time
    * in ns. However, the descriptor itself interprets timings in number of
    * memclk cycles. Therefore, we have to convert the absolute time required
    * by the card into memclk cycles.
    */

   *pPrimary = ( (*pPrimary & ~PCI_ISA_DESC_WRITE_WAIT_MASK) |
      ( ( (unsigned int)( ((unsigned int)Device->WriteAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_ISA_DESC_WRITE_WAIT_SHIFT));
   *pPrimary = ((*pPrimary & ~PCI_ISA_DESC_READ_WAIT_MASK) |
      ( ( (unsigned int)( ((unsigned int)Device->ReadAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_ISA_DESC_READ_WAIT_SHIFT));

   *pSecondary = (
      (( (unsigned int)( ((unsigned int)Device->IORegSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_REG_SETUP_TIME_SHIFT)  |
      (( (unsigned int)( ((unsigned int)Device->IORegAccessTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_REG_ACCESS_TIME_SHIFT) |
      (( (unsigned int)( ((unsigned int)Device->CSSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CS_SETUP_TIME_SHIFT)   |
      (( (unsigned int)( ((unsigned int)Device->CSHoldTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CS_HOLD_TIME_SHIFT)    |
      (( (unsigned int)( ((unsigned int)Device->CtrlSetupTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_CTRL_SETUP_TIME_SHIFT) |
      (( (unsigned int)( ((unsigned int)Device->CtrlHoldTime*100)/MemClkPeriod32 + 1) ) <<
         PCI_EXT_DESC_ADDR_HOLD_TIME_SHIFT));

   pReg = (LPREG)PCI_ROM_XOE_MASK_REG;
   *pReg = *pReg | (Device->XOEMask << Device->ChipSelect);
 
  return ;
}


static void __init config_isa_descriptors(void)
{
    LPREG pReg;
    int UsingSyncIO = FALSE;
    ISA_DEVICE Device;

    /*
     * Run-time select the Xtal frequency from the EEPROM data
     * <<<< Needs to be fixed ! >>>>
     */
#define    xtal_frequency 60000000UL

    /**********************************/
    /* Approximate the MemClk period. */
    /**********************************/

    CalcClkFreqAndPeriod (&ArmClkFreq32, &ArmClkPeriod32, ARM_PLL_SOURCE, xtal_frequency);
    CalcClkFreqAndPeriod (&MemClkFreq32, &MemClkPeriod32, FENRUS_PLL_SOURCE, xtal_frequency);
    //printk( KERN_INFO "ArmClkFreq32 = 0x%x\n",ArmClkFreq32 );
    //printk( KERN_INFO "ArmClkPeriod32 = 0x%x\n",ArmClkPeriod32 );
    //printk( KERN_INFO "MemClkFreq32 = 0x%x\n",MemClkFreq32 );
    //printk( KERN_INFO "MemClkPeriod32 = 0x%x\n",MemClkPeriod32 );

   #ifdef CONFIG_PROGRAM_DESC_2
      Device.DescNum         = 2;
      Device.ChipSelect      = CONFIG_CNXT_DESC_2_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_2_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_2_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_2_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_2_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_2_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_2_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_2_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_2_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_2_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_2_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_2_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_2_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_2_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_2_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_2_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_3
      Device.DescNum         = 3;
      Device.ChipSelect      = CONFIG_CNXT_DESC_3_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_3_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_3_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_3_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_3_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_3_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_3_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_3_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_3_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_3_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_3_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_3_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_3_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_3_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_3_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_3_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_4
      Device.DescNum         = 4;
      Device.ChipSelect      = CONFIG_CNXT_DESC_4_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_4_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_4_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_4_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_4_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_4_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_4_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_4_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_4_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_4_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_4_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_4_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_4_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_4_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_4_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_4_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_5
      Device.DescNum         = 5;
      Device.ChipSelect      = CONFIG_CNXT_DESC_5_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_5_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_5_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_5_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_5_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_5_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_5_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_5_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_5_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_5_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_5_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_5_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_5_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_5_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_5_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_5_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_6
      Device.DescNum         = 6;
      Device.ChipSelect      = CONFIG_CNXT_DESC_6_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_6_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_6_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_6_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_6_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_6_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_6_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_6_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_6_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_6_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_6_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_6_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_6_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_6_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_6_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_6_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

   #ifdef CONFIG_PROGRAM_DESC_7
      Device.DescNum         = 7;
      Device.ChipSelect      = CONFIG_CNXT_DESC_7_CHIP_SELECT;
      Device.bClocked        = (CONFIG_CNXT_DESC_7_SYNC_IO == 1);
      #ifdef CONFIG_CNXT_DESC_7_XOE
            Device.XOEMask         = 1;
      #else
            Device.XOEMask         = 0;
      #endif      
      Device.ExtWait         = CONFIG_CNXT_DESC_7_EXT_WAIT;
      Device.IOMemory        = CONFIG_CNXT_DESC_7_IO_ACCESS;
      Device.IORegMode       = CONFIG_CNXT_DESC_7_IO_REG_MODE;
      Device.WriteAccessTime = CONFIG_CNXT_DESC_7_WRITE_ACCESS;
      Device.ReadAccessTime  = CONFIG_CNXT_DESC_7_READ_ACCESS;
      Device.IORegSetupTime  = CONFIG_CNXT_DESC_7_IO_REG_SETUP;
      Device.IORegAccessTime = CONFIG_CNXT_DESC_7_IO_REG_ACCESS;
      Device.CSSetupTime     = CONFIG_CNXT_DESC_7_CS_SETUP;
      Device.CSHoldTime      = CONFIG_CNXT_DESC_7_CS_HOLD;
      Device.CtrlSetupTime   = CONFIG_CNXT_DESC_7_CONTROL_SETUP;
      Device.CtrlHoldTime    = CONFIG_CNXT_DESC_7_CONTROL_HOLD;
      Device.Width           = CONFIG_CNXT_DESC_7_WIDTH;
      Device.ISAIncr         = CONFIG_CNXT_DESC_7_INCR;
      UsingSyncIO            = (UsingSyncIO || Device.bClocked);
      program_descriptor( &Device );
   #endif

    /* If anything is using SyncIO, enable it */
    if(UsingSyncIO)
    {
        /*****************************************
         * Enable Program synchronous I/O clock  *
         *****************************************/
         pReg = (LPREG)ASX_TO_VIRT(PCI_ROM_MODE_REG);
         *pReg = *pReg | 0x400;
    }

}


static int __init cnxt_init( void )
{
    int i = 0;
    int ret = 0;
#ifndef CONFIG_ED_PECOS_UFP
    LPREG pReg;
#endif

//#ifdef CONFIG_MTD_PARTITIONS
#if 0
    physmap_set_partitions(cnxt_map_partitions, num_cnxt_map_partitions);
#endif

#ifndef CONFIG_ED_PECOS_UFP

    while ( i < ARRAY_SIZE(conf_pll) ) {
	pReg = conf_pll[i].preg;
	*pReg = conf_pll[i].val ; 
	i++;
    }

   /* Clear out GPIO registers. */
   /* Init all GPIO interrupt sources to known inactive state */
   pReg = (LPREG)GPI_POS_EDGE_REG; *pReg = 0;
   pReg = (LPREG)GPI_NEG_EDGE_REG; *pReg = 0;

   /* Make sure all GPIOs are inactive and in high impedance state */
   pReg = (LPREG)GPI_DRIVE_HIGH_REG; *pReg = 0;
   pReg = (LPREG)GPI_DRIVE_LOW_REG; *pReg = 0;
   pReg = (LPREG)GPI_DRIVE_OFF_REG; *pReg = 0xffffffff;

   /* Note: 2xMode Source clock from Pecos.
         By default MII0_CLK / MII1_CLK provides 5OMHz.
         If not, Update below main registers to get the required Clocks.
         FENRUS_PLL_CTRL,
         DIV_MUX_CTRL14
    */

   #ifdef CONFIG_CNXT_EMAC0_2xMODE_ENABLE
   /* EMAC0: enable 50MHz MII0_CLK */

   (*(LPREG)(0xE0440088)) = ((*(LPREG)(0xE0440088) & ~(0x0000FFFF)) | ((0x00000809) & (0x0000FFFF)));

   pReg = (LPREG)PLL_PIN_GPIO_MUX5_REG;
   *pReg |= 0x00000001;
   #endif

   #ifdef CONFIG_CNXT_EMAC1_2xMODE_ENABLE
   /* EMAC1: enable 50MHz MII1_CLK */

   (*(LPREG)(0xE0440088)) = ((*(LPREG)(0xE0440088) & ~(0xFFFF0000)) | ((0x08090000) & (0xFFFF0000)));

   pReg = (LPREG)0xE0440198;
   *pReg |= 0x00000001; 
   #endif

   /* EMAC :  PIO 42 Output HIGH to reset PHY connected to MII0 */
   DRIVE_GPIO_LOW_BANK_CLEAR(1,GPI_DRIVE_LOW_REG_BIT010_SHIFT);
   DRIVE_GPIO_HIGH_BANK_SET(1, GPI_DRIVE_HIGH_REG_BIT010_SHIFT);

   /* EMAC : PIO 127 Output High to reset PHY connected to MII1 */
   DRIVE_GPIO_LOW_BANK_CLEAR(3,GPI_DRIVE_LOW_REG_BIT031_SHIFT);
   DRIVE_GPIO_HIGH_BANK_SET(3, GPI_DRIVE_HIGH_REG_BIT031_SHIFT);

   config_isa_descriptors();

   /**************************************************/
   /* ARM 11 Peripheral Remap Register Value         */
   /**************************************************/
   /* Base address = 0xE8000000  Size = 256KiB       */
   /* Other combination can be addded appropriately.  */
   #define PERI_PORT_REMAP_REG_VALUE 0xE8000009 
   
     __asm__("ldr r0, =0xE8000009\n\t"
      "mcr   p15, 0, r0, c15, c2, 4\n\t");

     /** pecos**/
    #ifdef CONFIG_OUTER_CACHE
        l220_init((void *)0xE8020000,0x00730000, 0xfe000fff) ;
    #endif

#endif   /* !CONFIG_ED_PECOS_UFP */

   return ret;
}

__initcall( cnxt_init );

static unsigned long cnxt_gettimeoffset(void)
{
  unsigned long ticks;

  ticks =  __raw_readl(TIMER_NUM_VALUE_REG(SYS_TIMER << 4));
  if (__raw_readl(ITC_INTRIRQ_REG(0,PIC_2)) & ITC_TIMER0)
       ticks += TIMER_INTERVAL;

  return (ticks) / (uSEC_1);
}

extern struct s_latency latency;
extern unsigned long (*gettimeoffset)(void);
extern void idleAddCount(void);

static irqreturn_t  cnxt_sys_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
   volatile unsigned int *timer_count_reg;

   timer_count_reg = (volatile unsigned int *)ASX_TO_VIRT( TIM_VALUE_REG );
   write_seqlock(&xtime_lock);
     __raw_writel( (__raw_readl(TIMER_NUM_MODE_REG(SYS_TIMER << 4)) & ~(TIM_INT_ENABLE_MASK)),
                         TIMER_NUM_MODE_REG(SYS_TIMER << 4)) ; /* Disable IRQ */

   latency.delta[latency.count++] =  *timer_count_reg;
   if (latency.count > LATENCY_COUNT) latency.count = 0;

   /* Clearing not needed in case of Pecos */
    timer_tick(regs);
     __raw_writel( (__raw_readl(TIMER_NUM_MODE_REG(SYS_TIMER << 4)) | TIM_INT_ENABLE_MASK),
                         TIMER_NUM_MODE_REG(SYS_TIMER << 4)) ; /* Enable IRQ */
    write_sequnlock(&xtime_lock);

    idleAddCount();
    return IRQ_HANDLED;
}


static struct irqaction cnxt_sys_timer = {
        .name = "PECOS System Timer",
        .flags = IRQF_DISABLED | IRQF_TIMER,
        .handler = cnxt_sys_timer_interrupt
};

static inline void cnxt_setup_timer( void )
{
    int i;

    for (i=0; i < NUM_TIM_BANKS; i++) {
      __raw_writel(0x0, TIMER_NUM_MODE_REG(i << 4)); /* Disable  timers */
      __raw_writel(0x0, TIMER_NUM_VALUE_REG(i << 4)); /* Clear Counter Value  */
    }
    /* System Timer -- Progam to ~ 10ms */
    /* Program Limit to 10ms Controlled by timer Base register 54 Mhz (Not sure
     *  how to program it), as  read/write resets to 0's */
    __raw_writel(TIMER_INTERVAL, TIMER_NUM_LIMIT_REG(SYS_TIMER << 4));

   setup_irq(INT_SYSTIMER, &cnxt_sys_timer);

    /* Enable irq generation by timer 0 */
     __raw_writel(TIM_INT_ENABLE_MASK | TIM_ENABLE_MASK, TIMER_NUM_MODE_REG(SYS_TIMER << 4));
}



static struct sys_timer cnxt_timer ={
	.init = &cnxt_setup_timer,
	.offset = &cnxt_gettimeoffset,
 };

MACHINE_START( PECOS, "Conexant Presidio IRD" )
	/* maintainer "Conexant Systems Noida Design Center" */
	.phys_io	= 0xe0000000,
	.io_pg_offst	= ((0xfd000000) >> 18) & 0xfffc,
    /* boot_pararms will be placed (by the loader) at the megabyte boundary
       just below where the kernel is loaded.  Since the kernel is loaded 
       at a 0x18000 boundary (in conexant's case, 0x48000), we're always
       guaranteed to have a PTE so the kernel can find the ATAG list here. 
       The ATAG list is only temporary and will eventually get over-written
       so we copy it in our machine fixup call. */
	.boot_params	= ((TEXT_OFFSET + PHYS_OFFSET) & 0xfff00000),
        .map_io 	= cnxt_map_io,
        .fixup   	= cnxt_fixup, 
	.init_irq 	= cnxt_init_irq, 	
	.timer 		= &cnxt_timer
MACHINE_END

/**************************************************************************************
* Modifications:
* $Logs$
*
***************************************************************************************/








