/****************************************************************************
 *
 *  arch/arm/mach-pecos/mipidle.c
 *
 *  Copyright (C) 2007 Conexant Systems Inc, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/
/*$Id: mipidle.c,v 1.3, 2007-08-02 12:07:11Z, Har Yash Bahadur$
 ****************************************************************************/

#include <linux/kernel.h>
#include <asm/param.h> /* for HZ */

/**********
* Globals *
**********/
unsigned long gIdleCount = 0;

#define TICKS_PER_MIN  ((HZ) * 60)

static unsigned long idleSample[TICKS_PER_MIN];
static int index = 0;

void cpu_v6_do_idle(void);

void idleAddCount(void);
void cnxt_mip_idle(void)  __attribute__ ((pure));
   
void idleAddCount()
{
   idleSample[index++] = gIdleCount;
   if (index == TICKS_PER_MIN) index = 0;
   gIdleCount = 0;
}

int get_idle_list( char *buf ) {
    unsigned int i;
    unsigned int j; /* Cycles per second */
    unsigned int k; /* seconds per minute */
    unsigned int sum;
    unsigned int sum2=0;
    unsigned int avg_mips=0;
    unsigned int min_mips=667000000;
    unsigned int max_mips=0;

    char *p = buf;
    
    p += sprintf(p, "Idle MIPS for last minute:\n\n");

    if ( (index + 1) == TICKS_PER_MIN ) i = 0; else i = index;
    
    k = 60;
    while (k--)
    {
       sum = 0;
       j = HZ;
       while (j--)
       {
          sum += idleSample[i++];
          if (i == TICKS_PER_MIN) i = 0;
       }
       p += sprintf(p, "%2d: %10d\n", k, sum);
       sum2 += (sum/10000);
       if (sum < min_mips) min_mips=sum;
       if (sum > max_mips) max_mips=sum;
    }
    avg_mips = (sum2/60);

    p += sprintf(p, "Avg Idle MIPS = %5d  x 10^4\n", avg_mips);
    p += sprintf(p, "Min Idle MIPS = %10d\n", min_mips);
    p += sprintf(p, "Max Idle MIPS = %10d\n", max_mips);
    p += sprintf(p, "\n");

    return p - buf;
}

void cnxt_mip_idle() 
{
   /* TODO:  Fix these hardcoded register addresses.  Be more 
    * dyanmic with the use of which hwtimer to use.  Be more
    * dyanmic with detecticing CPU speed.
    */

   static volatile unsigned int *ptimVal  = (unsigned int *)0xe0430070;
   static volatile unsigned int *ptimLim  = (unsigned int *)0xe0430074;
   static volatile unsigned int *ptimMode = (unsigned int *)0xe0430078;
   static volatile unsigned int *ptimBase = (unsigned int *)0xe043007c;
   static int bInited = 0;
   int timein, timeout;
   
   if (!bInited)
   {
      bInited = 1;
      *ptimVal = 0;
      *ptimLim = -1;
      *ptimMode = 0x1;
      *ptimBase = 0;
   }
      
   timein = *ptimVal;
   cpu_v6_do_idle();
   timeout = *ptimVal;

   /* hw timer has 54 MHz clock input.  dT / 54 is number of elapsed
    * uSecs while idle.  Arm is running at 667 MHz, so you should, in 
    * theory, have about 667 instructions per microsecond
    */
   
   gIdleCount += ((timeout - timein) * 667) / 54;
}

/****************************************************************************
 * Modifications:
 * $Log:
 *  4    Linux_SDK 1.3         8/2/07 5:37:11 PM IST  Har Yash Bahadur Add
 *       calculation of Average, Minimum and Maximum Idle MIPS in the last one
 *        minute.
 *  3    Linux_SDK 1.2         6/5/07 3:15:16 PM IST  Vineet Seth     Cosmetic
 *       changes to remove warning
 *  2    Linux_SDK 1.1         3/27/07 5:16:05 PM IST Satpal Parmar   GPL
 *       header and starteam footer addition.
 *  1    Linux_SDK 1.0         3/1/07 10:53:09 PM IST Vineet Seth     
 * $
 *
 ****************************************************************************/

