/****************************************************************************
 *
 *  arch/arm/mach-pecos/pll.c
 *
 *  Copyright (C) 2007 Conexant Systems Inc, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/
/*$Id: pllc.c,v 1.2, 2007-10-10 00:17:06Z, Miles Bintz$
 ****************************************************************************/

/******************/
/* Include Files  */
/******************/
#ifdef __KERNEL__
#include <asm/arch/hardware.h>
#include <asm/arch/startup.h>

/* FIXME: TODO:  Need to use  a global definition of CNXT_GET */
#define CNXT_GET(a, b)  (*((volatile unsigned int*)ASX_TO_VIRT(a)) & b)
#define CNXT_GET_VAL(reg,mask)           ((*((volatile unsigned int*)ASX_TO_VIRT(reg)) & (mask)) >> RMO(mask))

typedef unsigned int u_int32;
typedef unsigned long long u_int64;

#else
#include "stbcfg.h"
#include "basetype.h"
#include "startup.h"
#include "stbcfg.h"
#endif


/********************************************************************/
/*  CalcFreqFromFracPart                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Calculate ( freq * pll_frac ) / ( 2 ^ frac_len )            */
/*      Note that ( freq * pll_frac ) is much bigger                */
/*      than unsigned 0xffffffff.  The algorithm used may result    */
/*      up to 3 less than what should be                            */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      See description                                             */
/*                                                                  */
/*  RETURNS:                                                        */
/*      result described above                                      */
/********************************************************************/
#if (PLL_TYPE == TRINITY)
static u_int32 CalcFreqFromFracPart ( u_int32 freq,
                                      u_int32 pll_frac,
                                      u_int32 frac_len )
{
   u_int32 k, l, freq0, frac0, retval;

   if ( frac_len == 25 )
   {
      k = 13;
      l = 12;
   }
   else if ( frac_len == 16 )
   {
      k = 8;
      l = 8;
   }
   else
   {
      return 0xffffffff;
   }

   freq0    = freq >> k;
   freq     = freq - ( freq0 << k );
   frac0    = pll_frac >> l;
   pll_frac = pll_frac - ( frac0 << l );

   retval = freq0 * frac0 + ( ( freq0 * pll_frac ) >> l )
                          + ( ( frac0 * freq ) >> k )
                          + ( ( freq * pll_frac ) >> frac_len );
 
   return retval;
}

#else
static u_int32 CalcFreqFromFracPart(
                    u_int32 uFreq,
                    u_int32 uPLLDivider,
                    u_int32 uDividerWidth)
{
    u_int64 uCalc;
 
    uCalc = ((u_int64)uFreq * (u_int64)(uPLLDivider & ((1 << uDividerWidth)-1)))/(u_int64)(1 << uDividerWidth);
 
    return((u_int32)uCalc);
}
#endif

/******************************************************************************/
/*  CalkClkFreqAndPeriod                                                      */
/*                                                                            */
/*  DESCRIPTION:                                                              */
/*      Calculate clock frequency and period for a specific PLL.              */
/*                                                                            */
/*  PARAMETERS:                                                               */
/*      pll_source - One of:  ARM_PLL_SOURCE, MEM_PLL_SOURCE, MPG0_PLL_SOURCE */
/*      xtal_freq  - Xtal frequency in Hz                                     */
/*                                                                            */
/*  RETURNS:                                                                  */
/*      frequency  - Returned clock frequency in Hz                           */
/*      period     - Returned clock period in 100ns increments                */
/******************************************************************************/
void CalcClkFreqAndPeriod (u_int32 *frequency,
                           u_int32 *period,
                           PLL_SOURCE pll_source,
                           u_int32 xtal_freq )
{
    u_int32 pll_int;
    u_int32 pll_frac;
    u_int32 pll_div;
    u_int32 pll_prescale = 0;
    u_int32 clk_freq;
    u_int32 remainder;
    u_int32 frac_shift = 0;
    u_int32 clk_period;
#if (PLL_TYPE == PECOS)
    u_int32 pll_sel;
#endif
    
    *frequency = 0;
    *period    = 0;

    /*
     * Calculate the clk_freq and clk_period
     */

    /* Calculation below depends on the PLL details of the chip.  */
    /* According to Eric Deal (1/29/04) Trinity uses a XTAL       */
    /* frequency of 74.25 MHz, but there is a divide by 2         */
    /* before the clock ever reaches the PLL logic                */
   
  #if (PLL_TYPE == TRINITY)
    xtal_freq >>= 1;
  #else
   /* We need to do a XTAL /2 for MEM and FENRUS PLLs only, the other Verve PLLs take the 60 Mhz crystal directly */
    if (pll_source == MEM_PLL_SOURCE)
    {
    xtal_freq >>= 1;
    }
  #endif


    /* In the following calculations, the frac value read from the*/
    /* PLL multiplier must be first converted to a floating       */
    /* point value, then divided by the number of bits used to    */
    /* represent the frac value (2^16) represented as a floating  */
    /* point value. This will cause the fixed-point integer       */
    /* representation of a fractional value to be converted to a  */
    /* floating-point fractional value.                           */

    /*
     * Read the PLL values from the chip
     */
    switch(pll_source)
    {
       case ARM_PLL_SOURCE:
       #if (PLL_TYPE == TRINITY)
          pll_int  = CNXT_GET_VAL(PLL_ARM_INTFRAC_REG, PLL_ARM_INTFRAC_INT_MASK);
          pll_frac = CNXT_GET_VAL(PLL_ARM_INTFRAC_REG, PLL_ARM_INTFRAC_FRAC_MASK);
          pll_div  = CNXT_GET_VAL(PLL_DIV0_REG, PLL_DIV0_ARMCLK_POSTDIV_MASK);
          pll_prescale = PLL_PRESCALE (CNXT_GET_VAL(PLL_PRESCALE_REG, PLL_PRESCALE_ARM_MASK));
          frac_shift = PLL_ARM_INTFRAC_FRAC_SHIFT;
        #else
	  /* For Pecos, there is no dedicated ARM PLL, any of the 9 PLLs can source it.
	   * We need to derive this from the PLL_SEL bits for the ARM clock 
	   * We are handling only the 7 Verve PLLs here.*/

          pll_sel = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_SEL_MASK); 

          switch(pll_sel)
          {
              /* Handle the Verve PLLs in a common fashion */
              /* All the INTFRAC and CTRL regs for the Verve PLLs start at offset 0 of 
	            * PLL_BASE with 0x4 increments. We derive the corresponding INTFRAC and
	            * CTRL reg offsets from the corresponding PLL_SEL value
               */
          case  MPG0PLL:
          case  MPG1PLL:
          case  HDPLL:
          case  AUDPLL:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * pll_sel * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
              break;
          case  PLL0:
          case  PLL1:
          case  PLL2:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL1_REG, DIV_MUX_CTRL1_ARM926_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
              break;
          default:
              return;
          }
          
        #endif
          break;
          
       case MEM_PLL_SOURCE:
       #if (PLL_TYPE == TRINITY)
         pll_int  = CNXT_GET_VAL(PLL_MEM_INTFRAC_REG, PLL_MEM_INTFRAC_INT_MASK);
         pll_frac = CNXT_GET_VAL(PLL_MEM_INTFRAC_REG, PLL_MEM_INTFRAC_FRAC_MASK);
         pll_div  = CNXT_GET_VAL(PLL_DIV0_REG, PLL_DIV0_MEMCLK_POSTDIV_MASK);
         pll_prescale = PLL_PRESCALE (CNXT_GET_VAL(PLL_PRESCALE_REG, PLL_PRESCALE_MEM_MASK));
         frac_shift = PLL_MEM_INTFRAC_FRAC_SHIFT;
       #else
       /* For Pecos, use the INT/FRAC masks from the MEM_PLL_CTRL2 register, and the
       * MEM_PLL_CTRL1 register for the POSTDIV/PREDIV masks */
         pll_int  = CNXT_GET_VAL(PLL_MEM_CTRL2_REG, PLL_MEM_CTRL2_PLL_N_VALUE_MASK);
         pll_frac = CNXT_GET_VAL(PLL_MEM_CTRL2_REG, PLL_MEM_CTRL2_PLL_FRAC_VALUE_MASK);
         pll_div  = CNXT_GET_VAL(PLL_MEM_CTRL1_REG, PLL_MEM_CTRL1_PLL_POST_DIV_MASK);
         pll_prescale = CNXT_GET_VAL(PLL_MEM_CTRL1_REG, PLL_MEM_CTRL1_PLL_PRE_DIV_MASK);
         frac_shift = PLL_MEM_CTRL2_PLL_FRAC_VALUE_SHIFT;
       #endif
         break;

       case MPG0_PLL_SOURCE:
         pll_int  = CNXT_GET_VAL(PLL_MPG0_INTFRAC_REG, PLL_MPG0_INTFRAC_INT_MASK);
         pll_frac = CNXT_GET_VAL(PLL_MPG0_INTFRAC_REG, PLL_MPG0_INTFRAC_FRAC_MASK);
         pll_div  = 1; /* We want the basic PLL rate here, not the MPG0 clock rate */
       #if (PLL_TYPE == TRINITY)
         pll_prescale = PLL_PRESCALE (CNXT_GET_VAL(PLL_PRESCALE_REG, PLL_PRESCALE_MPG0_MASK));
       #else
         pll_prescale = CNXT_GET_VAL(PLL_MPG0_CTRL_REG, PLL_MPG0_CTRL_PRE_DIV_MASK);
       #endif
         frac_shift = PLL_MPG0_INTFRAC_FRAC_SHIFT;
         break;
       #if (PLL_TYPE != TRINITY)
       case FENRUS_PLL_SOURCE:
     /* For Pecos, there is no dedicated I/O PLL, any of the 9 PLLs can source it.
      * We need to derive this from the PLL_SEL bits for the HSX0 clock*/

          pll_sel  = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_SEL_MASK);

          switch(pll_sel)
          {
              /* Handle the Verve PLLs in a common fashion */
              /* All the INTFRAC and CTRL regs for the Verve PLLs start at offset 0 of 
               * PLL_BASE with 0x4 increments. We derive the corresponding INTFRAC and
               * CTRL reg offsets from the corresponding PLL_SEL value
               *
               * INT_MASK = 0x7E000000 
               * FRAC_MASK = 0x01FFFFFF 
               * PREDIV_MASK = 0x000F0000 */
          case  MPG0PLL:
          case  MPG1PLL:
          case  HDPLL:
          case  AUDPLL:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * pll_sel * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
              break;
          case  PLL0:
          case  PLL1:
          case  PLL2:
              pll_int = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
              pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
              pll_div = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
              pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
              break;
          case  FENRUSPLL:
         pll_int  = CNXT_GET_VAL(PLL_FENRUS_CTRL_REG, PLL_FENRUS_CTRL_LOOP_DIV_MASK);
         pll_frac = 0;
         pll_div  = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
               xtal_freq    = CHIP_CRYSTAL_FREQUENCY;
         pll_prescale = XTAL_PRESCALE_FACTOR;
         break;
          default:
              return;
          }
         break;
       #endif
       default:
            return;
    }

  #ifdef DLOAD

    /*
     * If called from the Download (DLOAD) Utility, run-time select the
     * appropriate calculation function.
     */

    #if (PLL_TYPE == TRINITY)
    xtal_freq    = 37125000;
    pll_prescale = 3;
    #else
    xtal_freq    = CHIP_CRYSTAL_FREQUENCY;
    pll_prescale = XTAL_PRESCALE_FACTOR; 
    #endif

  #endif /* DLOAD */

    pll_div = pll_div * pll_prescale;

    /* 
     * calculate clk_freq = xtal_freq*(pll_int+pll_frac/2^n)/pll_div
     * note that algorithm used here may result up to 4Hz less than what it should be
     */
    clk_freq  = xtal_freq / pll_div;
    remainder = xtal_freq % pll_div;

    /* 
     * The no. of fractional bits have not changed across Pecos and Trinity, except
     * for the MEM PLL in Pecos which is 20 fractional bits, instead of the usual 25 bits
     */


      if (pll_source == MEM_PLL_SOURCE && PLL_TYPE == PECOS)
	   {
	    clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (32-frac_shift))
                      + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div);	
           }
#if (PLL_TYPE != TRINITY)
      else if (pll_source == FENRUS_PLL_SOURCE && PLL_TYPE == PECOS && pll_sel == FENRUSPLL)
           {
            clk_freq = (clk_freq /pll_div) * pll_int;
           }
#endif
	   else
	   {
	    clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (25-frac_shift))
                      + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div);	
	   }  	

    /*
     * Calculate clk_period = (10^11)/clk_freq
     * note that the algorithm will be broken if clk_freq is higher
     * than 429.5MHz.  it should not happen in the predictable future.
     */
    clk_period  = ( 1000000000 / clk_freq ) * 100;
    remainder   = ( 1000000000 % clk_freq ) * 10;

    clk_period += ( remainder / clk_freq ) * 10;
    remainder   = ( remainder % clk_freq ) * 10;

    clk_period += remainder / clk_freq;

    /*
     * Return the values
     */
    *frequency = clk_freq;
    *period    = clk_period;
}

/* Yet to see if this re-sourcing the PIX1CLK from MPG1PLL to MPG0PLL is needed 
*   in Pecos. For now we assume that we don't need this 
*/
#if (PLL_TYPE == TRINITY)

/********************************************************************/
/*  ClockDRM1FromMPG0PLL                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      This function reconfigures PIX1CLK such that it is sourced  */
/*      from MPG0PLL rather than MPG1PLL (i.e. the same PLL as      */
/*      PIX0CLK). This is needed when running in a mode where the   */
/*      DRMs need to be synchronised.                               */
/*                                                                  */
/*      The function takes into account the current frequency of    */
/*      the MPG0PLL and sets the PIXCLK1 divider appropriately to   */
/*      ensure that the output is 27MHz as required.                */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      uXTalFreq - the IRD crystal frequency in Hz.                */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Nothing                                                     */
/********************************************************************/
#define PIX1CLK_FREQUENCY 27000000

void ClockDRM1FromMPG0PLL(u_int32 uXtalFreq)
{
   u_int32 uFreq;
   u_int32 uPeriod;
   u_int32 uDivider;
   u_int32 uTemp;
   
   /* Unlock access to the PLL divider registers */
   PLL_UNLOCK ();
   #if (PLL_TYPE == TRINITY)
   CNXT_SET(PLL_LOCK_STAT_REG,
            PLL_LOCK_STAT_ALL_DIVIDER_MASK,
            PLL_LOCK_STAT_ALL_DIVIDER_WRITE_ENABLE);
   #else
      CNXT_SET(PLL_LOCK_STAT_REG,
            PLL_LOCK_STAT_PLL_CTRL_MASK,
            PLL_LOCK_STAT_PLL_CTRL_WRITE_ENABLE);
   #endif
   PLL_LOCK ();

   /* Set up PIX1CLK so that it is sourced from MPG0 PLL and has the 
      appropriate postdivide to generate 27MHz from whatever the current
      MPG0 PLL clock rate is. */
   
   /* What is the current MPG0 rate? */
   CalcClkFreqAndPeriod(&uFreq, &uPeriod, MPG0_PLL_SOURCE, uXtalFreq);
   
   /* What is the divider we need to use to provide the expected 
      PIX1CLK frequency? */
   
   uDivider = uFreq / PIX1CLK_FREQUENCY;
   
   /* Make sure that the MPG0 PLL rate is an integer multiple of the
      desired clock rate. If it isn't, we have a problem so don't change
      the clocking from the current setting! */
   if((uDivider * PIX1CLK_FREQUENCY) == uFreq)
   {
      /* We have an integer divider so update the PIX1CLK source and divider */
      uTemp = CNXT_GET(PLL_DIV2_REG, 0xFFFFFFFF);
      uTemp = uTemp & ~(PLL_DIV2_PIX1CLK_POSTDIV_MASK | PLL_DIV2_PIX1CLK_SEL_MASK);
      uTemp = uTemp | (uDivider << PLL_DIV2_PIX1CLK_POSTDIV_SHIFT);
      uTemp = uTemp | (MPG0PLL << PLL_DIV0_MEMCLK_SEL_SHIFT);
      CNXT_SET(PLL_DIV2_REG, 0xFFFFFFFF, uTemp);
   }
   
   /* Lock access to the PLL divider registers */
   PLL_UNLOCK ();
   #if (PLL_TYPE == TRINITY)
   CNXT_SET(PLL_LOCK_STAT_REG,
            PLL_LOCK_STAT_ALL_DIVIDER_MASK,
            PLL_LOCK_STAT_ALL_DIVIDER_WRITE_DISABLE);
    #else
      CNXT_SET(PLL_LOCK_STAT_REG,
            PLL_LOCK_STAT_PLL_CTRL_MASK,
            PLL_LOCK_STAT_PLL_CTRL_WRITE_DISABLE);
   #endif
   PLL_LOCK ();

}

#endif

/****************************************************************************
 * Modifications:
 * $Log:
 *  12   mpeg       1.11        9/7/07 5:23:59 PM CDT  Billy Jackman   CR(s)
 *       31572 : Added code to check the PLL source correctly for Fenrus (I/O
 *       clock) measurements.
 *  11   mpeg       1.10        2/5/07 5:56:57 AM CST  Nitin Garg      CR(s)
 *       23931 23932 : Checkpoint is an assembly function and it was used in C
 *        source code. When we switch ON the optimization, the registers were
 *       getting corrupted. Instead of Checkpoint, we should use CheckPointC
 *       function after mini_serial_init has happened.
 *  10   mpeg       1.9         1/19/07 4:37:16 PM CST Nitin Garg      CR(s)
 *       23462 : Divide by zero error as pll_prescale was not getting set for
 *       FENRUS_PLL.
 *  9    mpeg       1.8         12/10/06 10:57:02 AM CSTBilly Jackman   CR(s)
 *       22136 : Get rid of compiler warnings from RVDS.
 *  8    mpeg       1.7         12/5/06 4:32:34 PM CST RadhaKumar Pulyala CR(s)
 *        22227 : Change the MEM PLL source to FERNUS PLL source for FLASH
 *
 *       
 *  7    mpeg       1.6         10/27/06 3:41:58 PM CDTMahesh Mudireddy CR(s)
 *       21393 : Pecos Code Loader changes
 *       Pecos HWlib Changes
 *       Changes to build for UFP With basic functionality
 *       Changes to Build with RVDS/RVCT
 *  6    mpeg       1.5         10/4/06 5:09:47 PM CDT Bob Van Gulick  CR(s)
 *       21045 :  Drop initial changes required for Pecos bringup
 *
 *       
 *  5    mpeg       1.4         6/9/06 2:05:40 PM CDT  Dave Wilson     CR(s)
 *       19586 19585 : Added a function to allow PIX1CLK (for DRM1) to be
 *       moved to the MPG0 PLL (from MPG1). This is needed when using a
 *       sync-DRM version of the video microcode to ensure that both DRMs are
 *       driven from the same clock.
 *  4    mpeg       1.3         5/23/05 9:43:56 AM CDT Bobby Bradford  CR(s)
 *       14039 : Restore MEMCLK calcuations for RevB to be the same as for Rev
 *        A (per Eric Deal)
 *  3    mpeg       1.2         5/11/05 4:51:32 PM CDT Bobby Bradford  CR(s)
 *       14092 : Modify the calculation of MemClk(Freq/Period) to account for
 *       post divider changes in Trinity Rev B.
 *  2    mpeg       1.1         10/15/04 4:52:07 PM CDTLucy Allevato   CR(s)
 *       11118 : Removed legacy code and/or Cosmetic changes
 *  1    mpeg       1.0         4/7/04 9:42:58 PM CDT  Joe Kroesche    CR(s)
 *       8697 : pll support
 * $
 ****************************************************************************/


