/****************************************************************************/
/* CONEXANT PROPRIETARY AND CONFIDENTIAL                                    */
/* SOFTWARE FILE/MODULE HEADER                                              */
/* Conexant Systems Inc. (c) 2006-2007                                      */
/* Austin, TX                                                               */
/* All Rights Reserved                                                      */
/****************************************************************************/
/*
* Filename: nfc_pecos.h
*
*
* Description: Public header file defining hardware-specific values
* (such as register addresses, bit definitions, etc.) for NAND flash  
* controller module.
*
*
* DEPENDENCY: THIS HEADER MUST BE INCLUDED ONLY AFTER PCI DEFINITIONS ARE 
*    INCLUDED OR PLACED.
*
*
* Author:  RadhaKumar Pulyala
*
****************************************************************************/
/* $Id: nfc_pecos.h 101069 2009-07-02 06:00:07Z rathd $
****************************************************************************/

#ifndef _NFC_PECOS_H_
#define _NFC_PECOS_H_

#define  NFC_BASE                                              (PCI_BASE)
#define  NFC_IO_TYPE_BASE                             (NFC_BASE + 0x1000)

/*******************************************/
/* ROM type interface register definitions */
/*******************************************/

/* PCI ROM Descriptor and interrupt registers (enable, status) are    */
/* defined in PCI space. The below are just for reference and         */
/* completeness sake.                                                 */
#ifndef _PCI_PECOS_H

#define PCI_ROM_DESC0_REG                                (PCI_BASE + 0x00)
#define PCI_ISAROM_DESC1_REG                             (PCI_BASE + 0x04)
#define     PCI_ROM_DESC_WIDTH_MASK                            0x00000030
#define     PCI_ROM_DESC_WIDTH_SHIFT                                    4
#define         PCI_ROM_DESC_WIDTH_32                             (0UL<<4)
#define         PCI_ROM_DESC_WIDTH_8                              (1UL<<4)
#define         PCI_ROM_DESC_WIDTH_16                             (2UL<<4)

/* Added in Pecos rev B*/
#define     PCI_ROM_DESC_NOR_NAND_MODE_MASK                    0x000000C0
#define     PCI_ROM_DESC_NOR_NAND_MODE_SHIFT                            6
#define         PCI_ROM_DESC_NOR_NAND_MODE_NAND                   (0UL<<4)
#define         PCI_ROM_DESC_NOR_NAND_MODE_NOR                    (1UL<<4)

#define     PCI_ROM_DESC_CS_MASK                               0x00000700
#define     PCI_ROM_DESC_CS_SHIFT                                       8
#define     PCI_ROM_DESC_BURST_ENABLE_MASK                     0x00000800
#define     PCI_ROM_DESC_BURST_ENABLE_SHIFT                            11
#define         PCI_ROM_DESC_BURST_DISABLE                       (0UL<<11)
#define         PCI_ROM_DESC_BURST_ENABLE                        (1UL<<11)
#define     PCI_ROM_DESC_BURST_WAIT_MASK                       0x0000F000
#define     PCI_ROM_DESC_BURST_WAIT_SHIFT                              12
#define     PCI_ROM_DESC_READ_WAIT_MASK                        0x007F0000
#define     PCI_ROM_DESC_READ_WAIT_SHIFT                               16
#define     PCI_ROM_DESC_FLASH_PROGRAM_MASK                    0x00800000
#define     PCI_ROM_DESC_FLASH_PROGRAM_SHIFT                           23
#define         PCI_ROM_DESC_FLASH_NORMAL_MODE                   (0UL<<23)
#define         PCI_ROM_DESC_FLASH_PROGRAM_MODE                  (1UL<<23)
#define     PCI_ROM_DESC_WRITE_WAIT_MASK                       0x7F000000
#define     PCI_ROM_DESC_WRITE_WAIT_SHIFT                              24
#define     PCI_ROM_DESC_TYPE_MASK                             0x80000000
#define     PCI_ROM_DESC_TYPE_SHIFT                                    31
#define         PCI_ROM_DESC_TYPE_ROM                            (0UL<<31)
#define         PCI_ROM_DESC_TYPE_ISA                            (1UL<<31)

#define PCI_INTR_ENABLE_REG                              (PCI_BASE + 0x50)
#define PCI_INTR_STATUS_REG                              (PCI_BASE + 0x54)
#define     PCI_INTR_INTA_ENABLE_MASK                          0x00000001
#define     PCI_INTR_INTA_ENABLE_SHIFT                                  0
#define         PCI_INTR_INTA_ENABLE                              (1UL<<0)
#define     PCI_INTR_RESET_ASSERTED_MASK                       0x00000002
#define     PCI_INTR_RESET_ASSERTED_SHIFT                               1
#define         PCI_INTR_RESET_ASSERTED                           (1UL<<1)
#define     PCI_INTR_RESET_DEASSERTED_MASK                     0x00000004
#define     PCI_INTR_RESET_DEASSERTED_SHIFT                             2
#define         PCI_INTR_RESET_DEASSERTED                         (1UL<<2)
#define     PCI_INTR_RETRY_TIMEOUT_MASK                        0x00000008
#define     PCI_INTR_RETRY_TIMEOUT_SHIFT                                3
#define         PCI_INTR_RETRY_TIMEOUT                            (1UL<<3)
#define     PCI_INTR_PAR_ERR_MASK                              0x00000400
#define     PCI_INTR_PAR_ERR_SHIFT                                     10
#define         PCI_INTR_PAR_ERR                                 (1UL<<10)
#define     PCI_INTR_MTABT_MASK                                0x00001000
#define     PCI_INTR_MTABT_SHIFT                                       12
#define         PCI_INTR_MTABT                                   (1UL<<12)
#define     PCI_INTR_MABT_MASK                                 0x00002000
#define     PCI_INTR_MABT_SHIFT                                        13
#define         PCI_INTR_MABT                                    (1UL<<13)
#define     PCI_INTR_SERR_MASK                                 0x00004000
#define     PCI_INTR_SERR_SHIFT                                        14
#define         PCI_INTR_SERR                                    (1UL<<14)
#define     PCI_INTR_PERR_MASK                                 0x00008000
#define     PCI_INTR_PERR_SHIFT                                        15
#define         PCI_INTR_PERR                                    (1UL<<15)

/* Added in Pecos rev B*/
#define     PCI_INTR_WERR_MASK                                (0x00010000)
#define     PCI_INTR_WERR_SHIFT                                        16
#define         PCI_INTR_WERR                                    (1UL<<16)
#define     PCI_INTR_ECCERR_MASK                               0x00020000
#define     PCI_INTR_ECCERR_SHIFT                                      17
#define         PCI_INTR_ECCERR                                  (1UL<<17)
#define     PCI_INTR_SOFTRST_MASK                              0x00040000
#define     PCI_INTR_SOFTRST_SHIFT                                     18
#define         PCI_INTR_SOFTRST                                 (1UL<<18)

#endif  /*  _PCI_PECOS_H */

/* Added registers in Pecos rev B for NFC functionality */

#define  NFC_FLUSH_REG                                   (NFC_BASE + 0x5C)
#define     NFC_FLUSH_BUFFER_MASK                              0x00000001
#define     NFC_FLUSH_BUFFER_SHIFT                                      0
#define         NFC_FLUSH_BUFFER                                     (1UL)
#define     NFC_FLUSH_FIFO_MASK                                0x00010000
#define     NFC_FLUSH_FIFO_SHIFT                                       16
#define         NFC_FLUSH_FIFO                                       (1UL)

#define  NFC_ECC_DATA_OUT_REG                            (NFC_BASE + 0x60)

/* Number of device configuration registers */
#define  NFC_NUM_CONFIG                                                 4

#define  NFC_DEV_CONFIG0_REG                             (NFC_BASE + 0xA0)
#define  NFC_DEV_CONFIG1_REG                             (NFC_BASE + 0xA4)
#define  NFC_DEV_CONFIG2_REG                             (NFC_BASE + 0xA8)
#define  NFC_DEV_CONFIG3_REG                             (NFC_BASE + 0xAC)
#define  NFC_DEV_CONFIG_IO_REG                  (NFC_IO_TYPE_BASE + 0x238)
#define     NFC_FLASH_SIZE_MASK                                0x0000000F
#define     NFC_FLASH_SIZE_SHIFT                                        0
#define        NFC_FLASH_SIZE_32GBITS                                (0UL)
#define        NFC_FLASH_SIZE_16GBITS                                (1UL)
#define        NFC_FLASH_SIZE_8GBITS                                 (2UL)
#define        NFC_FLASH_SIZE_4GBITS                                 (3UL)
#define        NFC_FLASH_SIZE_2GBITS                                 (4UL)
#define        NFC_FLASH_SIZE_1GBITS                                 (5UL)
#define        NFC_FLASH_SIZE_512MBITS                               (6UL)
#define        NFC_FLASH_SIZE_256MBITS                               (7UL)
#define        NFC_FLASH_SIZE_128MBITS                               (8UL)

#define     NFC_PAGE_SIZE_MASK                                 0x00000010
#define     NFC_PAGE_SIZE_SHIFT                                         4
#define        NFC_PAGE_SIZE_512BYTES                                (0UL)
#define        NFC_PAGE_SIZE_2048BYTES                               (1UL)

#define     NFC_BUS_WIDTH_MASK                                 0x00000020
#define     NFC_BUS_WIDTH_SHIFT                                         5
#define        NFC_BUS_WIDTH_8BIT                                    (0UL)
#define        NFC_BUS_WIDTH_16BIT                                   (1UL)

#define     NFC_DEV_TYPE_MASK                                  0x00000040
#define     NFC_DEV_TYPE_SHIFT                                          6
#define        NFC_DEV_TYPE_SLC_SMALL                                (0UL)
#define        NFC_DEV_TYPE_SLC_LARGE                                (1UL)

#define     NFC_ADDR_CYCLES_MASK                               0x00000180
#define     NFC_ADDR_CYCLES_SHIFT                                       7
#define        NFC_ADDR_CYCLES_4CYCLES                               (1UL)
#define        NFC_ADDR_CYCLES_5CYCLES                               (2UL)
#define        NFC_ADDR_CYCLES_3CYCLES                               (3UL)

#define     NFC_ROM_BUF_INVALID_MASK                           0x00000200
#define     NFC_ROM_BUF_INVALID_SHIFT                                   9
#define        NFC_ROM_BUF_INVALID_NO                                (0UL)
#define        NFC_ROM_BUF_INVALID_YES                               (1UL)

#define     NFC_SPARE_AREA_SIZE_MASK                           0x00000400
#define     NFC_SPARE_AREA_SIZE_SHIFT                                  10
#define        NFC_SPARE_AREA_SIZE_16BYTES                           (0UL)
#define        NFC_SPARE_AREA_SIZE_64BYTES                           (1UL)

#define     NFC_ECC_INIT_MASK                                  0x00000800
#define     NFC_ECC_INIT_SHIFT                                         11
#define        NFC_ECC_INIT_OFF                                      (0UL)
#define        NFC_ECC_INIT_ON                                       (1UL)

#define     NFC_ECC_TYPE_MASK                                  0x00001000
#define     NFC_ECC_TYPE_SHIFT                                         12
#define        NFC_ECC_TYPE_HAMM                                     (0UL)
#define        NFC_ECC_TYPE_BCH                                      (1UL)

#define     NFC_ECC_524_MASK                                   0x00002000
#define     NFC_ECC_524_SHIFT                                          13
#define        NFC_ECC_524_OFF                                       (0UL)
#define        NFC_ECC_524_ON                                        (1UL)

#define     NFC_ECC_CORRECT_MASK                               0x00004000
#define     NFC_ECC_CORRECT_SHIFT                                      14
#define        NFC_ECC_CORRECT_OFF                                   (0UL)
#define        NFC_ECC_CORRECT_ON                                    (1UL)

#define     NFC_RDY_BUSY_TIMEOUT_ENBL_MASK                     0x00008000
#define     NFC_RDY_BUSY_TIMEOUT_ENBL_SHIFT                            15
#define        NFC_RDY_BUSY_TIMEOUT_DISBLE                           (0UL)
#define        NFC_RDY_BUSY_TIMEOUT_ENABLE                           (1UL)

/************************************************
TWP  = WE# pulse width
TCLH = CLE hold time
TWH  = WE# pulse width HIGH
TALH = ALE hold time
TWB  = WE# HIGH to Busy
TRR  = Ready to RE# LOW
TRP  = RE# pulse width
TRH  = RE# HIGH hold time
TCLR = CLE to RE# delay for Read Status
TADL = Address to Data Latch while page program
TAR  = ALE to RE# delay for READ ID
*************************************************/

#define  NFC_TIMINGS_PARAMS0_REG                         (NFC_BASE + 0xB0)
#define  NFC_TIMINGS_PARAMS1_REG                         (NFC_BASE + 0xB4)
#define  NFC_TIMINGS_PARAMS2_REG                         (NFC_BASE + 0xB8)
#define  NFC_TIMINGS_PARAMS3_REG                         (NFC_BASE + 0xBC)
#define  NFC_TIMINGS_PARAMS_IO_REG              (NFC_IO_TYPE_BASE + 0x234)
#define     NFC_TIMINGS_TWP_MASK                               0x00000007
#define     NFC_TIMINGS_TWP_SHIFT                                       0
#define     NFC_TIMINGS_TCLH_MASK                              0x00000038
#define     NFC_TIMINGS_TCLH_SHIFT                                      3
#define     NFC_TIMINGS_TWH_MASK                               0x000001C0
#define     NFC_TIMINGS_TWH_SHIFT                                       6
#define     NFC_TIMINGS_TALH_MASK                              0x00000E00
#define     NFC_TIMINGS_TALH_SHIFT                                      9
#define     NFC_TIMINGS_TWB_MASK                               0x00007000
#define     NFC_TIMINGS_TWB_SHIFT                                      12
#define     NFC_TIMINGS_TRR_MASK                               0x00038000
#define     NFC_TIMINGS_TRR_SHIFT                                      15
#define     NFC_TIMINGS_TRP_MASK                               0x001C0000
#define     NFC_TIMINGS_TRP_SHIFT                                      18
#define     NFC_TIMINGS_TRH_MASK                               0x00E00000
#define     NFC_TIMINGS_TRH_SHIFT                                      21
#define     NFC_TIMINGS_TCLR_MASK                              0x07000000
#define     NFC_TIMINGS_TCLR_SHIFT                                     24
#define     NFC_TIMINGS_TADL_MASK                              0x38000000
#define     NFC_TIMINGS_TADL_SHIFT                                     27
#define     NFC_TIMINGS_TAR_MASK                               0xC0000000
#define     NFC_TIMINGS_TAR_SHIFT                                      30

#define  NFC_RDY_BUSY_STATUS_REG                         (NFC_BASE + 0xC0)
#define  NFC_MAKER_ID_REG                                (NFC_BASE + 0xC4)
#define  NFC_DEVICE_ID_REG                               (NFC_BASE + 0xC8)
#define  NFC_BUFF_VALID_PAGE_ADDR_REG                    (NFC_BASE + 0xCC)

#define  NFC_LOCK_STRT_ADDR_DES0_REG                     (NFC_BASE + 0xD0)
#define  NFC_LOCK_END_ADDR_DES0_REG                      (NFC_BASE + 0xD4)
#define  NFC_LOCK_STRT_ADDR_DES1_REG                     (NFC_BASE + 0xD8)
#define  NFC_LOCK_END_ADDR_DES1_REG                      (NFC_BASE + 0xDC)
#define  NFC_LOCK_STRT_ADDR_DES2_REG                     (NFC_BASE + 0xE0)
#define  NFC_LOCK_END_ADDR_DES2_REG                      (NFC_BASE + 0xE4)
#define  NFC_LOCK_STRT_ADDR_DES3_REG                     (NFC_BASE + 0xE8)
#define  NFC_LOCK_END_ADDR_DES3_REG                      (NFC_BASE + 0xEC)

#define  NFC_UNLOCK_DES0_REG                             (NFC_BASE + 0xF0)
#define  NFC_UNLOCK_DES1_REG                             (NFC_BASE + 0xF4)
#define  NFC_UNLOCK_DES2_REG                             (NFC_BASE + 0xF8)
#define  NFC_UNLOCK_DES3_REG                             (NFC_BASE + 0xFC)

/* IO Sequencer register set for IO type interface */

#define  NFC_IRAM_BASE_REG                               (NFC_IO_TYPE_BASE)
#define  NFC_IO_SEQ_FIFO_REG                     (NFC_IO_TYPE_BASE + 0x200)
#define  NFC_IO_SEQ_ADDR_REG                     (NFC_IO_TYPE_BASE + 0x204)

#define  NFC_IO_SEQ_STATUS_REG                   (NFC_IO_TYPE_BASE + 0x208)
#define     NFC_IO_SEQ_BUSY_MASK                                0x00000001
#define     NFC_IO_SEQ_SOFT_HLT_ACK_MASK                        0x00000002

#define  NFC_IO_SEQ_GPR0                         (NFC_IO_TYPE_BASE + 0x20C)
#define  NFC_IO_SEQ_GPR1                         (NFC_IO_TYPE_BASE + 0x210)
#define  NFC_IO_SEQ_GPR2                         (NFC_IO_TYPE_BASE + 0x214)
#define  NFC_IO_SEQ_GPR3                         (NFC_IO_TYPE_BASE + 0x218)
#define  NFC_IO_SEQ_GPR4                         (NFC_IO_TYPE_BASE + 0x21C)
#define  NFC_IO_SEQ_GPR5                         (NFC_IO_TYPE_BASE + 0x220)
#define  NFC_IO_SEQ_GPR6                         (NFC_IO_TYPE_BASE + 0x224)
#define  NFC_IO_SEQ_GPR7                         (NFC_IO_TYPE_BASE + 0x228)
#define  NFC_IO_SEQ_GPR8                         (NFC_IO_TYPE_BASE + 0x22C)
#define  NFC_IO_SEQ_GPR9                         (NFC_IO_TYPE_BASE + 0x230)

/* These is covered with ROM type register set. Keeping it for Order.
#define  NFC_TIMINGS_PARAMS_IO_REG               (NFC_IO_TYPE_BASE + 0x234)
#define  NFC_DEV_CONFIG_IO_REG                   (NFC_IO_TYPE_BASE + 0x238)
*/
#define  NFC_IO_RB_TIMEOUT_REG                   (NFC_IO_TYPE_BASE + 0x23C)
#define  NFC_IO_PC_REG                           (NFC_IO_TYPE_BASE + 0x240)
#define  NFC_IO_RD_CNTR_REG                      (NFC_IO_TYPE_BASE + 0x244)
#define  NFC_IO_WR_CNTR_REG                      (NFC_IO_TYPE_BASE + 0x248)
#define  NFC_IO_CS_REG                           (NFC_IO_TYPE_BASE + 0x24C)

#define  NFC_IO_CONFIG_REG                       (NFC_IO_TYPE_BASE + 0x250)
#define     NFC_IO_CFG_PC_STRT_MASK                             0x00000001
#define     NFC_IO_CFG_PC_STRT_SHIFT                                     0
#define     NFC_IO_CFG_PC_STRT                                        (1UL)
#define     NFC_IO_CFG_SOFT_RST_MASK                            0x00000002
#define     NFC_IO_CFG_SOFT_RST_SHIFT                                    1
#define     NFC_IO_CFG_SOFT_RST                                    (1UL<<1)
#define     NFC_IO_CFG_SOFT_HLT_MASK                            0x00000004
#define     NFC_IO_CFG_SOFT_HLT_SHIFT                                    2
#define     NFC_IO_CFG_SOFT_HLT                                       (1UL)

#define  NFC_IO_DMA_CNT1_REG                     (NFC_IO_TYPE_BASE + 0x254)
#define  NFC_IO_DMA_CNT2_REG                     (NFC_IO_TYPE_BASE + 0x258)
#define  NFC_IO_LOCK_STRT_ADDR_REG               (NFC_IO_TYPE_BASE + 0x25C)
#define  NFC_IO_LOCK_END_ADDR_REG                (NFC_IO_TYPE_BASE + 0x260)
#define  NFC_IO_UNLOCK_REG                       (NFC_IO_TYPE_BASE + 0x264)

/* Host DMA controller defines used by NFC */

/* NFC DMA channels */
#define       DMA_CHANNEL6                                              (6)
#define       DMA_CHANNEL7                                              (7)

/* NFC DMA request selection */
#define       DMA_CTRL_REQ_PCI1                                  (49UL<<16)
#define       DMA_CTRL_REQ_PCI2                                  (50UL<<16)


#endif /* _NFC_PECOS_H_ */

/*******************************************************************************
 * Modifications:
 * $Log:
 * $
 *
 ******************************************************************************/
