#ifndef _NFC_PRV_H_
#define _NFC_PRV_H_

/******************************************************************************
 * IMPORTANT
 * ---------
 *
 * PLEASE ENSURE THAT EACH TIME ANY OF THE FOLLOWING FILES ARE MODIFIED, 
 * THE CHANGES ARE COMMITTED TO THE OTHER FILE(S) ALSO.
 *
 * - target/src/linux/kernel/linux-2.6.18.5/drivers/mtd/cnxtnfc/nfc_prv.h
 * - target/src/u-boot/drivers/mtd/nfc/nfc_prv.h
 *
 *****************************************************************************/

#include <asm/arch/hwopts.h>

#include "nfc_pecos.h"

extern unsigned long           ChipRevID; 
#define NFC_HAS_WAIT_RB_BUG   (ChipRevID <= REV_C_NEVIS)

#define FALSE                                (0)
#define TRUE                                 (1)

#define NFC_DMA_ALLOC_COHERENT               (NO)

#define NFC_PIO_ENABLE                       0

#if (NFC_PIO_ENABLE==1)
#define NFC_DMA_WRITE_ENABLE                 0 
#define NFC_DMA_READ_ENABLE                  0
#else //DMA mode
#define NFC_DMA_WRITE_ENABLE                 (NFC_HAS_WAIT_RB_BUG == NO)
#define NFC_DMA_READ_ENABLE                  (NFC_HAS_WAIT_RB_BUG == NO)
#endif

#define NFC_ECC_ENABLE                       (NO)

#define NFC_ERROR_FLAG unsigned __bErrorSeen = FALSE
#define NFC_ERROR_CHECK(seen)                      \
               do                                  \
               {                                   \
                  if((seen) < (0))                 \
                  {                                \
                     printk(KERN_ALERT             \
                        "%s:%d -> Error.\n",       \
                        __FUNCTION__, __LINE__);   \
                     __bErrorSeen = TRUE;          \
                  }                                \
               } while( 0 )
#define NFC_ERROR_OCCURRED __bErrorSeen

#define NFC_STATUS_WAIT(expression)                                  \
               {                                                     \
                  int i = NFC_POLL_WAIT_TIMEOUT;                     \
                  while (--i>=0)                                     \
                  {                                                  \
                     if (expression)                                 \
                        break;                                       \
                     udelay(100);                                    \
                  }                                                  \
                  if(i<0)                                            \
                  {                                                  \
                     printk(KERN_ALERT                               \
                        "%s:%d -> Timeout.\n",                       \
                        __FUNCTION__, __LINE__);                     \
                     return -EIO;                                    \
                  }                                                  \
               }

/* Status Register bit definitions for Samsung Flash */
#define  FLASH_STATUS_PROGRAM_ERASE_MASK           0x1
#define  FLASH_STATUS_DEV_OPERATION_MASK           0x40  /* 0: Busy   1: Ready*/
#define  FLASH_STATUS_WRITE_PROTECT_MASK           0x80

/* Timeouts */
#define  NFC_POLL_WAIT_TIMEOUT                     1000

/************************************/
/*    Micro Code Declarations       */
/************************************/
extern unsigned int NFCOpnReadId[];
extern unsigned int NFCOpnReadIdSize;
extern unsigned int NFCOpnReadStatus[];
extern unsigned int NFCOpnReadStatusSize;
extern unsigned int NFCOpnWaitForDevRdy[];
extern unsigned int NFCOpnWaitForDevRdySize;
extern unsigned int NFCOpnReadPage_WA[];
extern unsigned int NFCOpnReadPageSize_WA;
extern unsigned int NFCOpnReadPage[];
extern unsigned int NFCOpnReadPageSize;
extern unsigned int NFCOpnReadSpare_WA[];
extern unsigned int NFCOpnReadSpareSize_WA;
extern unsigned int NFCOpnReadSpare[];
extern unsigned int NFCOpnReadSpareSize;
extern unsigned int NFCOpnReadBadBlkMarker_WA[];
extern unsigned int NFCOpnReadBadBlkMarkerSize_WA;
extern unsigned int NFCOpnReadBadBlkMarker[];
extern unsigned int NFCOpnReadBadBlkMarkerSize;
extern unsigned int NFCOpnBlockErase_WA[];
extern unsigned int NFCOpnBlockEraseSize_WA;
extern unsigned int NFCOpnBlockErase[];
extern unsigned int NFCOpnBlockEraseSize;
extern unsigned int NFCOpnPageWrite_WA[];
extern unsigned int NFCOpnPageWriteSize_WA;
extern unsigned int NFCOpnPageWrite[];
extern unsigned int NFCOpnPageWriteSize;
extern unsigned int NFCOpnSpareWrite_WA[];
extern unsigned int NFCOpnSpareWriteSize_WA;
extern unsigned int NFCOpnSpareWrite[];
extern unsigned int NFCOpnSpareWriteSize;
extern unsigned int NFCOpnReset[];
extern unsigned int NFCOpnResetSize;
/*
extern unsigned int NFCOpnReadPageEccON[];
extern unsigned int NFCOpnReadPageEccONSize;
extern unsigned int NFCOpnPageWriteEccON[];
extern unsigned int NFCOpnPageWriteEccONSize;
*/
extern unsigned int NFCOpnTestUCode[];
extern unsigned int NFCOpnTestUCodeSize;


/************************************/
/*  Internal function declarations  */
/************************************/
void nfc_dma_read_setup(
               unsigned int   dest_addr,
               unsigned int   byte_count,
               unsigned int   request_sel,
               unsigned int   channel);
void nfc_dma_write_setup(
               unsigned int   src_addr,
               unsigned int   byte_count,
               unsigned int   request_sel,
               unsigned int   channel);
void nfc_dma_start(
               unsigned int   channel);
int nfc_dma_wait2complete(
               unsigned int   channel);
void nfc_dma_clear(
               unsigned       channel);
void nfc_dev_lock(void);
void nfc_dev_unlock(void);

#endif /* _NFC_PRV_H_ */

