/******************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                    */
/*                        SOFTWARE FILE/MODULE HEADER                         */
/*                 Copyright Conexant Systems Inc. 2007                       */
/*                            All Rights Reserved                             */
/******************************************************************************/
/*
 * Filename:      nfc_ucode.c
 *
 *
 * Description:   Micro code for NAND flash controller (NFC) driver.
 *
 *
 * Author:        Manish Kr. Dewangan, Deven Balani
 *
 ******************************************************************************/
/* $Id: nfc_ucode.c 72951 2009-01-21 09:10:36Z balanid $
 ******************************************************************************/

/******************************************************************************
 * IMPORTANT
 * ---------
 *
 * PLEASE ENSURE THAT EACH TIME ANY OF THE FOLLOWING FILES ARE MODIFIED, 
 * THE CHANGES ARE COMMITTED TO ALL FILES.
 *
 * - target/src/linux/kernel/linux-2.6.18.5/drivers/mtd/cnxtnfc/nfc_ucode.c
 * - target/src/u-boot/drivers/mtd/nfc/nfc_ucode.c
 * - target/src/cnxt_base/drivers/nfc/nfc_ucode.c
 *
 *****************************************************************************/

/*****************/
/* Include Files */
/*****************/

/*******************/
/* Local Variables */
/*******************/

/********************************
; send READ ID command
CMDD 0x90
;Start address cycle and send 0x00 as address
ADDRD 0x00
WAIT TWB
WAIT TAR
;Read Maker ID and Device ID
READ1 R0
READ1 R1
HLT
*********************************/
const unsigned int NFCOpnReadId[] = {
   0x0,
   0x90,
   0x2,
   0x00,
   0x1407,
   0x1a07,
   0xeb,
   0x1eb,
   0xe4,
};
const unsigned int NFCOpnReadIdSize = sizeof(NFCOpnReadId)/sizeof(unsigned int);

/********************************
;Send Read Status Command
CMDD 0x70
;Wait for Command Latch Reset
WAIT TCLR
;Read the status register (1 byte)
; into R0
READ1 R0
HLT
*********************************/
const unsigned int NFCOpnReadStatus[] = {
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4
};
const unsigned int NFCOpnReadStatusSize = sizeof(NFCOpnReadStatus)/sizeof(unsigned int);

/********************************  
MOVD R2 0x40
CMDD 0x70  #LABEL1
WAIT TCLR
READ1 R1
AND R1 R2
CMP R1 R2
BRNZ LABEL1
XOR R0 R0
HLT
*********************************/
const unsigned int NFCOpnWaitForDevRdy[] = {
   0x204,
   0x40,
   0x0,
   0x70,
   0x1807,
   0x1eb,
   0x810d,
   0x8115,
   0xea,
   0x38004402,
   0xf,
   0xe4
};
const unsigned int NFCOpnWaitForDevRdySize = sizeof(NFCOpnWaitForDevRdy)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; CMD for page read
CMDD 0x00
; send column address as Zero
; as we read entire page
ADDRD 0x00
ADDRD 0x00
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
; wait till the command completes
WAIT TWB
;WAITRB
MOVD R5 0x0
WAITD 0x07  #LABEL1
ADDD R5 0x1
CMPD R5 0x200
BRNZ LABEL1
WAIT TRR
; read 2K bytes page
READD 0x800
;Enable Spare Area
SPAREEN
; read 64 bytes spare area
READD 0x40
;Disable spare area
SPAREDIS
HLT
**********************************************/
const unsigned int NFCOpnReadPage_WA[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x00,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x508,
   0x1,
   0x514,
   0x200,
   0xea,
   0x38004412,
   0x1507,
   0x16,
   0x800,
   0xee,
   0x16,
   0x40,
   0xef,
   0xe4,
};
const unsigned int NFCOpnReadPageSize_WA = sizeof(NFCOpnReadPage_WA)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; CMD for page read
CMDD 0x00
; send column address as Zero
; as we read entire page
ADDRD 0x00
ADDRD 0x00
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
; wait till the command completes
WAIT TWB
WAITRB
WAIT TRR
; read 2K bytes page
READD 0x800
;Enable Spare Area
SPAREEN
; read 64 bytes spare area
READD 0x40
;Disable spare area
SPAREDIS
HLT
**********************************************/
const unsigned int NFCOpnReadPage[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x00,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0xe5,
   0x1507,
   0x16,
   0x800,
   0xee,
   0x16,
   0x40,
   0xef,
   0xe4,
};
const unsigned int NFCOpnReadPageSize = sizeof(NFCOpnReadPage)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; Command for page read
CMDD 0x00
; send column address as with
; CA[11] = 1 to read spare area.
ADDRD 0x00
ADDRD 0x08
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
WAIT TWB
;WAITRB
MOVD R5 0x0
WAITD 0x07  #LABEL1
ADDD R5 0x1
CMPD R5 0x200
BRNZ LABEL1
WAIT TRR
; read 64 bytes spare area
READD 0x40
HLT
**********************************************/
const unsigned int NFCOpnReadSpare_WA[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x508,
   0x1,
   0x514,
   0x200,
   0xea,
   0x38004412,
   0x1507,
   0x16,
   0x40,
   0xe4,
};
const unsigned int NFCOpnReadSpareSize_WA = sizeof(NFCOpnReadSpare_WA)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; Command for page read
CMDD 0x00
; send column address as with
; CA[11] = 1 to read spare area.
ADDRD 0x00
ADDRD 0x08
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
WAIT TWB
WAITRB
WAIT TRR
; read 64 bytes spare area
READD 0x40
HLT
**********************************************/
const unsigned int NFCOpnReadSpare[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0xe5,
   0x1507,
   0x16,
   0x40,
   0xe4,
};
const unsigned int NFCOpnReadSpareSize = sizeof(NFCOpnReadSpare)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; Command for page read
CMDD 0x00
; send column address as with
; CA[11] = 1 to read spare area.
ADDRD 0x00
ADDRD 0x08
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
WAIT TWB
;WAITRB
MOVD R5 0x0
WAITD 0x07  #LABEL1
ADDD R5 0x1
CMPD R5 0x200
BRNZ LABEL1
WAIT TRR
; read a byte into R1
READ1 R1
READ1 R2
HLT
**********************************************/
const unsigned int NFCOpnReadBadBlkMarker_WA[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x508,
   0x1,
   0x514,
   0x200,
   0xea,
   0x38004412,
   0x1507,
   0x1eb,
   0x2eb,
   0xe4,
};
const unsigned int NFCOpnReadBadBlkMarkerSize_WA = sizeof(NFCOpnReadBadBlkMarker_WA)/sizeof(unsigned int);

/*********************************************
; R0 contains the row address
; Command for page read
CMDD 0x00
; send column address as with
; CA[11] = 1 to read spare area.
ADDRD 0x00
ADDRD 0x08
; send the row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
WAIT TWB
WAITRB
WAIT TRR
; read a byte into R1
READ1 R1
READ1 R2
HLT
**********************************************/
const unsigned int NFCOpnReadBadBlkMarker[] = {
   0x0,
   0x00,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0xe5,
   0x1507,
   0x1eb,
   0x2eb,
   0xe4,
};
const unsigned int NFCOpnReadBadBlkMarkerSize = sizeof(NFCOpnReadBadBlkMarker)/sizeof(unsigned int);

#if 0
/******************************************************
; R0 contains the page address.
; Copy it to R1 for safe keeping.
MOV R1 R0
; Send command 0 to point to first half page.
CMDD 0x0
; Send the column address.
; We always read pages, so column address is zero.
ADDRD 0x0
ADDRD 0x0
; Send Row address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
CMDD 0x30
;Wait for ready signal
WAIT TWB
WAITRB
; Read the 2KB page area
READD 0x200
;Copy first ECC word
MOV R1 ECALC
READD 0x200
;Copy second ECC word
MOV R2 ECALC
READD 0x200
;Copy third ECC word
MOV R3 ECALC
READD 0x200
;Copy forth ECC word
MOV R4 ECALC
; Enable spare area.
SPAREEN
; Read the spare area.
READD 0x40
; Disable the spare area
SPAREDIS
HLT
*******************************************************/
const unsigned int NFCOpnReadPageEccON[] = {
   0x105,
   0x0,
   0x0,
   0x2,
   0x0,
   0x2,
   0x0,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0x30,
   0x1407,
   0xe5,
   0x16,
   0x200,
   0x84105,
   0x16,
   0x200,
   0x84205,
   0x16,
   0x200,
   0x84305,
   0x16,
   0x200,
   0x84405,
   0xee,
   0x16,
   0x40,
   0xef,
   0xe4,
};
const unsigned int NFCOpnReadPageEccONSize = sizeof(NFCOpnReadPageEccON)/sizeof(unsigned int);
#endif

/**********************************************
;R0 contains the address of the block to be erased
; Send Auto Block Erase setup command
CMDD 0x60
; Send the block address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
;Send Erase command
CMDD 0xD0
WAIT TWB
; Wait for the erase to complete
MOVD R5 0x0
WAITD 0x07  #LABEL1
ADDD R5 0x1
CMPD R5 0x5000    
BRNZ LABEL1
; Get status into R0
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
**********************************************/
const unsigned int NFCOpnBlockErase_WA[] = {
   0x0,
   0x60,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0xD0,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x508,
   0x1,
   0x514,
   0x5000,
   0xea,
   0x3800440e,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnBlockEraseSize_WA = sizeof(NFCOpnBlockErase_WA)/sizeof(unsigned int);

/**********************************************
;R0 contains the address of the block to be erased
; Send Auto Block Erase setup command
CMDD 0x60
; Send the block address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
;Send Erase command
CMDD 0xD0
WAIT TWB
WAITRB
; Get status into R0
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
**********************************************/
const unsigned int NFCOpnBlockErase[] = {
   0x0,
   0x60,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x0,
   0xD0,
   0x1407,
   0xe5,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnBlockEraseSize = sizeof(NFCOpnBlockErase)/sizeof(unsigned int);

#if 0
/**********************************************
; send command 0 to point to
; first half of the page.
; with this, we can write to first half,
; second half and spare area.
CMDD 0x00
; send sequential data input command
CMDD 0x80
; wait for TWC
WAITD 0x100 ; TWC
; send column address.
; zero because we would write entire page
ADDRD 0x00
ADDRD 0x00
; send page (row) address from R0
ADDR R0
SHRD R0 8
ADDR R0
SHRD R0 8
ADDR R0
; wait for TRR
WAIT TRR
;write data on the IO lines
;write both the pages
WRITED 0x800
;Enable spare area
SPAREEN
WRITED 0x40
;Disable Spare area
SPAREDIS
; Issue Program command.
CMDD 0x10
;wait for the write completion
WAITD 0x110
WAIT TWB
WAIT TWB
WAIT TWB
WAITRB
; read status
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
****************************************************/
const unsigned int NFCOpnPageWriteEccON[] = {
   0x0,
   0x00,
   0x0,
   0x80,
   0x6,
   0x100,
   0x2,
   0x00,
   0x2,
   0x00,
   0x3,
   0x1c,
   8,
   0x3,
   0x1c,
   8,
   0x3,
   0x1507,
   0x18,
   0x800,
   0xee,
   0x18,
   0x40,
   0xef,
   0x0,
   0x10,
   0x6,
   0x110,
   0x1407,
   0x1407,
   0x1407,
   0xe5,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnPageWriteEccONSize = sizeof(NFCOpnPageWriteEccON)/sizeof(unsigned int);
#endif

/**********************************************
; send sequential data input command
CMDD 0x80
; wait for TWC
WAITD 0x100 ; TWC
; send column address.
; zero because we would write entire page
ADDRD 0x00
ADDRD 0x00
; send page (row) address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
; wait for TRR
WAIT TRR
;write the main page
WRITED 0x800
; Issue Program command.
CMDD 0x10
;wait for the write completion
WAITD 0x110
WAIT TWB
WAIT TWB
;WAITRB
MOVD R5 0x0
WAITD 0x07  #LABEL1
ADDD R5 0x1
CMPD R5 0x4000
BRNZ LABEL1
; read status
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
****************************************************/
const unsigned int NFCOpnPageWrite_WA[] = {
   0x0,
   0x80,
   0x6,
   0x100,
   0x2,
   0x00,
   0x2,
   0x00,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1507,
   0x18,
   0x800,
   0x0,
   0x10,
   0x6,
   0x110,
   0x1407,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x1,
   0x508,
   0x514,
   0x4000,
   0xea,
   0x3800441a,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnPageWriteSize_WA = sizeof(NFCOpnPageWrite_WA)/sizeof(unsigned int);

/**********************************************
; send sequential data input command
CMDD 0x80
; wait for TWC
WAITD 0x100 ; TWC
; send column address.
; zero because we would write entire page
ADDRD 0x00
ADDRD 0x00
; send page (row) address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
; wait for TRR
WAIT TRR
;write the main page
WRITED 0x800
; Issue Program command.
CMDD 0x10
;wait for the write completion
WAITD 0x110
WAIT TWB
WAIT TWB
WAITRB
; read status
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
****************************************************/
const unsigned int NFCOpnPageWrite[] = {
   0x0,
   0x80,
   0x6,
   0x100,
   0x2,
   0x00,
   0x2,
   0x00,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1507,
   0x18,
   0x800,
   0x0,
   0x10,
   0x6,
   0x110,
   0x1407,
   0x1407,
   0xe5,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnPageWriteSize = sizeof(NFCOpnPageWrite)/sizeof(unsigned int);

/**********************************************
; send sequential data input command
CMDD 0x80
; wait for TWC
WAITD 0x100 ; TWC
; send column address.
; zero because we would write entire page
ADDRD 0x00
ADDRD 0x08
; send page (row) address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
; wait for TRR
WAIT TRR
;write the main page
WRITED 0x40
; Issue Program command.
CMDD 0x10
;wait for the write completion
WAITD 0x110
WAIT TWB
WAIT TWB
;WAITRB
MOVD R5 0x0
WAITD 0x07   #LABEL1
ADDD R5 0x1
CMPD R5 0x4000
BRNZ LABEL1
; read status
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
****************************************************/
const unsigned int NFCOpnSpareWrite_WA[] = {
   0x0,
   0x80,
   0x6,
   0x100,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1507,
   0x18,
   0x40,
   0x0,
   0x10,
   0x6,
   0x110,
   0x1407,
   0x1407,
   0x504,
   0x0,
   0x6,
   0x07,
   0x508,
   0x1,
   0x514,
   0x4000,
   0xea,
   0x3800441a,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnSpareWriteSize_WA = sizeof(NFCOpnSpareWrite_WA)/sizeof(unsigned int);

/**********************************************
; send sequential data input command
CMDD 0x80
; wait for TWC
WAITD 0x100 ; TWC
; send column address.
; zero because we would write entire page
ADDRD 0x00
ADDRD 0x08
; send page (row) address from R0
ADDR R0
SHRD R0 0x8
ADDR R0
SHRD R0 0x8
ADDR R0
; wait for TRR
WAIT TRR
;write the main page
WRITED 0x40
; Issue Program command.
CMDD 0x10
;wait for the write completion
WAITD 0x110
WAIT TWB
WAIT TWB
WAITRB
; read status
CMDD 0x70
WAIT TCLR
READ1 R0
HLT
****************************************************/
const unsigned int NFCOpnSpareWrite[] = {
   0x0,
   0x80,
   0x6,
   0x100,
   0x2,
   0x00,
   0x2,
   0x08,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1c,
   0x8,
   0x3,
   0x1507,
   0x18,
   0x40,
   0x0,
   0x10,
   0x6,
   0x110,
   0x1407,
   0x1407,
   0xe5,
   0x0,
   0x70,
   0x1807,
   0xeb,
   0xe4,
};
const unsigned int NFCOpnSpareWriteSize = sizeof(NFCOpnSpareWrite)/sizeof(unsigned int);

/**********************************************
; Send RESET command
CMDD 0xFF
; Wait for the longest time of all - TRST
WAITD 0xFFFF ; TRST
; Send Status command
CMDD 0x70
WAIT TRR
READ1 R0
HLT
***********************************************/
const unsigned int NFCOpnReset[] = {
   0x0,
   0xFF,
   0x6,
   0xFFFF,
   0x0,
   0x70,
   0x1507,
   0xeb,
   0xe4,

};
const unsigned int NFCOpnResetSize = sizeof(NFCOpnReset)/sizeof(unsigned int);

const unsigned int NFCOpnTestUCode[] = {
   0x1e6,
   0x2e7,
   0xe4
};
const unsigned int NFCOpnTestUCodeSize = sizeof(NFCOpnTestUCode)/sizeof(unsigned int);

/****************************************************************************
 * Modifications:
 * $Log:
 * $
 *
 ****************************************************************************/

