/*
 * $Id: cnxt_map_funcs.c,v 1.1, 2007-08-09 14:51:48Z, RadhaKumar Pulyala$
 *
 * Out-of-line map I/O functions for cnxt maps when CONFIG_COMPLEX_MAPPINGS
 * is enabled.
 * 
 * modified version of map_funcs.c with CNXT specitic APIs.
 */

#include <linux/kernel.h>
#include <linux/module.h>

#include <linux/mtd/map.h>

#include <asm/arch/platform.h>
extern int  GetCalcClkFreqAndPeriod( unsigned int *pMemClkFreq32 ,unsigned *pMemClkPeriod32);
static int boot_from_nand;


inline void enter_flash_program_mode(void)
{
   writel(readl(PCI_ROM_DESC0_REG)| PCI_ROM_DESC_FLASH_PROGRAM_MODE , PCI_ROM_DESC0_REG);
   writel(readl(PCI_ISAROM_DESC1_REG)| PCI_ROM_DESC_FLASH_PROGRAM_MODE , PCI_ISAROM_DESC1_REG);
}

inline void exit_flash_program_mode(void)
{
   writel(readl(PCI_ROM_DESC0_REG) &  ~(PCI_ROM_DESC_FLASH_PROGRAM_MODE) , PCI_ROM_DESC0_REG);
   writel(readl(PCI_ISAROM_DESC1_REG) &  ~(PCI_ROM_DESC_FLASH_PROGRAM_MODE) ,PCI_ISAROM_DESC1_REG);
}

/*****************************************/
/*       CNXT Flash map APIs             */
/*****************************************/

static map_word cnxt_map_read(struct map_info *map, unsigned long ofs)
{
   map_word r;
   r.x[0] = readw(map->virt + ofs);
   return r;
}

static void cnxt_map_write(struct map_info *map, const map_word datum, unsigned long ofs)
{
   enter_flash_program_mode();
   writew(datum.x[0], map->virt + ofs);
   exit_flash_program_mode();
   mb();
   /*printk("[0x%x]<=0x%x\n", map->virt + ofs, datum.x[0]);*/
}

static void cnxt_map_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
   uint16_t buf;
   unsigned char *to_lval = to;
   while(len >0 )
   {
      buf = readw(map->virt + (from&0xFFFFFFFE));
      if ( !(from & 1) )
         *((unsigned char*)to_lval) = (buf & 0xff);
      else
         *((unsigned char*)to_lval) = ((buf & 0xff00)>>8);
      to_lval++;
      len--;
      from++;
   }
}

static void cnxt_map_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
   uint16_t buf;
   uint16_t *from_lval = (uint16_t *)from;
   enter_flash_program_mode();
   while(len >0 )
   {
      buf = readw(map->virt + (to&0xFFFFFFFE));
      if ( !(to & 1) )
         buf = (buf & 0xff00) | *((unsigned char*)from_lval); 
      else
         buf = (buf & 0xff) | (((uint16_t)(*((unsigned char*)from_lval))) << 8); 
      writew(buf, map->virt + (to&0xFFFFFFFE));
      from_lval++;
      len--;
      to++;
   }
   exit_flash_program_mode();
}

static void rom_early_init(void)
{
   LPREG pPrimary, pSecondary;
   ulong uMaxDiv;
   unsigned int MemClkFreq32 =0,
                MemClkPeriod32 =0,
                val =0;  

   ulong uMaxReadAccess, data;
   u8    bank =0;
   LPREG pMap;

   /* if we are booting from NAND mode then we need disable Primary descriptor
     as we will not be accessing nand in ROM mode. 
    */

   pPrimary  = (LPREG)PCI_ISA_PRIMARY_DESC_ADDR(0);      
   *pPrimary = *pPrimary |PCI_ROM_DESC_TYPE_ISA;

   /* Set up Bank 1 prinamy and pMap Register */ 
   GetCalcClkFreqAndPeriod( &MemClkFreq32 ,&MemClkPeriod32);

   /****************************************************/
   /* Determine the maximum read access time required. */
   /****************************************************/
   uMaxReadAccess = 126; /* this value should be configurable via config option 
                            this value has been taken from uboot code 
                          */

   data = uMaxReadAccess * 100;
   val = uMaxDiv = 0;
   while(val < data)
   {
      ++uMaxDiv;
      val += MemClkPeriod32;
   }
   /*
     setup Second pci rom registers for bank1 (NOR)
   */
   bank = 1;
   pPrimary  = (LPREG)PCI_ISA_PRIMARY_DESC_ADDR(bank);       
   pSecondary =(LPREG)PCI_ISA_SECONDARY_DESC_ADDR(bank); 

   /* Set default values to guarantee access. */
   *pPrimary = (PCI_ROM_DESC_WIDTH_16                         |
         (bank<< PCI_ROM_DESC_CS_SHIFT)                       |
         (15 << PCI_ROM_DESC_BURST_WAIT_SHIFT)                |
         (uMaxDiv << PCI_ROM_DESC_READ_WAIT_SHIFT)            |
         (uMaxDiv << PCI_ROM_DESC_WRITE_WAIT_SHIFT)           |
         PCI_ROM_DESC_TYPE_ROM                                |
         PCI_ROM_DESC_BURST_DISABLE)                          &
      ~(PCI_ROM_DESC_NOR_NAND_MODE_MASK);
   *pSecondary = 0x0;
     /*
     Initially we dont the size of the NOR flash so set iy default 32MB.
     actual  valye will be updated once we get the actual size of the flash 
     after probing  it.
      */
   pMap       = (LPREG) PCI_ROM_MAP_BASE  + bank;
   *pMap = ((((~((0x2000000>>16)-1)) << PCI_ROM_MAP_SIZE_SHIFT) & PCI_ROM_MAP_SIZE_MASK) |
         ((0xF0000000>>16) << PCI_ROM_MAP_OFFSET_SHIFT));

}
void cnxt_map_init(struct map_info *map)
{
   map->read = cnxt_map_read;
   map->write = cnxt_map_write;
   map->copy_from = cnxt_map_copy_from;
   map->copy_to = cnxt_map_copy_to;
   
  /* (NEED MORE NAND/NOR FLASH)*/ 
   /* check if booting from nand flash */
   if (IS_NANDBOOT)
   {
      boot_from_nand =1;
      rom_early_init();
   }
 
  writel(0x1000, PCI_ROM_DESC0_REG2);
}

void cnxt_setup_rom(unsigned long bank_size)
{ 
   /* we are only updating bank1 as bank0 is NAND flash (NEED MORE NAND/NOR FLASH)*/ 
   u8 bank =1;
   LPREG pMap;
   boot_from_nand = 1;
   if(boot_from_nand)
   {
      pMap       = (LPREG) PCI_ROM_MAP_BASE  + bank;
      *pMap = ((((~((bank_size>>16)-1)) << PCI_ROM_MAP_SIZE_SHIFT) & PCI_ROM_MAP_SIZE_MASK) |
            ((0xF0000000>>16) << PCI_ROM_MAP_OFFSET_SHIFT));
   } 
}
EXPORT_SYMBOL(cnxt_map_init);
MODULE_LICENSE("GPL");
