/*
 * cnxt_udc0.c -- for Pecos/Nevis Rev B later usb device controller 0  udc0; 
 * Copyright (C) 2006 Conexant systems.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 */


#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/usb_ch9.h>
#include <linux/usb_gadget.h>
#include <asm/cacheflush.h>
#include <linux/platform_device.h>
#include "./tdi/devapi.h"

#include "udc-cnxt.h"

#include <asm/arch/hardware.h>
/* #include <asm/arch/hardware/usb.h> */

#define USB2_MAX_PACKET_SIZE 64
#define USB2_NUM_ENDPOINTS 8

#define xprintk(level,fmt,args...) \
	printk(level "%s: " fmt , driver_name , \
## args)

#ifdef DEBUG_ENUMERATION
#define DBG_ENUMERATION(fmt,args...) \
xprintk(KERN_ALERT  , fmt , ## args)
#else 
#define DBG_ENUMERATION(fmt,args...) \
do { } while (0)
#endif /* DEBUG */

 /* NOTE : On the variable direction used accross the code.
 * direction = 0, implies receive from host to device;
 * direction = 1, implies send from device to host;
*/

#define USB_DEV_PRINT(x...)   
#define printk_spl(x...)		

#ifdef USB_TASKLET
void *usb_TrCompleteTasklet;
#endif

#ifdef CONFIG_USB_CNXT_OTG	
	struct otg_transceiver	*transceiver;
#endif


static const char driver_name [] = "cnxt_udc";

struct vusbhs_req
{
	struct usb_request    req;
	struct list_head    queue;
};

struct vusbhs_ep
{
	struct usb_ep          ep;
	struct list_head       queue;
	char                   ep_name[14];
	u16                    maxpacket;
	u8                     bEndpointAddress;
	u8                     bmAttributes;
	struct vusbhs_udc      *udc;
};

struct vusbhs_udc
{
	struct              usb_gadget gadget;
	struct              usb_gadget_driver *gadget_driver;
	struct              vusbhs_ep ep[USB2_NUM_ENDPOINTS];
	_usb_device_handle  handle;
	spinlock_t          lock;
};

static struct vusbhs_udc *udc;

struct usbblock {
	int irq;
	u32 base_addr;
};

/* syncs-up zero-len packets and setup packets for setup transactions */
unsigned char setup_state;

/* Keeps track of enqueue count in ep0 */
int ep0_cnt;

/* Interrupt handler provided by tdi */
extern void _usb_dci_vusb20_isr(void* arg);
static void nuke(struct vusbhs_ep *ep, int status);
static void done(struct vusbhs_ep *ep, struct vusbhs_req *req, int status);
static int handleClearFeature(_usb_device_handle handle, struct usb_ctrlrequest *setup_ptr);

/* Intializes VUSB HS 2.0 controller */
int usb_initTdi(void);

/* Local irq Handler */
static irqreturn_t 
usb_udc_irq(int irq, void *dev_id, struct pt_regs *regs)
{
	_usb_dci_vusb20_isr((void*) NULL);
	return IRQ_HANDLED;
}

unsigned char 
get_ep_num(const char *ep_name)
{
	if (strcmp(ep_name,"ep0") == 0)
		return 0;
	else if (strcmp(ep_name,"ep2out-bulk") == 0)
		return 2;
	else if (strcmp(ep_name,"ep3in-bulk") == 0)
		return 3;
	else if (strcmp(ep_name,"ep1in-int") == 0)
		return 1;
	else
		return -EPERM;
}

/* Endpoint Operations */
static int 
vusbhs_ep_enable(struct usb_ep *_ep, const struct usb_endpoint_descriptor *desc)
{
	unsigned char ep_num = 0;
	ep_num = get_ep_num(_ep->name);
	
	USB_DEV_PRINT(KERN_INFO "vusbhs_ep_enable called\n");
	USB_DEV_PRINT(KERN_INFO "ep_num :: %d\n",ep_num);
	USB_DEV_PRINT(KERN_INFO "mxp :: %d\n",desc->wMaxPacketSize);
	USB_DEV_PRINT(KERN_INFO "direction %x, type :: %x\n",
		((desc->bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_IN)? 0 : 1,
		desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK);
	
	/* we have to keep trying to init the endpoint...if it fails 
	* and we don't retry then we may not be able to unplug and 
	* replug the USB cable properly. 
	*/
	while(_usb_device_init_endpoint ( udc->handle,
		ep_num,
		desc->wMaxPacketSize,
		((desc->bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_IN)? 1 : 0,
		desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK,(ep_num==1)?1:0) ){
		_usb_device_cancel_transfer ( udc->handle, ep_num, 
			((desc->bEndpointAddress & USB_ENDPOINT_DIR_MASK) == USB_DIR_IN)? 1 : 0 );
	}
	return 0;
}

static int 
vusbhs_ep_disable(struct usb_ep *_ep)
{
	struct vusbhs_ep  *ep = container_of(_ep, struct vusbhs_ep, ep);
	unsigned char ep_num = 0;
	unsigned long flags;
	
	USB_DEV_PRINT(KERN_INFO "vusbhs_ep_disable called\n");
	
	ep_num = get_ep_num(_ep->name);
	
	spin_lock_irqsave(&ep->udc->lock, flags);
	nuke (ep, -ESHUTDOWN);
	spin_unlock_irqrestore(&ep->udc->lock, flags);
	
	USB_DEV_PRINT(KERN_INFO "ep_num :: %d\n",ep_num);
	return 0;
}


static struct usb_request *
vusbhs_alloc_request(struct usb_ep *ep, int gfp_flags)
{
	struct vusbhs_req* req;
	
	USB_DEV_PRINT(KERN_INFO "%s\n",__FUNCTION__);
	req = kmalloc(sizeof *req, gfp_flags);
	
	if (req) {
		memset (req, 0, sizeof *req);
		req->req.dma = 0;
		INIT_LIST_HEAD (&req->queue);
	}
	
	return &req->req;
}

static void
vusbhs_free_request(struct usb_ep *ep, struct usb_request *_req)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_free_request called\n");
	if (_req)
		kfree (_req);
}

static void *
vusbhs_alloc_buffer(struct usb_ep  *_ep, unsigned  bytes, dma_addr_t  *dma,int gfp_flags)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_alloc_buffer called\n");
	return kmalloc(bytes, gfp_flags);
}


static void 
vusbhs_free_buffer(struct usb_ep *_ep,	void *buf, dma_addr_t dma, unsigned  bytes
				   )
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_free_buffer called\n");
	if(buf == NULL) return;
	kfree (buf);
}


/* dequeue ALL requests; caller holds udc->lock */
static void 
nuke(struct vusbhs_ep *ep, int status)
{
	struct vusbhs_req *req;
	
	while (!list_empty(&ep->queue)) {
		req = list_entry(ep->queue.next, struct vusbhs_req, queue);
		done(ep, req, status);
	}
}

static void
done(struct vusbhs_ep *ep, struct vusbhs_req *req, int status)
{
	list_del_init(&req->queue);
	
	if (req->req.status == -EINPROGRESS)
		req->req.status = status;
	else
		status = req->req.status;
	
	req->req.actual = req->req.length;
	
	if (status && status != -ESHUTDOWN)
	{
		printk_spl("complete %s req %p stat %d len %u/%u\n",
			ep->ep.name, &req->req, status,
			req->req.actual, req->req.length);
	}
	
	spin_unlock(&ep->udc->lock);
	req->req.complete(&ep->ep, &req->req);
	spin_lock(&ep->udc->lock);
}

static int
vusbhs_ep_queue(struct usb_ep *_ep, struct usb_request *_req, int gfp_flags)
{
	
	unsigned char ep_num = 0;
	u8 ret_val = 0;
	struct vusbhs_udc *udc;
	struct vusbhs_ep  *ep = container_of(_ep, struct vusbhs_ep, ep);
	struct vusbhs_req *req = container_of(_req, struct vusbhs_req, req);
	unsigned long flags;
	
	ep_num = get_ep_num(_ep->name);
	
	/* catch various bogus parameters */
	if (!_req || !req->req.complete || !req->req.buf) {
		printk_spl("%s: ep %d, bad params\n", __FUNCTION__, ep_num);
		return -EINVAL;
	}
	
	if (!_ep ) {
		printk_spl("%s, bad ep, _ep=%p, ep->bEndpointAddress: %x\n", __FUNCTION__, _ep, ep->bEndpointAddress);
		return -EINVAL;
	}
	
	udc = ep->udc;
	
	/* Filling usb request status to be used by completion handler*/
	spin_lock_irqsave(&udc->lock, flags);
	req->req.status = -EINPROGRESS;
	req->req.actual = 0;
	
	if ( req->req.buf )
	{
		if(strcmp(ep->ep.name, "ep2out-bulk") == 0){
			ret_val= _usb_device_recv_data( udc->handle, 
				get_ep_num(ep->ep.name), 
				(unsigned char *)req->req.buf, 
				req->req.length); 
			if (ret_val != USB_OK){
				printk_spl("%s: Rx error,%d \n", __FUNCTION__, ep_num);
			}
		}
		else if (strcmp(ep->ep.name, "ep3in-bulk") == 0)
		{
			USB_DEV_PRINT("Tx packet params complete %p!! buffer: %p, len :%d\n", 
				req->req.complete, req->req.buf, req->req.length);
			ret_val = _usb_device_send_data(udc->handle, 
				ep_num, 
				(u8 *)((u32)req->req.buf & 0xfffff000), 
				req->req.length, 
				(u32)req->req.buf & 0x00000fff);
			if (ret_val != USB_OK){
				printk_spl("%s: Transmission error,%d \n", __FUNCTION__, ep_num);
			}
		}
		else 
		{
          		if(ep_num == 0){
				/* Keep track of ep0 enqueue count */
				//	if(ep0_cnt == 1){
				//		printk("twice queueing up in ep0\n");
				//	}
					ep0_cnt++;
				}
				ret_val = _usb_device_send_data(udc->handle, ep_num,
					(u8 *)((u32)req->req.buf & 0xfffff000), req->req.length, 
					(u32)req->req.buf & 0x00000fff);
				
				if (ret_val != USB_OK){
					printk_spl("%s: Transmission error %d, \n", __FUNCTION__, ep_num);
				}
		}
	}
	
	if (req != 0){
		/* If ep0 enqueue cnt is more than one, skip adding the req buf as this 
		   will corrupt the list since only one req buf is used for ep0 */

		if((ep_num==0)&&(ep0_cnt>1))
			goto done;		
		list_add_tail(&req->queue, &ep->queue);
	}
done:
	spin_unlock_irqrestore(&udc->lock, flags);
	return 0;
}

static int 
vusbhs_ep_dequeue(struct usb_ep *_ep, struct usb_request *_req)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_ep_dequeue called\n");
	printk(KERN_INFO "vusbhs_ep_dequeue called\n");
	return 0;
}


static int 
vusbhs_ep_set_halt(struct usb_ep *_ep, int value)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_ep_set_halt called\n");
	return 0;
}

/* Gadget Operations */
static int 
vusbhs_get_frame(struct usb_gadget *gadget)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_get_frame called\n");
	return 0;
}

static int 
vusbhs_wakeup(struct usb_gadget *gadget)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_wakeup called\n");
	return 0;
}

static int
vusbhs_set_selfpowered(struct usb_gadget *gadget, int is_selfpowered)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_set_selfpowered called\n");
	return 0;
}

static int 
vusbhs_vbus_session(struct usb_gadget *gadget, int is_active)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_vbus_session called\n");
	return 0;
}

static int 
vusbhs_vbus_draw(struct usb_gadget *gadget, unsigned mA)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_vbus_draw called\n");
	return 0;
}


static int 
vusbhs_pullup(struct usb_gadget *gadget, int is_on)
{
	USB_DEV_PRINT(KERN_INFO "vusbhs_pullup called\n");
	return 0;
}

static struct usb_ep_ops vusbhs_ep_ops = {
	.enable    = vusbhs_ep_enable,
	.disable  = vusbhs_ep_disable,
		
	.alloc_request  = vusbhs_alloc_request,
	.free_request  = vusbhs_free_request,
	
	.alloc_buffer  = vusbhs_alloc_buffer,
	.free_buffer  = vusbhs_free_buffer,
		
	.queue    = vusbhs_ep_queue,
	.dequeue  = vusbhs_ep_dequeue,
		
	.set_halt  = vusbhs_ep_set_halt,
	// fifo_status ... report bytes in fifo
	// fifo_flush ... flush fifo
};


static struct usb_gadget_ops vusbhs_gadget_ops = {
	.get_frame    = vusbhs_get_frame,
	.wakeup      = vusbhs_wakeup,
	.set_selfpowered  = vusbhs_set_selfpowered,
	.vbus_session    = vusbhs_vbus_session,
	.vbus_draw    = vusbhs_vbus_draw,
	.pullup      = vusbhs_pullup,
};


#define PIO_OUT_TIMEOUT (jiffies + HZ/3)


static unsigned 
ep_setup(char *name, u8 addr, u8 type, unsigned buf, unsigned maxp, int dbuf)
{
	struct vusbhs_ep  *ep;
	
	/* OUT endpoints first, then IN */
	ep = &udc->ep[addr & 0xf];
	if (addr & USB_DIR_IN)
		ep += 4;
	
	/* next endpoint's buffer starts after this one's */
	USB_DEV_PRINT("name %s, ep->name %s\n",name,ep->name);
	buf += maxp;
	
	/* set up driver data structures */
	if(strlen(name) < 14 )
		strlcpy(ep->ep_name, name, 14);
	else
		return 0;
	INIT_LIST_HEAD(&ep->queue);
	ep->udc = udc;
	ep->bEndpointAddress = addr;
	ep->bmAttributes = type;
	
	ep->ep.name = ep->ep_name;
	ep->ep.ops = &vusbhs_ep_ops; 
	ep->ep.maxpacket =  maxp;
	
	list_add_tail (&ep->ep.ep_list, &udc->gadget.ep_list);
	
	return buf;
}


int usb_gadget_register_driver (struct usb_gadget_driver *driver)
{
	
	int status = 0;
	unsigned buf;
	struct vusbhs_ep  *ep;
	ep = &((struct vusbhs_udc *)udc)->ep[0];
	
	USB_DEV_PRINT( KERN_INFO "usb_gadget_register_driver\n");
	if (!driver
		|| driver->speed < USB_SPEED_FULL
		|| !driver->bind
		|| !driver->unbind
		|| !driver->setup)
		return -EINVAL;
	
	/* Populating gadget structure */
	device_initialize(&udc->gadget.dev);
	udc->gadget.dev.driver = &driver->driver;
	udc->gadget.name = "vusbhs";
	udc->gadget.ep0 = &ep->ep;
	udc->gadget.ops = &vusbhs_gadget_ops;
	udc->gadget_driver = driver;
	udc->gadget.is_otg = 0;  
	udc->gadget.is_dualspeed = 1;
	
	/* Populating ep list */
	INIT_LIST_HEAD(&udc->gadget.ep_list);
	buf = ep_setup("ep0", 0, USB_ENDPOINT_XFER_CONTROL,
		8 /* after SETUP */, 64 /* maxpacket */, 0);
	
#define BULK_EP(name,addr) \
	buf = ep_setup(name "-bulk", addr, \
	USB_ENDPOINT_XFER_BULK, buf, 64, 1);
#define INT_EP(name,addr, maxp) \
	buf = ep_setup(name "-int", addr, \
	USB_ENDPOINT_XFER_INT, buf, maxp, 0);
#define ISO_EP(name,addr, maxp) \
	buf = ep_setup(name "-iso", addr, \
	USB_ENDPOINT_XFER_ISOC, buf, maxp, 1);
	
	BULK_EP("ep2out",  USB_DIR_OUT  | 2);
	BULK_EP("ep3in", USB_DIR_IN | 3);
	INT_EP("ep1in",   USB_DIR_IN  | 1, 64);
	
	USB_DEV_PRINT(KERN_INFO "ep setup complete\n");
	
	
	/* calling ether bind function */
	status = driver->bind (&udc->gadget);
	if (status) {
		printk_spl(KERN_INFO"bind to %s --> %d\n", driver->driver.name, status);
		udc->gadget.dev.driver = 0;
		goto done;
	}
	usb_initTdi(); //moved here from probe to avoid crash on first boot up
	USB_DEV_PRINT(KERN_INFO"bound to driver %s\n", driver->driver.name);
	
done:
	return status;
	return 0;
}

void handle_packet(_usb_device_handle handle, boolean  setup, 
				   uint_8 direction,uint_32 length , 
				   struct usb_ctrlrequest local_setup_packet);

void service_ep0(_usb_device_handle handle, 
				 boolean        setup,
				 uint_8         direction,
				 uint_8_ptr     buffer,
				 uint_32        length,
				 uint_8         error)
{ 
	static struct usb_ctrlrequest local_setup_packet;
	struct vusbhs_ep  *ep0 = &udc->ep[0];
	struct vusbhs_req *req = 0;
	
	DBG_ENUMERATION( KERN_INFO "service ep0, setup = %d, direction = %d, length = %d\n",
		setup,direction,length);
	
	if(direction == 1){
		USB_DEV_PRINT("name---> %s\n", ep0->name);
		if (likely (!list_empty(&ep0->queue))){
			
			ep0_cnt--;
			if (likely (!ep0_cnt)){
				req = container_of(ep0->queue.next, struct vusbhs_req, queue);
				
				list_del_init(&req->queue); 
				spin_unlock(&ep0->udc->lock);
				if (likely (req))
				{
					if (likely (req->req.complete))
					{
						req->req.complete(&ep0->ep, &req->req);
					}
				}
				spin_lock(&ep0->udc->lock);
			}
        	}
		return;
	}
	
	if (likely (setup))
	{
		_usb_device_read_setup_data(handle, 0, (uchar_ptr)&local_setup_packet);
		le16_to_cpus (&local_setup_packet.wValue);
		le16_to_cpus (&local_setup_packet.wIndex);
		le16_to_cpus (&local_setup_packet.wLength);
	}
	handle_packet(handle,setup,direction,length,local_setup_packet);
}

void handle_standard_request(_usb_device_handle handle, 
							 boolean  setup, 
							 uint_8 direction, 
							 struct usb_ctrlrequest local_setup_packet)
{
	u8 request = local_setup_packet.bRequest;
	u16 usb_status;
	u8 endpoint;
	static u8 new_address;
	int isGet=0, status = 0;
	
	if(setup)
	{
		switch(request)
		{
		case USB_REQ_GET_DESCRIPTOR:
			DBG_ENUMERATION("USB_REQ_GET_DESCRIPTOR\n");
			if( udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
				status = EINVAL;
			isGet = 1;
			break;
			
		case USB_REQ_GET_STATUS:
			DBG_ENUMERATION("USB_REQ_GET_STATUS\n");
			if (local_setup_packet.bRequestType != 0x80) {
				DBG_ENUMERATION("Unknown status request\n");
				status = EINVAL;
			}
			else {
				_usb_device_get_status(handle, USB_STATUS_DEVICE, &usb_status);
				_usb_device_send_data(handle, 
					0, 
					(unsigned char *)&usb_status, 
					sizeof(usb_status), 
					0);
				isGet = 1;
			}
			break;
			
		case USB_REQ_GET_INTERFACE:
			DBG_ENUMERATION("USB_REQ_GET_INTERFACE\n");
			if(udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
				status = EINVAL;
			
			isGet = 1;
			break;
			
		case USB_REQ_GET_CONFIGURATION:
			DBG_ENUMERATION("USB_REQ_GET_CONFIGURATION\n");
			if( udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
				status = EINVAL;
			isGet = 1;
			break;
			
		case USB_REQ_SET_ADDRESS:
			DBG_ENUMERATION("USB_REQ_SET_ADDRESS\n");
			new_address = local_setup_packet.wValue;    
			
			/* if hardware assitance is enabled for set_address (see
			* hardware rev for details) we need to do the set_address
			* before queuing the status phase.
			*/
			
			_usb_device_set_status(handle, USB_STATUS_ADDRESS, new_address);
			_usb_device_set_status(handle, USB_STATUS_DEVICE_STATE, USB_STATE_ADDRESS);
			
			_usb_device_send_data(handle, 0, 0, 0, 0);
			setup_state = TX_ZERO_LEN;
			break;
			
		case USB_REQ_SET_FEATURE:
			DBG_ENUMERATION("USB_REQ_SET_FEATURE\n");
			switch (local_setup_packet.bRequestType){
			case 0:
				/* DEVICE */
				if (local_setup_packet.wValue == 1)
				{
					/* set remote wakeup */
					_usb_device_get_status(handle, USB_STATUS_DEVICE, &usb_status);
					usb_status |= USB_REMOTE_WAKEUP;
					_usb_device_set_status(handle, USB_STATUS_DEVICE, usb_status);
				}
				else
					status = EINVAL;
				break;
				
			case 2:
				/* ENDPOINT */
				if (local_setup_packet.wValue != 0){
					return;
				} /* Endif */
				endpoint = local_setup_packet.wIndex & USB_STATUS_ENDPOINT_NUMBER_MASK;
				_usb_device_get_status(handle, USB_STATUS_ENDPOINT | endpoint,
					&usb_status);
				/* set stall */
				_usb_device_set_status(handle, USB_STATUS_ENDPOINT | endpoint, 1);
				break;
			default:
				status = EINVAL;
				
			} /* Endswitch */
			_usb_device_send_data(handle, 0, 0, 0, 0); 
			setup_state = TX_ZERO_LEN;
			break;
			
			case USB_REQ_SET_DESCRIPTOR:
				DBG_ENUMERATION("USB_REQ_SET_DESCRIPTOR\n");
				status = EINVAL;
				break;
				
			case USB_REQ_SET_CONFIGURATION:
				DBG_ENUMERATION("USB_REQ_SET_CONFIGURATION\n");
				if( udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
					status = EINVAL;
				setup_state = TX_ZERO_LEN;
				break;
				
			case USB_REQ_SET_INTERFACE:
				DBG_ENUMERATION("USB_REQ_SET_INTERFACE\n");
				if( udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
					status = EINVAL;
				setup_state = TX_ZERO_LEN;
				break;
				
			case USB_REQ_SYNCH_FRAME:
				DBG_ENUMERATION("USB_REQ_SYNCH_FRAME\n");
				status = EINVAL;
				break;
				
			case USB_REQ_CLEAR_FEATURE:
				DBG_ENUMERATION("USB_REQ_CLEAR_FEATURE\n");
				status = handleClearFeature(handle, &local_setup_packet);
				break;      
				
			default:
				DBG_ENUMERATION("Unknown request\n");  
				status = EINVAL;
				break;
		}/* end switch */
	
		if (status)
		{
			DBG_ENUMERATION("Error processing request 0x%x\n", request);
			//_usb_device_stall_endpoint(handle, 0, 0);
			return;
		}
	
		if (isGet)
		{
			/* The host should now get the data and send a 0 byte packet back */
			_usb_device_recv_data(handle, 0, 0, 0);
			setup_state = RX_ZERO_LEN;
		}
	}/* end if setup */
}

static int handleClearFeature(_usb_device_handle handle,
							  struct usb_ctrlrequest *setup_ptr)
{
    switch (setup_ptr->bRequestType)
    {
	case 0:     // device
		if (setup_ptr->wValue == 1)
		{
			// clear remote wakeup
		}
		else
		{
			return EINVAL;
		} /* Endif */
		break;
		
	case 2:     // endpoint
		if (setup_ptr->wValue != 0)
			return EINVAL;
		
		_usb_device_unstall_endpoint(handle,
			setup_ptr->wIndex & 0x000f,
			(setup_ptr->wIndex & 0x0080) ? USB_SEND : USB_RECV);
		break;
		
	default:
		return EINVAL;
		
    };
    return 0;
}


void handle_class_request(_usb_device_handle handle, 
						  boolean  setup, 
						  uint_8 direction, 
						  uint_32 length,
						  struct usb_ctrlrequest local_setup_packet)
{
	static struct vusbhs_req req1;
	struct vusbhs_req *req = &req1;
	
	/* CDC_GET_ENCAPSULATED_RESPONSE */
	if (local_setup_packet.bRequestType & 0x80){
		if(setup){
			if( udc->gadget_driver->setup(&udc->gadget,&local_setup_packet) < 0)
				USB_DEV_PRINT("ch9ClassSet: error in setup\n");
			_usb_device_recv_data(handle, 0, 0, 0);
			setup_state = RX_ZERO_LEN;
			return;
		}
		else{
			return;
		}
	}
	
	if(setup){
		/* CDC_SEND_ENCAPSULATED_COMMAND */
		printk_spl("CDC_SEND_ENCAPSULATED_COMMAND\n");
		if(local_setup_packet.wLength == 0){
			printk_spl("ch9ClassSet: 0 length packet\n");
			_usb_device_send_data(handle, 0, 0, 0, 0);
			setup_state = TX_ZERO_LEN;
			return;
		}
		else {
			
			if((req->req.buf = kmalloc(local_setup_packet.wLength + sizeof(local_setup_packet), GFP_ATOMIC)) == NULL)
			{
				printk_spl("Buf alloc failed\n");
				return;
			}
			req->req.length = local_setup_packet.wLength;          
			_usb_device_recv_data(handle, 0, (unsigned char *)req->req.buf + sizeof(local_setup_packet),local_setup_packet.wLength);
		}
	}
	else if(direction == 0)
	{
	        if (unlikely ((length != local_setup_packet.wLength))){
			/* Zero length packet for GET CDC received now, do nothing */	
			return;	
		}	
		
            	if (likely(req && req->req.buf))
            	{
                	memcpy(req->req.buf, (unsigned char *)&local_setup_packet, sizeof(local_setup_packet));
	                printk_spl("send cdc encapsulated rndis message,length %d\n", local_setup_packet.wLength);
        	        if(udc->gadget_driver->setup(&udc->gadget,req->req.buf) < 0 ) {
	                    USB_DEV_PRINT("ch9ClassSet: error in setup\n"); 
                	}
	                kfree(req->req.buf); 
			setup_state = TX_ZERO_LEN;
        	}
	}
}


void handle_packet(_usb_device_handle handle, 
				   boolean                  setup, 
				   uint_8                   direction,
				   uint_32                  length, 
				   struct usb_ctrlrequest   local_setup_packet)
{
	
	u8 requestType = (local_setup_packet.bRequestType & 0x60);
	
	switch (requestType)
	{
	case USB_TYPE_STANDARD:
		USB_DEV_PRINT("standard request\n");
		DBG_ENUMERATION("standard request\n");
		handle_standard_request(handle,setup,direction,local_setup_packet);
		break;
		
	case USB_TYPE_CLASS:
		DBG_ENUMERATION("class specific request\n");
		handle_class_request(handle,setup,direction,length,local_setup_packet);  
		break;
		
	case USB_TYPE_VENDOR:
		DBG_ENUMERATION("vendor specific request\n");
		
	default:
		break;
	};
}

void service_ep1(_usb_device_handle handle, 
				 boolean        setup,
				 uint_8         direction,
				 uint_8_ptr     buffer,
				 uint_32        length,
				 uint_8         error)
{
	struct vusbhs_ep  *ep1_in_int = &udc->ep[5];
	struct vusbhs_req *req;
	
	if (!list_empty(&ep1_in_int->queue)) {
		
		req = container_of(ep1_in_int->queue.next, struct vusbhs_req, queue);
		
		if(req){
			list_del_init(&req->queue);  
			
			USB_DEV_PRINT("calling tx complete handler %p!! buffer: %p, len :%d\n",
				req->req.complete, req->req.buf, req->req.length);
			req->req.buf = buffer;
			req->req.length = length;
			req->req.actual = length;
			req->req.dma = 0;
			req->req.status = 0;
			spin_unlock(&ep1_in_int->udc->lock);
			if( req->req.complete)
				req->req.complete(&ep1_in_int->ep, &req->req);
			spin_lock(&ep1_in_int->udc->lock);
		}
	}
	//Call the complete function here for the status msg sent
	printk_spl( KERN_INFO "service ep1, setup=%d, direction=%d\n", setup, direction);
}

void service_ep2(_usb_device_handle handle, 
				 boolean        setup,
				 uint_8         direction,
				 uint_8_ptr     buffer,
				 uint_32        length,
				 uint_8         error)
{ 
	
	struct vusbhs_ep  *ep2_out_bulk = &udc->ep[2];
	struct vusbhs_req *req;
	
	USB_DEV_PRINT( "service ep2, , setup=%d, direction=%d, error = %d\n", 
		setup, direction, error);
	
	/* rx from host */
	if (likely (direction == 0))
	{
		if (likely (!list_empty(&ep2_out_bulk->queue))) {
			
			USB_DEV_PRINT("name---> %s\n", ep2_out_bulk->name);
			req = container_of(ep2_out_bulk->queue.next, struct vusbhs_req, queue);
			list_del_init(&req->queue); 
			
			req->req.buf = buffer;
			req->req.length = length;
			req->req.actual = length;
			req->req.dma = 0;
			req->req.status = 0;
			USB_DEV_PRINT("calling rx complete handler %p!! buffer: %p, len :%d\n", 
				req->req.complete, req->req.buf, req->req.length);
			spin_unlock(&ep2_out_bulk->udc->lock);
			if (likely (!error))
			{
#ifdef DEBUG_VUSBHS
				printk("Received packet!\n");
#endif /* DEBUG_VUSBHS */
				req->req.complete(&ep2_out_bulk->ep, &req->req);
			}
			else
			{
				printk_spl("%s,Dropping the received packet! vusbhs error: %x\n", __FUNCTION__, error);
#ifdef DEBUG_VUSBHS
				req->req.length = error;
				req->req.actual = error;
				
				req->req.complete(&ep2_out_bulk->ep, &req->req);
#endif /* DEBUG_VUSBHS */
			}
			spin_lock(&ep2_out_bulk->udc->lock);
		}
	}
	else if( unlikely (direction == 1))
	{
		/* tx to host */
		printk_spl("%s: Invalid data flow!\n", __FUNCTION__);
	}
}


void service_ep3(_usb_device_handle handle, 
				 boolean         setup,
				 uint_8         direction,
				 uint_8_ptr     buffer,
				 uint_32     length,
				 uint_8         error)
{ 
	
	struct vusbhs_ep  *ep3_in_bulk = &udc->ep[7];
	struct vusbhs_req *req;
	
	USB_DEV_PRINT( "service ep3, , setup=%d, direction=%d, buffer=%p, \
		len=%d, err=%d\n", 
		setup, direction, buffer, length, error);
	
	/* tx to host */
	if (likely (direction == 1))
	{
		if (!list_empty(&ep3_in_bulk->queue)) {
			
			USB_DEV_PRINT("name---> %s\n", ep3_in_bulk->ep.name);
			req = container_of(ep3_in_bulk->queue.next, struct vusbhs_req, queue);
			list_del_init(&req->queue); 
			USB_DEV_PRINT("calling tx complete handler %p!! buffer: %p, len :%d\n", 
				req->req.complete, req->req.buf, req->req.length);
			req->req.buf = buffer;
			req->req.length = length;
			req->req.actual = length;
			req->req.dma = 0;
			req->req.status = 0;
			spin_unlock(&ep3_in_bulk->udc->lock);
			req->req.complete(&ep3_in_bulk->ep, &req->req);
			spin_lock(&ep3_in_bulk->udc->lock);
		}
	}
	/* rx from host */
	else if (unlikely (direction == 0))
	{
		printk_spl("%s: Invalid data flow!\n", __FUNCTION__);
		printk_spl("ep name ---> %s\n", ep3_in_bulk->ep.name);
	}
}


void reset_ep0(_usb_device_handle handle, 
			   boolean        setup,
			   uint_8         direction,
			   uint_8_ptr     buffer,
			   uint_32     length,
			   uint_8         error)
{
	
	int endpoint, dir, ret=0;
	unsigned long flags;
	
	
	/*on a reset always cancel all transfers all EP 0 */
	for (endpoint = 0; endpoint < 4; endpoint++)
	{
		for (dir = 0; dir < 2; dir++)
		{
			ret = _usb_device_cancel_transfer(handle, endpoint, dir);
			if (ret)
				break;
		}
		if (ret)
			break;
	}
	
	spin_lock_irqsave(&udc->lock, flags);
	nuke (&udc->ep[0], -ESHUTDOWN);
	nuke (&udc->ep[7], -ESHUTDOWN);
	nuke (&udc->ep[2], -ESHUTDOWN);
	nuke (&udc->ep[5], -ESHUTDOWN);
	spin_unlock_irqrestore(&udc->lock, flags);
	
	if (ret)
	{
		USB_DEV_PRINT("Can't cancel transfer on endpoint %d dir %d\n", endpoint, dir);
		printk("Can't cancel transfer on endpoint %d dir %d\n", endpoint, dir);
		usb_initTdi();
		return;
	}
	
	/* Initialize the endpoint 0 in both directions */
	_usb_device_init_endpoint(handle, 0, USB2_MAX_PACKET_SIZE, 0,
		USB_ENDPOINT_XFER_CONTROL, 1);
	_usb_device_init_endpoint(handle, 0, USB2_MAX_PACKET_SIZE, 1,
		USB_ENDPOINT_XFER_CONTROL, 0);
	
	return;
}


void service_suspend(_usb_device_handle handle, 
					 boolean  setup,
					 uint_8 direction,
					 uint_8_ptr buffer,
					 uint_32 length,
					 uint_8 error)
{
	printk_spl( KERN_INFO "service_suspend : setup=%ld dir=%d len=%ld\n", 
		setup, direction, length);
}


void service_speed_detection(_usb_device_handle handle, uint_32 speed)
{
	
	switch(speed) /* TODO */
	{
	case 0:   
		udc->gadget.speed = 2;
		break;
		
	case 1:
		udc->gadget.speed = 1;
		break;
		
	case 2:
		udc->gadget.speed = 3;
		break;
		
	default:
		udc->gadget.speed = 0;
		break;
		
	}
	USB_DEV_PRINT("gadget.speed=%d\n",udc->gadget.speed);
}

int usb_initTdi(void)
{
	
	int ret = 0;
	_usb_device_handle   handle;
	
	// Initialize the USB interface
	ret = _usb_device_init(0, &handle, 8);
	
	udc->handle = handle;
	
	//Uncomment the following line to force full speed operation
	//_usb_dci_vusb20_set_speed_full(handle, 0);
	
	if (ret != 0)
	{
		printk_spl(KERN_INFO "USB Initialization failed.");
		return -1;
	}
	
	/* Initialize the endpoint 0 in both directions */
	_usb_device_init_endpoint(handle, 0, USB2_MAX_PACKET_SIZE, 0,
		USB_ENDPOINT_XFER_CONTROL, 1);
	_usb_device_init_endpoint(handle, 0, USB2_MAX_PACKET_SIZE, 1,
		USB_ENDPOINT_XFER_CONTROL, 0);
	return 0;
	
}
//extern int v6_dma_inv_range(unsigned long, unsigned long);

#ifdef	CONFIG_USB_CNXT_OTG
extern int trans;
#endif

static int __init cnxt_udc_probe(struct platform_device *pdev)
{
	int ret = 0,temp;
	
	USB_DEV_PRINT( KERN_INFO "cnxt_udc_probe udc1\n");
	
	udc = kmalloc (sizeof *udc, SLAB_KERNEL);
	if (!udc)
		return -ENOMEM;
	
	memset(udc, 0, sizeof *udc);
	spin_lock_init (&udc->lock);  

 /* Enable the  Port Power for all device controller ports */

    temp = *(unsigned int *)0xe0440138;
    temp = temp | 0x763;
    *(unsigned int *)0xe0440138 =  temp;

    msleep(100);


	ret = request_irq(pdev->resource[1].start, usb_udc_irq,
			SA_SHIRQ, "cnxt_udc", udc);
	
	
#ifdef USB_TASKLET
    	usb_TrCompleteTasklet = (struct tasklet_struct *)kmalloc(
				sizeof(struct tasklet_struct),
				GFP_KERNEL);
	
	tasklet_init((struct tasklet_struct *)usb_TrCompleteTasklet, usb_TrCompleteTaskletHandler , (unsigned long)NULL);
#endif
	
	USB_DEV_PRINT("request interrupt ret = %d\n" ,ret);

#ifdef	CONFIG_USB_CNXT_OTG
	
		trans= 1;
		transceiver = otg_get_transceiver();
		if (transceiver) {
			int	status = otg_set_peripheral(transceiver,
						&udc->gadget);
			
		} else {
			printk( "can't find transceiver%d \n",trans);
			return -ENODEV;
		}
	
#endif

	return 0;
}



static int __exit cnxt_udc_remove(struct platform_device *pdev)
{
	USB_DEV_PRINT( KERN_INFO "cnxt_udc_remove\n");
	return 0;
}


static int cnxt_udc_suspend(struct platform_device *dev)
{
	USB_DEV_PRINT( KERN_INFO "cnxt_udc_suspend\n");
	return 0;
}

static int cnxt_udc_resume(struct platform_device *dev)
{
	USB_DEV_PRINT( KERN_INFO "cnxt_udc_resume\n");
	return 0;
}


/*-------------------------------------------------------------------------*/

static void usb_release(struct device *dev)
{
	/* normally not freed */
}


/* The dmamask must be set for usb device controller to work */
static u64 cnxt_udc_dmamask =(u64) (u64*)-1;

static struct resource cnxt_udc_resources1[] = {
	{
		.start	= ASX_USB1_VIRT,
		.end	= ASX_USB1_VIRT + 4096 - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= ITC_USB1_POS,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device udc_device1 = {
	.name			= "cnxt_udc1",
	.id			= -1,
	.dev = {
		.release		= usb_release,
		.dma_mask		= &cnxt_udc_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(cnxt_udc_resources1),
	.resource		= cnxt_udc_resources1,
};

static struct platform_driver cnxt_udc1_driver = {
	.probe		= cnxt_udc_probe,
	.remove		= __exit_p(cnxt_udc_remove),
	.suspend	= cnxt_udc_suspend,
	.resume		= cnxt_udc_resume,
	.driver		= {
		.owner	= THIS_MODULE,
		.name	= (char *) driver_name,
	},
};

static int __init cnxt_udc1_init(void)
{
	int rc = 0;

	rc = platform_driver_register(&cnxt_udc1_driver);
	if (rc < 0)
		return rc;
		
	rc = platform_device_register(&udc_device1);
	if (rc)
		printk("can't register cnxt usb  device controller1 , %d\n", rc);

	return rc;
	
}

module_init(cnxt_udc1_init);

static void __exit cnxt_udc1_exit(void)
{
	platform_driver_unregister(&cnxt_udc1_driver);
}
module_exit(cnxt_udc1_exit);

MODULE_DESCRIPTION("USB Device Controller Driver");
MODULE_LICENSE("GPL");


