/* ###########################################################################
# (c) Copyright Conexant Limited 2006
#
# Conexant Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Limited ("Conexant").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Conexant.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Conexant.  If you have not entered into a
# confidentiality agreement with Conexant granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@conexant.com
# ##########################################################################*/

/**HEADER*********************************************************
-----------------------------------------------------------------------------
 CONFIDENTIAL AND PROPRIETARY INFORMATION OF TRANSDIMENSION, INC. AND/OR ITS
 WHOLLY OWNED SUBSIDIARY SOFTCONNEX TECHNOLOGIES, INC. THIS NOTICE IS NOT TO
 BE DELETED, MODIFIED, MOVED OR CHANGED IN ANY WAY.

 (TransDimension, Inc. and SoftConnex Technologies, Inc. are referred to
 collectively below as TransDimension, Inc.) 
 Copyright (c) 1999 - 2004 by TransDimension, Inc. 

 This software is protected by copyright laws and international copyright 
 treaties, as well as other intellectual property laws and treaties.  This
 software is a CONFIDENTIAL, unpublished work of authorship, and with portions 
 constituting TRADE SECRETS of TransDimension, Inc..  Any unauthorized use, 
 disclosure, and/or reproduction of this software, or any part of this software;
 or distribution of this software in any form or by any means; or storage of 
 this software in any database or retrieval system, without the express written
 consent of, and license from, TransDimension, Inc. is strictly prohibited.  
 This software is protected under the copyright and/or trade secret laws in 
 other countries in addition to USA.  All Rights Reserved.  Failure to abide by
 the use, disclosure and/or reproduction restrictions may result in civil and
 or criminal penalties, and will be prosecuted to the maximum extent of the law.
 -------------------------------------------------------------------------------

*** Description:      
***  This file contains the VUSB_HS Device Controller interface 
***  function to cancel a transfer.
***                                                               
**************************************************************************
**END*********************************************************/
#include "devapi.h"
#include "usbprv_dev.h"
#include <linux/time.h>
#include <linux/dma-mapping.h>
#ifdef __USB_OS_MQX__
   #include "mqx_arc.h"
#endif

extern u32 disable_printks;

#ifdef ORIGINAL_CODE

uint_8 _usb_dci_vusb20_cancel_transfer
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [IN] direction */
      uint_8                     direction
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, check_dTD_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_32                                      temp, bit_pos;
   int                                          timeout;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   bit_pos = (1 << (16 * direction + ep_num));
   temp = (2*ep_num + direction);

#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      temp;
#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      temp;
#endif

   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];
   
   if (dTD_ptr) {
#ifdef _VIRT_MEM
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
	  ((uint_32)phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK)) ;
#else
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)((uint_32)dTD_ptr->NEXT_TR_ELEM_PTR & 
         VUSBHS_TD_ADDR_MASK);
#endif
      if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
         /* Flushing will halt the pipe */
         /* Write 1 to the Flush register */
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;

         /* Wait until flushing completed */
         timeout = 1000;
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
            /* ENDPTFLUSH bit should be cleared to indicate this 
            ** operation is complete 
            */
             if (timeout-- == 0)
             {
                 USBERR("Timeout while cancelling td\n");
                 return USBERR_UNKNOWN_ERROR;
             }
             
         } /* EndWhile */
         
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
            bit_pos) 
         {
            /* Write 1 to the Flush register */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;
         
            /* Wait until flushing completed */
            while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
               /* ENDPTFLUSH bit should be cleared to indicate this 
               ** operation is complete 
               */
            } /* EndWhile */
         } /* EndWhile */
      } /* Endif */
      
      /* Retire the current dTD */
      dTD_ptr->SIZE_IOC_STS = 0;
      dTD_ptr->NEXT_TR_ELEM_PTR = VUSBHS_TD_NEXT_TERMINATE; //vOK
      
      /* The transfer descriptor for this dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      /* Free the dTD */
      _usb_dci_vusb20_free_dTD((pointer)dTD_ptr);
      
      /* Update the dTD head and tail for specific endpoint/direction */
      if (!check_dTD_ptr) {
         usb_dev_ptr->EP_DTD_HEADS[temp] = NULL;
         usb_dev_ptr->EP_DTD_TAILS[temp] = NULL;
         if (xd_ptr) {
            /* Free the transfer descriptor */
            _usb_device_free_XD((pointer)xd_ptr);
         } /* Endif */
         /* No other transfers on the queue */
         ep_queue_head_ptr->NEXT_DTD_PTR = VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE; //vOK
         ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
      } else {
         usb_dev_ptr->EP_DTD_HEADS[temp] = check_dTD_ptr; //v2v
            
         if (xd_ptr) {
            if ((uint_32)check_dTD_ptr->SCRATCH_PTR->\
               XD_FOR_THIS_DTD != (uint_32)xd_ptr) 
            {
               /* Free the transfer descriptor */
               _usb_device_free_XD((pointer)xd_ptr);
            } /* Endif */
         } /* Endif */

         
         if (check_dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {

            /* Start CR 1015 */
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
            if (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
               bit_pos)) 
            {
               while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & bit_pos) {
                  /* Wait for the ENDPTPRIME to go to zero */
               } /* EndWhile */

               if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
                  bit_pos) 
               {
                  /* The endpoint was not not primed so no other transfers on 
                  ** the queue 
                  */
                  goto done;
               } /* Endif */
            } else {
               goto done;
            } /* Endif */

            /* No other transfers on the queue */
#ifdef _VIRT_MEM
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)virt2phys((void *)check_dTD_ptr) ;
#else
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)check_dTD_ptr;
#endif
            ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
   
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
         } /* Endif */
      } /* Endif */
   } /* Endif */
done:
   
   return USB_OK;

   /* End CR 1015 */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer, SUCCESSFUL");
   #endif
   
} /* EndBody */

/* EOF */

#endif

u32 extern readwriteflag ,activestatus_flag,big_flag;
u32 extern mulqtd ,out_sync_flag ,write_10;


extern unsigned char scsi_cmd[31];

extern u32 out_scsi_cmd_cnt;
 //u32 dTD_status[8]= { 0,0,0,0,0,0,0,0,0};

struct timeval add_qtd_statusclear_time_start,add_qtd_statusclear_time_end;

extern unsigned char in_512[64];
extern unsigned int flag_transfer;


//int gval =0;
#if 1

#ifdef  USB_PROP_FILE_OP

//extern void read_file(char *filename);
extern void read_file(char *filename , char * buf, size_t count);
extern int flt_open_file (const char *fname, unsigned int mode, void **hdl);
extern unsigned char read_buf[64];
extern unsigned char read_buff[64];
extern unsigned int read_buff_cnt;
extern int get_dir(char *dir );
extern int flt_write_file (void *hdl, void *buffer, unsigned long long offset, 
		    unsigned int length, unsigned int *bytes_written);
extern void flt_close_file (void *hdl);
unsigned char write_buf[256];
extern unsigned char  prop_cmd[8];
extern unsigned char total_file_string_size[64];
extern unsigned int read_stream;
extern unsigned int read_file_open_flag , IsEOF;
extern char file_name[64];

#endif


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_cancel_transfer
*  Returned Value : USB_OK or error code
*  Comments       :
*        Cancels a transfer
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_cancel_transfer
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [IN] direction */
      uint_8                     direction
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, check_dTD_ptr,temp_dTD_ptr;  //June13th
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_32                                      temp, bit_pos;
   int                                          /* timeout ,*/active=0 , /* active_temp=0,i=0 ,val=0 , */transfer_len;

   /* static	char								tmp_buf[32 * 1024];									*/

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer");
   #endif

#ifdef  USB_PROP_FILE_OP

/*
  int fstatus;
  void *hdl;	
  unsigned int bytes_wr=0 ,bytes_rd=0 ,cnt=0; */
  	

#endif

   
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   bit_pos = (1 << (16 * direction + ep_num));
   temp = (2*ep_num + direction);

   //printk(" _usb_dci_vusb20_cancel_transfer ep_num=%d, direction = %d temp=%d\n",ep_num,direction,temp);
#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      temp;
#else
   
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      temp;
#endif

  // v6_inv_data_cache_all();

   
   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];


 

 //  temp_dTD_ptr->NEXT_TR_ELEM_PTR = dTD_ptr->NEXT_TR_ELEM_PTR;

   if(readwriteflag)
   {
#if 0
		if(!disable_printks) //June13th
		{
			if(dTD_ptr)
		/* 	printk( " _usb_dci_vusb20_cancel_transfer    dTD_ptr->NEXT_TR_ELEM_PTR = %x  temp=%x  ep_num = %x \n",dTD_ptr->NEXT_TR_ELEM_PTR,temp,ep_num); */
		}
#endif

   }
   if (dTD_ptr) {
#ifdef _VIRT_MEM
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
	  ((uint_32)phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK)) ;
		
	  //Senthil Added May26th
  

#else
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)((uint_32)dTD_ptr->NEXT_TR_ELEM_PTR & 
         VUSBHS_TD_ADDR_MASK);
#endif

		

	  //printk(" before IF dTD_ptr->SIZE_IOC_STS = %x \n",dTD_ptr->SIZE_IOC_STS);
	  //v6_dma_inv_range(dTD_ptr , dTD_ptr + sizeof(dTD_ptr) -1 );

	//  v6_flush_kern_cache_all();

	 //  consistent_sync((void*)dTD_ptr, sizeof(VUSB20_EP_TR_STRUCT), DMA_FROM_DEVICE); 

      if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
		

		//  gval= val = dTD_ptr->SIZE_IOC_STS;
		
		 // if(big_flag)
		//	active_temp =1 ;
	

	//	  printk(" _usb_dci_vusb20_cancel_transfer dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE \n");
			
#if 0
//		  	local_irq_save(flags);
		 while((dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) == VUSBHS_TD_STATUS_ACTIVE)
		  {
 //			v6_inv_data_cache_all();
			//	printk(" ######################## VINEET \n" );
			//	memset(tmp_buf, 0xf0, sizeof(32 * 1024));
			// __udelay(1000);
		//	 v6_flush_kern_cache_all();
		//	 v6_inv_data_cache_all();
	//	  	printk(" ######################## VINEET \n" );
			
	//
		  }
#endif

//		 local_irq_restore(flags);

//	  do_gettimeofday(&add_qtd_statusclear_time_start);	  
	//  printk(" dTD_ptr->SIZE_IOC_STS = %x \n",dTD_ptr->SIZE_IOC_STS);
	
#if 0	
	  while ( (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) ==  (VUSBHS_TD_STATUS_ACTIVE))	  {
		  
//		v6_inv_data_cache_all();
			 printk(" ######################## VINEET \n" );  
	
	 }
#endif
		

//	 printk(" ********************SENTHIL*************  \n" );  
	  

//	do_gettimeofday(&add_qtd_statusclear_time_end);	
//	printk("QTD Status Clear  start sec:%08li start usec:%08li end sec:%08li end usec:%08li\n",add_qtd_statusclear_time_start.tv_sec,(long)add_qtd_statusclear_time_start.tv_usec,add_qtd_statusclear_time_end.tv_sec,(long)add_qtd_statusclear_time_end.tv_usec);
	  
		 // printk(" _usb_dci_vusb20_cancel_transfer TD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE \n");

		  //if(big_flag)
		//	activestatus_flag=1;

		  if(/* activestatus_flag */0) 
		  {
		
		  do_gettimeofday(&add_qtd_statusclear_time_start);	  
		  while((dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) == VUSBHS_TD_STATUS_ACTIVE)
		  {

				printk("Waiting**************for dTD Status to clear \n");
		  }


			do_gettimeofday(&add_qtd_statusclear_time_end);


			 printk("QTD Status Clear  start sec:%08li start usec:%08li end sec:%08li end usec:%08li\n",add_qtd_statusclear_time_start.tv_sec,(long)add_qtd_statusclear_time_start.tv_usec,add_qtd_statusclear_time_end.tv_sec,(long)add_qtd_statusclear_time_end.tv_usec);

			 activestatus_flag=0;
		  }
		  //printk("DTD Status is cleared \n");

#if 0
		  

		  printk(" _usb_dci_vusb20_cancel_transfer TD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE \n");



		  active = 1; //June13th
         /* Flushing will halt the pipe */
         /* Write 1 to the Flush register */
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;

         /* Wait until flushing completed */
         timeout = 1000;
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
            /* ENDPTFLUSH bit should be cleared to indicate this 
            ** operation is complete 
            */
             if (timeout-- == 0)
             {
                 USBERR("Timeout while cancelling td\n");
				 printk("Timeout while cancelling td\n");
                 return USBERR_UNKNOWN_ERROR;
             }
             
         } /* EndWhile */
         
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
            bit_pos) 
         {
            /* Write 1 to the Flush register */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;
         
            /* Wait until flushing completed */
            while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
               /* ENDPTFLUSH bit should be cleared to indicate this 
               ** operation is complete 
               */
            } /* EndWhile */
         } /* EndWhile */

#endif
      } /* Endif */

	//Senthil Added June13th
	//Hi
	 if(readwriteflag  &&   (dTD_ptr->NEXT_TR_ELEM_PTR != VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE )  && !active)
     {
	//	printk("Cleanup \n");

		// if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
		//	printk( "INSIDE dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE \n");
		// }
		 
		temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR) phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR);
		xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;

	//	if(active_temp && big_flag)
	//	{
	//		dTD_status[active_temp]= dTD_ptr->SIZE_IOC_STS;
	//		active_temp++;
	//	}
			

		if(dTD_ptr)
		_usb_dci_vusb20_free_dTD((pointer)dTD_ptr);

		if (xd_ptr) {
            /* Free the transfer descriptor */
            _usb_device_free_XD((pointer)xd_ptr);

			
         } 

		while(temp_dTD_ptr != (VUSB20_EP_TR_STRUCT_PTR)VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE)
		{
			if(temp_dTD_ptr->NEXT_TR_ELEM_PTR != VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE)
			{
				temp_dTD_ptr =  (VUSB20_EP_TR_STRUCT_PTR) phys2virt(temp_dTD_ptr->NEXT_TR_ELEM_PTR);

		//		if(active_temp && big_flag)
		//		{
			//		dTD_status[active_temp]= dTD_ptr->SIZE_IOC_STS;
			//		active_temp++;
			//	}
			

				xd_ptr = (XD_STRUCT_PTR)temp_dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;

				if (xd_ptr) {
            /* Free the transfer descriptor */
            _usb_device_free_XD((pointer)xd_ptr);

			
				 } 

			if(dTD_ptr)
			_usb_dci_vusb20_free_dTD((pointer)temp_dTD_ptr);
			
			}else
			{
				temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE;
			//	printk(" Terminate ***********\n");
			}
		
			
			



		//	printk(" Next test_dTD_ptr =%x \n",test_dTD_ptr);
		}
		

	//	if(active_temp && big_flag)
	//	{
	//		active_temp=0;

	//	printk(" TEST \n");
			
	//	}

		active=0;

		usb_dev_ptr->EP_DTD_HEADS[temp] = NULL;
        usb_dev_ptr->EP_DTD_TAILS[temp] = NULL;

	 if ( write_10   ) //June06th
	 {	

		   out_scsi_cmd_cnt++;
		  _usb_device_recv_data(handle, 2, scsi_cmd, 31);
    	  
	 }
	  
		
		goto done;

	 }
	
      
      /* Retire the current dTD */
      dTD_ptr->SIZE_IOC_STS = 0;
      dTD_ptr->NEXT_TR_ELEM_PTR = VUSBHS_TD_NEXT_TERMINATE; //vOK july 1
      
      /* The transfer descriptor for this dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;


	  transfer_len = xd_ptr->WTOTALLENGTH; //July7th

	  //senthil
      /* Free the dTD */
      //_usb_dci_vusb20_free_dTD((pointer)dTD_ptr);
      
//Senthil Added May26th
      /* Update the dTD head and tail for specific endpoint/direction */
      if (dTD_ptr->NEXT_TR_ELEM_PTR == VUSBHS_TD_NEXT_TERMINATE)  /* Senthil !check_dTD_ptr)*/ {  
		if(disable_printks) //June13th
		{
	//	  printk(" No Next DTD \n ");
		}
		// _usb_dci_vusb20_free_dTD((pointer)dTD_ptr); //June13th

	//July 07	check_dTD_ptr=NULL; //Senthil Added May26th

         usb_dev_ptr->EP_DTD_HEADS[temp] = NULL;
         usb_dev_ptr->EP_DTD_TAILS[temp] = NULL;
         if (xd_ptr) {
            /* Free the transfer descriptor */
            _usb_device_free_XD((pointer)xd_ptr);

			
         } /* Endif */
		
		  if(dTD_ptr)
		   _usb_dci_vusb20_free_dTD((pointer)dTD_ptr); //June13th

         /* No other transfers on the queue */
         ep_queue_head_ptr->NEXT_DTD_PTR = VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE; //vOK
         ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;

#if 1
	 if (write_10  && (ep_num == 2) && ( transfer_len != 31)) //June06th
	 {	

		   out_scsi_cmd_cnt++;
		  _usb_device_recv_data(handle, 2, scsi_cmd, 31);
    	  
	 }
#endif	  
		


		if(!disable_printks)
		{
			 printk(" _usb_dci_vusb20_cancel_transfer No other Transfer onthe queue \n");
		}
      } else {
		

		  if(disable_printks) //June13th
		  {
		       printk(" _usb_dci_vusb20_cancel_transfer Try Next QTD \n");
		  }
		  
         usb_dev_ptr->EP_DTD_HEADS[temp] = check_dTD_ptr; //v2v

		 if(!disable_printks)
		 {
		     printk(" _usb_dci_vusb20_cancel_transfer Try Next QTD 1\n");
		 }


  //Senthil Test May26th          
#if 0  //June13th
         if (xd_ptr) {
            if ((uint_32)check_dTD_ptr->SCRATCH_PTR->\
               XD_FOR_THIS_DTD != (uint_32)xd_ptr) 
            {
               /* Free the transfer descriptor */
               _usb_device_free_XD((pointer)xd_ptr);
            } /* Endif */
         } /* Endif */

#endif
	
		 if(!disable_printks)
		 {
             printk(" _usb_dci_vusb20_cancel_transfer Try Next QTD 2\n");
		 }


         if (check_dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {

			 printk(" _usb_dci_vusb20_cancel_transfer Try Next QTD VUSBHS_TD_STATUS_ACTIVE \n");

            /* Start CR 1015 */
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
            if (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
               bit_pos)) 
            {
               while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & bit_pos) {
                  /* Wait for the ENDPTPRIME to go to zero */
               } /* EndWhile */

               if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
                  bit_pos) 
               {
                  /* The endpoint was not not primed so no other transfers on 
                  ** the queue 
                  */
                  goto done;
               } /* Endif */
            } else {
               goto done;
            } /* Endif */

            /* No other transfers on the queue */
#ifdef _VIRT_MEM

#ifdef DMA_POOL
			ep_queue_head_ptr->NEXT_DTD_PTR = check_dTD_ptr->phyaddr;
#else
		   ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)virt2phys((void *)check_dTD_ptr) ;
#endif
            

#else
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)check_dTD_ptr;
#endif
            ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
   
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
         } /* Endif */
		if(!disable_printks)
		{
		 printk( " _usb_dci_vusb20_cancel_transfer Try Next QTD OUT1 \n");
		}
		
		//Senthil Need to Add to Cleanup the QTD resource

		


      } /* Endif */
	
	  if(!disable_printks)
	  {
	     printk( " _usb_dci_vusb20_cancel_transfer Try Next QTD OUT2 \n");
	  }

#ifdef USB_PROP_FILE_OP


	  if(read_stream  & (ep_num == 3))
	  {

#if 0
			read_file("/1.ts",read_buff,64);
			_usb_device_send_data(handle,3,(u8 *)((u32)read_buff & 0xfffff000) ,64,(u32)read_buff & 0x00000fff); 
#endif

		//	printk(" Inside read_stream IN  \n");

			read_file(file_name,read_buff,64);
			
		//	read_file("/test/1.ts",read_buff,64);
		//	printk(" Inside read_stream IN  1\n");
			 ; /* file open is required just once */

			if(!IsEOF )
			{
				read_file_open_flag = 1;
				_usb_device_send_data(handle,3,(u8 *)((u32)read_buff & 0xfffff000) ,64,(u32)read_buff & 0x00000fff); 

			}
			else
			{
				read_file_open_flag = 0; 	
				_usb_device_send_data(handle,3,(u8 *)((u32)read_buf & 0xfffff000) ,64,(u32)read_buf & 0x00000fff); 
					IsEOF = 0;

			}
		//	printk(" Inside read_stream IN  2\n");
		//	printk(" Inside read_stream OUT\n");
	  }

	  if(flag_transfer)
		{

		
		
		//	_usb_device_send_data(handle,3,(u8 *)((u32)total_file_string_size & 0xfffff000) ,64,(u32)total_file_string_size & 0x00000fff);

		  _usb_device_recv_data(handle, 2, prop_cmd, 8);
		  flag_transfer =0;
#if 0		
		  	read_file("/1.txt");


		//	read_file("/jan08.txt"); 

	//	  get_dir("/test");

		

		//	fstatus = flt_open_file ("/jan08.txt", O_RDONLY, &hdl);


		//	flt_read_file(hdl,read_buf,0,10,&bytes_rd);
		//	printk(" flt_read_file bytes_rd =%d \nt",bytes_rd);

	
		//	for(cnt=0;cnt < bytes_rd ; cnt++)
		//	{
		
		///		printk(" read_buf[cnt] = %x \n",read_buf[cnt]);
		
		///	}
			
		  //WriteFile
		printk(" REPROGRAM THE  OUT dTD \n");


		//	_usb_device_send_data(handle,3,(u8 *)((u32)read_buff & 0xfffff000) ,read_buff_cnt,(u32)read_buff & 0x00000fff); //Send zero bytes data

		_usb_device_recv_data(handle, 2, write_buf, 256);


		 

		fstatus = flt_open_file ("/test/write.txt", O_WRONLY|O_CREAT, &hdl);

		printk(" flt_open_file hdl=%x \n",hdl);
	


		flt_write_file( hdl, write_buf ,0,256, &bytes_wr);

		printk(" flt_write_file bytes_wr=%d \nt",bytes_wr);


		flt_close_file(hdl);
#endif

#if 0
		  for( cnt = 0 ; cnt < 256 ; cnt++)
			  printk(" %x \t ", write_buf[cnt]);

		    printk("\n");
#endif


		  //ReadFile
#if 0
			printk(" REPROGRAM THE  IN_dTD \n");
			_usb_device_send_data(handle,3,(u8 *)((u32)read_buff & 0xfffff000) ,64,(u32)read_buff & 0x00000fff); //Send zero bytes data
#endif


		}
#endif	  


   } /* Endif */
done:
   
   return USB_OK;

   /* End CR 1015 */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer, SUCCESSFUL");
   #endif

#ifdef DEBUG_USB
	  printk("_usb_dci_vusb20_cancel_transfer, SUCCESSFUL  OUT \n"); 

#endif
   
} /* EndBody */


#endif
/* EOF */
