/****************************************************************************
 * linux/drivers/usb/host/ehci-cnxt.c
 *
 * Copyright (C) 2007 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/

/****************************************************************************
 * $Header: ehci-cnxt.c, 3, 10/10/07 1:35:42 PM IST, Nishant Pandit$
 ****************************************************************************/

#define EHCI_IAA_JIFFIES        (HZ/100)        /* arbitrary; ~10 msec */
#define EHCI_IO_JIFFIES         (HZ/10)         /* io watchdog > irq_thresh */
#define EHCI_ASYNC_JIFFIES      (HZ/20)         /* async idle timeout */
#define EHCI_SHRINK_JIFFIES     (HZ/200)        /* async qh unlink delay */

#define USB_ENABLE_REG          (0xE0440138)
#define USB0_PHY_PDN            (1<<0) 
#define USB1_PHY_PDN            (1<<1) 
#define USB_PHY_CLK             (1<<2) 
#define PORT_LS         (3<<10) /* Line State Mask */
#define PORT_HS         (1<<9)  /* Port Speed Mask */

#include <linux/config.h>
#include <linux/reboot.h>
#include <linux/usb.h>
#include <asm/io.h>
#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/arch/hardware.h>
/*#include <asm/arch/hardware/usb.h> */
/*#include <asm/arch/hardware/devconf.h> */


#ifdef CONFIG_ARCH_PECOS
#include "asm/arch/platform.h"
#endif

#ifdef CONFIG_ARCH_NEVIS
#include "asm/arch/platform.h"
#endif

#include "ehci.h"

#include "ehci-cnxt.h" 

#include <linux/platform_device.h>

#define CONFIG_CNXT_USB_PLUGFEST

#ifndef CONFIG_TEST_MODE
#define CONFIG_TEST_MODE
#endif

#ifdef	CONFIG_CNXT_USB_PLUGFEST
#include <linux/proc_fs.h>
#endif

static int ehci_cnxt_setup(struct usb_hcd *hcd);

static const struct hc_driver ehci_cnxt_hc_driver = {
	.description = hcd_name_cnxt,
	.product_desc = "conexant EHCI",
	.hcd_priv_size = sizeof(struct ehci_hcd),

	/*
	 * generic hardware linkage
	 */
	.irq = ehci_irq,
	.flags = HCD_MEMORY | HCD_USB2,

	/*
	 * basic lifecycle operations
	 */
	.reset = ehci_cnxt_setup,
	.start = ehci_run,
	.stop = ehci_stop,

	/*
	 * managing i/o requests and associated device resources
	 */
	.urb_enqueue = ehci_urb_enqueue,
	.urb_dequeue = ehci_urb_dequeue,
	.endpoint_disable = ehci_endpoint_disable,

	/*
	 * scheduling support
	 */
	.get_frame_number = ehci_get_frame,

	/*
	 * root hub support
	 */
	.hub_status_data = ehci_hub_status_data,
	.hub_control = ehci_hub_control,
//#ifdef	CONFIG_PM /* Commented for Plugfest USB Host Mode Test:  Joseph 13-Jan-2008 */
#if 1
	.hub_suspend = ehci_hub_suspend,
	.hub_resume = ehci_hub_resume,
#endif
	.bus_suspend = ehci_bus_suspend,
	.bus_resume = ehci_bus_resume,

#ifdef CONFIG_TEST_MODE
	.port_test = ehci_port_test_control,
#endif
};

/* AIX_3RDPARTY_CODE_CHANGED: AK/JS */
#define MAX_HCDS 3
static struct usb_hcd * g_hcd[MAX_HCDS] = {0};;
static int              g_hcd_devs = 0;

/********************************************************************/
/*  FUNCTION:    ehci_regs_dump										*/
/*                                                                  */
/*  PARAMETERS:  None											    */
/*                                                                  */
/*  DESCRIPTION:  Dump Ehci Registers								*/
/*													                */
/*                                                                  */
/*  RETURNS:     None					                            */
/*                                                                  */
/********************************************************************/

void ehci_regs_dump(void)
{


    unsigned int *x;

    x = (unsigned int *)USB_CAP_LEN;
        printk("CAPLENGTH = 0x%02x\n", *(unsigned short *)x);

    x = (unsigned int *)USB_HCI_VER;
        printk("HCIVERSION = 0x%02x\n",*(unsigned short *)x);

    x = (unsigned int *)USB_HSCPARM;
        printk("HCSPARAMS = 0x%02x\n", *(unsigned short *)x);

    x = (unsigned int *)USB_HCCPARM;
        printk("HCCPARAMS = 0x%02x\n", *(unsigned short *)x);

    x = (unsigned int *)USB_CMD_REG;
        printk("USBCMD = 0x%08x\n",*x);

    x = (unsigned int *)USB_STS_REG;
        printk("USBSTS = 0x%08x\n",*x);

    x = (unsigned int *)USB_INE_REG;
        printk("USBINTR = 0x%08x\n",*x);

    x = (unsigned int *)USB_FRI_REG;
        printk("FRINDEX = 0x%08x\n",*x);

    x = (unsigned int *)USB_CTL_SEG;
        printk("CTRLDSEGMENT = 0x%08x\n",*x);

    x = (unsigned int *)USB_PRL_REG;
        printk("PERIODICLISTBASE = 0x%08x\n",*x);

    x = (unsigned int *)USB_ASY_REG;
        printk("ASYNCLISTBASE = 0x%08x\n",*x);

    x = (unsigned int *)USB_TTL_REG;
        printk("TTCTRL = 0x%08x\n",*x);

    x = (unsigned int *)USB_BUR_REG;
        printk("BURSTSIZE = 0x%08x\n",*x);

    x = (unsigned int *)USB_COF_REG;
        printk("USB_COF_REG = 0x%08x\n",*x);

    x = (unsigned int *)USB_PST_REG;
        printk("USB_PST_REG = 0x%08x\n",*x);

    x = (unsigned int *)USB_USM_REG;
        printk("USBMODE = 0x%08x\n",*x);

}


/********************************************************************/
/*  FUNCTION:    usb_read_pciconfig									*/
/*                                                                  */
/*  PARAMETERS:	 devicenum- Device Number		 				    */
/*               offset - offset value                              */
/*  DESCRIPTION:  usb_read_pciconfig								*/
/*													                */
/*                                                                  */
/*  RETURNS: Returns the 8th Byte of PCI Bridges's                  */
/*          Configuration Space, To get the REV ID                  */
/********************************************************************/


unsigned int usb_read_pciconfig(unsigned int devicenum, unsigned int offset)
{

    unsigned int temp;
    *(unsigned int *)PCI_CFG_ADDR_REG = ((devicenum<<11) | offset);
    temp = *(unsigned int *)PCI_CFG_ADDR_REG;

	    return temp;
}


/********************************************************************/
/*  FUNCTION:    usb_read_pciconfig									*/
/*                                                                  */
/*  PARAMETERS:	None 		 				                        */
/*										                            */
/*  DESCRIPTION:  Use PCI RevID to differentiate					*/
/*				  between RevB & RevC				                */
/*                                                                  */
/*  RETURNS: None													*/
/*																	*/
/********************************************************************/

void usb_configure_clkdly(void)
{
     unsigned int temp;
     unsigned int usb_read_pciconfig(unsigned int, unsigned int);

#ifdef CONFIG_ARCH_CACTUS
     temp = usb_read_pciconfig(0,8);
     if ((temp & 0xFF) >= 4)
     {
#endif


             temp = *(unsigned int *)0xe0440150;
			 temp = temp & 0x3;
             temp = temp | 0x00151500;
             *(unsigned int *)0xe0440150 =  temp;

#ifdef CONFIG_ARCH_CACTUS
       }      
#endif

}



/********************************************************************/
/*  FUNCTION:    usb_release										*/
/*                                                                  */
/*  PARAMETERS:	dev - pointer to struct device                      */
/*										                            */
/*  DESCRIPTION:  usb_release										*/
/*													                */
/*                                                                  */
/*  RETURNS: None													*/
/*																	*/
/********************************************************************/

static void usb_release(struct device *dev)
{
	/* normally not freed */
}

/* The dmamask must be set for EHCI to work */
static u64 ehci_dmamask =(u64) (u64*)-1;

static struct resource ehci_resources0[] = {
	{
		.start	= ASX_USB0_VIRT,
		.end	= ASX_USB0_VIRT + 4096 - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= ITC_USB0_POS,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ehci_device0 = {
	.name			= "cnxt0",
	.id			= -1,
	.dev = {
		.release		= usb_release,
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(ehci_resources0),
	.resource		= ehci_resources0,
};

#ifdef CONFIG_EHCI_CNXT_3CTRLER

static struct resource ehci_resources1[] = {
	{
		.start	= ASX_USB1_VIRT,
		.end	= ASX_USB1_VIRT + 4096 - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= ITC_USB1_POS,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ehci_device1 = {
	.name			= "cnxt1",
	.id			= -1,
	.dev = {
		.release		= usb_release,
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(ehci_resources1),
	.resource		= ehci_resources1,
};

static struct resource ehci_resources2[] = {
	{
		.start	= ASX_USB2_VIRT,
		.end	= ASX_USB2_VIRT + 4096 - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= ITC_USB2_POS,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ehci_device2 = {
	.name			= "cnxt2",
	.id			= -1,
	.dev = {
		.release		= usb_release,
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(ehci_resources2),
	.resource		= ehci_resources2,
};

#ifndef SUPPORT_SUSPEND_RESUME
#define SUPPORT_SUSPEND_RESUME
#endif
#ifdef SUPPORT_SUSPEND_RESUME
struct platform_device *PLATFORM_DEVICE_1_ADDR ( void );
struct platform_device *PLATFORM_DEVICE_2_ADDR ( void );
struct platform_device *PLATFORM_DEVICE_3_ADDR ( void );

struct platform_device *PLATFORM_DEVICE_1_ADDR ( void )
{
	return &ehci_device0;
}
struct platform_device *PLATFORM_DEVICE_2_ADDR ( void )
{
	return &ehci_device1;
}
struct platform_device *PLATFORM_DEVICE_3_ADDR ( void )
{
	return &ehci_device2;
}
#endif

#endif


/********************************************************************/
/*  FUNCTION:    ehci_cnxt_reinit									*/
/*                                                                  */
/*  PARAMETERS:	ehci - pointer to struct ehci_hcd                   */
/*										                            */
/*  DESCRIPTION:called after powerup,by probe or system-pm "wakeup"	*/
/*													                */
/*                                                                  */
/*  RETURNS: Zero for success										*/
/*																	*/
/********************************************************************/

static int ehci_cnxt_reinit(struct ehci_hcd *ehci)
{
	int temp;

    usb_configure_clkdly();

#ifdef CONFIG_EHCI_CNXT_3CTRLER
    // AIX_3RDPART_CODE_CHANGED: configurable USB_ENABLE bits (see also Kconfig)
	 /* Enable the  Port Power for all 3 ports */
    *(volatile unsigned int *)0xe0440138 = 
      ( 0x0003 
#ifdef CONFIG_EHCI_CNXT_USB11_SLPN
      | 0x0400
#endif
#ifdef CONFIG_EHCI_CNXT_USB11_RSLPN
      | 0x0200
#endif
#ifdef CONFIG_EHCI_CNXT_USB0_VBUSFAULT_SEL
      | 0x0008
#endif
#ifdef CONFIG_EHCI_CNXT_USB1_VBUSFAULT_SEL
      | 0x0010
#endif
#ifdef CONFIG_EHCI_CNXT_USB2_VBUSFAULT_SEL
      | 0x0020
#endif
#ifdef CONFIG_EHCI_CNXT_USB0_VBUSDRIVE_SEL
      | 0x0040
#endif
#ifdef CONFIG_EHCI_CNXT_USB1_VBUSDRIVE_SEL
      | 0x0080
#endif
#ifdef CONFIG_EHCI_CNXT_USB2_VBUSDRIVE_SEL
      | 0x0100
#endif
      );
    msleep(100);
#endif

    ehci_port_power(ehci, 0);

#if 1 /* PIO_77 (bit 13) as VBUSDRIVE: Added for Chirp Timing testing Power On Reset (POR) failure: Joseph 24-Jan-2009 */
	temp = *(unsigned int *)0xE0440188;
	temp = (temp | 0x00002000);
	*(unsigned int *)0xE0440188  = temp;
#endif

	return 0;
}

/********************************************************************/
/*  FUNCTION:    ehci_cnxt_setup									*/
/*                                                                  */
/*  PARAMETERS:	hcd - pointer to struct usb_hcd	                    */
/*										                            */
/*  DESCRIPTION:called ehci_reset									*/
/*													                */
/*                                                                  */
/*  RETURNS: retval returns zero for sucess							*/
/*																	*/
/********************************************************************/
static int ehci_cnxt_setup(struct usb_hcd *hcd)
{
	struct ehci_hcd *ehci = hcd_to_ehci(hcd);
	int retval;
	u32 temp;

		
	ehci->caps = hcd->regs ;
	ehci->regs = hcd->regs + HC_LENGTH(readl(&ehci->caps->hc_capbase));

	/* cache this readonly data; minimize chip reads */
	ehci->hcs_params = readl(&ehci->caps->hcs_params);
 
    tdi_reset(ehci);
	retval = ehci_halt(ehci);
	if (retval)
		return retval;

	/* data structure init */
	retval = ehci_init(hcd);
	if (retval)
		return retval;

	
	ehci->is_tdi_rh_tt = 1;
	

 	if (ehci_is_TDI(ehci))
		ehci_reset(ehci); 



	temp = HCS_N_CC(ehci->hcs_params) * HCS_N_PCC(ehci->hcs_params);
	temp &= 0x0f;
	if (temp && HCS_N_PORTS(ehci->hcs_params) > temp) {
		printk( "bogus port configuration: "
			"cc=%d x pcc=%d < ports=%d\n",
			HCS_N_CC(ehci->hcs_params),
			HCS_N_PCC(ehci->hcs_params),
			HCS_N_PORTS(ehci->hcs_params));
	}

	retval = ehci_cnxt_reinit(ehci);

	return retval;
}


#ifdef	CONFIG_USB_CNXT_OTG

extern int trans;

void start_hnp(struct ehci_hcd *ehci ,u16 index)
{
	const unsigned	port = ehci_to_hcd(ehci)->self.otg_port - 1;
	unsigned long	flags;
	u32 temp;

	otg_start_hnp(ehci->transceiver);

	local_irq_save(flags);
	ehci->transceiver->state = OTG_STATE_A_SUSPEND;

    temp = readl (&ehci->regs->port_status [index]);
	temp |= PORT_SUSPEND;
	writel (temp, &ehci->regs->port_status [index]);
	local_irq_restore(flags);
}
#endif



/********************************************************************/
/*  FUNCTION:    usb_ehci_cnxt_probe								*/
/*                                                                  */
/*  PARAMETERS:	driver - pointer to struct hc_driver                */
/*				hcd_out - pointer to pointer of struct usb_hcd		*/
/*				pdev   - Platform device pointer                    */
/*  DESCRIPTION: Probe fuction for cnxt ehci driver					*/
/*													                */
/*                                                                  */
/*  RETURNS: retval returns zero for sucess							*/
/*																	*/
/********************************************************************/

int usb_ehci_cnxt_probe(const struct hc_driver *driver,
			  struct usb_hcd **hcd_out, struct platform_device *pdev)
{
	int retval;
	struct usb_hcd *hcd;
	struct ehci_hcd *ehci;
	struct resource *res;
#ifdef CONFIG_USB_CNXT_OTG
	u32 uTransCnt=-1;
#endif
		
	
	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (!res) {
		dev_err(&pdev->dev,
			"Found HC with no register addr. Check %s setup!\n",
			pdev->dev.bus_id);
		retval = -ENODEV;
	
	}

	if (pdev->resource[1].flags != IORESOURCE_IRQ) {
		pr_debug("resource[1] is not IORESOURCE_IRQ");
		retval = -ENOMEM;
	}
	hcd = usb_create_hcd(driver, &pdev->dev, "cnxt" /* pdev->dev.bus_id */);

  // AIX_3RDPARTY_CODE_CHANGED: AK
  *hcd_out = hcd;

	if (!hcd)
		return -ENOMEM;


	hcd->rsrc_start = (pdev->resource[0].start + CNXT_USB_CAP_OFFSET);
	hcd->rsrc_len = pdev->resource[0].end - pdev->resource[0].start + 1;

	hcd->regs = (void *)(pdev->resource[0].start + CNXT_USB_CAP_OFFSET);

	ehci = hcd_to_ehci(hcd);

	retval =
	    usb_add_hcd(hcd, pdev->resource[1].start, /* IRQF_DISABLED | */IRQF_SHARED);

#ifdef	CONFIG_USB_CNXT_OTG
	
		trans= uTransCnt++;
		ehci->transceiver = otg_get_transceiver();
		if (ehci->transceiver) {
			int	status = otg_set_host(ehci->transceiver,
						&ehci_to_hcd(ehci)->self);
			dev_dbg(&pdev->dev, "init %s transceiver, status %d\n",
					ehci->transceiver->label, status);
			if (status) {
				if (ehci->transceiver)
					put_device(ehci->transceiver->dev);
				return status;
			}
		} else {
			printk( "can't find transceiver%d \n",uTransCnt);
			return -ENODEV;
		}
	
#endif

#ifdef	CONFIG_CNXT_USB_PLUGFEST

    cnxt_usb_host_create_proc_entry();

#endif

	if (retval == 0)
		return retval;

	return retval;
}


/********************************************************************/
/*  FUNCTION:    ehci_hcd_cnxt_drv_probe							*/
/*                                                                  */
/*  PARAMETERS:										                */
/*																	*/
/*				pdev   - Platform device pointer                    */
/*  DESCRIPTION: probe function for cnxt ehci driver				*/
/*													                */
/*                                                                  */
/*  RETURNS: ret returns zero for sucess							*/
/*																	*/
/********************************************************************/
static int ehci_hcd_cnxt_drv_probe(struct platform_device *pdev)
{
	struct usb_hcd *hcd = NULL;
	int ret = 0;

	pr_debug("In ehci_hcd_cnxt_drv_probe\n");
	
	if (usb_disabled())
		return -ENODEV;

	ret = usb_ehci_cnxt_probe(&ehci_cnxt_hc_driver, &hcd, pdev);
  if( g_hcd_devs < MAX_HCDS )
  {
    printk( "%s(), g_hcd[%d]=%08p, ret=%d\n", __FUNCTION__, g_hcd_devs, hcd, ret );
    g_hcd[g_hcd_devs] = hcd;
    g_hcd_devs++;
  }
  return ret;
}

/********************************************************************/
/*  FUNCTION:    ehci_hcd_cnxt_drv_remove							*/
/*                                                                  */
/*  PARAMETERS:										                */
/*																	*/
/*				pdev   - Platform device pointer                    */
/*  DESCRIPTION: Remove function for cnxt ehci driver				*/
/*													                */
/*                                                                  */
/*  RETURNS:  returns zero for sucess							*/
/*																	*/
/********************************************************************/
static int ehci_hcd_cnxt_drv_remove(struct platform_device *pdev)
{
	struct usb_hcd *hcd = platform_get_drvdata(pdev);

	usb_remove_hcd(hcd);
	usb_put_hcd(hcd);
	return 0;
}


/* conexant ehci hcd platform driver strucuture */
static struct platform_driver ehci_hcd_cnxt_driver = {
	.probe = ehci_hcd_cnxt_drv_probe,
	.remove = ehci_hcd_cnxt_drv_remove,
	.driver = {
		.name = "cnxt-ehci",
		.bus = &platform_bus_type
	}
};

#define EHCI_CNXT_USE_CLOCK_GATING 0
int  ehci_cnxt_phy_reset(struct usb_device * udev, int portnum)
{ 
   struct usb_bus * bus = udev->bus;
   struct usb_hcd * hcd = (struct usb_hcd*)bus->hcpriv;
   struct ehci_hcd *ehci = hcd_to_ehci(hcd);
   u32 phyresetwait = 0;
   u32 isPhyreset = 0;
   u32 phy_port = 0;
   u32 temp;

   printk( "js.aix: %s(bus=%d) called\n", __FUNCTION__, bus->busnum );

   /* Check the bus number.  This will tell us which PHY to power cycle */
   if((bus->busnum) == 2) 
   {
      /* BUS 2 is controller 1 */
      phy_port = USB1_PHY_PDN;
   }
   else if((bus->busnum) == 1) 
   {
      /* BUS 1 is controller 0 */
      phy_port = USB0_PHY_PDN;
   }
   else
   {
      /* there are only 2 ports with PHY power control */
      /* just use 0 so no bits are manipulated */
   }
   
   /* Begin a busywait loop to wait for the end of the real RESET.  We must */
   /* do this as close as possible to the actual end of RESET otherwise we  */
   /* will run out of time and the device will begin it's RESET retry       */
   /* counter and fall back into full speed mode.                           */
   while ( 1 ) 
   {
      temp = readl(&ehci->regs->port_status[portnum-1]);
      /* CI13611-a implements auto-clearing reset feature. Get ready to     */
      /* quit the loop when it clears itself.  It *will* clear by itself.   */
      if( !(temp & PORT_RESET) ) 
      {
         /* If the port is not in High Speed, then quit.  No reset is       */
         /* required for low speed and full speed devices.                  */
         if(!(temp & PORT_HS))
         {
            break;
         }
         if((temp & PORT_HS)&&(!(temp & PORT_LS)) && (temp & PORT_PE))
         {
            /* If:                                                          */
            /*    1. The port identifies as High Speed                      */
            /*    2. The LINE STATE is in the SE0 state                     */
            /*    3. The port is finally set to appear as enabled           */
            /*                                                              */
            /*    then quit and perform the PHY reset.                      */
            break;
         }
      }
      phyresetwait++;
   }
   /* Check to make sure we still think the port is in high speed.             */
   temp = readl(&ehci->regs->port_status[portnum-1]);
   if(temp & PORT_HS)
   {
      u32 cmdval = 0;
      /* ONLY perform this workaround if we chirped up into high speed.        */
   
      /* This operation is sensitive to timing -- make this absolutely atomic  */
      spin_lock_irq(&ehci->lock);
      /* Read the USB_CMD Register for a  RMW operation to be done down the line */
      cmdval = readl(&ehci->regs->command);

      temp = readl(USB_ENABLE_REG);
      {
         u32 address  = USB_ENABLE_REG;
#if (EHCI_CNXT_USE_CLOCK_GATING == 1)
      /* Clock gate the PHY to trick the FSM's into resetting themselves */
         u32 step1 = (temp |   USB_PHY_CLK);   /* Gate   the clock */
         u32 step2 = (temp & (~USB_PHY_CLK));  /* Ungate the clock */
#else
      /* Power Cycle the PHY */
         u32 step1 = (temp &= (~phy_port));    /* Power down PHY  */
         u32 step2 = (temp |    phy_port);     /* Power up   PHY  */
#endif
      /* Do the Magic Reset Sequence back to back- Very Sensitive-Don't Alter  */  
      /* "HC STOP" ->"PHY PDN"->"PHY PUP"->"HC RUN"  back to back sequence     */
      /* clear run/stop to disable SOF's and guarantee no SOF's are            */
      /* transmitted during PHY reset                                          */
      	 cmdval &= ~CMD_RUN;
      	 writel(cmdval,&ehci->regs->command);
         __asm( "str %[s1],[%[usb_enable]]\n"
                "str %[s2],[%[usb_enable]]\n"
                : /* no output */
                : [s1] "r" (step1), 
                  [s2] "r" (step2), 
                  [usb_enable] "r" (address)
                :"memory");
      }
      /* restore SOF generation - set run/stop */
      cmdval |= CMD_RUN;
      writel(cmdval,&ehci->regs->command);
      
      /* Indicate we have reset the PHY */
      isPhyreset=1;
   
      /* End atomic lock */
      spin_unlock_irq(&ehci->lock);
   }
   return isPhyreset;
}

#ifdef CONFIG_USB_AIX_EXTERNAL_POWER_ENABLE
// AIX_3RDPARTY_CODE_CHANGED
/* AIX AK wrapper to en/disable port power */
int aix_usb_ehci_power_enable( int device, int on )
{
    struct ehci_hcd *ehci;
    if( device < 0 || device > g_hcd_devs ) return -ENODEV;
    ehci = hcd_to_ehci( g_hcd[device] );
    if( !ehci ) return -ENODEV;

    ehci_port_power ( ehci, on );
    printk("--- usb: ehci port %d set power to: %d\n", device, on);
    return 0;
}

EXPORT_SYMBOL( aix_usb_ehci_power_enable );
#endif

#ifdef	CONFIG_CNXT_USB_PLUGFEST

int cnxt_proc_usb_host1(char *buffer,char **buffer_location,off_t offset,
                             int buffer_length, int *eof, void *data)
{

		printk(KERN_INFO "cnxt_proc_usb_host1\n");

		return 0;
		
}

int cnxt_proc_usb_host2(char *buffer,char **buffer_location,off_t offset,
                             int buffer_length, int *eof, void *data)
{

		printk(KERN_INFO "cnxt_proc_usb_host2\n");

		return 0;
		
}

void cnxt_usb_host_create_proc_entry()
{
    static struct proc_dir_entry *host_proc_file;

    host_proc_file = create_proc_entry("usb_host1", 0644, NULL);
    if (host_proc_file == NULL) {
             remove_proc_entry("usb_host1", &proc_root);
             printk(KERN_ALERT "Error: Could not initialize /proc/%s\n","usb_host1");
             return;
    }
    host_proc_file->read_proc = cnxt_proc_usb_host1;
    host_proc_file->owner = THIS_MODULE;
    host_proc_file->mode = S_IFREG | S_IRUGO;
    host_proc_file->uid = 0;
    host_proc_file->gid = 0;
    host_proc_file->size = 100;

    host_proc_file = create_proc_entry("usb_host2", 0644, NULL);
    if (host_proc_file == NULL) {
             remove_proc_entry("usb_host2", &proc_root);
             printk(KERN_ALERT "Error: Could not initialize /proc/%s\n","usb_host2");
             return;
    }
    host_proc_file->read_proc = cnxt_proc_usb_host2;
    host_proc_file->owner = THIS_MODULE;
    host_proc_file->mode = S_IFREG | S_IRUGO;
    host_proc_file->uid = 0;
    host_proc_file->gid = 0;
    host_proc_file->size = 100;



}
void cnxt_usb_host_remove_proc_entry()
{

    remove_proc_entry("usb_host1", &proc_root);
    printk(KERN_INFO "/proc/%s removed\n", "usb_host1");
    remove_proc_entry("usb_host2", &proc_root);
    printk(KERN_INFO "/proc/%s removed\n", "usb_host2");
}

EXPORT_SYMBOL(cnxt_proc_usb_host1);
EXPORT_SYMBOL(cnxt_proc_usb_host2);
EXPORT_SYMBOL(cnxt_usb_host_create_proc_entry);
EXPORT_SYMBOL(cnxt_usb_host_remove_proc_entry);

#endif

/**************************************************************************************
* Modifications:
* $Logs$
*
***************************************************************************************/

