/*
 * cnxt_otg_transiever.h - Conexant USB OTG transiever driver
 *
 * Copyright (C) 2007 Conexant Systems Inc
 * Copyright (C) 2007 P S Senthil Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#undef	DEBUG
#undef	VERBOSE

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/usb_ch9.h>
#include <linux/usb_gadget.h>
#include <linux/usb.h>
#include <linux/usb_otg.h>
#include <linux/workqueue.h>

//#include <asm/irq.h>
//#include <asm/arch/usb.h>

#ifdef CONFIG_ARCH_PECOS
#include "asm/arch/platform.h"
#endif

#ifdef CONFIG_ARCH_NEVIS
#include "asm/arch/platform.h"
#endif
#include <asm/mach-types.h>


#ifndef	DEBUG
#undef	VERBOSE
#endif


#define	DRIVER_VERSION	"24 November 2008"

MODULE_DESCRIPTION("CNXT  USB OTG Transceiver Driver");
MODULE_LICENSE("GPL");

enum usb_otg_state_internal {
	OTG_STATE_INTERNAL_UNDEFINED = 0,
	OTG_STATE_INTERNAL_B_SESSION_END,
	OTG_STATE_INTERNAL_B_SESSION_VALID,
	OTG_STATE_INTERNAL_A_SESSION_VALID,
	OTG_STATE_INTERNAL_A_ABUS_VALID,
	OTG_STATE_INTERNAL_A_VBUS_END
};


struct cnxt_otg_trans {
	struct otg_transceiver	otg;
	
	int			irq;

	u32			last_otg_ctrl;
	unsigned		working:1;

	struct timer_list	timer;

	/* use keventd context to change the state for us */
	struct work_struct	work;
	
	unsigned long		todo;
#		define WORK_UPDATE_OTG	1	/* update OTG  */
#		define WORK_HOST_RESUME	4	/* resume host */
#		define WORK_STOP	7	/* don't resubmit */

	u32  otg_controller_id;
	enum usb_otg_state_internal	state_internal;
};


#define OTGBASE      0xE8000000

#define OTG_BASE(i)			(OTGBASE +  ( (i * 1000) + 0x01A4))

#define OTG_REG(i)		(OTG_BASE(i))

#define OTG_DPIE_IE	 (1 << 30)
#define OTG_1MS_IE	 (1 << 29)
#define OTG_BSE_IE	 (1 << 28)
#define OTG_BSV_IE	 (1 << 27)
#define OTG_ASV_IE	 (1 << 26)
#define OTG_AVV_IE	 (1 << 25)
#define OTG_ID_IE	 (1 << 24)


#define OTG_DP_IS     (1 << 22)
#define OTG_BSE_IS     (1 << 20)
#define OTG_BSV_IS     (1 << 19)
#define OTG_ASV_IS     (1 << 18)
#define OTG_AVV_IS     (1 << 17)
#define OTG_ID_IS	 (1 << 16)
#define OTG_DPS		 (1 << 14)
#define OTG_BSE		 (1 << 12)
 
#define OTG_ASV		 (1 << 10)
#define OTG_AVV		 (1 << 9)
#define OTG_ID		 (1 << 8)
#define OTG_HABA	 (1 << 7)
#define OTG_HADP	 (1 << 6)
#define OTG_IDPU	 (1 << 5)
#define OTG_DP		 (1 << 4)
#define OTG_VC		 (1 << 1)
#define OTG_VD		 (1 << 0)

#define IRQ_USB_OTG0	13
#define IRQ_USB_OTG1    17

#define OTG_ID_0		0
#define OTG_ID_1		1



#define OTG_TRANSIEVER_ID_0  0
#define OTG_TRANSIEVER_ID_1  1

//#include <asm/arch/gpio.h>
//#include <asm/arch/mux.h>
