/****************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  */
/*                       SOFTWARE FILE/MODULE HEADER                        */
/*          Conexant Systems Inc. (c) 1998, 1999, 2000, 2001, 2002          */
/*                              Austin, TX                                  */
/*                         All Rights Reserved                              */
/****************************************************************************/
/*
 * Filename:        hwconfig.h
 *
 *
 * Description:     Public header file containing labels for all the
 *                  hardware configuration values passed into code
 *                  during compilation using the config file mechanism
 *
 *
 * Author:          Dave Wilson
 *
 ****************************************************************************/
/* $Header$
 ****************************************************************************/

#ifndef __STBCFG_H__
   /* Should generate a "warning" here, but don't recall how */
   #include "stbcfg.h"
#endif   /*#ifndef __STBCFG_H__*/

#ifndef  __HWCONFIG_H__
#define  __HWCONFIG_H__

/********************************************************************************/
/********************************************************************************/
/**	                                                                           **/
/** It is ABSOLUTELY VITAL that definitions in this file match definitions of  **/
/** the same label in HWCONFIG.VXA and HWCONFIG.A!!!!!! If this is not the     **/
/** case, build inconsistencies will occur.                                    **/
/**                                                                            **/
/********************************************************************************/
/********************************************************************************/


/****************************************************************************/
/*                                                                          */
/* This file contains definitions for all the strings that may be passed in */
/* Sabine build process configuration files. Labels are passed to source    */
/* files using -D<label>=<value> command line switches and source files     */
/* may then use these labels to decide which sections of code or parameters */
/* are required for a given build, for example, using the #if macro as in:  */
/*                                                                          */
/* #if <label> == <value>                                                   */
/*                                                                          */
/* By defining macros for all the possible <value>s, these lines can be     */
/* made more readable, for example:                                         */
/*                                                                          */
/* #if RTOS == PSOS                                                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* NOTE:                                                                    */
/*                                                                          */
/* Please DO NOT use double-slash comments in this file. It is used by some */
/* OpenTV o-code applications and the o-code compiler falls over whenever   */
/* it hits a non-standard comment.                                          */
/*                                                                          */
/****************************************************************************/

#include "warnfix.h"

/****************************************************************************/
/*                                                                          */
/* Hardware register access macros                                          */
/*                                                                          */
/****************************************************************************/

#define RMO(y)                   ( ((y) & 0x00000001) ?  0 : \
                                   ( ((y) & 0x00000002) ?  1 : \
                                     ( ((y) & 0x00000004) ?  2 : \
                                       ( ((y) & 0x00000008) ?  3 : \
                                         ( ((y) & 0x00000010) ?  4 : \
                                           ( ((y) & 0x00000020) ?  5 : \
                                             ( ((y) & 0x00000040) ?  6 : \
                                               ( ((y) & 0x00000080) ?  7 : \
                                                 ( ((y) & 0x00000100) ?  8 : \
                                                   ( ((y) & 0x00000200) ?  9 : \
                                                     ( ((y) & 0x00000400) ? 10 : \
                                                       ( ((y) & 0x00000800) ? 11 : \
                                                         ( ((y) & 0x00001000) ? 12 : \
                                                           ( ((y) & 0x00002000) ? 13 : \
                                                             ( ((y) & 0x00004000) ? 14 : \
                                                               ( ((y) & 0x00008000) ? 15 : \
                                                                 ( ((y) & 0x00010000) ? 16 : \
                                                                   ( ((y) & 0x00020000) ? 17 : \
                                                                     ( ((y) & 0x00040000) ? 18 : \
                                                                       ( ((y) & 0x00080000) ? 19 : \
                                                                         ( ((y) & 0x00100000) ? 20 : \
                                                                           ( ((y) & 0x00200000) ? 21 : \
                                                                             ( ((y) & 0x00400000) ? 22 : \
                                                                               ( ((y) & 0x00800000) ? 23 : \
                                                                                 ( ((y) & 0x01000000) ? 24 : \
                                                                                   ( ((y) & 0x02000000) ? 25 : \
                                                                                     ( ((y) & 0x04000000) ? 26 : \
                                                                                       ( ((y) & 0x08000000) ? 27 : \
                                                                                         ( ((y) & 0x10000000) ? 28 : \
                                                                                           ( ((y) & 0x20000000) ? 29 : \
                                                                                             ( ((y) & 0x40000000) ? 30 : \
                                                                                               ( ((y) & 0x80000000) ? 31 : 0 ))))))))))))))))))))))))))))))))

/*
 * Access macros used to get, set/clear bits within a hardware register.
 * These macros *do not* perform any automatic shifting of bits and are
 * meant to be used with bit definitions which include their encoded bit
 * position within the register definition (e.g. an enable bit).
 */
#define CNXT_GET(reg,mask)               (*(LPREG)(reg) & (mask))
#define CNXT_SET(reg,mask,val)           (*(LPREG)(reg)) =                     \
                                  ((*(LPREG)(reg) & ~(mask)) | ((val) & (mask)))

/*
 * Access macros used to get & set a numerical value within a hardware
 * register.  These macros perform automatic shifting (based on the mask)
 * of the numerical value used.  These macros are useful for setting a
 * numerical value into a multi-bit contiguous field within a register.
 */
#define CNXT_GET_VAL(reg,mask)           ((*(LPREG)(reg) & (mask)) >> RMO(mask))
#define CNXT_SET_VAL(reg,mask,val)       (*(LPREG)(reg)) =                     \
                   ((*(LPREG)(reg) & ~(mask)) | (((unsigned long)(val) << RMO(mask)) & (mask)))

typedef unsigned int    BIT_FLD;
typedef unsigned long   HW_DWORD;      /* was u_int32; */
typedef unsigned short  HW_WORD;       /* was u_int16; */
typedef unsigned char   HW_BYTE;       /* was u_int8; */
typedef unsigned int    HW_BOOL;       /* was bool; */
typedef void            HW_VOID;

/********************************************************************************/
/* The following header file, generated automatically from HWCONFIG.CFG during  */
/* the build process, contains definitions of all the values that can be        */
/* taken by all config file keys.                                               */
/********************************************************************************/
#include "hwopts.h"

/*********************************************************************/
/* Used for any run-time chip detection code (i.e. download utility) */
/*********************************************************************/
extern unsigned long ChipID;
#define ISCOLORADO     (((ChipID&0xFFF0FFFF)==(PCI_VEND_DEV_ID_COLO_4900 & 0xFFF0FFFF))?1:0)
#define ISHONDO        (((ChipID&0xFFF0FFFF)==(PCI_VEND_DEV_ID_HONDO_2492 & 0xFFF0FFFF))?1:0)
#define ISWABASH       (((ChipID&0xFFF0FFFF)==(PCI_VEND_DEV_ID_WABASH_4430 & 0xFFF0FFFF))?1:0)
#define ISBRAZOS       (((ChipID&0xFFF0FFFF)==(PCI_VEND_DEV_ID_BRAZOS & 0xFFF0FFFF))?1:0)

/***************************************/
/* Include Vendor Specific Definitions */
/* For runtime selection, all files    */
/* must be included.  If a compile-time*/
/* solution is used, specific vendor   */
/* files can be added or removed here. */
/***************************************/
#include <conexant.h>
#if CUSTOMER == VENDOR_A
#include <vendor_a.h>
#endif
#if CUSTOMER == VENDOR_B
#include <vendor_b.h>
#endif
#if CUSTOMER == VENDOR_C
#include <vendor_c.h>
#endif
#if CUSTOMER == VENDOR_D
#include <vendor_d.h>
#endif

/****************************************************************************/
/* Here, include the BOXCFG file, translated from the CONFIG file           */
/*                                                                          */
/* NOTE:  All FEATURE type definitions must have been defined before this   */
/*    file is included.  The only thing after this line should be the       */
/*    assignment of default values                                          */
/*                                                                          */
/****************************************************************************/
#include "boxcfg.h"

/****************************************************************************/
/* Here, all newly defined FEATURE (from CONFIG files) must be assigned a   */
/* default value.                                                           */
/*                                                                          */
/* NOTE:  All default values should be assigned such that the behavior for  */
/*    the specific feature will revert to Colorado/Klondike type behavior   */
/*                                                                          */
/* There MUST not be ANY chip-specific default values defined here.         */
/* They should ONLY be defined in each chip header file.                    */
/****************************************************************************/

/* This file is generated automatically from HWCONFIG.CFG during the */
/* build process.                                                    */
#include "hwdefault.h"

/************************************************************************/
/* Some helper macros that depend upon definitions from the main config */
/* file and the current hardware config file.                           */
/************************************************************************/
#define IRD_HAS_EXTERNAL_SAT_DEMOD ((EXTERNAL_TS_SRC0 == EXT_TS_SRC_SAT_24106_24110) ||  \
                                    (EXTERNAL_TS_SRC1 == EXT_TS_SRC_SAT_24106_24110) ||  \
                                    (EXTERNAL_TS_SRC2 == EXT_TS_SRC_SAT_24106_24110) ||  \
                                    (EXTERNAL_TS_SRC3 == EXT_TS_SRC_SAT_24106_24110) ||  \
                                    (EXTERNAL_TS_SRC0 == EXT_TS_SRC_SAT_24121_24130) ||  \
                                    (EXTERNAL_TS_SRC1 == EXT_TS_SRC_SAT_24121_24130) ||  \
                                    (EXTERNAL_TS_SRC2 == EXT_TS_SRC_SAT_24121_24130) ||  \
                                    (EXTERNAL_TS_SRC3 == EXT_TS_SRC_SAT_24121_24130))

#define IRD_HAS_INTERNAL_SAT_DEMOD ((EXTERNAL_TS_SRC0 == EXT_TS_SRC_SAT_CX2415X) ||  \
                                    (EXTERNAL_TS_SRC1 == EXT_TS_SRC_SAT_CX2415X) ||  \
                                    (EXTERNAL_TS_SRC2 == EXT_TS_SRC_SAT_CX2415X) ||  \
                                    (EXTERNAL_TS_SRC3 == EXT_TS_SRC_SAT_CX2415X))
    
#define IRD_HAS_SAT_DEMOD ( IRD_HAS_EXTERNAL_SAT_DEMOD | IRD_HAS_INTERNAL_SAT_DEMOD )

#define IRD_HAS_INTERNAL_CABLE_DEMOD  ((EXTERNAL_TS_SRC0 == EXT_TS_SRC_CAB_24430) ||  \
                                       (EXTERNAL_TS_SRC1 == EXT_TS_SRC_CAB_24430) ||  \
                                       (EXTERNAL_TS_SRC2 == EXT_TS_SRC_CAB_24430) ||  \
                                       (EXTERNAL_TS_SRC3 == EXT_TS_SRC_CAB_24430))
    
/* There are no external cable demods as of yet */
#define IRD_HAS_EXTERNAL_CABLE_DEMOD  0

#define IRD_HAS_CABLE_DEMOD   (IRD_HAS_INTERNAL_CABLE_DEMOD | IRD_HAS_EXTERNAL_CABLE_DEMOD)

#define IRD_HAS_TER_DEMOD ((EXTERNAL_TS_SRC0 == EXT_TS_SRC_TER_CX22702_TDLB7)   ||  \
                             (EXTERNAL_TS_SRC1 == EXT_TS_SRC_TER_CX22702_TDLB7) ||  \
                             (EXTERNAL_TS_SRC2 == EXT_TS_SRC_TER_CX22702_TDLB7) ||  \
                             (EXTERNAL_TS_SRC3 == EXT_TS_SRC_TER_CX22702_TDLB7) ||  \
                             (EXTERNAL_TS_SRC0 == EXT_TS_SRC_TER_CX22702_T7575) ||  \
                             (EXTERNAL_TS_SRC1 == EXT_TS_SRC_TER_CX22702_T7575) ||  \
                             (EXTERNAL_TS_SRC2 == EXT_TS_SRC_TER_CX22702_T7575) ||  \
                             (EXTERNAL_TS_SRC3 == EXT_TS_SRC_TER_CX22702_T7575) ||  \
                             (EXTERNAL_TS_SRC0 == EXT_TS_SRC_TER_CX22702_T7579) ||  \
                             (EXTERNAL_TS_SRC1 == EXT_TS_SRC_TER_CX22702_T7579) ||  \
                             (EXTERNAL_TS_SRC2 == EXT_TS_SRC_TER_CX22702_T7579) ||  \
                             (EXTERNAL_TS_SRC3 == EXT_TS_SRC_TER_CX22702_T7579) ||  \
                             (EXTERNAL_TS_SRC0 == EXT_TS_SRC_TER_CX22702_T7580) ||  \
                             (EXTERNAL_TS_SRC1 == EXT_TS_SRC_TER_CX22702_T7580) ||  \
                             (EXTERNAL_TS_SRC2 == EXT_TS_SRC_TER_CX22702_T7580) ||  \
                             (EXTERNAL_TS_SRC3 == EXT_TS_SRC_TER_CX22702_T7580))

#define IRD_HAS_DVB_DEMOD ((EXTERNAL_TS_SRC0 == EXT_TS_SRC_BASEBAND_DVB) ||  \
                             (EXTERNAL_TS_SRC1 == EXT_TS_SRC_BASEBAND_DVB) ||  \
                             (EXTERNAL_TS_SRC2 == EXT_TS_SRC_BASEBAND_DVB) ||  \
                             (EXTERNAL_TS_SRC3 == EXT_TS_SRC_BASEBAND_DVB))

#define  FRONT_PANEL_IS_SCANBTN  \
 ( (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_KLONDIKE) || \
   (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_ABILENE)  || \
   (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_ATHENS)   || \
   (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_BRONCO)   || \
   (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_EUREKA)   || \
   (FRONT_PANEL_KEYPAD_TYPE == FRONT_PANEL_KEYPAD_VEND_D_PROD_1) )

#if (EXTERNAL_ENCODER == DETECT_BT861_BT865) || (EXTERNAL_ENCODER == BT861) || (INTERNAL_ENCODER == INTERNAL_BT861_LIKE)
  #define VIDEO_ENCODER_INCLUDE_BT861
#endif

#if (EXTERNAL_ENCODER == DETECT_BT861_BT865) || (EXTERNAL_ENCODER == BT865)
  #define VIDEO_ENCODER_INCLUDE_BT865
#endif

/***************************************************************/
/* Include the automatically generated file which pulls in the */
/* appropriate chip header file.                               */
/*                                                             */
/* This MUST follow the box configuration file inclusion AND   */
/* the default configuration definitions.                      */
/*                                                             */
/* There MUST not be ANY chip-specific default values defined  */
/* here. They should ONLY be defined in each chip header file. */
/***************************************************************/
#include "chiphdr.h"

#endif   /* #ifndef  __HWCONFIG_H__ */

/****************************************************************************
 * $Log$
 * 
 *    Rev 1.162   19 Jun 2003 09:34:38   mitchei
 * SCR(s): 6773 
 * IRD_HAS_TER_DEMOD macro modified to check for new terrestrial nim types.
 * 
 *    Rev 1.161   25 Mar 2003 13:51:40   bradforw
 * SCR(s) 5864 :
 * Added EUREKA keypad type to list of FRONT_PANEL_IS_SCANBTN definition
 * 
 *    Rev 1.160   24 Jan 2003 08:21:56   joness
 * SCR(s): 5308 
 * Add useful VIDEO_ENCODER macros.
 * 
 *    Rev 1.159   21 Jan 2003 23:42:26   dawilson
 * SCR(s) 5099 :
 * Corrected ISBRAZOS and other chip identification macros.
 * 
 *    Rev 1.158   13 Jan 2003 13:57:02   bradforw
 * SCR(s) 5103 :
 * Added BRONCO front panel to list that will define FRONT_PANEL_IS_SCANBTN
 * 
 *    Rev 1.157   12 Dec 2002 17:15:48   whiteth
 * SCR(s) 5157 :
 * Added all run-time chip type macros which can be used for any run-time chip type
 * detection code (e.g. download).
 * 
 * 
 *    Rev 1.156   05 Dec 2002 16:11:12   bintzmf
 * SCR(s) 5074 :
 * changed int_ts_src to ext_ts_src
 * 
 * 
 *    Rev 1.155   05 Dec 2002 16:05:14   bintzmf
 * SCR(s) 5074 :
 * Added a few more macros to differentiate between internal and external demods
 * 
 * 
 *    Rev 1.154   20 Nov 2002 13:24:52   dawilson
 * SCR(s) 4904 :
 * Removed extraneous definitions of supported RTOSs which had been added to
 * debug a problem since fixed.
 * 
 *    Rev 1.153   20 Nov 2002 09:36:22   dawilson
 * SCR(s) 4904 :
 * Replaced previous header with a wrapper which pulls in the automatically
 * generated config files headers.
 * 
 *    Rev 1.152   06 Nov 2002 16:09:02   donaheb
 * SCR(s) 4912 :
 * Added "SMCD" task info for SC12 (8004) smart card driver
 * 
 * 
 *    Rev 1.151   06 Nov 2002 10:18:22   dawilson
 * SCR(s) 4896 :
 * Added Brazos options
 * 
 *    Rev 1.150   30 Oct 2002 13:37:32   bradforw
 * SCR(s) 4866 :
 * Add support for BURNET keypad type (single GPIO)
 * 
 *    Rev 1.149   22 Oct 2002 12:43:30   bintzmf
 * SCR(s) 4819 :
 * Added definitions for QAM Demod IF types
 * 
 * 
 *    Rev 1.148   18 Oct 2002 15:07:30   wangl2
 * SCR(s) 4811 :
 * Remove a space from the end of name strings for some tasks.
 * 
 *    Rev 1.147   11 Oct 2002 11:20:26   bintzmf
 * SCR(s) 4431 :
 * Added config option for hsx arbiter type
 * 
 * 
 *    Rev 1.146   17 Sep 2002 16:53:24   vancec
 * SCR(s) 4613 :
 * Added EXTERNAL_PCI_IO_MODE and it's possible values.
 * 
 *    Rev 1.145   12 Sep 2002 11:37:48   bradforw
 * SCR(s) 4111 :
 * Added default defintion of REDIRECTOR_PORT to the list of HWCONFIG values.  This value is used by the REDIRECTOR task (for modem test applications) to determine which serial port to connect/redirect the modem to.
 * 
 *    Rev 1.144   09 Sep 2002 15:09:42   vangulr
 * SCR(s) 4403 :
 * Raise priority of PCM tasks to be higher than cable modem.  Lower priority
 * was causing glitches with PCM audio when running in a release build
 * 
 * 
 *    Rev 1.143   30 Aug 2002 17:35:24   velusws
 * SCR(s) 4502 :
 * Defines for using the New IIC interface
 * 
 *    Rev 1.142   30 Aug 2002 14:01:00   wangl2
 * SCR(s) 4499 :
 * Add the definitions of INTEXP_WABASH, INTEXP_COLORADO and INTEXP_HONDO.
 * 
 *    Rev 1.141   22 Aug 2002 15:06:32   wangl2
 * SCR(s) 4459 :
 * Define PCI_ROM_MODE_REG_TYPE_COLORADO, PCI_ROM_MODE_REG_TYPE_HONDO and PCI_ROM_MODE_REG_TYPE_WABASH.
 * 
 *    Rev 1.140   15 Aug 2002 18:24:52   rossst
 * SCR(s) 4409 :
 * Removed inclusion of chip header from boxcfg and placed it at end of file after box default definitions so that CHIP_REV can bve used within the chip header. Also removed all defrault eature definitions for chip-specific features and placed them in the chip header files.
 * 
 *    Rev 1.139   15 Aug 2002 14:55:16   whiteth
 * SCR(s) 4398 :
 * Added Wabash Rev_B identifier.
 * 
 * 
 *    Rev 1.138   02 Aug 2002 14:50:42   velusws
 * SCR(s) 4315 :
 * Reduced NDS Smart card task's priority from above the NDS Verifier tasks to below the NDS Verifier tasks as that high a priority was not necessary. This task is still time critical. Should be kept above all the other "normal" system tasks. 
 * 
 *    Rev 1.137   24 Jul 2002 14:52:04   velusws
 * SCR(s) 3280 :
 * Increased the NDS task priorities to be above the NDS Verifier tasks which are now > 218
 * 
 *    Rev 1.136   22 Jul 2002 17:07:42   velusws
 * SCR(s) 4253 :
 * Increased NDS Smart Card task's priority to be near the other NDS tasks
 * 
 *    Rev 1.135   16 Jul 2002 18:47:48   bintzmf
 * SCR(s) 4209 :
 * Added IRD_HAS_DVB_DEMOD macro
 * 
 * 
 *    Rev 1.134   12 Jul 2002 07:17:02   joness
 * SCR(s): 4176 
 * Implement / use config for Brady and other boxes with a terrestrial NIM.
 *
 *    Rev 1.133   01 Jul 2002 15:57:10   mooreda
 * SCR(s) 3889 :
 * Added additional Cable Modem task, lowered priority of CMCT task.
 * 
 * 
 *    Rev 1.132   01 Jul 2002 15:11:58   whiteth
 * SCR(s) 4103 :
 * Hondo video ucode does not support bit 18 in the mpeg video size register waiting
 * for the 2nd frame to decode.  Therefore on Hondo, we simply proceed without waiting
 * in WaitForLiveVideoToStart() with bInSync == TRUE.
 * 
 * 
 *    Rev 1.131   21 Jun 2002 14:02:08   mooreda
 * SCR(s) 4069 :
 * Added Cable Modem Tasks
 *
 *
 *    Rev 1.130   21 Jun 2002 10:52:54   kortemw
 * SCR(s) 4072 :
 * Bump priority of PCMM and SSRC tasks to 1 above default.
 *
 *
 *    Rev 1.129   21 Jun 2002 10:27:08   dawilson
 * SCR(s) 4061 :
 * Added default for chip feature DRM_FILTER_PHASE_0_PROBLEM
 *
 *    Rev 1.128   18 Jun 2002 11:00:00   joness
 * SCR(s): 3960
 * Update Demod API.
 *
 *    Rev 1.123   06 Jun 2002 16:46:36   dawilson
 * SCR(s) 3952 :
 * Moved new/old remote selection flags from here to CONFMGR.H
 *
 *    Rev 1.122   16 May 2002 14:58:24   glennon
 * SCR(s): 3807
 * Changed priority of SSRC and PCMM "PCM Audio"
 * tasks back to default. This should allow tuning to work now.
 *
 *
 *    Rev 1.121   15 May 2002 02:18:44   glennon
 * SCR(s): 2438
 * Added another set of task ID's etc. for PCM Mixing task. Adjusted the
 * priorities of PCMM and SSRC to be above default to ensure no audio breakup.
 *
 *
 *    Rev 1.120   14 May 2002 11:12:48   wangl2
 * SCR(s) 3585 1238924 :
 * Move the definitions of GPIO_EXTEND_IIC and GPIO_EXTEND_IO out of the condition.  Change the value of GPIO_EXTEND_IO from 0 to 2.
 *
 *    Rev 1.119   07 May 2002 16:09:36   whiteth
 * SCR(s) 3721 :
 * Remove CNXT_REG_SET & CNXT_REG_GET macros.
 * Added (unsigned long) cast in the CNXT_SET_VAL macro.
 *
 *
 *    Rev 1.118   01 May 2002 14:30:54   whiteth
 * SCR(s) 3673 :
 * Remove PLL_ bitfields usage from codebase.
 *
 *
 *    Rev 1.117   26 Apr 2002 14:20:10   whiteth
 * SCR(s) 3562 :
 * Add support for Colorado Rev_F
 *
 *
 *    Rev 1.116   26 Apr 2002 13:28:56   wangl2
 * SCR(s) 3638 :
 * Remove definition of CABLE_MODEM_TUNER_TYPE_NONE.
 *
 *    Rev 1.115   24 Apr 2002 23:43:58   rossst
 * SCR(s) 3619 :
 * Changed names of cable-modem tuner definitions, added a default definition, and a definition for no tuner type.
 *
 *    Rev 1.114   22 Apr 2002 14:11:50   whiteth
 * SCR(s) 3585 :
 * Add definition of NUM_OF_GPIO for COLORADO, HONDO and WABASH chips.  Define the default GPIO extender type in hwconfig.h
 *
 *    Rev 1.113   12 Apr 2002 18:50:10   whiteth
 * SCR(s) 3558 :
 * Add new hardware register access macro definitions.  One set of macros now...
 *
 *    Rev 1.112   11 Apr 2002 14:15:14   whiteth
 * SCR(s) 3543 :
 * Changed BITFIELD #define to be in the <chipname>.h header files since
 * nothing in these files can depend on config options from config files or
 * hw/swconfig.h
 *
 *
 *    Rev 1.111   10 Apr 2002 16:15:18   whiteth
 * SCR(s) 3509 :
 * Eradicate external bus (PCI) bitfield usage.
 *
 *
 *    Rev 1.110   03 Apr 2002 02:53:44   mitchei
 * SCR(s): 3363
 * Removed APP_RESERVED_PRIO_X from file because they are unused
 *
 *    Rev 1.109   02 Apr 2002 16:50:08   bintzmf
 * SCR(s) 3469 :
 * removed a reference to front_panel_keypad_zapata
 *
 *
 *    Rev 1.108   02 Apr 2002 16:39:20   bintzmf
 * SCR(s) 3469 :
 * added defines for vendor_d's front panel
 *
 *
 *    Rev 1.107   02 Apr 2002 12:25:32   dawilson
 * SCR(s) 3488 :
 * Sorted out video encoder labels and added default for EXTERNAL_ENCODER.
 *
 *    Rev 1.106   27 Mar 2002 15:43:24   dawilson
 * SCR(s) 3460 :
 * Fixed problem in IRD_HAS_SAT_DEMOD macro.
 *
 *    Rev 1.105   26 Mar 2002 15:50:02   dawilson
 * SCR(s) 3447 :
 * Removed SATELLITE_DEMOD and renamed CABLE_DEVICE as CABLE_MODEM. Reworked
 * the existing EXT_TS_SRCn definitions to be more meaninful and added a couple
 * of macros to make it easier to determine if a given configuration has a
 * cable and/or satellite demod installed.
 *
 *    Rev 1.104   26 Mar 2002 08:43:12   dawilson
 * SCR(s) 3422 :
 * Ensured that information in C and assembler versions of HWCONFIG headers
 * is consistent and complete.
 *
 *    Rev 1.103   25 Mar 2002 10:28:00   whiteth
 * SCR(s) 3433 :
 * Removed NEW definition which may conflict with RTOS include files.
 *
 *
 *    Rev 1.102   15 Mar 2002 10:13:54   mitchei
 * SCR(s): 3329
 * I didn't actually make the last changes... I have now!
 *
 *
 *    Rev 1.100   15 Mar 2002 09:50:28   whiteth
 * SCR(s) 3352 :
 * Added "NEW" label for swconfig DMXVER.
 *
 *
 *    Rev 1.99   12 Mar 2002 17:49:00   whiteth
 * SCR(s) 3360 :
 * Use PLL_TYPE_COLORADO for Hondo since it has the same PLL setup as Colorado.
 *
 *
 *    Rev 1.98   11 Mar 2002 10:56:40   mitchei
 * SCR(s): 3342
 * added new defines
 *
 *
 *    Rev 1.97   08 Mar 2002 16:16:36   dawilson
 * SCR(s) 3290 :
 * Added task priority, stack size and name #defines for the 2 tasks created
 * by IOFUNCS.
 *
 *    Rev 1.96   08 Mar 2002 04:59:28   mitchei
 * SCR(s): 3330
 * Add definitions for the stack, name and priority to file
 *
 *    Rev 1.95   08 Mar 2002 04:31:04   mitchei
 * SCR(s): 3329
 * added APP_RESERVED_PRI_X's for p93test
 *
 *    Rev 1.94   06 Mar 2002 07:07:36   mitchei
 * SCR(s): 3312
 * changed the defined vale of APP_RESERVED_PRIO_3
 *
 *    Rev 1.93   05 Mar 2002 16:40:34   bintzmf
 * SCR(s) 3306 :
 * Added the default number of button columns and rows
 *
 *
 *    Rev 1.92   05 Mar 2002 07:33:22   banghag
 * SCR(s): 3301
 * ramoved UCOS definitions and placed into a header file ucosconf.h which is part of the UCOS BSP module.
 *
 *    Rev 1.91   04 Mar 2002 17:50:06   whiteth
 * SCR(s) 3299 :
 * Add ATA_TYPE definitions used in the chip header files.
 *
 *
 *    Rev 1.90   04 Mar 2002 11:11:42   jackmaw
 * SCR(s) 3237 :
 * Double the size of the teletext task stack to avoid reports of stack overflow.
 *
 *    Rev 1.89   28 Feb 2002 14:59:14   raymack
 * SCR(s) 3270 :
 * added default values for when the USB power control isn't specified.
 *
 *    Rev 1.88   28 Feb 2002 14:29:46   raymack
 * SCR(s) 3270 :
 * added USB power labels
 *
 *    Rev 1.87   28 Feb 2002 10:34:24   banghag
 * SCR(s): 3268
 * added APP_RESERVED_PRIO_X for applications to ppickup definitions.
 *
 *    Rev 1.85   Feb 25 2002 09:04:52   bradforw
 * SCR(s) 3201 :
 * Assorted changes to support the use of Software Configuration Files
 *
 ****************************************************************************/

