/****************************************************************************/
/*                    Conexant Systems Inc - CN8600                         */
/****************************************************************************/
/*                                                                          */
/* Filename:           INTID.H                                              */
/*                                                                          */
/* Description:        Public header file defining interrupt identitication */
/*                     macros and                                           */
/*                                                                          */
/* Author:             Miles Bintz                                          */
/*                                                                          */
/* Copyright Conexant Systems Inc, 2004                                     */
/* All Rights Reserved.                                                     */
/*                                                                          */
/****************************************************************************/
/* $ID:  */
/****************************************************************************/
#ifndef _INTID_H_
#define _INTID_H_

/**********************************************************************/
/* Use this macro to generate the dwIntID parameter to any of the     */
/* KAL interrupt service functions. Parameter x is either 0 or 1 and  */
/* indicates the PIC in use, parameter y is the bit number associated */
/* with the requested interrupt.                                      */
/**********************************************************************/

    #define PIC_0      0x00
    #define PIC_1      0x01
//    #define PIC_CENTRAL      0x00
    /* Assume all expansion bits are used */
//    #define PIC_EXPANSION    0x01
    /* Assume 32 timers even though brazos only has 8 and trinity has 16 */
    #define PIC_TIMERS       0x02
    /* Unfortunately, even though we only use ~6 allow the possibility
     * of 128 GPIO interrupts...
     */
    #define PIC_GPIO         0x03
    
    /* This is not a fool-proof macro.  GIGO... */
    #define IN_PIC_RANGE(x, y)   ( y == PIC_GPIO ? ((x > (PIC_GPIO*32)) && (x < 256)) : \
                                 ( (x > (y*32)) & (x < ((y+1)*32)) ) )
    #define CALC_INT_ID(x,y) ((x * 32) + (y))
    #define PIC_FROM_INT_ID(x) ( IN_PIC_RANGE(x, PIC_CENTRAL)   ? PIC_CENTRAL   : \
                                 IN_PIC_RANGE(x, PIC_EXPANSION) ? PIC_EXPANSION : \
                                 IN_PIC_RANGE(x, PIC_TIMERS)    ? PIC_TIMERS    : \
                                 IN_PIC_RANGE(x, PIC_GPIO)      ? PIC_GPIO      : \
                                 -1 /* holy shnikees! we're in trouble now! */    )
    #define INT_FROM_INT_ID(x) ( x - (32 * PIC_FROM_INT_ID(x)) )

/************************/
/* Useful Interrupt IDs */
/************************/

/* Main interrupt status register - sources common to all chips */
#define INT_UART1         CALC_INT_ID(ITC_UART1_PIC,       ITC_UART1_POS)
#define INT_UART2         CALC_INT_ID(ITC_UART2_PIC,       ITC_UART2_POS)
#define INT_UART3         CALC_INT_ID(ITC_UART3_PIC,       ITC_UART3_POS)
#define INT_SCD0        CALC_INT_ID(ITC_SCR0_PIC, ITC_SCR0_POS)
#define INT_SCD1        CALC_INT_ID(ITC_SCR1_PIC, ITC_SCR1_POS)
#define INT_SCD2        CALC_INT_ID(ITC_SCR2_PIC, ITC_SCR2_POS)
#define INT_IIC0        CALC_INT_ID(ITC_IIC0_PIC, ITC_IIC0_POS)
#define INT_IIC1        CALC_INT_ID(ITC_IIC1_PIC, ITC_IIC1_POS)
#define INT_SYNCSER     CALC_INT_ID(ITC_SYNCSERIAL_PIC, ITC_SYNCSERIAL_POS)
#define INT_IR0         CALC_INT_ID(ITC_IR0_PIC, ITC_IR0_POS)
#define INT_IR1         CALC_INT_ID(ITC_IR1_PIC, ITC_IR1_POS)
#define INT_PT0         CALC_INT_ID(ITC_PT0_PIC, ITC_PT0_POS)
#define INT_PT1         CALC_INT_ID(ITC_PT1_PIC, ITC_PT1_POS)
#define INT_USB         CALC_INT_ID(ITC_USB_PIC, ITC_USB_POS)
#define INT_GPIOS       CALC_INT_ID(ITC_GPIO_PIC, ITC_GPIO_POS)
#define INT_PWM         CALC_INT_ID(ITC_PWM_PIC, ITC_PWM_POS)
#define INT_HSUART0     CALC_INT_ID(ITC_HSUART0_PIC, ITC_HSUART0_POS)

#define INT_DBGCOMMTX   CALC_INT_ID(ITC_CMTX_PIC, ITC_CMTX_POS)
#define INT_DBGCOMMRX   CALC_INT_ID(ITC_CMRX_PIC, ITC_CMRX_POS)
#define INT_CRYPTO      CALC_INT_ID(ITC_CRYPTO_PIC, ITC_CRYPTO_POS)
#define INT_ATAPI       CALC_INT_ID(ITC_ATAPI_PIC, ITC_ATAPI_POS)
#define INT_DMA         CALC_INT_ID(ITC_DMA_PIC, ITC_DMA_POS)
#define INT_PCI         CALC_INT_ID(ITC_PCI_PIC, ITC_PCI_POS)
#define INT_DVT         CALC_INT_ID(ITC_DVT_PIC, ITC_DVT_POS)
#define INT_TIMERS      CALC_INT_ID(ITC_TIMER_PIC, ITC_TIMER_POS)
#define INT_AFE         CALC_INT_ID(ITC_AFE_PIC, ITC_AFE_POS)
#define INT_DAA         CALC_INT_ID(ITC_DAA_PIC, ITC_DAA_POS)
#define INT_MCX         CALC_INT_ID(ITC_MCX_PIC, ITC_MCX_POS)
#define INT_IMP         CALC_INT_ID(ITC_IMP_PIC, ITC_IMP_POS)
#define INT_HDCP        CALC_INT_ID(ITC_HDCP_PIC, ITC_HDCP_POS)
#define INT_TSS         CALC_INT_ID(ITC_TSS_PIC, ITC_TSS_POS)
#define INT_GXA         CALC_INT_ID(ITC_GXA_PIC, ITC_GXA_POS)
#define INT_DEINT       CALC_INT_ID(ITC_DEINT_PIC, ITC_DEINT_POS)
#define INT_DRM0        CALC_INT_ID(ITC_DRM0_PIC, ITC_DRM0_POS)
#define INT_DRM1        CALC_INT_ID(ITC_DRM1_PIC, ITC_DRM1_POS)
#define INT_SSP         CALC_INT_ID(ITC_SSP_PIC, ITC_SSP_POS)
#define INT_ICAM0       CALC_INT_ID(ITC_ICAM0_PIC, ITC_ICAM0_POS)
#define INT_ICAM1       CALC_INT_ID(ITC_ICAM1_PIC, ITC_ICAM1_POS)
#define INT_TSP0        CALC_INT_ID(ITC_TSP0_PIC, ITC_TSP0_POS)
#define INT_TSP1        CALC_INT_ID(ITC_TSP1_PIC, ITC_TSP1_POS)
#define INT_TSP2        CALC_INT_ID(ITC_TSP2_PIC, ITC_TSP2_POS)
#define INT_TSP3        CALC_INT_ID(ITC_TSP3_PIC, ITC_TSP3_POS)
#define INT_AMP         CALC_INT_ID(ITC_AMP_PIC, ITC_AMP_POS)
#define INT_APP         CALC_INT_ID(ITC_APP_PIC, ITC_APP_POS)
#define INT_PVC         CALC_INT_ID(ITC_PVC_PIC, ITC_PVC_POS)

/***************************/
/* Timer interrupt sources */
/***************************/
#define INT_SYSTIMER  CALC_INT_ID(PIC_TIMERS, SYS_TIMER)
#define INT_WATCHDOG  CALC_INT_ID(PIC_TIMERS, WATCHDOG_TIMER)
#define INT_TICKTIMER CALC_INT_ID(PIC_TIMERS, TICK_TIMER)
#define INT_TIMER(x)  CALC_INT_ID(PIC_TIMERS, (x))

/**************************/
/* GPIO interrupt sources */
/**************************/
#define INT_GPIO(x)  CALC_INT_ID(PIC_GPIO, (x))

//#define INT_INVALID  CALC_INT_ID(0, 33)


/* These are tied to GPIO's so they belong in a hwconfig file
 * but since linux doesn't have hwconfig files (yet), they 
 * will go here for now...
 */
//#define INT_IDE         INT_GPIO45
//#define INT_ETHERNET    INT_GPIO42

#endif /* _INTID_H_ */
/****************************************************************************
 * Modifications:
 * $Log$
 ****************************************************************************/

