#ifndef _MEMMAP_H_
#define _MEMMAP_H_

#define PHYS_RAM_SIZE (128UL * 1024 * 1024)
#define KERNEL_RAM_SIZE (64UL * 1024 * 1024)

/*
            Fixed Mappings
*/

   // Virtual Addresses 
#define FLASH_VADDR_BASE     0xF0000000  /* FC00.0000 - FCFF.FFFF */
#define FLASHNC_VADDR_BASE   0xF8000000  /* FC00.0000 - FCFF.FFFF */
#define ASX_VADDR_BASE       0xE0000000  /* E000.0000 - E07F.FFFF */
#define EXT_IO_VADDR_BASE    0xE1000000  /* E100.0000 - E17F.FFFF */
#define TEMPEST_VADDR_BASE   0xE4000000  /* E400.0000 - E407.FFFF */
#define PCI_MEM_VADDR_BASE   0x80000000  /* 8000.0000 - 9FFF.FFFF */

/* TODO:  REVISIT:  How can we do this without hard-coding numbers?
 * num_phys_pages is calculated at runtime based on PAGE_SIZE (4k on arm)
 * and bootline parameter "RAM"... Furthermore, we don't know what the
 * physical amount of RAM is at compile time.  Need configuration hooks...
 */

#define IO_BASE              (ASX_VADDR_BASE)

#define IO_ADDRESS(x)        (((x) & 0x00ffffff) | IO_BASE)

   // Mapping Range 
#define FLASH_SIZE           0x02000000
#define ASX_IO_SIZE          0x00800000  
#define EXT_IO_SIZE          0x00800000 /* EXT IO ecompases PCI IO and ISA IO */
#define TEMPEST_IO_SIZE      0x00800000
#define PCI_MEM_SIZE         0x08000000
#define PCI_IO_SIZE          0x00100000

#define KAL_MEM_SIZE         (PHYS_RAM_SIZE - KERNEL_RAM_SIZE)

// Physical Addresses 
#define FLASH_PHYS_BASE      0xF0000000
#define ASX_PHYS_BASE        0xE0000000
#define EXT_IO_PHYS_BASE     0xE1000000 /* Include PCI IO and ISA IO */
#define TEMPEST_PHYS_BASE    0xE4000000
#define PCI_MEM_PHYS_BASE    0x80000000
#define KAL_MEM_PHYS_BASE    (KERNEL_RAM_SIZE)

#define KAL_PHYSICAL_ADDRESS(ptr) ((u_int32)(ptr) & 0x07ffffff)
#endif

