/******************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                    */
/*                        SOFTWARE FILE/MODULE HEADER                         */
/*                 Copyright Conexant Systems Inc. 2004-2006                  */
/*                                 Austin, TX                                 */
/*                            All Rights Reserved                             */
/******************************************************************************/
/*
 * Filename:        TRINITY.H
 *
 *
 * Description:     Public header file defining hardware-specific values
 *                  (such as register addresses, bit definitions, etc.)
 *                  for Trinity.
 *
 *
 * Author:          Dave Wilson
 *
 ******************************************************************************/
/* $Id: trinity.h,v 1.4, 2007-10-10 09:25:31Z, Jaychander Thirunavukkarasu$
 ******************************************************************************/

#ifndef _TRINITY_H_
#define _TRINITY_H_


/*********************************/
/* General Types used everywhere */
/*********************************/
#ifndef __ASSEMBLY__
   typedef unsigned long HW_DWORD;
   typedef unsigned short HW_WORD;
   typedef unsigned char HW_BYTE;
   typedef volatile HW_DWORD *LPREG;
#endif /* __ASSEMBLY__ */

/************************/
/* Interrupt Controller */
/************************/

#define ITC_BASE       (ASX0_BASE + 0x50000)
#define ITC_BANK_SIZE  0x20
#define NUM_ITC_BANKS  2

/* itcNum is always 0 for Trinity, but can vary on Pecos.  To make the
   code compatible between Pecos and Trinity we add a dummy itcNum
   parameter to the macros below
*/
/* Interrupt Destination Register, x = 0~1 */
#define ITC_INTDEST_REG(itcNum ,x)    (ITC_BASE + (x)*ITC_BANK_SIZE + 0x00)

/* Interrupt Enable Register, x = 0~1 */
#define ITC_INTENABLE_REG(itcNum ,x)  (ITC_BASE + (x)*ITC_BANK_SIZE + 0x04)

/* Interrupt Request IRQ Register, x = 0~1 */
#define ITC_INTRIRQ_REG(itcNum ,x)    (ITC_BASE + (x)*ITC_BANK_SIZE + 0x08)

/* Interrupt Request FIQ Register, x = 0~1 */
#define ITC_INTRFIQ_REG(itcNum ,x)    (ITC_BASE + (x)*ITC_BANK_SIZE + 0x0C)

/* Interrupt Status/Clear Register, x = 0~1 */
#define ITC_INTSTATCLR_REG(itcNum, x) (ITC_BASE + (x)*ITC_BANK_SIZE + 0x10)

/* Interrupt Status/Set Register, x= 0~1 */
#define ITC_INTSTATSET_REG(itcNum, x) (ITC_BASE + (x)*ITC_BANK_SIZE + 0x14)

/* ITC0 status register */
#define ITC_UART1                 0x00000001
#define ITC_UART1_POS                   0
#define ITC_UART1_PIC                (PIC_0)
#define ITC_UART2                 0x00000002
#define ITC_UART2_POS                   1
#define ITC_UART2_PIC                (PIC_0)
#define ITC_UART3                 0x00000004
#define ITC_UART3_POS                   2
#define ITC_UART3_PIC                (PIC_0)
#define ITC_PCI                   0x00000020
#define ITC_PCI_POS                     5
#define ITC_PCI_PIC                  (PIC_1)
#define ITC_TIMER                 0x00000080
#define ITC_TIMER_POS                   7
#define ITC_TIMER_PIC                (PIC_1)
#define ITC_USB0                  0x00002000
#define ITC_USB0_POS                   13
#define ITC_USB0_PIC                 (PIC_0)
#define ITC_GPIO                  0x00004000
#define ITC_GPIO_POS                   14
#define ITC_GPIO_PIC                 (PIC_0)
#define ITC_USB1                  0x00020000
#define ITC_USB1_POS                   17
#define ITC_USB1_PIC                 (PIC_0)


#define HSX_BASE                0xE0000000
#define ASX0_BASE               0xE0400000
#define ASX1_BASE               0xE0500000

#define RST_BASE (ASX0_BASE + 0)
#define RST_SOFTRESET_REG                                      (RST_BASE + 0x1C)


#define SER_BASE                (ASX0_BASE + 0x10000)
#define HSUART_BASE             (HSX_BASE  + 0x40000)
/* TODO: Change to 4 when HSUART support is added to LLSER */
#define SER_NUM_PORTS           3
/* #define NUM_SER_BANKS           3 seems redundant */
#define SER_BANK_SIZE           0x1000
#define SER_BASE_UNIT(n)        ((n == 3) ? (HSUART_BASE) : (SER_BASE+SER_BANK_SIZE*(n)))
#define SER_BASE_UART1          (SER_BASE_UNIT(0))
#define SER_BASE_UART2          (SER_BASE_UNIT(1))
#define SER_BASE_UART3          (SER_BASE_UNIT(2))
#define SER_BASE_HSUART1        (SER_BASE_UNIT(3))

#define SER_NUM_BASE(x)                 ( (x == 3) ? (HSUART_BASE) : (SER_BASE + (x)*SER_BANK_SIZE) )

/* UART FIFO Register */
#define SER_DATA_REG                                   0x00000000
#define SER_NUM_DATA_REG(x)             (SER_DATA_REG           + (SER_NUM_BASE(x)))
#define     SER_DATA_DATA_MASK                           0x000000FF
#define     SER_DATA_DATA_SHIFT                                 0
/* HSUART */
#define     SER_HSUART_DATA_DATA_MASK                    0xFFFFFFFF
#define     SER_HSUART_DATA_DATA_SHIFT                          0

/* UART Low Baud Rate Divisor Register */
#define SER_BRD_LOW_REG                                0x00000000
#define SER_NUM_BRD_LOW_REG(x)          (SER_BRD_LOW_REG        + (SER_NUM_BASE(x)))
#define     SER_BRD_LOW_VALUE_MASK                     0x000000FF
#define     SER_BRD_LOW_VALUE_SHIFT                           0
/* HSUART */
#define     SER_HSUART_BRD_LOW_VALUE_MASK                0x000000FF
#define     SER_HSUART_BRD_LOW_VALUE_SHIFT                      0


/* UART Interrupt Enable Register */
#define SER_INT_ENABLE_REG                             0x00000004
#define SER_NUM_INT_ENABLE_REG(x)       (SER_INT_ENABLE_REG     + (SER_NUM_BASE(x)))
#define     SER_INT_ENABLE_RXFIFO_MASK                 0x00000001
#define     SER_INT_ENABLE_RXFIFO_SHIFT                       0
#define        SER_INT_ENABLE_RXFIFO_DISABLE               (0UL<<0)
#define        SER_INT_ENABLE_RXFIFO_ENABLE                (1UL<<0)
#define     SER_INT_ENABLE_RXOVERRUN_MASK              0x00000002
#define     SER_INT_ENABLE_RXOVERRUN_SHIFT                    1
#define        SER_INT_ENABLE_RXOVERRUN_DISABLE          (0UL<<1)
#define        SER_INT_ENABLE_RXOVERRUN_ENABLE           (1UL<<1)
#define     SER_INT_ENABLE_PARITYERROR_MASK            0x00000004
#define     SER_INT_ENABLE_PARITYERROR_SHIFT                  2
#define        SER_INT_ENABLE_PARITYERROR_DISABLE        (0UL<<2)
#define        SER_INT_ENABLE_PARITYERROR_ENABLE         (1UL<<2)
#define     SER_INT_ENABLE_FRAMINGERROR_MASK           0x00000008
#define     SER_INT_ENABLE_FRAMINGERROR_SHIFT                 3
#define        SER_INT_ENABLE_FRAMINGERROR_DISABLE        (0UL<<3)
#define        SER_INT_ENABLE_FRAMINGERROR_ENABLE         (1UL<<3)
#define     SER_INT_ENABLE_BREAKRECEIVED_MASK          0x00000010
#define     SER_INT_ENABLE_BREAKRECEIVED_SHIFT                4
#define        SER_INT_ENABLE_BREAKRECEIVED_DISABLE      (0UL<<4)
#define        SER_INT_ENABLE_BREAKRECEIVED_ENABLE       (1UL<<4)
#define     SER_INT_ENABLE_TXFIFO_MASK                 0x00000020
#define     SER_INT_ENABLE_TXFIFO_SHIFT                       5
#define        SER_INT_ENABLE_TXFIFO_DISABLE             (0UL<<5)
#define        SER_INT_ENABLE_TXFIFO_ENABLE              (1UL<<5)
#define     SER_INT_ENABLE_TXIDLE_MASK                 0x00000040
#define     SER_INT_ENABLE_TXIDLE_SHIFT                       6
#define        SER_INT_ENABLE_TXIDLE_DISABLE             (0UL<<6)
#define        SER_INT_ENABLE_TXIDLE_ENABLE              (1UL<<6)

#define     SER_INT_ENABLE_RXFIFOERROR_MASK            0x00000080
#define     SER_INT_ENABLE_RXFIFOERROR_SHIFT                  7
#define        SER_INT_ENABLE_RXFIFOERROR_DISABLE        (0UL<<7)
#define        SER_INT_ENABLE_RXFIFOERROR_ENABLE         (1UL<<7)


/* UART Upper Baud Rate Divisor Register */
#define SER_BRD_HIGH_REG                               0x00000004
#define SER_NUM_BRD_HIGH_REG(x)         (SER_BRD_HIGH_REG       + (SER_NUM_BASE(x)))
#define     SER_BRD_HIGH_VALUE_MASK                    0x000000FF
#define     SER_BRD_HIGH_VALUE_SHIFT                          0

/* UART FIFO Control Register */
#define SER_FIFO_CTRL_REG                              0x00000008
#define SER_NUM_FIFO_CTRL_REG(x)        (SER_FIFO_CTRL_REG      + (SER_NUM_BASE(x)))
#define     SER_FIFO_CTRL_RXFIFOCLEAR_MASK             0x00000002
#define     SER_FIFO_CTRL_RXFIFOCLEAR_SHIFT                   1
#define       SER_FIFO_CTRL_RXFIFOCLEAR                   (1UL<<1)
#define     SER_FIFO_CTRL_TXFIFOCLEAR_MASK             0x00000004
#define     SER_FIFO_CTRL_TXFIFOCLEAR_SHIFT                   2
#define        SER_FIFO_CTRL_TXFIFOCLEAR                   (1UL<<2)
#define     SER_FIFO_CTRL_TXFIFOTHRESHOLD_MASK         0x00000030
#define     SER_FIFO_CTRL_TXFIFOTHRESHOLD_SHIFT               4
#define        SER_TXTHRESH_25                             (0UL<<4)
#define        SER_TXTHRESH_50                             (1UL<<4)
#define        SER_TXTHRESH_75                             (2UL<<4)
#define        SER_TXTHRESH_EMPTY                          (3UL<<4)
#define     SER_FIFO_CTRL_RXFIFOTHRESHOLD_MASK         0x000000C0
#define     SER_FIFO_CTRL_RXFIFOTHRESHOLD_SHIFT               6
#define        SER_RXTHRESH_CHAR                           (0UL<<6)
#define        SER_RXTHRESH_25                             (1UL<<6)
#define        SER_RXTHRESH_50                             (2UL<<6)
#define        SER_RXTHRESH_75                             (3UL<<6)

/* UART Frame Control Register */
#define SER_FRAME_CTRL_REG                             0x0000000C
#define SER_NUM_FRAME_CTRL_REG(x)       (SER_FRAME_CTRL_REG     + (SER_NUM_BASE(x)))
#define     SER_FRAME_CTRL_DATA8BIT_MASK               0x00000001
#define     SER_FRAME_CTRL_DATA8BIT_SHIFT                     0
#define        SER_FRAME_CTRL_DATA7BIT                     (0UL<<0)
#define        SER_FRAME_CTRL_DATA8BIT                     (1UL<<0)
#define     SER_FRAME_CTRL_LOOPBACK_MASK               0x00000002
#define     SER_FRAME_CTRL_LOOPBACK_SHIFT                     1
#define        SER_FRAME_CTRL_LOOPBACK_ENA               (1UL<<1)
#define     SER_FRAME_CTRL_TX2STOPBITS_MASK            0x00000004
#define     SER_FRAME_CTRL_TX2STOPBITS_SHIFT                  2
#define        SER_FRAME_CTRL_TX1STOPBITS                  (0UL<<2)
#define        SER_FRAME_CTRL_TX2STOPBITS                  (1UL<<2)
#define     SER_FRAME_CTRL_PARITYCHECK_MASK            0x00000008
#define     SER_FRAME_CTRL_PARITYCHECK_SHIFT                  3
#define        SER_FRAME_CTRL_PARITYCHECK_DISABLE        (0UL<<3)
#define        SER_FRAME_CTRL_PARITYCHECK_ENABLE         (1UL<<3)
#define     SER_FRAME_CTRL_EVENPARITY_MASK             0x00000010
#define     SER_FRAME_CTRL_EVENPARITY_SHIFT                   4
#define        SER_FRAME_CTRL_ODDPARITY                    (0UL<<4)
#define        SER_FRAME_CTRL_EVENPARITY                   (1UL<<4)
#define        SER_FRAME_CTRL_DISABLE_PARITY               (0UL<<4)
#define     SER_FRAME_CTRL_PARITYOVERRIDE_MASK         0x00000020
#define     SER_FRAME_CTRL_PARITYOVERRIDE_SHIFT               5
#define        SER_FRAME_CTRL_PARITYOVERRIDE_DISABLE       (0UL<<5)
#define        SER_FRAME_CTRL_PARITYOVERRIDE_ENABLE        (1UL<<5)
#define     SER_FRAME_CTRL_TXBREAK_MASK                0x00000040
#define     SER_FRAME_CTRL_TXBREAK_SHIFT                      6
#define        SER_FRAME_CTRL_TXBREAK_DISABLE            (0UL<<6)
#define        SER_FRAME_CTRL_TXBREAK_ENABLE             (1UL<<6)
#define     SER_FRAME_CTRL_BAUDDIVISORSELECT_MASK      0x00000080
#define     SER_FRAME_CTRL_BAUDDIVISORSELECT_SHIFT            7
#define        SER_FRAME_CTRL_BAUDDIVISORSELECT_DISABLE  (0UL<<7)
#define        SER_FRAME_CTRL_BAUDDIVISORSELECT_ENABLE   (1UL<<7)


/* UART Status Register */
#define SER_STATUS_REG                                 0x00000014
#define SER_NUM_STATUS_REG(x)           (SER_STATUS_REG         + (SER_NUM_BASE(x)))
#define     SER_STATUS_RXFIFOSERVICE_MASK              0x00000001
#define     SER_STATUS_RXFIFOSERVICE_SHIFT                    0
#define        SER_STATUS_RXFIFOSERVICE_NO               (0UL<<0)
#define        SER_STATUS_RXFIFOSERVICE_YES              (1UL<<0)
#define     SER_STATUS_RXFIFOOVERRUN_MASK              0x00000002
#define     SER_STATUS_RXFIFOOVERRUN_SHIFT                    1
#define        SER_STATUS_RXFIFOOVERRUN_NO               (0UL<<1)
#define        SER_STATUS_RXFIFOOVERRUN_YES              (1UL<<1)
#define     SER_STATUS_PARITYERROR_MASK                0x00000004
#define     SER_STATUS_PARITYERROR_SHIFT                      2
#define        SER_STATUS_PARITYERROR_NO                 (0UL<<2)
#define        SER_STATUS_PARITYERROR_YES                (1UL<<2)
#define     SER_STATUS_FRAMINGERROR_MASK               0x00000008
#define     SER_STATUS_FRAMINGERROR_SHIFT                     3
#define        SER_STATUS_FRAMINGERROR_NO                (0UL<<3)
#define        SER_STATUS_FRAMINGERROR_YES               (1UL<<3)
#define     SER_STATUS_BREAKRECEIVED_MASK              0x00000010
#define     SER_STATUS_BREAKRECEIVED_SHIFT                    4
#define        SER_STATUS_BREAKRECEIVED_NO               (0UL<<4)
#define        SER_STATUS_BREAKRECEIVED_YES              (1UL<<4)
#define     SER_STATUS_TXFIFOSERVICE_MASK              0x00000020
#define     SER_STATUS_TXFIFOSERVICE_SHIFT                    5
#define        SER_STATUS_TXFIFOSERVICE_NO               (0UL<<5)
#define        SER_STATUS_TXFIFOSERVICE_YES              (1UL<<5)
#define     SER_STATUS_TXIDLE_MASK                     0x00000040
#define     SER_STATUS_TXIDLE_SHIFT                           6
#define        SER_STATUS_TXIDLE_DATA                    (0UL<<6)
#define        SER_STATUS_TXIDLE_EMPTY                   (1UL<<6)
#define     SER_STATUS_RXERROR_MASK                    0x00000080
#define     SER_STATUS_RXERROR_SHIFT                          7
#define        SER_STATUS_RXERROR_NO                     (0UL<<7)
#define        SER_STATUS_RXERROR_YES                    (1UL<<7)

/* UART Interrupt Level Register */
#define SER_IRLVL_REG                                  0x00000018
#define SER_NUM_IRLVL_REG(x)            (SER_IRLVL_REG          + (SER_NUM_BASE(x)))
#define     SER_IRLVL_LEVEL_MASK                       0x0000000F
#define     SER_IRLVL_LEVEL_SHIFT                             0
#define        SER_IRLVL_LEVEL_NO_ACT                    (1UL<<0)
#define        SER_IRLVL_LEVEL_OPFB                      (6UL<<0)
#define        SER_IRLVL_LEVEL_RFSR                      (4UL<<0)
#define        SER_IRLVL_LEVEL_IDLE                     (12UL<<0)
#define        SER_IRLVL_LEVEL_TFSR                      (2UL<<0)

/* UART IrDA Control Register */
#define SER_IRDA_CTRL_REG                              0x00000020
#define SER_NUM_IRDA_CTRL_REG(x)        (SER_IRDA_CTRL_REG      + (SER_NUM_BASE(x)))
#define     SER_IRDA_CTRL_ENABLE_MASK                  0x00000001
#define     SER_IRDA_CTRL_ENABLE_SHIFT                        0
#define        SER_IRDA_CTRL_DISABLE                       (0UL<<0)
#define        SER_IRDA_CTRL_ENABLE                        (1UL<<0)


/* UART Transimit FIFO Status Register */
#define SER_TX_FIFO_STATUS_REG                         0x00000028
#define SER_NUM_TX_FIFO_STATUS_REG(x)   (SER_TX_FIFO_STATUS_REG + (SER_NUM_BASE(x)))




#define     SER_FIFO_STATUS_NUMDATA_MASK               0x0000001F
#define     SER_FIFO_STATUS_NUMDATA_SHIFT                     0

/* UART Expansion Register */
#define SER_EXP_REG                                    0x0000002C
#define SER_NUM_EXP_REG(x)              (SER_EXP_REG + (SER_NUM_BASE(x)))
#define    SER_EXP_BAUD_RATE_DIVISOR_MASK              0x00000003
#define    SER_EXP_BAUD_RATE_DIVISOR_SHIFT                    0
#define       SER_EXP_RATE_DIVISOR_ZERO                  (0UL<<0)
#define       SER_EXP_RATE_DIVISOR_QUA                   (1UL<<0)
#define       SER_EXP_RATE_DIVISOR_HALF                  (2UL<<0)
#define       SER_EXP_RATE_DIVISOR_3QUA                  (3UL<<0)

/* UART Receive FIFO Status Register. This register has same address as
 * SER_NUM_EXP_REG(x) */
#define SER_RX_FIFO_STATUS_REG                         0x0000002C
#define SER_NUM_RX_FIFO_STATUS_REG(x)   (SER_RX_FIFO_STATUS_REG + (SER_NUM_BASE(x)))
#define    SER_RX_FIFO_DATA_NUM_MASK                   0x0000001F
#define    SER_RX_FIFO_DATA_NUM_SHIFT                         0

/* There is no register defined at this address in the spec */
#define SER_BRD_FRACTIONAL_REG                         0x00000030
#define SER_NUM_BRD_FRACTIONAL_REG(x)   (SER_BRD_FRACTIONAL_REG + (SER_NUM_BASE(x)))
#define     SER_BRD_FRACTIONAL_MASK                      0x00000003
#define     SER_BRD_FRACTIONAL_SHIFT                            0

/* UART Expansion Control Register, there is no register defined
 * at this address in the spec */
#define SER_EXT_CTRL_REG                               0x0000003C
#define SER_NUM_EXT_CRTL_REG(x)         ( (x != 3) ? (NULL) : (SER_EXT_CTRL_REG + SER_NUM_BASE(x)) )
#define     SER_EXT_CTRL_HSM_MASK                      0x00000080
#define     SER_EXT_CTRL_HSM_SHIFT                            7
#define        SER_EXT_CTRL_HSM_DIS                      (0UL<<7)
#define        SER_EXT_CTRL_HSM_ENA                      (1UL<<7)
#define     SER_EXT_CTRL_OSP_MASK                      0x00000040
#define     SER_EXT_CTRL_OSP_SHIFT                            6
#define        SER_EXT_CTRL_OSP_X16                      (0UL<<6)
#define        SER_EXT_CTRL_OSP_X8                       (1UL<<6)
#define     SER_EXT_CTRL_RTS_MASK                      0x00000020
#define     SER_EXT_CTRL_RTS_SHIFT                            5
#define        SER_EXT_CTRL_RTS_DIS                      (0UL<<5)
#define        SER_EXT_CTRL_RTS_ENA                      (1UL<<5)
#define     SER_EXT_CTRL_DMATX_MASK                    0x00000010
#define     SER_EXT_CTRL_DMATX_SHIFT                          4
#define        SER_EXT_CTRL_DMATX_DIS                    (0UL<<4)
#define        SER_EXT_CTRL_DMATX_ENA                    (1UL<<4)
#define     SER_EXT_CTRL_DMARX_MASK                    0x00000008
#define     SER_EXT_CTRL_DMARX_SHIFT                          3
#define        SER_EXT_CTRL_DMARX_DIS                    (0UL<<3)
#define        SER_EXT_CTRL_DMARX_ENA                    (1UL<<3)
#define     SER_EXT_CTRL_FCEN_MASK                     0x00000004
#define     SER_EXT_CTRL_FCEN_SHIFT                           2
#define        SER_EXT_CTRL_FCEN_DIS                     (0UL<<2)
#define        SER_EXT_CTRL_FCEN_ENA                     (1UL<<2)
#define     SER_EXT_CTRL_RXFC_MASK                     0x00000003
#define     SER_EXT_CTRL_RXFC_SHIFT                           0
#define        SER_EXT_CTRL_RXFC_1BYTE                   (0UL<<0)
#define        SER_EXT_CTRL_RXFC_4BYTES                  (1UL<<0)
#define        SER_EXT_CTRL_RXFC_8BYTES                  (2UL<<0)
#define        SER_EXT_CTRL_RXFC_14BYTES                 (3UL<<0)
/* Bit positions to use when writing whole register at once */
#define SER_INT_RX_FIFO     0x01
#define SER_INT_RX_OVERRUN  0x02
#define SER_INT_PARITY_ERR  0x04
#define SER_INT_FRAME_ERR   0x08
#define SER_INT_BREAK       0x10
#define SER_INT_TX_FIFO     0x20
#define SER_INT_TX_IDLE     0x40
#define SER_INT_RX_FIFO_ERR 0x80

#define SER_FRAME_CTRL_PARITY_EVEN  1
#define SER_FRAME_CTRL_PARITY_ODD   0
#define SER_FRAME_CTRL_STOPBITS_1   0
#define SER_FRAME_CTRL_STOPBITS_2   1
#define SER_FRAME_CTRL_DATA_7BITS   0
#define SER_FRAME_CTRL_DATA_8BITS   1

#define SER_FIFO_DEPTH              16



#define PCI_BASE        (HSX_BASE + 0x10000)

#define ISA_BANK_SIZE 0x100000

#define PCI_DESC_BASE                                          (PCI_BASE + 0x00)
#define PCI_ROM_DESC_BASE                                      (PCI_BASE + 0x00)
#define PCI_ROM_DESC0_REG                                      (PCI_BASE + 0x00)
#define PCI_ISAROM_DESC1_REG                                   (PCI_BASE + 0x04)
#define     PCI_ROM_DESC_HOLD_TIME_MASK                            0x00000003
#define     PCI_ROM_DESC_HOLD_TIME_SHIFT                                  0
#define     PCI_ROM_DESC_SETUP_TIME_MASK                           0x0000000C
#define     PCI_ROM_DESC_SETUP_TIME_SHIFT                                 2
#define     PCI_ROM_DESC_WIDTH_MASK                                0x00000030
#define     PCI_ROM_DESC_WIDTH_SHIFT                                      4
#define         PCI_ROM_DESC_WIDTH_32                                (0UL<<4)
#define         PCI_ROM_DESC_WIDTH_8                                 (1UL<<4)
#define         PCI_ROM_DESC_WIDTH_16                                (2UL<<4)
#define     PCI_ROM_DESC_CS_MASK                                   0x00000700
#define     PCI_ROM_DESC_CS_SHIFT                                         8
#define     PCI_ROM_DESC_BURST_ENABLE_MASK                         0x00000800
#define     PCI_ROM_DESC_BURST_ENABLE_SHIFT                              11
#define         PCI_ROM_DESC_BURST_DISABLE                          (0UL<<11)
#define         PCI_ROM_DESC_BURST_ENABLE                           (1UL<<11)
#define     PCI_ROM_DESC_BURST_WAIT_MASK                           0x0000F000
#define     PCI_ROM_DESC_BURST_WAIT_SHIFT                                12
#define     PCI_ROM_DESC_READ_WAIT_MASK                            0x007F0000
#define     PCI_ROM_DESC_READ_WAIT_SHIFT                                 16
#define     PCI_ROM_DESC_FLASH_PROGRAM_MASK                        0x00800000
#define     PCI_ROM_DESC_FLASH_PROGRAM_SHIFT                             23
#define         PCI_ROM_DESC_FLASH_NORMAL_MODE                      (0UL<<23)
#define         PCI_ROM_DESC_FLASH_PROGRAM_MODE                     (1UL<<23)
#define     PCI_ROM_DESC_WRITE_WAIT_MASK                           0x7F000000
#define     PCI_ROM_DESC_WRITE_WAIT_SHIFT                                24
#define     PCI_ROM_DESC_TYPE_MASK                                 0x80000000
#define     PCI_ROM_DESC_TYPE_SHIFT                                      31
#define         PCI_ROM_DESC_TYPE_ROM                               (0UL<<31)
#define         PCI_ROM_DESC_TYPE_ISA                               (1UL<<31)

#define PCI_ISA_DESC_BASE                                      (PCI_BASE + 0x00)
#define PCI_ISAROM_DESC2_REG                                   (PCI_BASE + 0x08)
#define PCI_ISAROM_DESC3_REG                                   (PCI_BASE + 0x0C)
#define PCI_ISA_DESC4_REG                                      (PCI_BASE + 0x10)
#define PCI_ISA_DESC5_REG                                      (PCI_BASE + 0x14)
#define PCI_ISA_DESC6_REG                                      (PCI_BASE + 0x18)
#define PCI_ISA_DESC7_REG                                      (PCI_BASE + 0x1C)
#define     PCI_ISA_DESC_WIDTH_MASK                                0x00000030
#define     PCI_ISA_DESC_WIDTH_SHIFT                                      4
#define         PCI_ISA_DESC_WIDTH_8BIT                            0x00000010
#define         PCI_ISA_DESC_WIDTH_16BIT                           0x00000020
#define     PCI_ISA_DESC_INCR_MASK                                 0x00000800
#define     PCI_ISA_DESC_INCR_SHIFT                                      11
#define     PCI_ISA_DESC_HOLD_TIME_MASK                            0x00000003
#define     PCI_ISA_DESC_HOLD_TIME_SHIFT                                  0
#define     PCI_ISA_DESC_SETUP_TIME_MASK                           0x0000000C
#define     PCI_ISA_DESC_SETUP_TIME_SHIFT                                 2
#define     PCI_ISA_DESC_SYNC_IO_ENABLE_MASK                       0x00000010
#define     PCI_ISA_DESC_SYNC_IO_ENABLE_SHIFT                             4
#define         PCI_ISA_DESC_SYNC_IO_DISABLE                         (0UL<<4)
#define         PCI_ISA_DESC_SYNC_IO_ENABLE                          (1UL<<4)
#define     PCI_ISA_DESC_REG_PROG_MASK                             0x00000020
#define     PCI_ISA_DESC_REG_PROG_SHIFT                                   5
#define         PCI_ISA_DESC_REG_IS_PROG                             (1UL<<5)
#define     PCI_ISA_DESC_IO_TYPE_MASK                              0x00000040
#define     PCI_ISA_DESC_IO_TYPE_SHIFT                                    6
#define         PCI_ISA_DESC_IO_TYPE_OE_WE                           (0UL<<6)
#define         PCI_ISA_DESC_IO_TYPE_IORD_IOWR                       (1UL<<6)
#define         PCI_IS_IO                                            (1UL<<6)
#define     PCI_ISA_DESC_REG_ASSERT_ENABLE_MASK                    0x00000080
#define     PCI_ISA_DESC_REG_ASSERT_ENABLE_SHIFT                          7
#define         PCI_ISA_DESC_REG_ASSERT_DISABLE                      (0UL<<7)
#define         PCI_ISA_DESC_REG_ASSERT_ENABLE                       (1UL<<7)
#define         PCI_NEEDS_ISA_REG                                    (1UL<<7)
#define     PCI_ISA_DESC_CS_MASK                                   0x00000700
#define     PCI_ISA_DESC_CS_SHIFT                                         8
#define     PCI_ISA_DESC_EXT_WAIT_MASK                             0x00003000
#define     PCI_ISA_DESC_EXT_WAIT_SHIFT                                  12
#define         PCI_ISA_DESC_NO_EXT_WAIT_STATES                     (0UL<<12)
#define         PCI_ISA_DESC_ISA_RDY_WAIT_IS_0                      (2UL<<12)
#define         PCI_ISA_DESC_ISA_RDY_WAIT_IS_1                      (3UL<<12)
#define     PCI_ISA_DESC_READ_WAIT_MASK                            0x007F0000
#define     PCI_ISA_DESC_READ_WAIT_SHIFT                                 16
#define     PCI_ISA_DESC_WRITE_WAIT_MASK                           0x7F000000
#define     PCI_ISA_DESC_WRITE_WAIT_SHIFT                                24
#define     PCI_ISA_DESC_TYPE_MASK                                 0x80000000
#define     PCI_ISA_DESC_TYPE_SHIFT                                      31
#define         PCI_ISA_DESC_TYPE_ROM                               (0UL<<31)
#define         PCI_ISA_DESC_TYPE_ISA                               (1UL<<31)
#define         PCI_IS_ISA                                          (1UL<<31)

#define PCI_ROM_MAP_BASE                                       (PCI_BASE + 0x20)
#define PCI_ROM0_MAP_REG                                       (PCI_BASE + 0x20)
#define PCI_ROM1_MAP_REG                                       (PCI_BASE + 0x24)
#define PCI_ROM2_MAP_REG                                       (PCI_BASE + 0x28)
#define PCI_ROM3_MAP_REG                                       (PCI_BASE + 0x2C)
#define     PCI_ROM_MAP_SIZE_MASK                                  0x00000FFF
#define     PCI_ROM_MAP_SIZE_SHIFT                                        0
#define     PCI_ROM_MAP_OFFSET_MASK                                0x0FFF0000
#define     PCI_ROM_MAP_OFFSET_SHIFT                                     16

#define PCI_ROM_MODE_REG                                       (PCI_BASE + 0x30)
#define     PCI_ROM_GLOBAL_FLASH_PROGRAM_MASK                      0x00000001
#define     PCI_ROM_GLOBAL_FLASH_PROGRAM_SHIFT                            0
#define         PCI_ROM_GLOBAL_FLASH_NORMAL_MODE                     (0UL<<0)
#define         PCI_ROM_GLOBAL_FLASH_PROGRAM_MODE                    (1UL<<0)
#define     PCI_ROM_PREFETCH_ENABLE_MASK                           0x00000002
#define     PCI_ROM_PREFETCH_ENABLE_SHIFT                                 1
#define         PCI_ROM_PREFETCH_DISABLE                             (0UL<<1)
#define         PCI_ROM_PREFETCH_ENABLE                              (1UL<<1)
#define     PCI_ROM_SYNC_BURST_FLASH_ENABLE_MASK                   0x00000010
#define     PCI_ROM_SYNC_BURST_FLASH_ENABLE_SHIFT                         4
#define         PCI_ROM_SYNC_BURST_FLASH_DISABLE                     (0UL<<4)
#define         PCI_ROM_SYNC_BURST_FLASH_ENABLE                      (1UL<<4)
#define     PCI_ROM_SYNC_FLASH_CTRL_ENABLE_MASK                    0x00000020
#define     PCI_ROM_SYNC_FLASH_CTRL_ENABLE_SHIFT                          5
#define         PCI_ROM_SYNC_FLASH_CTRL_DISABLE                      (0UL<<5)
#define         PCI_ROM_SYNC_FLASH_CTRL_ENABLE                       (1UL<<5)
#define     PCI_ROM_SYNC_IO_CLOCK_DIVIDER_MASK                     0x00000F00
#define     PCI_ROM_SYNC_IO_CLOCK_DIVIDER_SHIFT                           8
#define     PCI_ROM_SYNC_IO_CLOCK_DIVIDER_ASYNC                      (0UL<<8)
/* the following are for Rev B */
#define     PCI_ROM_REQ0_SYNC_SHIFT                                      12
#define     PCI_ROM_REQ1_SYNC_SHIFT                                      14
#define     PCI_ROM_REQ2_SYNC_SHIFT                                      16

#define PCI_ROM_XOE_MASK_REG                                   (PCI_BASE + 0x34)
#define     PCI_ROM_XOE_CS_MASK                                    0x0000003F
#define     PCI_ROM_XOE_CS_SHIFT                                          0

#define PCI_PAGE_BOUNDARY_REG                                  (PCI_BASE + 0x38)
#define     PCI_PAGE_BOUNDARY_PAGE_SIZE_MASK                       0x000000FF
#define     PCI_PAGE_BOUNDARY_PAGE_SIZE_SHIFT                             0

#define PCI_IO_TIMEOUT_REG                                     (PCI_BASE + 0x3C)

#define PCI_CFG_ADDR_REG                                       (PCI_BASE + 0x40)
#define     PCI_CFG_ADDR_REGISTER_MASK                             0x000000FC
#define     PCI_CFG_ADDR_REGISTER_SHIFT                                   2
#define     PCI_CFG_ADDR_FUNCTION_MASK                             0x00000700
#define     PCI_CFG_ADDR_FUNCTION_SHIFT                                   8
#define     PCI_CFG_ADDR_DEVICE_MASK                               0x0000F800
#define     PCI_CFG_ADDR_DEVICE_SHIFT                                    11
#define     PCI_CFG_ADDR_BUS_MASK                                  0x00FF0000
#define     PCI_CFG_ADDR_BUS_SHIFT                                       16
#define     PCI_CFG_ADDR_ENABLE_BIT_MASK                           0x80000000
#define     PCI_CFG_ADDR_ENABLE_BIT_SHIFT                                31
#define         PCI_CFG_ADDR_ENABLE_BIT_DISABLE                     (0UL<<31)
#define         PCI_CFG_ADDR_ENABLE_BIT_ENABLE                      (1UL<<31)

#define PCI_CFG_DATA_REG                                       (PCI_BASE + 0x44)

#define PCI_RESET_REG                                          (PCI_BASE + 0x48)
#define     PCI_RESET_MASK                                         0x00000001
#define     PCI_RESET_SHIFT                                               0
#define         PCI_RESET_DEASSERTED                                 (0UL<<0)
#define         PCI_RESET_ASSERTED                                   (1UL<<0)
#define     PCI_RESET_IN_MASK                                      0x00000002
#define     PCI_RESET_IN_SHIFT                                            1
#define         PCI_RESET_IN_ASSERTED                                (0UL<<1)
#define         PCI_RESET_IN_DEASSERTED                              (1UL<<1)

#define PCI_EXTINT_REG                                         (PCI_BASE + 0x4C)
#define     PCI_EXTINT_INTA_ENABLE_MASK                            0x00000001
#define     PCI_EXTINT_INTA_ENABLE_SHIFT                                  0
#define         PCI_EXTINT_INTA_ENABLE_DEASSERTED                    (0UL<<0)
#define         PCI_EXTINT_INTA_ENABLE_ASSERTED                      (1UL<<0)
#define     PCI_EXTINT_INTA_STATUS_MASK                            0x00000002
#define     PCI_EXTINT_INTA_STATUS_SHIFT                                  1
#define         PCI_EXTINT_INTA_STATUS_ASSERTED                      (0UL<<1)
#define         PCI_EXTINT_INTA_STATUS_DEASSERTED                    (1UL<<1)

#define PCI_INTR_ENABLE_REG                                    (PCI_BASE + 0x50)
#define PCI_INTR_STATUS_REG                                    (PCI_BASE + 0x54)
#define     PCI_INTR_INTA_ENABLE_MASK                              0x00000001
#define     PCI_INTR_INTA_ENABLE_SHIFT                                    0
#define         PCI_INTR_INTA_ENABLE                                 (1UL<<0)
#define     PCI_INTR_RESET_ASSERTED_MASK                           0x00000002
#define     PCI_INTR_RESET_ASSERTED_SHIFT                                 1
#define         PCI_INTR_RESET_ASSERTED                              (1UL<<1)
#define     PCI_INTR_RESET_DEASSERTED_MASK                         0x00000004
#define     PCI_INTR_RESET_DEASSERTED_SHIFT                               2
#define         PCI_INTR_RESET_DEASSERTED                            (1UL<<2)
#define     PCI_INTR_MASTER_PARITY_ERROR_MASK                      0x00000400
#define     PCI_INTR_MASTER_PARITY_ERROR_SHIFT                           10
#define         PCI_INTR_MASTER_PARITY_ERROR                        (1UL<<10)
#define     PCI_INTR_SLAVE_SIGNALLED_TARGET_ABORT_MASK             0x00000800
#define     PCI_INTR_SLAVE_SIGNALLED_TARGET_ABORT_SHIFT                  11
#define         PCI_INTR_SLAVE_SIGNALLED_TARGET_ABORT               (1UL<<11)
#define     PCI_INTR_MASTER_RECEIVED_TARGET_ABORT_MASK             0x00001000
#define     PCI_INTR_MASTER_RECEIVED_TARGET_ABORT_SHIFT                  12
#define         PCI_INTR_MASTER_RECEIVED_TARGET_ABORT               (1UL<<12)
#define     PCI_INTR_MASTER_DETECTED_MASTER_ABORT_MASK             0x00002000
#define     PCI_INTR_MASTER_DETECTED_MASTER_ABORT_SHIFT                  13
#define         PCI_INTR_MASTER_DETECTED_MASTER_ABORT               (1UL<<13)
#define     PCI_INTR_SYSTEM_ERROR_MASK                             0x00004000
#define     PCI_INTR_SYSTEM_ERROR_SHIFT                                  14
#define         PCI_INTR_SYSTEM_ERROR                               (1UL<<14)
#define     PCI_INTR_PARITY_ERROR_MASK                             0x00008000
#define     PCI_INTR_PARITY_ERROR_SHIFT                                  15
#define         PCI_INTR_PARITY_ERROR                               (1UL<<15)

#define PCI_REMAP_REG                                          (PCI_BASE + 0x58)
#define     PCI_REMAP_IO_REMAP_MASK                                0x0000FFF0
#define     PCI_REMAP_IO_REMAP_SHIFT                                      4
#define     PCI_REMAP_MEM_REMAP_MASK                               0xE0000000
#define     PCI_REMAP_MEM_REMAP_SHIFT                                    29

#define PCI_EXT_DESC_BASE                                      (PCI_BASE + 0x80)
#define PCI_ROM_DESC0_REG2                                     (PCI_BASE + 0x80)
#define PCI_EXT_ISAROM_DESC1_REG2                              (PCI_BASE + 0x84)
#define PCI_ISAROM_DESC2_REG2                                  (PCI_BASE + 0x88)
#define PCI_ISAROM_DESC3_REG2                                  (PCI_BASE + 0x8C)
#define PCI_ISA_DESC4_REG2                                     (PCI_BASE + 0x90)
#define PCI_ISA_DESC5_REG2                                     (PCI_BASE + 0x94)
#define PCI_ISA_DESC6_REG2                                     (PCI_BASE + 0x98)
#define PCI_ISA_DESC7_REG2                                     (PCI_BASE + 0x9C)
#define     PCI_EXT_DESC_ADDR_HOLD_TIME_MASK                       0x0000001F
#define     PCI_EXT_DESC_ADDR_HOLD_TIME_SHIFT                             0
#define     PCI_EXT_DESC_CTRL_SETUP_TIME_MASK                      0x00000FE0
#define     PCI_EXT_DESC_CTRL_SETUP_TIME_SHIFT                            5
#define     PCI_EXT_DESC_CS_HOLD_TIME_MASK                         0x0001F000
#define     PCI_EXT_DESC_CS_HOLD_TIME_SHIFT                              12
#define     PCI_EXT_DESC_CS_SETUP_TIME_MASK                        0x00FE0000
#define     PCI_EXT_DESC_CS_SETUP_TIME_SHIFT                             17
#define     PCI_EXT_DESC_REG_ACCESS_TIME_MASK                      0x0F000000
#define     PCI_EXT_DESC_REG_ACCESS_TIME_SHIFT                           24
#define     PCI_EXT_DESC_REG_SETUP_TIME_MASK                       0xF0000000
#define     PCI_EXT_DESC_REG_SETUP_TIME_SHIFT                            28
/* Macro allowing the address of a descriptor to be determined from the
   descriptor number. */
#define PCI_ISA_PRIMARY_DESC_ADDR(desc_num) (PCI_ISA_DESC_BASE + 4*(desc_num))
#define PCI_ISA_SECONDARY_DESC_ADDR(desc_num) (PCI_EXT_DESC_BASE + 4*(desc_num))

/* Use this macro to determine an ISA device base address from its
   descriptor number. */
#define PCI_ISA_DEVICE_BASE(desc_num) (PCI_IO_BASE_ADDR + ((desc_num)*ISA_BANK_SIZE))

/*
 * PCI Device Config Space Offsets (by dword)
 */

#define PCI_ID_OFF                                             0x00
#define PCI_CMD_STAT_OFF                                       0x01
#define PCI_REV_ID_CLASS_CODE_OFF                              0x02
#define PCI_CLS_LT_HT_BIST_OFF                                 0x03
#define PCI_BASE_ADDR_0_OFF                                    0x04
#define PCI_BASE_ADDR_1_OFF                                    0x05
#define PCI_BASE_ADDR_2_OFF                                    0x06
#define PCI_BASE_ADDR_3_OFF                                    0x07
#define PCI_BASE_ADDR_4_OFF                                    0x08
#define PCI_BASE_ADDR_5_OFF                                    0x09
#define PCI_SUBSYSTEM_IDS                                      0x0b
#define PCI_ROM_BASE_ADDR_OFF                                  0x0c
#define PCI_IL_IP_MING_MAXG_OFF                                0x0f
#define PCI_BASE_ENABLE_SIZE                                   0x10
#define PCI_BASE_REMAP                                         0x11
#define PCI_ENABLE_TRDY                                        0x12

/*
 * PCI Device Config Space Command Register
 */

#define     PCI_CMD_IO_ACCESS_ENABLE_MASK                          0x00000001
#define     PCI_CMD_IO_ACCESS_ENABLE_SHIFT                                0
#define         PCI_CMD_IO_ACCESS_ENABLE                             (1UL<<0)
#define     PCI_CMD_MEM_ACCESS_ENABLE_MASK                         0x00000002
#define     PCI_CMD_MEM_ACCESS_ENABLE_SHIFT                               1
#define         PCI_CMD_MEM_ACCESS_ENABLE                            (1UL<<1)
#define     PCI_CMD_MASTER_ENABLE_MASK                             0x00000004
#define     PCI_CMD_MASTER_ENABLE_SHIFT                                   2
#define         PCI_CMD_MASTER_ENABLE                                (1UL<<2)
#define     PCI_CMD_SPECIAL_CYCLE_RECOGNITION_ENABLE_MASK          0x00000008
#define     PCI_CMD_SPECIAL_CYCLE_RECOGNITION_ENABLE_SHIFT                3
#define         PCI_CMD_SPECIAL_CYCLE_RECOGNITION_ENABLE             (1UL<<3)
#define     PCI_CMD_MEM_WRITE_AND_INVALIDATE_ENABLE_MASK           0x00000010
#define     PCI_CMD_MEM_WRITE_AND_INVALIDATE_ENABLE_SHIFT                 4
#define         PCI_CMD_MEM_WRITE_AND_INVALIDATE_ENABLE              (1UL<<4)
#define     PCI_CMD_PALETTE_SNOOP_ENABLE_MASK                      0x00000020
#define     PCI_CMD_PALETTE_SNOOP_ENABLE_SHIFT                            5
#define         PCI_CMD_PALETTE_SNOOP_ENABLE                         (1UL<<5)
#define     PCI_CMD_PARITY_ERROR_RESPONSE_ENABLE_MASK              0x00000040
#define     PCI_CMD_PARITY_ERROR_RESPONSE_ENABLE_SHIFT                    6
#define         PCI_CMD_PARITY_ERROR_RESPONSE_ENABLE                 (1UL<<6)
#define     PCI_CMD_WAIT_CYCLE_ENABLE_MASK                         0x00000080
#define     PCI_CMD_WAIT_CYCLE_ENABLE_SHIFT                               7
#define         PCI_CMD_WAIT_CYCLE_ENABLE                            (1UL<<7)
#define     PCI_CMD_SYSTEM_ERROR_ENABLE_MASK                       0x00000100
#define     PCI_CMD_SYSTEM_ERROR_ENABLE_SHIFT                             8
#define         PCI_CMD_SYSTEM_ERROR_ENABLE                          (1UL<<8)
#define     PCI_CMD_FAST_BACK_TO_BACK_ENABLE_MASK                  0x00000200
#define     PCI_CMD_FAST_BACK_TO_BACK_ENABLE_SHIFT                        9
#define         PCI_CMD_FAST_BACK_TO_BACK_ENABLE                     (1UL<<9)
/*
 * PCI-PCCard/CardBus Bridge Config Space Offsets
 */

#define PCI_PCCARD_BASE_ADDR_OFF                               0x04
#define PCI_BUS_NUM_OFF                                        0x06
#define PCI_CARDBUS_MEM_BASE_0_OFF                             0x07
#define PCI_CARDBUS_MEM_LIMIT_0_OFF                            0x08
#define PCI_CARDBUS_MEM_BASE_1_OFF                             0x09
#define PCI_CARDBUS_MEM_LIMIT_1_OFF                            0x0A
#define PCI_CARDBUS_IO_BASE_0_OFF                              0x0B
#define PCI_CARDBUS_IO_LIMIT_0_OFF                             0x0C
#define PCI_CARDBUS_IO_BASE_1_OFF                              0x0D
#define PCI_CARDBUS_IO_LIMIT_1_OFF                             0x0E
#define PCI_CARDBUS_BRIDGE_CTRL_OFF                            0x0F
#define PCI_PCCARD_IO_BASE_OFF                                 0x11
#define PCI_PCCARD_SYS_CTRL_OFF                                0x20
#define PCI_PCCARD_GPIO_OFF                                    0x23
#define PCI_PCCARD_RTY_DC_CC_DR_OFF                            0x24
#define PCI_PCCARD_SKT_DMA_OFF                                 0x25
#define PCI_PCCARD_DDMA_OFF                                    0x26
#define PCI_CARDBUS_POWER_OFF                                  0x29
#define PCI_PCCARD_GPE_OFF                                     0x2a
#define PCI_PCCARD_GPIO_IO_OFF                                 0x2b

/*
 * PCI ATA Subsystem
 */

#define PCI_ATA_DMA_CTRL_REG                                  (PCI_BASE + 0x100)
#define     PCI_ATA_DMA_ENABLE_MASK                                0x00000001
#define     PCI_ATA_DMA_ENABLE_SHIFT                                      0
#define         PCI_ATA_DMA_DISABLE                                  (0UL<<0)
#define         PCI_ATA_DMA_ENABLE                                   (1UL<<0)
#define     PCI_ATA_DMA_MODE_MASK                                  0x00000002
#define     PCI_ATA_DMA_MODE_SHIFT                                        1
#define         PCI_ATA_DMA_MODE_MWDMA                               (0UL<<1)
#define         PCI_ATA_DMA_MODE_UDMA                                (1UL<<1)
#define     PCI_ATA_DMA_DIR_MASK                                   0x00000004
#define     PCI_ATA_DMA_DIR_SHIFT                                         2
#define         PCI_ATA_DMA_DIR_READ                                 (0UL<<2)
#define         PCI_ATA_DMA_DIR_WRITE                                (1UL<<2)
#define     PCI_ATA_INTR_ENABLE_MASK                               0x00000008
#define     PCI_ATA_INTR_ENABLE_SHIFT                                     3
#define         PCI_ATA_INTR_DISABLE                                 (0UL<<3)
#define         PCI_ATA_INTR_ENABLE                                  (1UL<<3)
#define     PCI_ATA_DES_ENABLE_MASK                                0x00000010
#define     PCI_ATA_DES_ENABLE_SHIFT                                      4
#define         PCI_ATA_DES_DISABLE                                  (0UL<<4)
#define         PCI_ATA_DES_ENABLE                                   (1UL<<4)
#define     PCI_ATA_DES_TYPE_MASK                                  0x00000020
#define     PCI_ATA_DES_TYPE_SHIFT                                        5
#define         PCI_ATA_DES_TYPE_ALL_DATA                            (0UL<<5)
#define         PCI_ATA_DES_TYPE_PKT_DATA                            (1UL<<5)
#define     PCI_ATA_XFER_COUNT_MASK                                0xFFFFFF00
#define     PCI_ATA_XFER_COUNT_SHIFT                                      8
#define PCI_ATA_DMA_STATUS_REG                                (PCI_BASE + 0x104)
#define     PCI_ATA_DMA_ACTIVE_MASK                                0x00000001
#define     PCI_ATA_DMA_ACTIVE_SHIFT                                      0
#define         PCI_ATA_DMA_INACTIVE                                 (0UL<<0)
#define         PCI_ATA_DMA_ACTIVE                                   (1UL<<0)
#define     PCI_ATA_DMA_COMPLETE_MASK                              0x00000002
#define     PCI_ATA_DMA_COMPLETE_SHIFT                                    1
#define         PCI_ATA_DMA_INCOMPLETE                               (0UL<<1)
#define         PCI_ATA_DMA_COMPLETE                                 (1UL<<1)
#define     PCI_ATA_PIO_ERROR_MASK                                 0x00000004
#define     PCI_ATA_PIO_ERROR_SHIFT                                       2
#define         PCI_ATA_PIO_NO_ERROR                                 (0UL<<2)
#define         PCI_ATA_PIO_ERROR                                    (1UL<<2)
#define PCI_ATA_DMA_TARGET_REG                                (PCI_BASE + 0x108)

#define PCI_ATA_DMA_MODE_REG                                  (PCI_BASE + 0x10C)
#define     PCI_ATA_DMAREQ_PINCTL_MASK                             0x00000001
#define     PCI_ATA_DMAREQ_PINCTL_SHIFT                                   0
#define         PCI_ATA_DMAREQ_PINCTL_USE_ISA                        (0UL<<0)
#define         PCI_ATA_DMAREQ_PINCTL_USE_PIO                        (1UL<<0)
#define     PCI_ATA_BREAK_MWDMA_BURSTS_AT_16_MASK                  0x00000002
#define     PCI_ATA_BREAK_MWDMA_BURSTS_AT_16_SHIFT                        1
#define         PCI_ATA_DONT_BREAK_MWDMA_BURSTS_AT_16                (0UL<<1)
#define         PCI_ATA_BREAK_MWDMA_BURSTS_AT_16                     (1UL<<1)
#define     PCI_ATA_PRESMPL_SEL_MASK                               0x0000001C
#define     PCI_ATA_PRESMPL_SEL_SHIFT                                     2
#define         PCI_ATA_PRESMPL_SEL_RAW                              (0UL<<2)
#define         PCI_ATA_PRESMPL_SEL_DSYNC0                           (1UL<<2)
#define         PCI_ATA_PRESMPL_SEL_DSYNC1                           (2UL<<2)
#define         PCI_ATA_PRESMPL_SEL_DSYNC2                           (3UL<<2)
#define         PCI_ATA_PRESMPL_SEL_DSYNC3                           (4UL<<2)
#define     PCI_ATA_ENABLE_PIO_READ_TIMEOUT_MASK                   0x00000020
#define     PCI_ATA_ENABLE_PIO_READ_TIMEOUT_SHIFT                         5
#define         PCI_ATA_DISABLE_PIO_READ_TIMEOUT                     (0UL<<5)
#define         PCI_ATA_ENABLE_PIO_READ_TIMEOUT                      (1UL<<5)
#define     PCI_ATA_DBG_SEL_MASK                                   0x000000C0
#define     PCI_ATA_DBG_SEL_SHIFT                                         6
#define     PCI_ATA_DMA_BURST_BOUNDARY_MASK                        0x0000FF00
#define     PCI_ATA_DMA_BURST_BOUNDARY_SHIFT                              8
#define     PCI_ATA_UDMA_READ_FIFO_FULL_LEVEL_MASK                 0x000F0000
#define     PCI_ATA_UDMA_READ_FIFO_FULL_LEVEL_SHIFT                      16
#define     PCI_ATA_DATA_OUT_DELAY_CTRL_MASK                       0x00300000
#define     PCI_ATA_DATA_OUT_DELAY_CTRL_SHIFT                            20
#define         PCI_ATA_DATA_OUT_DELAY_CTRL_0_0_NS                  (0UL<<20)
#define         PCI_ATA_DATA_OUT_DELAY_CTRL_2_5_NS                  (1UL<<20)
#define         PCI_ATA_DATA_OUT_DELAY_CTRL_5_0_NS                  (2UL<<20)
#define         PCI_ATA_DATA_OUT_DELAY_CTRL_7_5_NS                  (3UL<<20)
#define     PCI_ATA_DATA_IN_DELAY_CTRL_MASK                        0x00C00000
#define     PCI_ATA_DATA_IN_DELAY_CTRL_SHIFT                             22
#define         PCI_ATA_DATA_IN_DELAY_CTRL_0_0_NS                   (0UL<<22)
#define         PCI_ATA_DATA_IN_DELAY_CTRL_2_5_NS                   (1UL<<22)
#define         PCI_ATA_DATA_IN_DELAY_CTRL_5_0_NS                   (2UL<<22)
#define         PCI_ATA_DATA_IN_DELAY_CTRL_7_5_NS                   (3UL<<22)
#define PCI_ATA_MWDMA_TIMING_REG                              (PCI_BASE + 0x110)
#define     PCI_ATA_MWDMA_ACTIVE_TIME_MASK                         0x000000FF
#define     PCI_ATA_MWDMA_ACTIVE_TIME_SHIFT                               0
#define     PCI_ATA_MWDMA_WREC_TIME_MASK                           0x0000FF00
#define     PCI_ATA_MWDMA_WREC_TIME_SHIFT                                 8
#define     PCI_ATA_MWDMA_RREC_TIME_MASK                           0x00FF0000
#define     PCI_ATA_MWDMA_RREC_TIME_SHIFT                                16
#define     PCI_ATA_MWDMA_CSSETUP_TIME_MASK                        0x0F000000
#define     PCI_ATA_MWDMA_CSSETUP_TIME_SHIFT                             24
#define     PCI_ATA_MWDMA_CSHOLD_TIME_MASK                         0xF0000000
#define     PCI_ATA_MWDMA_CSHOLD_TIME_SHIFT                              28

#define PCI_ATA_UDMA_TIMING_REG                               (PCI_BASE + 0x114)
#define     PCI_ATA_UDMA_SETUP_TIME_MASK                           0x000000FF
#define     PCI_ATA_UDMA_SETUP_TIME_SHIFT                                 0
#define     PCI_ATA_UDMA_HOLD_TIME_MASK                            0x0000FF00
#define     PCI_ATA_UDMA_HOLD_TIME_SHIFT                                  8
#define     PCI_ATA_UDMA_RP_TIME_MASK                              0x00FF0000
#define     PCI_ATA_UDMA_RP_TIME_SHIFT                                   16
#define     PCI_ATA_UDMA_INIT_TIME_MASK                            0x0F000000
#define     PCI_ATA_UDMA_INIT_TIME_SHIFT                                 24
#define     PCI_ATA_UDMA_CRC_SETUP_AND_STROBE_TO_DMA_ACK_TIME_MASK 0xF0000000
#define     PCI_ATA_UDMA_CRC_SETUP_AND_STROBE_TO_DMA_ACK_TIME_SHIFT      28
#define PCI_ATA_PIO_TIMING_REG                                (PCI_BASE + 0x118)
#define     PCI_ATA_PIO_HOLD_TIME_MASK                             0x000000FF
#define     PCI_ATA_PIO_HOLD_TIME_SHIFT                                   0
#define     PCI_ATA_PIO_ACCESS_TIME_MASK                           0x0000FF00
#define     PCI_ATA_PIO_ACCESS_TIME_SHIFT                                 8
#define     PCI_ATA_PIO_SETUP_TIME_MASK                            0x00FF0000
#define     PCI_ATA_PIO_SETUP_TIME_SHIFT                                 16
#define     PCI_ATA_PIO_DELAY_TIME_MASK                            0xFF000000
#define     PCI_ATA_PIO_DELAY_TIME_SHIFT                                 24

#define PCI_ATA_CS_SELECT_REG                                 (PCI_BASE + 0x11C)
#define     PCI_ATA_DESC_MASK                                      0x000000FF
#define     PCI_ATA_DESC_SHIFT                                            0
#define         PCI_ATA_DESC0                                        (1UL<<0)
#define         PCI_ATA_DESC1                                        (1UL<<1)
#define         PCI_ATA_DESC2                                        (1UL<<2)
#define         PCI_ATA_DESC3                                        (1UL<<3)
#define         PCI_ATA_DESC4                                        (1UL<<4)
#define         PCI_ATA_DESC5                                        (1UL<<5)
#define         PCI_ATA_DESC6                                        (1UL<<6)
#define         PCI_ATA_DESC7                                        (1UL<<7)
#define     PCI_ATA_CS_MASK                                        0x0000FF00
#define     PCI_ATA_CS_SHIFT                                              8
#define         PCI_ATA_CS0                                          (1UL<<8)
#define         PCI_ATA_CS1                                          (1UL<<9)
#define         PCI_ATA_CS2                                         (1UL<<10)
#define         PCI_ATA_CS3                                         (1UL<<11)
#define         PCI_ATA_CS4                                         (1UL<<12)
#define         PCI_ATA_CS5                                         (1UL<<13)
#define         PCI_ATA_CS6                                         (1UL<<14)
#define         PCI_ATA_CS7                                         (1UL<<15)
#define PCI_ATA_DESC(desc)                                           (1<<(desc))
#define PCI_ATA_CS(cs)                             ((1UL<<(cs))<<PCI_ATA_CS_SHIFT)

#define PCI_ATA_SCRAMBLE_KEY_REG                              (PCI_BASE + 0x120)
#define PCI_ATA_INTR_ENABLE_REG                               (PCI_BASE + 0x124)
#define     PCI_ATA_DMA_COMPLETE_INTR_ENABLE_MASK                  0x00000002
#define     PCI_ATA_DMA_COMPLETE_INTR_ENABLE_SHIFT                        1
#define         PCI_ATA_DMA_COMPLETE_INTR_DISABLE                    (0UL<<1)
#define         PCI_ATA_DMA_COMPLETE_INTR_ENABLE                     (1UL<<1)
#define     PCI_ATA_PIO_ERROR_INTR_ENABLE_MASK                     0x00000004
#define     PCI_ATA_PIO_ERROR_INTR_ENABLE_SHIFT                           2
#define         PCI_ATA_PIO_ERROR_INTR_DISABLE                       (0UL<<2)
#define         PCI_ATA_PIO_ERROR_INTR_ENABLE                        (1UL<<2)

#define PCI_ATA_COMMAND_REG                                   (PCI_BASE + 0x130)
#define     PCI_ATA_COMMAND_ENTRIES_MASK                           0x000000FF
#define     PCI_ATA_COMMAND_ENTRIES_SHIFT                                 0
#define     PCI_ATA_QUEUE_SIZE_MASK                                0x0000FF00
#define     PCI_ATA_QUEUE_SIZE_SHIFT                                      8
#define     PCI_ATA_CMD_DEVICE_MASK                                0x000000FF
#define     PCI_ATA_CMD_DEVICE_SHIFT                                      0
#define     PCI_ATA_CMD_FEATURE_MASK                               0x0000FF00
#define     PCI_ATA_CMD_FEATURE_SHIFT                                     8
#define     PCI_ATA_CMD_SECTOR_COUNT_MASK                          0x00FF0000
#define     PCI_ATA_CMD_SECTOR_COUNT_SHIFT                               16
#define     PCI_ATA_CMD_NUM_SECTORS_MASK                           0xFF000000
#define     PCI_ATA_CMD_NUM_SECTORS_SHIFT                                24
#define     PCI_ATA_CMD_CYL_LOW_MASK                               0x000000FF
#define     PCI_ATA_CMD_CYL_LOW_SHIFT                                     0
#define     PCI_ATA_CMD_CYL_HIGH_MASK                              0x0000FF00
#define     PCI_ATA_CMD_CYL_HIGH_SHIFT                                    8
#define     PCI_ATA_CMD_DEV_HEAD_MASK                              0x00FF0000
#define     PCI_ATA_CMD_DEV_HEAD_SHIFT                                   16
#define     PCI_ATA_CMD_COMMAND_MASK                               0xFF000000
#define     PCI_ATA_CMD_COMMAND_SHIFT                                    24
#define     PCI_ATA_CMD_ENABLE_MASK                                0x000000FF
#define     PCI_ATA_CMD_ENABLE_SHIFT                                      0
#define     PCI_ATA_CMD_TAG_MASK                                   0x0000FF00
#define PCI_ATA_CMD0(numsec,seccnt,feat,dev)                                  \
                     (((numsec)<<PCI_ATA_CMD_NUM_SECTORS_SHIFT)  |            \
                      ((seccnt)<<PCI_ATA_CMD_SECTOR_COUNT_SHIFT) |            \
                      ((feat)<<PCI_ATA_CMD_FEATURE_SHIFT) | (dev))
#define PCI_ATA_CMD1(cmd,devhead,cylhigh,cyllow)                              \
                     (((cmd)<<PCI_ATA_CMD_COMMAND_SHIFT)      |               \
                      ((devhead)<<PCI_ATA_CMD_DEV_HEAD_SHIFT) |               \
                      ((cylhigh)<<PCI_ATA_CMD_CYL_HIGH_SHIFT) | (cyllow))
#define PCI_ATA_CMD2(addr) (addr)
#define PCI_ATA_CMD3(tag,en)                                                  \
                     (((tag)<<PCI_ATA_CMD_TAG_SHIFT) | (en))
#define PCI_ATA_STATUS_REG                                    (PCI_BASE + 0x134)
#define     PCI_ATA_STATUS_TAG_MASK                                0x000000FF
#define     PCI_ATA_STATUS_TAG_SHIFT                                      0
#define     PCI_ATA_STATUS_MASK                                    0x0000FF00
#define     PCI_ATA_STATUS_SHIFT                                          8
#define         PCI_ATA_STATUS_NOT_COMPLETE                          (0UL<<8)
#define         PCI_ATA_STATUS_NORMAL_COMPLETION                     (1UL<<8)
#define         PCI_ATA_STATUS_PIO1_ERROR                            (5UL<<8)
#define         PCI_ATA_STATUS_PIO2_ERROR                            (6UL<<8)
#define         PCI_ATA_STATUS_PIO3_ERROR                            (7UL<<8)
#define         PCI_ATA_STATUS_PIO1_TIMEOUT                          (9UL<<8)
#define         PCI_ATA_STATUS_PIO2_TIMEOUT                         (10UL<<8)
#define         PCI_ATA_STATUS_PIO3_TIMEOUT                         (11UL<<8)
#define     PCI_ATA_READ_DATA_MASK                                 0x00FF0000
#define     PCI_ATA_READ_DATA_SHIFT                                      16
#define     PCI_ATA_STATUS_ENTRIES_MASK                            0xFF000000
#define     PCI_ATA_STATUS_ENTRIES_SHIFT                                 24

#define PCI_ATA_DEBUG_REG                                     (PCI_BASE + 0x170)
#define     PCI_ATA_DMA_REQ_STATUS_MASK                            0x00000001
#define     PCI_ATA_DMA_REQ_STATUS_SHIFT                                  0
#define         PCI_ATA_DMA_REQ_STATUS_INACTIVE                      (0UL<<0)
#define         PCI_ATA_DMA_REQ_STATUS_ACTIVE                        (1UL<<0)
#define     PCI_ATA_DMA_ACK_STATUS_MASK                            0x00000002
#define     PCI_ATA_DMA_ACK_STATUS_SHIFT                                  1
#define         PCI_ATA_DMA_ACK_STATUS_INACTIVE                      (0UL<<1)
#define         PCI_ATA_DMA_ACK_STATUS_ACTIVE                        (1UL<<1)
#define     PCI_ATA_IF_STATE_MASK                                  0x000FF000
#define     PCI_ATA_IF_STATE_SHIFT                                       12
#define     PCI_ATA_MWDMA_STATE_MASK                               0x00F00000
#define     PCI_ATA_MWDMA_STATE_SHIFT                                    20
#define     PCI_ATA_UDMA_STATE_MASK                                0xFF000000
#define     PCI_ATA_UDMA_STATE_SHIFT                                     24
#define PCI_ATA_DEBUG2_REG                                    (PCI_BASE + 0x174)
#define     PCI_ATA_FIFO_ENTRIES_MASK                              0x0000003F
#define     PCI_ATA_FIFO_ENTRIES_SHIFT                                    0
#define     PCI_ATA_DMA_COUNT_MASK                                 0xFFFFFFC0
#define     PCI_ATA_DMA_COUNT_SHIFT                                       6

#define PCI_ATA_KEY_LOOKUP_BASE                               (PCI_BASE + 0x180)
#define PCI_ATA_KEY_LOOKUP_0_REG                              (PCI_BASE + 0x180)
#define PCI_ATA_KEY_LOOKUP_1_REG                              (PCI_BASE + 0x184)
#define PCI_ATA_KEY_LOOKUP_2_REG                              (PCI_BASE + 0x188)
#define PCI_ATA_KEY_LOOKUP_3_REG                              (PCI_BASE + 0x18C)
#define PCI_ATA_KEY_LOOKUP_4_REG                              (PCI_BASE + 0x190)
#define PCI_ATA_KEY_LOOKUP_5_REG                              (PCI_BASE + 0x194)
#define PCI_ATA_KEY_LOOKUP_6_REG                              (PCI_BASE + 0x198)
#define PCI_ATA_KEY_LOOKUP_7_REG                              (PCI_BASE + 0x19C)
#define PCI_ATA_KEY_LOOKUP_8_REG                              (PCI_BASE + 0x1A0)
#define PCI_ATA_KEY_LOOKUP_9_REG                              (PCI_BASE + 0x1A4)
#define PCI_ATA_KEY_LOOKUP_A_REG                              (PCI_BASE + 0x1A8)
#define PCI_ATA_KEY_LOOKUP_B_REG                              (PCI_BASE + 0x1AC)
#define PCI_ATA_KEY_LOOKUP_C_REG                              (PCI_BASE + 0x1B0)
#define PCI_ATA_KEY_LOOKUP_D_REG                              (PCI_BASE + 0x1B4)
#define PCI_ATA_KEY_LOOKUP_E_REG                              (PCI_BASE + 0x1B8)
#define PCI_ATA_KEY_LOOKUP_F_REG                              (PCI_BASE + 0x1BC)
#define     PCI_ATA_KEY_LOOKUP_PID_MASK                            0x00001FFF
#define     PCI_ATA_KEY_LOOKUP_PID_SHIFT                                  0

#define PCI_ATA_KEY_BASE                                      (PCI_BASE + 0x200)
#define PCI_ATA_KEY_0L_REG                                    (PCI_BASE + 0x200)
#define PCI_ATA_KEY_0H_REG                                    (PCI_BASE + 0x204)
#define PCI_ATA_KEY_1L_REG                                    (PCI_BASE + 0x208)
#define PCI_ATA_KEY_1H_REG                                    (PCI_BASE + 0x20C)
#define PCI_ATA_KEY_2L_REG                                    (PCI_BASE + 0x210)
#define PCI_ATA_KEY_2H_REG                                    (PCI_BASE + 0x214)
#define PCI_ATA_KEY_3L_REG                                    (PCI_BASE + 0x218)
#define PCI_ATA_KEY_3H_REG                                    (PCI_BASE + 0x21C)
#define PCI_ATA_KEY_4L_REG                                    (PCI_BASE + 0x220)
#define PCI_ATA_KEY_4H_REG                                    (PCI_BASE + 0x224)
#define PCI_ATA_KEY_5L_REG                                    (PCI_BASE + 0x228)
#define PCI_ATA_KEY_5H_REG                                    (PCI_BASE + 0x22C)
#define PCI_ATA_KEY_6L_REG                                    (PCI_BASE + 0x230)
#define PCI_ATA_KEY_6H_REG                                    (PCI_BASE + 0x234)
#define PCI_ATA_KEY_7L_REG                                    (PCI_BASE + 0x238)
#define PCI_ATA_KEY_7H_REG                                    (PCI_BASE + 0x23C)
#define PCI_ATA_KEY_8L_REG                                    (PCI_BASE + 0x240)
#define PCI_ATA_KEY_8H_REG                                    (PCI_BASE + 0x244)
#define PCI_ATA_KEY_9L_REG                                    (PCI_BASE + 0x248)
#define PCI_ATA_KEY_9H_REG                                    (PCI_BASE + 0x24C)
#define PCI_ATA_KEY_AL_REG                                    (PCI_BASE + 0x250)
#define PCI_ATA_KEY_AH_REG                                    (PCI_BASE + 0x254)
#define PCI_ATA_KEY_BL_REG                                    (PCI_BASE + 0x258)
#define PCI_ATA_KEY_BH_REG                                    (PCI_BASE + 0x25C)
#define PCI_ATA_KEY_CL_REG                                    (PCI_BASE + 0x260)
#define PCI_ATA_KEY_CH_REG                                    (PCI_BASE + 0x264)
#define PCI_ATA_KEY_DL_REG                                    (PCI_BASE + 0x268)
#define PCI_ATA_KEY_DH_REG                                    (PCI_BASE + 0x26C)
#define PCI_ATA_KEY_EL_REG                                    (PCI_BASE + 0x270)
#define PCI_ATA_KEY_EH_REG                                    (PCI_BASE + 0x274)
#define PCI_ATA_KEY_FL_REG                                    (PCI_BASE + 0x278)
#define PCI_ATA_KEY_FH_REG                                    (PCI_BASE + 0x27C)

#define RST_GRESET_REG                                         (RST_BASE + 0x2C)
#define     RST_GRESET_SATA_MASK                                   0x02000000
#define         RST_GRESET_SATA                                      (1UL<<25)

#define PCI_CLK_DELAY_REG                                      (PLL_BASE + 0xB0)
#define     PCI_CLK_DELAY_SMALL_MASK                               0x000F0000
#define     PCI_CLK_DELAY_SMALL_SHIFT                                    16
#define     PCI_CLK_DELAY_BIG_MASK                                 0x00700000
#define     PCI_CLK_DELAY_BIG_SHIFT                                      20


#define SYS_TIMER           0

#define TIM_BASE            (ASX0_BASE + 0x30000)
#define NUM_TIM_BANKS       16
#define TIM_BANK_SIZE       0x0010

#define TIM_VALUE_REG                                         (TIM_BASE + 0x00)
#define TIMER_NUM_VALUE_REG(x) (TIM_VALUE_REG + ((x) << 4))

#define TIM_LIMIT_REG                                         (TIM_BASE + 0x04)
#define TIMER_NUM_LIMIT_REG(x) (TIM_LIMIT_REG + ((x) << 4))

#define TIM_MODE_REG                                          (TIM_BASE + 0x08)
#define TIMER_NUM_MODE_REG(x)  (TIM_MODE_REG  + ((x) << 4))
#define     TIM_ENABLE_MASK                                       0x00000001
#define     TIM_INT_ENABLE_MASK                                   0x00000008
#define TIM_INT_STATUS_REG                                    (TIM_BASE + 0x100)

#define GPI_BASE            (ASX0_BASE + 0x70000)
#define GPI_BANK_SIZE       0x40
#define NUM_GPI_BANKS       6
#define NUM_OF_GPIO         96

#define GPI_INT_STAT_REG     (GPI_BASE + 0x10)

#define GPI_INT_ENABLE0_REG  (GPI_BASE + 0x14)
#define GPI_POS_EDGE_REG     (GPI_BASE + 0x18)
#define GPI_NEG_EDGE_REG     (GPI_BASE + 0x1C)

#define SREGS_BASE          (ASX0_BASE + 0x40000)
#define PLL_BASE            (SREGS_BASE + 0)
#define PLL_PIN_GPIO_MUX0_REG                                  (PLL_BASE + 0x180)
#define PLL_PIN_GPIO_MUX1_REG                                  (PLL_BASE + 0x184)
#define PLL_PIN_GPIO_MUX2_REG                                  (PLL_BASE + 0x188)
#define PLL_PIN_GPIO_MUX3_REG                                  (PLL_BASE + 0x18C)
#define PLL_PIN_GPIO_MUX4_REG                                  (PLL_BASE + 0x190)
#define PLL_PIN_GPIO_MUX5_REG                                  (PLL_BASE + 0x194)

#define SREG_2ND_PIN_MUX0_REG                                  (PLL_BASE + 0x1C0)
#define SREG_2ND_PIN_MUX1_REG                                  (PLL_BASE + 0x1C4)
#define SREG_2ND_PIN_MUX2_REG                                  (PLL_BASE + 0x1C8)
#define SREG_2ND_PIN_MUX3_REG                                  (PLL_BASE + 0x1CC)
#define SREG_2ND_PIN_MUX4_REG                                  (PLL_BASE + 0x1D0)
#define SREG_2ND_PIN_MUX5_REG                                  (PLL_BASE + 0x1D4)

#define PLL_PIN_ALT_FUNC_REG                                   (PLL_BASE + 0x110)
#define     PLL_PIN_ALT_FUNC_IO_RW_MASK                            0x10000000
#define     PLL_PIN_ALT_FUNC_PCI_REQGNT1_MASK                      0x20000000
#define         PLL_PIN_ALT_FUNC_IO_RW_NORMAL                       (0UL<<28)
#define         PLL_PIN_ALT_FUNC_PCI_REQGNT1_PIO_7_6                (1UL<<29)

#define GPI_READ_REG         (GPI_BASE + 0x00)
#define GPI_DRIVE_HIGH_REG   (GPI_BASE + 0x04)
#define GPI_DRIVE_LOW_REG    (GPI_BASE + 0x08)
#define GPI_DRIVE_OFF_REG    (GPI_BASE + 0x0C)


#define PLL_ARM_INTFRAC_REG                                    (PLL_BASE + 0x08)
#define     PLL_ARM_INTFRAC_FRAC_MASK                              0x01FFFFFF
#define     PLL_ARM_INTFRAC_FRAC_SHIFT                                    0
#define     PLL_ARM_INTFRAC_INT_MASK                               0x7E000000
#define     PLL_ARM_INTFRAC_INT_SHIFT                                    25

#define PLL_DIV0_REG                                           (PLL_BASE + 0x20)
#define     PLL_DIV0_MEMCLK_POSTDIV_MASK                          0x0000003F
#define     PLL_DIV0_MEMCLK_POSTDIV_SHIFT                                0
#define     PLL_DIV0_ARMCLK_POSTDIV_MASK                          0x00003F00
#define     PLL_DIV0_ARMCLK_POSTDIV_SHIFT                                8

#define PLL_PRESCALE_REG                                       (PLL_BASE + 0x94)
#define     PLL_PRESCALE_ARM_MASK                                  0x00000030
#define     PLL_PRESCALE_ARM_SHIFT                                        4
#define     PLL_PRESCALE_MEM_MASK                                  0x000000C0
#define     PLL_PRESCALE_MEM_SHIFT                                        6
#define     PLL_PRESCALE_USB_MASK                                  0x00000300
#define     PLL_PRESCALE_USB_SHIFT                                        8
#define     PLL_PRESCALE(val) (((val)==0)?2:(((val)==1)?1:(((val)==2)?4:3)))

/* There is no such a register at this address defined in the spec */
#define PLL_LOCK_CMD_REG                                       (PLL_BASE + 0x120)
#define     PLL_LOCK_CMD_MASK                                      0x000000FF
#define     PLL_LOCK_CMD_SHIFT                                            0
#define         PLL_UNLOCK_CMD1                                        0xF8
#define         PLL_UNLOCK_CMD2                                        0x2B
#define         PLL_LOCK_CMD_VALUE                                     0x00

#define PLL_LOCK_STAT_REG                                      (PLL_BASE + 0x124)
#define     PLL_LOCK_STAT_GLOBAL_MASK                              0x00002000
#define         PLL_LOCK_STAT_GLOBAL_WRITE_ENABLE                   (0UL<<13)

#define         PLL_LOCK_STAT_GLOBAL_WRITE_DISABLE                  (1UL<<13)
#define ITC_SATA_POS                   10
#define ITC_SATA_PIC                 (PIC_1)

#define PLL_MEM_INTFRAC_REG                                    (PLL_BASE + 0x0C)
#define     PLL_MEM_INTFRAC_FRAC_MASK                              0x01FFFFFF
#define     PLL_MEM_INTFRAC_FRAC_SHIFT                                    0
#define     PLL_MEM_INTFRAC_INT_MASK                               0x7E000000
#define     PLL_MEM_INTFRAC_INT_SHIFT                                    25

#define PLL_USB_CONFIG_REG                                     (PLL_BASE + 0x10)
#define PLL_USB_INTFRAC_REG                                    (PLL_USB_CONFIG_REG)
#define     PLL_USB_INTFRAC_FRAC_MASK                              0x01FFFFFF
#define     PLL_USB_INTFRAC_FRAC_SHIFT                                    0
#define     PLL_USB_INTFRAC_INT_MASK                               0x7E000000
#define     PLL_USB_INTFRAC_INT_SHIFT                                    25

/********/
/* SATA */
/********/

#define SAT_BASE                                   (HSX_BASE + 0x200000)
#define SAT_DMA_CONFIG_BASE                        (SAT_BASE)
#define SAT_BASE_CH0_START                         (SAT_BASE + 0x100)
#define SAT_BASE_CH1_OFFSET                        (0x100000)
#define SAT_MAX_UNITS                              (1)

/* SATA Bugs */
#define SATA_HAS_HOST_CONTROLLER_LOCKUP_BUG        (NO)
#define SATA_HAS_UNALIGNED_DMA_BUG                 (NO)
#define SATA_HAS_HSX_ERR_DURING_PIO_READ_BUG       (NO)

/* tx_ch_mode rx_ch_mode */
#define SAT_DMA_CH_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x00)
#define SAT_DMA_CH_MODE_RX_REG                     (SAT_DMA_CH_MODE_TX_REG + 0x80)
#define     SAT_DMA_CH_MODE_GO_MASK                                   0x8000
#define     SAT_DMA_CH_MODE_GO_SHIFT                                      15
#define         SAT_DMA_CH_MODE_GO                                   (1UL<<15)
#define     SAT_DMA_CH_MODE_ABORT_MASK                                0x4000
#define     SAT_DMA_CH_MODE_ABORT_SHIFT                                   14
#define         SAT_DMA_CH_MODE_ABORT                                (1UL<<14)
#define     SAT_DMA_CH_MODE_HALT_MASK                                 0x2000
#define     SAT_DMA_CH_MODE_HALT_SHIFT                                    13
#define         SAT_DMA_CH_MODE_HALT                                 (1UL<<13)
#define     SAT_DMA_CH_MODE_INT_ENABLE_MASK                           0x0080
#define     SAT_DMA_CH_MODE_INT_ENABLE_SHIFT                               7
#define         SAT_DMA_CH_MODE_INT_ENABLE_ENABLED                   (1UL<<7)
#define         SAT_DMA_CH_MODE_INT_ENABLE_DISABLED                  (0UL<<7)
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_MASK                     0x0040
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_SHIFT                         6
#define         SAT_DMA_CH_MODE_ABORT_INT_ENABLE_ENABLED             (1UL<<6)
#define         SAT_DMA_CH_MODE_ABORT_INT_ENABLE_DISABLED            (0UL<<6)
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_MASK                      0x0020
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_SHIFT                          5
#define         SAT_DMA_CH_MODE_HALT_INT_ENABLE_ENABLED              (1UL<<5)
#define         SAT_DMA_CH_MODE_HALT_INT_ENABLE_DISABLED             (0UL<<5)
#define     SAT_DMA_CH_MODE_MODE_MASK                                 0x0008
#define     SAT_DMA_CH_MODE_MODE_SHIFT                                     3
#define         SAT_DMA_CH_MODE_MODE_ONLY_BD0                        (0UL<<3)
#define         SAT_DMA_CH_MODE_MODE_BD0_AND_BD1                     (1UL<<3)
#define     SAT_DMA_CH_MODE_BURST_SIZE_MASK                           0x0007
#define     SAT_DMA_CH_MODE_BURST_SIZE_SHIFT                               0
#define         SAT_DMA_CH_MODE_BURST_4_BEAT                         (0UL<<0)
#define         SAT_DMA_CH_MODE_BURST_8_BEAT                         (1UL<<0)
#define         SAT_DMA_CH_MODE_BURST_16_BEAT                        (2UL<<0)
#define         SAT_DMA_CH_MODE_BURST_24_BEAT                        (3UL<<0)
#define         SAT_DMA_CH_MODE_BURST_32_BEAT                        (4UL<<0)
#define         SAT_DMA_CH_MODE_BURST_64_BEAT                        (5UL<<0)
#define         SAT_DMA_CH_MODE_BURST_128_BEAT                       (6UL<<0)
#define         SAT_DMA_CH_MODE_BURST_256_BEAT                       (7UL<<0)

/* tx_b0_mode, tx_b0_al, tx_b0_ah, tx_b0_size */
/* rx_b0_mode, rx_b0_al, rx_b0_ah, rx_b0_size */
#define SAT_DMA_B0_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x08)
#define SAT_DMA_B0_MODE_RX_REG                     (SAT_DMA_B0_MODE_TX_REG + 0x80)
#define     SAT_DMA_B0_MODE_EOP_MASK                                     0x0002
#define     SAT_DMA_B0_MODE_EOP_SHIFT                                         1
#define         SAT_DMA_B0_MODE_EOP                                      (1UL<<1)
#define     SAT_DMA_B0_MODE_ENABLE_MASK                                  0x0001
#define     SAT_DMA_B0_MODE_ENABLE_SHIFT                                      0
#define         SAT_DMA_B0_MODE_ENABLED                                  (1UL<<0)
#define         SAT_DMA_B0_MODE_DISABLED                                 (0UL<<0)
#define         SAT_DMA_B0_MODE_INUSE                                    (1UL<<0)
#define         SAT_DMA_B0_MODE_FREE                                     (0UL<<0)

#define SAT_DMA_B0_ADR_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x10)
#define SAT_DMA_B0_ADR_RX_REG                      (SAT_DMA_B0_ADR_TX_REG + 0x80)
#define SAT_DMA_B0_ADL_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x10)
#define SAT_DMA_B0_ADL_RX_REG                      (SAT_DMA_B0_ADL_TX_REG + 0x80)
#define SAT_DMA_B0_ADH_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x12)
#define SAT_DMA_B0_ADH_RX_REG                      (SAT_DMA_B0_ADH_TX_REG + 0x80)
#define SAT_DMA_B0_SIZE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x18)
#define SAT_DMA_B0_SIZE_RX_REG                     (SAT_DMA_B0_SIZE_TX_REG + 0x80)

/* tx_b1_mode, tx_b1_al, tx_b1_ah, tx_b1_size */
/* rx_b1_mode, rx_b1_al, rx_b1_ah, rx_b1_size */
#define SAT_DMA_B1_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x20)
#define SAT_DMA_B1_MODE_RX_REG                     (SAT_DMA_B1_MODE_TX_REG + 0x80)
#define     SAT_DMA_B1_MODE_EOP_MASK                                     0x0002
#define     SAT_DMA_B1_MODE_EOP_SHIFT                                         1
#define         SAT_DMA_B1_MODE_EOP                                      (1UL<<1)
#define     SAT_DMA_B1_MODE_ENABLE_MASK                                  0x0001
#define     SAT_DMA_B1_MODE_ENABLE_SHIFT                                      0
#define         SAT_DMA_B1_MODE_ENABLED                                  (1UL<<0)
#define         SAT_DMA_B1_MODE_DISABLED                                 (0UL<<0)
#define         SAT_DMA_B1_MODE_INUSE                                    (1UL<<0)
#define         SAT_DMA_B1_MODE_FREE                                     (0UL<<0)
#define SAT_DMA_B1_ADR_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x28)
#define SAT_DMA_B1_ADR_RX_REG                      (SAT_DMA_B1_ADR_TX_REG + 0x80)
#define SAT_DMA_B1_ADL_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x28)
#define SAT_DMA_B1_ADL_RX_REG                      (SAT_DMA_B1_ADL_TX_REG + 0x80)
#define SAT_DMA_B1_ADH_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x2A)
#define SAT_DMA_B1_ADH_RX_REG                      (SAT_DMA_B1_ADH_TX_REG + 0x80)
#define SAT_DMA_B1_SIZE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x30)
#define SAT_DMA_B1_SIZE_RX_REG                     (SAT_DMA_B1_SIZE_TX_REG + 0x80)

/* tx_ch_sts rx_ch_sts */
#define SAT_DMA_CH_STATUS_TX_REG                   (SAT_DMA_CONFIG_BASE + 0x38)
#define SAT_DMA_CH_STATUS_RX_REG                   (SAT_DMA_CH_STATUS_TX_REG + 0x80)
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_MASK                         0x0040
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_SHIFT                             6
#define         SAT_DMA_CH_STATUS_ABORT_INT_ACK_STILLACTIVE              (0UL<<6)
#define         SAT_DMA_CH_STATUS_ABORT_INT_ACK_ABORTED                  (1UL<<6)
#define         SAT_DMA_CH_STATUS_ABORT_INT_ACK_CLEAR                    (1UL<<6)
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_MASK                          0x0020
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_SHIFT                              5
#define         SAT_DMA_CH_STATUS_HALT_INT_ACK_STILLACTIVE               (0UL<<5)
#define         SAT_DMA_CH_STATUS_HALT_INT_ACK_HALTED                    (1UL<<5)
#define         SAT_DMA_CH_STATUS_HALT_INT_ACK_CLEAR                     (1UL<<5)
#define     SAT_DMA_CH_STATUS_B1_EOP_DONE_MASK                           0x0010
#define     SAT_DMA_CH_STATUS_B1_EOP_DONE_SHIFT                               4
#define         SAT_DMA_CH_STATUS_B1_EOP_DONE                            (1UL<<4)
#define         SAT_DMA_CH_STATUS_B1_EOP_CLEAR                           (1UL<<4)
#define     SAT_DMA_CH_STATUS_B1_DONE_MASK                               0x0008
#define     SAT_DMA_CH_STATUS_B1_DONE_SHIFT                                   3
#define         SAT_DMA_CH_STATUS_B1_DONE                                (1UL<<3)
#define         SAT_DMA_CH_STATUS_B1_CLEAR                               (1UL<<3)
#define         SAT_DMA_CH_STATUS_B1_INUSE                               (0UL<<3)
#define     SAT_DMA_CH_STATUS_B0_EOP_DONE_MASK                           0x0004
#define     SAT_DMA_CH_STATUS_B0_EOP_DONE_SHIFT                               2
#define         SAT_DMA_CH_STATUS_B0_EOP_DONE                            (1UL<<2)
#define         SAT_DMA_CH_STATUS_B0_EOP_CLEAR                           (1UL<<2)
#define     SAT_DMA_CH_STATUS_B0_DONE_MASK                               0x0002
#define     SAT_DMA_CH_STATUS_B0_DONE_SHIFT                                   1
#define         SAT_DMA_CH_STATUS_B0_DONE                                (1UL<<1)
#define         SAT_DMA_CH_STATUS_B0_CLEAR                               (1UL<<1)
#define         SAT_DMA_CH_STATUS_B0_INUSE                               (0UL<<1)
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_MASK                         0x0001
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_SHIFT                             0
#define         SAT_DMA_CH_STATUS_ACTIVE_BUFFER_B0                       (0UL<<0)
#define         SAT_DMA_CH_STATUS_ACTIVE_BUFFER_B1                       (1UL<<0)

/* SATA Channel 0 Shadow Block Register 1 */
#define SAT_CH0_SB1_REG                            (SAT_BASE_CH0_START + 0x000)
#define SAT_CH0_SB_DATA_REG                        SAT_CH0_SB1_REG
#define    SAT_CH0_SB_DATA_MASK                               0x0000FFFF
#define    SAT_CH0_SB_DATA_SHIFT                                     0

/* SATA Channel 0 Shadow Block Register 2 */
#define SAT_CH0_SB2_REG                            (SAT_BASE_CH0_START + 0x008)
#define SAT_CH0_SB_FEATURES_REG                    SAT_CH0_SB2_REG
#define    SAT_CH0_SB_FEATURES_MASK                           0x000000FF
#define    SAT_CH0_SB_FEATURES_SHIFT                                 0
/* This register has same address as above one */
#define SAT_CH0_SB_ERROR_REG                       SAT_CH0_SB2_REG
#define    SAT_CH0_SB_ERROR_MASK                              0x000000FF
#define    SAT_CH0_SB_ERROR_SHIFT                                    0

/* SATA Channel 0 Shadow Block Register 3 */
#define SAT_CH0_SB3_REG                            (SAT_BASE_CH0_START + 0x010)
#define SAT_CH0_SB_SECTOR_COUNT_REG                SAT_CH0_SB3_REG
#define    SAT_CH0_SB_SECTOR_COUNT_MASK                       0x000000FF
#define    SAT_CH0_SB_SECTOR_COUNT_SHIFT                             0

/* SATA Channel 0 Shadow Block Register 4 */
#define SAT_CH0_SB4_REG                            (SAT_BASE_CH0_START + 0x018)
#define SAT_CH0_SB_SECTOR_NUM_REG                  SAT_CH0_SB4_REG
#define    SAT_CH0_SB_SECTOR_NUM_MASK                         0x0000FF00
#define    SAT_CH0_SB_SECTOR_NUM_SHIFT                               8
#define SAT_CH0_SB_LBA_LOW_REG                     SAT_CH0_SB4_REG
#define    SAT_CH0_SB_LBA_LOW_MASK                            0x000000FF
#define    SAT_CH0_SB_LBA_LOW_SHIFT                                  0

/* SATA Channel 0 Shadow Block Register 5 */
#define SAT_CH0_SB5_REG                            (SAT_BASE_CH0_START + 0x020)
#define SAT_CH0_SB_CYL_LOW_REG                     SAT_CH0_SB5_REG
#define    SAT_CH0_SB_CYL_LOW_MASK                            0x000000FF
#define    SAT_CH0_SB_CYL_LOW_SHIFT                                  0
#define SAT_CH0_SB_LBA_MID_REG                     SAT_CH0_SB5_REG
#define    SAT_CH0_SB_LBA_MID_MASK                            0x0000FF00
#define    SAT_CH0_SB_LBA_MID_SHIFT                                  8

/* SATA Channel 0 Shadow Block Register 6 */
#define SAT_CH0_SB6_REG                            (SAT_BASE_CH0_START + 0x028)
#define SAT_CH0_SB_CYL_HIGH_REG                    SAT_CH0_SB6_REG
#define    SAT_CH0_SB_CYL_HIGH_MASK                           0x0000FF00
#define    SAT_CH0_SB_CYL_HIGH_SHIFT                                 8
#define SAT_CH0_SB_LBA_HIGH_REG                    SAT_CH0_SB6_REG
#define  SAT_CH0_SB_LBA_HIGH_MASK                             0x000000FF
#define  SAT_CH0_SB_LBA_HIGH_SHIFT                                   0

/* SATA Channel 0 Shadow Block Register 7 */
#define SAT_CH0_SB7_REG                            (SAT_BASE_CH0_START + 0x030)
#define SAT_CH0_SB_DEV_HEAD_REG                    SAT_CH0_SB7_REG
#define    SAT_CH0_SB_DEV_HEAD_MASK                           0x000000FF
#define    SAT_CH0_SB_DEV_HEAD_SHIFT                                 0
#define SAT_CH0_SB_DEVICE_REG                      SAT_CH0_SB7_REG
#define    SAT_CH0_SB_DEVICE_MASK                             0x000000FF
#define    SAT_CH0_SB_DEVICE_SHIFT                                   0

/* SATA Channel 0 Shadow Block Register 8 */
#define SAT_CH0_SB8_REG                            (SAT_BASE_CH0_START + 0x038)
#define SAT_CH0_SB_COMMAND_REG                     SAT_CH0_SB8_REG
#define    SAT_CH0_SB_COMMAND_MASK                            0x000000FF
#define    SAT_CH0_SB_COMMAND_SHIFT                                  0
#define SAT_CH0_SB_STATUS_REG                      SAT_CH0_SB8_REG
#define    SAT_CH0_SB_STATUS_MASK                             0x000000FF
#define    SAT_CH0_SB_STATUS_SHIFT                                   0

/* SATA Channel 0 Shadow Block Register 9 */
#define SAT_CH0_SB9_REG                            (SAT_BASE_CH0_START + 0x040)
#define SAT_CH0_SB_ASTATUS_REG                     SAT_CH0_SB9_REG
#define    SAT_CH0_SB_ASTATUS_MASK                            0x000000FF
#define    SAT_CH0_SB_ASTATUS_SHIFT                                  0
#define SAT_CH0_SB_DEVICE_CONTROL_REG              SAT_CH0_SB9_REG
#define    SAT_CH0_SB_DEVICE_CONTROL_MASK                     0x000000FF
#define    SAT_CH0_SB_DEVICE_CONTROL_SHIFT                           0

/* SATA Channel 0 DMA Request FIFO Water Mark Register: CH0DMAREQWMREG */
#define SAT_CH0_DMA_REQWM_REG                      (SAT_BASE_CH0_START + 0x048)
#define     SAT_CH0_DMA_REQWM_MASK                                   0x000001FF
#define     SAT_CH0_DMA_REQWM_SHIFT                                           0
#define     SAT_CH0_DMA_REQWM_DEFAULT                                     0x0FF

/* SATA Channel 0 FIS Transmission FIFO Water Mark Register: CH0FISTXWMREG */
#define SAT_CH0_FIS_TXWM_REG                       (SAT_BASE_CH0_START + 0x050)
#define     SAT_CH0_FIS_TXWM_MASK                                    0x000001FF
#define     SAT_CH0_FIS_TXWM_SHIFT                                            0
#define     SAT_CH0_FIS_TXWM_DEFAULT                                      0x0FF

/* SATA Channel 0 PIO Status Update FIFO Water Mark Register: CH0PIOSTSWMREG */
#define SAT_CH0_PIO_STSWM_REG                      (SAT_BASE_CH0_START + 0x058)
#define     SAT_CH0_PIO_STSWM_MASK                                   0x000001FF
#define     SAT_CH0_PIO_STSWM_SHIFT                                           0
#define     SAT_CH0_PIO_STSWM_DEFAULT                                     0x0FF

/* SATA Bist FIS Register 1: BISTFISREG1 */
#define SAT_CH0_FIS_BIST1_REG                      (SAT_BASE_CH0_START + 0x060)
#define     SAT_CH0_FIS_BIST1_FE_VENDSPCLB_MASK                          0x0001
#define     SAT_CH0_FIS_BIST1_FE_VENDSPCLB_SHIFT                              0
#define        SAT_CH0_FIS_BIST1_FE_VENDSPCLB_DIS                      (0UL<<0)
#define        SAT_CH0_FIS_BIST1_FE_VENDSPCLB_ENA                      (1UL<<0)
#define     SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_MASK                       0x0004
#define     SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_SHIFT                           2
#define        SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_DIS                   (0UL<<2)
#define        SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_ENA                   (1UL<<2)
#define     SAT_CH0_FIS_BIST1_FE_ANALOGLB_MASK                           0x0008
#define     SAT_CH0_FIS_BIST1_FE_ANALOGLB_SHIFT                               3
#define        SAT_CH0_FIS_BIST1_FE_ANALOGLB_DIS                       (0UL<<3)
#define        SAT_CH0_FIS_BIST1_FE_ANALOGLB_ENA                       (1UL<<3)
#define     SAT_CH0_FIS_BIST1_FE_RETIMEDLB_MASK                          0x0010
#define     SAT_CH0_FIS_BIST1_FE_RETIMEDLB_SHIFT                              4
#define        SAT_CH0_FIS_BIST1_FE_RETIMEDLB_DIS                      (0UL<<4)
#define        SAT_CH0_FIS_BIST1_FE_RETIMEDLB_ENA                      (1UL<<4)
#define     SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_MASK                     0x0020
#define     SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_SHIFT                         5
#define        SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_DIS                 (0UL<<5)
#define        SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_ENA                 (1UL<<5)
#define     SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_MASK                        0x0040
#define     SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_SHIFT                            6
#define        SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_DIS                    (0UL<<6)
#define        SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_ENA                    (1UL<<6)
#define     SAT_CH0_FIS_BIST1_FE_TX_MASK                                 0x0080
#define     SAT_CH0_FIS_BIST1_FE_TX_SHIFT                                     7
#define        SAT_CH0_FIS_BIST1_FE_TX_NO                              (0UL<<7)
#define        SAT_CH0_FIS_BIST1_FE_TX_YES                             (1UL<<7)

/* SATA BIST FIS Register 2: BISTFISREG2 */
#define SAT_CH0_FIS_BIST2_REG                      (SAT_BASE_CH0_START + 0x068)
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_MASK                            0xFFFF
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_SHIFT                                0
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_DEFAULT                         0x0000

/* SATA BIST FIS Register 3: BISTFISREG3 */
#define SAT_CH0_FIS_BIST3_REG                      (SAT_BASE_CH0_START + 0x070)
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_MASK                            0xFFFF
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_SHIFT                                0
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_DEFAULT                         0x0000

/* SATA BIST FIS Register 4: BISTFISREG4 */
#define SAT_CH0_FIS_BIST4_REG                      (SAT_BASE_CH0_START + 0x078)
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_MASK                            0xFFFF
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_SHIFT                                0
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_DEFAULT                         0x0000

/* SATA BIST FIS Register 5: BISTFISREG5 */
#define SAT_CH0_FIS_BIST5_REG                      (SAT_BASE_CH0_START + 0x080)
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_MASK                            0xFFFF
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_SHIFT                                0
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_DEFAULT                         0x0000

/* SATA DMA SETUP FIS Register 1: DMASETUPREG1 */
#define SAT_CH0_DMA_SETUP1_REG                     (SAT_BASE_CH0_START + 0x088)
#define     SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_MASK                        0x0040
#define     SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_SHIFT                            6
#define         SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_ENABLE                  (1<<6)
#define         SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_DISABLE                 (0<<6)
#define     SAT_CH0_DMA_SETUP1_FPDMA_DIR_MASK                            0x0020
#define     SAT_CH0_DMA_SETUP1_FPDMA_DIR_SHIFT                                5
#define         SAT_CH0_DMA_SETUP1_FPDMA_DIR_HOST2DEV                    (1<<5)
#define         SAT_CH0_DMA_SETUP1_FPDMA_DIR_DEV2HOST                    (0<<5)

/* SATA DMA SETUP FIS Register 2: DMASETUPREG2 */
#define SAT_CH0_DMA_SETUP2_REG                     (SAT_BASE_CH0_START + 0x090)
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_MASK                         0xFFFF
#define        SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_TAG_MASK                    0x1F
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_SHIFT                             0
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_DEFAULT                      0x0000

/* SATA DMA SETUP FIS Register 3: DMASETUPREG3 */
#define SAT_CH0_DMA_SETUP3_REG                     (SAT_BASE_CH0_START + 0x098)
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_MASK                         0xFFFF
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_SHIFT                             0
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_DEFAULT                      0x0000

/* SATA DMA SETUP FIS Register 4: DMASETUPREG4 */
#define SAT_CH0_DMA_SETUP4_REG                     (SAT_BASE_CH0_START + 0x0A0)
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_MASK                         0xFFFF
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_SHIFT                             0
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_DEFAULT                      0x0000

/* SATA DMA SETUP FIS Register 5: DMASETUPREG5 */
#define SAT_CH0_DMA_SETUP5_REG                     (SAT_BASE_CH0_START + 0x0A8)
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_MASK                         0xFFFF
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_SHIFT                             0
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_DEFAULT                      0x0000

/* SATA DMA SETUP FIS Register 6: DMASETUPREG6 */
#define SAT_CH0_DMA_SETUP6_REG                     (SAT_BASE_CH0_START + 0x0B0)
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_MASK                           0xFFFF
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_SHIFT                               0
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_DEFAULT                        0x0000

/* SATA DMA SETUP FIS Register 7: DMASETUPREG7 */
#define SAT_CH0_DMA_SETUP7_REG                     (SAT_BASE_CH0_START + 0x0B8)
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_MASK                           0xFFFF
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_SHIFT                               0
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_DEFAULT                        0x0000

/* SATA DMA SETUP FIS Register 8: DMASETUPREG8 */
#define SAT_CH0_DMA_SETUP8_REG                     (SAT_BASE_CH0_START + 0x0C0)
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_MASK                           0xFFFF
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_SHIFT                               0
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_DEFAULT                        0x0000

/* SATA DMA SETUP FIS Register 9: DMASETUPREG9 */
#define SAT_CH0_DMA_SETUP9_REG                     (SAT_BASE_CH0_START + 0x0C8)
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_MASK                           0xFFFF
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_SHIFT                               0
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_DEFAULT                        0x0000

/* SATA Transmit FIS Request Register: TXFISREQREG */
#define SAT_CH0_FIS_TXREQ_REG                      (SAT_BASE_CH0_START + 0x0D0)
#define     SAT_CH0_FIS_TXREQ_DMASETUP_MASK                              0x0002
#define     SAT_CH0_FIS_TXREQ_DMASETUP_SHIFT                                  1
#define        SAT_CH0_FIS_TXREQ_DMASETUP_                             (1UL<<1)
#define     SAT_CH0_FIS_TXREQ_BIST_MASK                                  0x0001
#define     SAT_CH0_FIS_TXREQ_BIST_SHIFT                                      0
#define        SAT_CH0_FIS_TXREQ_BIST_ENA                              (1UL<<0)

/* SATA BIST Mode Control Register: BISTMODECTRLREG */
#define SAT_CH0_BIST_CONTROL_REG                   (SAT_BASE_CH0_START + 0x0D8)
#define     SAT_CH0_BIST_CONTROL_HOSTACCESS_MASK                         0x0001
#define     SAT_CH0_BIST_CONTROL_HOSTACCESS_SHIFT                             0
#define        SAT_CH0_BIST_CONTROL_HOSTACCESS_NO                      (0UL<<0)
#define        SAT_CH0_BIST_CONTROL_HOSTACCESS_YES                     (1UL<<0)
#define     SAT_CH0_BIST_CONTROL_LOADING_MASK                            0x0002
#define     SAT_CH0_BIST_CONTROL_LOADING_SHIFT                                1
#define        SAT_CH0_BIST_CONTROL_LOADING_NO                         (0UL<<1)
#define        SAT_CH0_BIST_CONTROL_LOADING_YES                        (1UL<<1)
#define     SAT_CH0_BIST_CONTROL_VALIDMODE_MASK                          0x0004
#define     SAT_CH0_BIST_CONTROL_VALIDMODE_SHIFT                              2
#define         SAT_CH0_BIST_CONTROL_VALIDMODE                           (1<<2)
#define     SAT_CH0_BIST_CONTROL_INVALIDMODE_MASK                        0x0008
#define     SAT_CH0_BIST_CONTROL_INVALIDMODE_SHIFT                            3
#define         SAT_CH0_BIST_CONTROL_INVALIDMODE                         (1<<3)
#define     SAT_CH0_BIST_CONTROL_FISRCVD_MASK                            0x0010
#define     SAT_CH0_BIST_CONTROL_FISRCVD_SHIFT                                4
#define         SAT_CH0_BIST_CONTROL_FISRCVD_CLEAR                       (1<<4)

/* SATA FPDMA Control Register: FPDMACTRLREG */
#define SAT_CH0_FPDMA_CONTROL_REG                  (SAT_BASE_CH0_START + 0x0E0)
#define     SAT_CH0_FPDMA_CONTROL_HOSTACCESS_MASK                        0x0001
#define     SAT_CH0_FPDMA_CONTROL_HOSTACCESS_SHIFT                            0
#define        SAT_CH0_FPDMA_CONTROL_HOSTACCESS_ENA                    (0UL<<0)
#define     SAT_CH0_FPDMA_CONTROL_LOADING_MASK                           0x0002
#define     SAT_CH0_FPDMA_CONTROL_LOADING_SHIFT                               1
#define        SAT_CH0_FPDMA_CONTROL_LOADING_NO                        (0UL<<1)
#define        SAT_CH0_FPDMA_CONTROL_LOADING_YES                       (1UL<<1)
#define     SAT_CH0_FPDMA_CONTROL_VALIDDMA_MASK                          0x0004
#define     SAT_CH0_FPDMA_CONTROL_VALIDDMA_SHIFT                              2
#define         SAT_CH0_FPDMA_CONTROL_VALIDDMA                           (1<<2)
#define     SAT_CH0_FPDMA_CONTROL_INVALIDDMA_MASK                        0x0008
#define     SAT_CH0_FPDMA_CONTROL_INVALIDDMA_SHIFT                            3
#define         SAT_CH0_FPDMA_CONTROL_INVALIDDMA                         (1<<3)
#define     SAT_CH0_FPDMA_CONTROL_DMARCVD_MASK                           0x0010
#define     SAT_CH0_FPDMA_CONTROL_DMARCVD_SHIFT                               4
#define         SAT_CH0_FPDMA_CONTROL_DMARCVD_CLEAR                      (1<<4)

/* SATA FIS Transmission Retry Control Register: FISTXRTRYREG */
#define SAT_CH0_FIS_TX_RETRY_REG                   (SAT_BASE_CH0_START + 0x0E8)
#define     SAT_CH0_FIS_TX_RETRY_ENABLE_MASK                             0x0001
#define     SAT_CH0_FIS_TX_RETRY_ENABLE_SHIFT                                 0
#define         SAT_CH0_FIS_TX_RETRY_ENABLED                             (1<<0)
#define         SAT_CH0_FIS_TX_RETRY_DISABLED                            (0<<0)
#define     SAT_CH0_FIS_TX_RETRY_STATUS_MASK                             0x0002
#define     SAT_CH0_FIS_TX_RETRY_STATUS_SHIFT                                 1
#define        SAT_CH0_FIS_TX_RETRY_STATUS_YES                         (1UL<<1)

/* SATA Data Transfer Control Register: DATTSFRCTRLREG */
#define SAT_CH0_DATA_XFER_CONTROL_REG              (SAT_BASE_CH0_START + 0x0F0)
#define     SAT_CH0_DATA_XFER_CONTROL_ABORTRX_MASK                       0x0001
#define     SAT_CH0_DATA_XFER_CONTROL_ABORTRX_SHIFT                           0
#define         SAT_CH0_DATA_XFER_CONTROL_ABORTRX                        (1<<0)

/* SATA FIFO Control Register: FIFCTRLREG */
#define SAT_CH0_FIFO_CONTROL_REG                   (SAT_BASE_CH0_START + 0x0F8)
#define     SAT_CH0_FIFO_CONTROL_FLUSHTX_MASK                            0x0001
#define     SAT_CH0_FIFO_CONTROL_FLUSHTX_SHIFT                                0
#define         SAT_CH0_FIFO_CONTROL_FLUSHTX                             (1<<0)
#define     SAT_CH0_FIFO_CONTROL_FLUSHRX_MASK                            0x0002
#define     SAT_CH0_FIFO_CONTROL_FLUSHRX_SHIFT                                1
#define         SAT_CH0_FIFO_CONTROL_FLUSHRX                             (1<<1)

/* SATA FIFO Status Register: FIFOSTATREG */
#define SAT_CH0_FIFO_STATUS_REG                    (SAT_BASE_CH0_START + 0x100)
#define     SAT_CH0_FIFO_STATUS_TXFIFOFULL_MASK                          0x8000
#define     SAT_CH0_FIFO_STATUS_TXFIFOFULL_SHIFT                             15
#define         SAT_CH0_FIFO_STATUS_TXFIFOFULL                          (1<<15)
#define     SAT_CH0_FIFO_STATUS_TXFIFOCOUNT_MASK                         0x3F00
#define     SAT_CH0_FIFO_STATUS_TXFIFOCOUNT_SHIFT                             8
#define     SAT_CH0_FIFO_STATUS_RXFIFOEMPTY_MASK                           0x80
#define     SAT_CH0_FIFO_STATUS_RXFIFOEMPTY_SHIFT                             7
#define         SAT_CH0_FIFO_STATUS_RXFIFOEMPTY                          (1<<7)
#define     SAT_CH0_FIFO_STATUS_RXFIFOCOUNT_MASK                           0x3F
#define     SAT_CH0_FIFO_STATUS_RXFIFOCOUNT_SHIFT                             0

/* SATA HOLD Configuration Register: HOLDCFGREG */
/* This register is for Rev C and later devices */
#define SAT_CH0_HOLD_CONFIG_REG                    (SAT_BASE_CH0_START + 0x108)
#define     SAT_CH0_HOLD_CONFIG_THRESHOLD_MASK                             0xFF
#define     SAT_CH0_HOLD_CONFIG_THRESHOLD_SHIFT                               0

/* SATA Serial ATA Interrupt Enable Register: SIENBREG1 */
#define SAT_CH0_INT_ENABLE_REG                     (SAT_BASE_CH0_START + 0x110)
#define     SAT_CH0_INT_ENABLE_PHYRDYCHNG_MASK                           0x0001
#define     SAT_CH0_INT_ENABLE_PHYRDYCHNG_SHIFT                               0
#define        SAT_CH0_INT_ENABLE_PHYRDYCHNG                           (1UL<<0)
#define        SAT_CH0_INT_DISABLE_PHYRDYCHNG                          (0UL<<0)
#define     SAT_CH0_INT_ENABLE_PHYINTERR_MASK                            0x0002
#define     SAT_CH0_INT_ENABLE_PHYINTERR_SHIFT                                1
#define        SAT_CH0_INT_ENABLE_PHYINTERR                            (1UL<<1)
#define        SAT_CH0_INT_DISABLE_PHYINTERR                           (0UL<<1)
#define     SAT_CH0_INT_ENABLE_COMWAKDET_MASK                            0x0004
#define     SAT_CH0_INT_ENABLE_COMWAKDET_SHIFT                                2
#define        SAT_CH0_INT_ENABLE_COMWAKDET                            (1UL<<2)
#define     SAT_CH0_INT_ENABLE_CRCERRRCVD_MASK                           0x0008
#define     SAT_CH0_INT_ENABLE_CRCERRRCVD_SHIFT                               3
#define        SAT_CH0_INT_ENABLE_CRCERRRCVD                           (1UL<<3)
#define     SAT_CH0_INT_ENABLE_HSKERRRCVD_MASK                           0x0010
#define     SAT_CH0_INT_ENABLE_HSKERRRCVD_SHIFT                               4
#define        SAT_CH0_INT_ENABLE_HSKERRRCVD                           (1UL<<4)
#define     SAT_CH0_INT_ENABLE_LNKSEQERR_MASK                            0x0020
#define     SAT_CH0_INT_ENABLE_LNKSEQERR_SHIFT                                5
#define        SAT_CH0_INT_ENABLE_LNKSEQERR                            (1UL<<5)
#define     SAT_CH0_INT_ENABLE_TRNSPTSEQERR_MASK                         0x0040
#define     SAT_CH0_INT_ENABLE_TRNSPTSEQERR_SHIFT                             6
#define        SAT_CH0_INT_ENABLE_TRNSPTSEQERR                         (1UL<<6)
#define     SAT_CH0_INT_ENABLE_UNRECFISRCVD_MASK                         0x0080
#define     SAT_CH0_INT_ENABLE_UNRECFISRCVD_SHIFT                             7
#define        SAT_CH0_INT_ENABLE_UNRECFISRCVD                         (1UL<<7)
#define     SAT_CH0_INT_ENABLE_PMODECHANGE_MASK                          0x0100
#define     SAT_CH0_INT_ENABLE_PMODECHANGE_SHIFT                              8
#define        SAT_CH0_INT_ENABLE_PMODECHANGE                          (1UL<<8)
#define     SAT_CH0_INT_ENABLE_BISTRCVD_MASK                             0x0200
#define     SAT_CH0_INT_ENABLE_BISTRCVD_SHIFT                                 9
#define        SAT_CH0_INT_ENABLE_BISTRCVD                             (1UL<<9)
#define     SAT_CH0_INT_ENABLE_FPDMARCVD_MASK                            0x0400
#define     SAT_CH0_INT_ENABLE_FPDMARCVD_SHIFT                               10
#define        SAT_CH0_INT_ENABLE_FPDMARCVD                            (1UL<<10)
#define     SAT_CH0_INT_ENABLE_FISTXRETRY_MASK                           0x0800
#define     SAT_CH0_INT_ENABLE_FISTXRETRY_SHIFT                              11
#define        SAT_CH0_INT_ENABLE_FISTXRETRY                           (1UL<<11)

/* SATA Port 0 Serial ATA Power Mode Status and Control Register: P0PMREG */
#define SAT_CH0_POWERMODE_REG                      (SAT_BASE_CH0_START + 0x118)
#define     SAT_CH0_POWERMODE_SLUMBERSTATUS_MASK                         0x0001
#define     SAT_CH0_POWERMODE_SLUMBERSTATUS_SHIFT                             0
#define         SAT_CH0_POWERMODE_SLUMBERSTATUS_CLEAR                    (1<<0)
#define     SAT_CH0_POWERMODE_PARTIALSTATUS_MASK                         0x0002
#define     SAT_CH0_POWERMODE_PARTIALSTATUS_SHIFT                             1
#define         SAT_CH0_POWERMODE_PARTIALSTATUS_CLEAR                    (1<<1)
#define     SAT_CH0_POWERMODE_SLUMBERREQ_MASK                            0x0004
#define     SAT_CH0_POWERMODE_SLUMBERREQ_SHIFT                                2
#define        SAT_CH0_POWERMODE_SLUMBERREQ                              (1<<2)
#define     SAT_CH0_POWERMODE_PARTIALREQ_MASK                            0x0008
#define     SAT_CH0_POWERMODE_PARTIALREQ_SHIFT                                3
#define        SAT_CH0_POWERMODE_PARTIALREQ                              (1<<3)
#define     SAT_CH0_POWERMODE_WAKEUPREQ_MASK                             0x0010
#define     SAT_CH0_POWERMODE_WAKEUPREQ_SHIFT                                 4
#define        SAT_CH0_POWERMODE_WAKEUPREQ                               (1<<4)
#define     SAT_CH0_POWERMODE_CHANGESTATUS_MASK                          0x0020
#define     SAT_CH0_POWERMODE_CHANGESTATUS_SHIFT                              5
#define         SAT_CH0_POWERMODE_CHANGESTATUS_CLEAR                     (1<<5)

/* SATA Port 0 Serial ATA Interface Status Register 1: P0SISREG1 */
#define SAT_CH0_SCR0_SSTATUS_REG                   (SAT_BASE_CH0_START + 0x120)
#define     SAT_CH0_SCR0_SSTATUS_DET_MASK                                0x000F
#define     SAT_CH0_SCR0_SSTATUS_DET_SHIFT                                    0
#define         SAT_CH0_SCR0_SSTATUS_DET_NOTPRESENT                      (0<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_NOTESTABLISHED                  (0<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICENOPHY                     (1<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICEANDPHY                    (2<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICEANDPHY1                   (3<<0)   /* ? */
#define         SAT_CH0_SCR0_SSTATUS_DET_PHYOFFLINE                      (4<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_PHYINBISTLB                     (4<<0)
#define     SAT_CH0_SCR0_SSTATUS_SPD_MASK                                0x00F0
#define     SAT_CH0_SCR0_SSTATUS_SPD_SHIFT                                    4
#define         SAT_CH0_SCR0_SSTATUS_SPD_NOTPRESENT                      (0<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_NOTESTABLISHED                  (0<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_GEN1                            (1<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_GEN2                            (2<<4)   /* ? */
#define     SAT_CH0_SCR0_SSTATUS_IPM_MASK                                0x0F00
#define     SAT_CH0_SCR0_SSTATUS_IPM_SHIFT                                    8
#define         SAT_CH0_SCR0_SSTATUS_IPM_NOTPRESENT                      (0<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_NOTESTABLISHED                  (0<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_ACTIVE                          (1<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_PARTIALPM                       (2<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_SLUMBERPM                       (4<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_SLUMBERPM1                      (6<<8)

/* Note that the 32-bit SERROR Register is split */
/* into two separate 16-bit registers!           */

/* P0SIEREG1 */
#define SAT_CH0_SCR1_SERROR_ERR_REG                (SAT_BASE_CH0_START + 0x128)
#define     SAT_CH0_SCR1_SERROR_ERR_I_DINTREC_MASK                       0x0001
#define     SAT_CH0_SCR1_SERROR_ERR_I_DINTREC_SHIFT                           0
#define     SAT_CH0_SCR1_SERROR_ERR_M_COMMREC_MASK                       0x0002
#define     SAT_CH0_SCR1_SERROR_ERR_M_COMMREC_SHIFT                           1
#define     SAT_CH0_SCR1_SERROR_ERR_T_TRANSDINT_MASK                     0x0100
#define     SAT_CH0_SCR1_SERROR_ERR_T_TRANSDINT_SHIFT                         8
#define     SAT_CH0_SCR1_SERROR_ERR_C_PERSCOM_MASK                       0x0200
#define     SAT_CH0_SCR1_SERROR_ERR_C_PERSCOM_SHIFT                           9
#define     SAT_CH0_SCR1_SERROR_ERR_P_PROTOCOL_MASK                      0x0400
#define     SAT_CH0_SCR1_SERROR_ERR_P_PROTOCOL_SHIFT                         10
#define     SAT_CH0_SCR1_SERROR_ERR_E_INT_MASK                           0x0800
#define     SAT_CH0_SCR1_SERROR_ERR_E_INT_SHIFT                              11

/* P0SIEREG2 */
#define SAT_CH0_SCR1_SERROR_DIAG_REG               (SAT_BASE_CH0_START + 0x130)
#define     SAT_CH0_SCR1_SERROR_DIAG_N_PHYRDY_MASK                       0x0001
#define     SAT_CH0_SCR1_SERROR_DIAG_N_PHYRDY_SHIFT                           0
#define     SAT_CH0_SCR1_SERROR_DIAG_I_PHYINT_MASK                       0x0002
#define     SAT_CH0_SCR1_SERROR_DIAG_I_PHYINT_SHIFT                           1
#define     SAT_CH0_SCR1_SERROR_DIAG_W_COMMWAKE_MASK                     0x0004
#define     SAT_CH0_SCR1_SERROR_DIAG_W_COMMWAKE_SHIFT                         2
#define     SAT_CH0_SCR1_SERROR_DIAG_B_DECODE_MASK                       0x0008
#define     SAT_CH0_SCR1_SERROR_DIAG_B_DECODE_SHIFT                           3
#define     SAT_CH0_SCR1_SERROR_DIAG_D_DISPARITY_MASK                    0x0010
#define     SAT_CH0_SCR1_SERROR_DIAG_D_DISPARITY_SHIFT                        4
#define     SAT_CH0_SCR1_SERROR_DIAG_C_CRC_MASK                          0x0020
#define     SAT_CH0_SCR1_SERROR_DIAG_C_CRC_SHIFT                              5
#define     SAT_CH0_SCR1_SERROR_DIAG_H_HANDSHAKE_MASK                    0x0040
#define     SAT_CH0_SCR1_SERROR_DIAG_H_HANDSHAKE_SHIFT                        6
#define     SAT_CH0_SCR1_SERROR_DIAG_S_LINKSEQ_MASK                      0x0080
#define     SAT_CH0_SCR1_SERROR_DIAG_S_LINKSEQ_SHIFT                          7
#define     SAT_CH0_SCR1_SERROR_DIAG_T_TRNSPTSEQ_MASK                    0x0100
#define     SAT_CH0_SCR1_SERROR_DIAG_T_TRNSPTSEQ_SHIFT                        8
#define     SAT_CH0_SCR1_SERROR_DIAG_F_UNRECFIS_MASK                     0x0200
#define     SAT_CH0_SCR1_SERROR_DIAG_F_UNRECFIS_SHIFT                         9

/* P0SICREG1 */
#define SAT_CH0_SCR2_SCONTROL_REG                  (SAT_BASE_CH0_START + 0x138)
#define     SAT_CH0_SCR2_SCONTROL_DET_MASK                               0x000F
#define     SAT_CH0_SCR2_SCONTROL_DET_SHIFT                                   0
#define         SAT_CH0_SCR2_SCONTROL_DET_NODETECT                       (0<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_REINIT                         (1<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_RESET                          (1<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_DISABLE                        (4<<0)
#define     SAT_CH0_SCR2_SCONTROL_SPD_MASK                               0x00F0
#define     SAT_CH0_SCR2_SCONTROL_SPD_SHIFT                                   4
#define         SAT_CH0_SCR2_SCONTROL_SPD_NORESTRICT                     (0<<4)
#define         SAT_CH0_SCR2_SCONTROL_SPD_RESTRICT_GEN1                  (1<<4)
#define     SAT_CH0_SCR2_SCONTROL_IPM_MASK                               0x0F00
#define     SAT_CH0_SCR2_SCONTROL_IPM_SHIFT                                   8
#define         SAT_CH0_SCR2_SCONTROL_IPM_NORESTRICT                     (0<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIAL                      (1<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOSLUMBER                      (2<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIALSLUMBER               (4<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIALSLUMBER1              (3<<8)

/* SATA Reserved Registers 3 through 15 here */
#define SAT_CH0_SCR3_RESERVED_REG                  (SAT_BASE_CH0_START + 0x140)
#define SAT_CH0_SCR4_RESERVED_REG                  (SAT_BASE_CH0_START + 0x148)
#define SAT_CH0_SCR5_RESERVED_REG                  (SAT_BASE_CH0_START + 0x150)
#define SAT_CH0_SCR6_RESERVED_REG                  (SAT_BASE_CH0_START + 0x158)
#define SAT_CH0_SCR7_RESERVED_REG                  (SAT_BASE_CH0_START + 0x160)
#define SAT_CH0_SCR8_RESERVED_REG                  (SAT_BASE_CH0_START + 0x168)
#define SAT_CH0_SCR9_RESERVED_REG                  (SAT_BASE_CH0_START + 0x170)
#define SAT_CH0_SCR10_RESERVED_REG                 (SAT_BASE_CH0_START + 0x178)
#define SAT_CH0_SCR11_RESERVED_REG                 (SAT_BASE_CH0_START + 0x180)
#define SAT_CH0_SCR12_RESERVED_REG                 (SAT_BASE_CH0_START + 0x188)
#define SAT_CH0_SCR13_RESERVED_REG                 (SAT_BASE_CH0_START + 0x190)
#define SAT_CH0_SCR14_RESERVED_REG                 (SAT_BASE_CH0_START + 0x198)
#define SAT_CH0_SCR15_RESERVED_REG                 (SAT_BASE_CH0_START + 0x1A0)

#define SAT_CH0_EXT_IENB_REG                       (SAT_BASE_CH0_START + 0x1A8)
#define SAT_CH0_EXT_STS_REG                        (SAT_BASE_CH0_START + 0x1A8)
#define    SAT_CH0_EXT_STS_FPDMA_TSFREND_MASK                            0x0002
#define    SAT_CH0_EXT_STS_FPDMA_TSFREND_SHIFT                                1
#define       SAT_CH0_EXT_STS_FPDMA_TSFREND_CLEAR                        (1<<1)
#define       SAT_CH0_EXT_STS_FPDMA_TSFREND_END                          (1<<1)
#define       SAT_CH0_EXT_STS_FPDMA_TSFREND_ACTIVE                       (0<<1)
#define    SAT_CH0_EXT_STS_DRIVE_MASK                                    0x0001
#define    SAT_CH0_EXT_STS_DRIVE_SHIFT                                        0


#define SAT_VSP_BASE                               (ASX1_BASE + 0xEC000)
#define    SAT_VSP_00_REG                          (SAT_VSP_BASE + 0x00)
#define       SAT_VSP_00_60MHZ_INIT                                    0x020000
#define       SAT_VSP_00_74MHZ_INIT                                    0x020000
#define       SAT_VSP_00_CLK_DTCT_OVR_ENA_MASK                         0x080000
#define       SAT_VSP_00_CLK_DTCT_OVR_ENA_SHIFT                              19
#define       SAT_VSP_00_CLK_DTCT_OVR_ENA_DISABLED                      (0<<19)
#define       SAT_VSP_00_CLK_DTCT_OVR_ENA_ENABLED                       (1<<19)
#define       SAT_VSP_00_CLK_DTCT_OVR_MASK                             0x040000
#define       SAT_VSP_00_CLK_DTCT_OVR_SHIFT                                  18
#define       SAT_VSP_00_CLK_DTCT_OVR_FORCE_74                          (0<<18)
#define       SAT_VSP_00_CLK_DTCT_OVR_FORCE_60                          (1<<18)
#define    SAT_VSP_01_REG                          (SAT_VSP_BASE + 0x04)
#define       SAT_VSP_01_60MHZ_INIT                                    0x000800
#define       SAT_VSP_01_74MHZ_INIT                                    0x000800
#define    SAT_VSP_02_REG                          (SAT_VSP_BASE + 0x08)
#define       SAT_VSP_02_60MHZ_INIT                                    0x001FFF
#define       SAT_VSP_02_74MHZ_INIT                                    0x002772
#define    SAT_VSP_03_REG                          (SAT_VSP_BASE + 0x0C)
#define       SAT_VSP_03_60MHZ_INIT                                    0x6ACD93
#define       SAT_VSP_03_74MHZ_INIT                                    0x6DD097
#define    SAT_VSP_04_REG                          (SAT_VSP_BASE + 0x10)
#define       SAT_VSP_04_60MHZ_INIT                                    0x6B4889
#define       SAT_VSP_04_74MHZ_INIT                                    0x6DCA8B
#define    SAT_VSP_05_REG                          (SAT_VSP_BASE + 0x14)
#define       SAT_VSP_05_60MHZ_INIT                                    0x1A0C00
#define       SAT_VSP_05_74MHZ_INIT                                    0x1A0C00
#define    SAT_VSP_06_REG                          (SAT_VSP_BASE + 0x18)
#define       SAT_VSP_06_60MHZ_INIT                                    0x114AFD
#define       SAT_VSP_06_74MHZ_INIT                                    0x114AFD
#define    SAT_VSP_07_REG                          (SAT_VSP_BASE + 0x1C)
#define       SAT_VSP_07_60MHZ_INIT                                    0x003D58
#define       SAT_VSP_07_74MHZ_INIT                                    0x002738
#define    SAT_VSP_08_REG                          (SAT_VSP_BASE + 0x20)
#define       SAT_VSP_08_60MHZ_INIT                                    0x000000
#define       SAT_VSP_08_74MHZ_INIT                                    0x000000
#define    SAT_VSP_09_REG                          (SAT_VSP_BASE + 0x24)
#define       SAT_VSP_09_60MHZ_INIT                                    0xC55C00
#define       SAT_VSP_09_74MHZ_INIT                                    0xC55C00
#define    SAT_VSP_10_REG                          (SAT_VSP_BASE + 0x28)
#define       SAT_VSP_10_60MHZ_INIT                                    0x030251
#define       SAT_VSP_10_74MHZ_INIT                                    0x000250
#define    SAT_VSP_11_REG                          (SAT_VSP_BASE + 0x2C)
#define       SAT_VSP_11_60MHZ_INIT                                    0xBBF000
#define       SAT_VSP_11_74MHZ_INIT                                    0xB97000
#define    SAT_VSP_12_REG                          (SAT_VSP_BASE + 0x30)
#define       SAT_VSP_12_60MHZ_INIT                                    0x000471
#define       SAT_VSP_12_74MHZ_INIT                                    0x000434
#define    SAT_VSP_13_REG                          (SAT_VSP_BASE + 0x34)
#define       SAT_VSP_13_60MHZ_INIT                                    0x000000
#define       SAT_VSP_13_74MHZ_INIT                                    0x000000
#define    SAT_VSP_14_REG                          (SAT_VSP_BASE + 0x38)
#define       SAT_VSP_14_60MHZ_INIT                                    0x000001
#define       SAT_VSP_14_74MHZ_INIT                                    0x000001
#define    SAT_VSP_15_REG                          (SAT_VSP_BASE + 0x3C)
#define       SAT_VSP_15_RX_LOCKED_MASK                                0x000010
#define       SAT_VSP_15_RX_LOCKED_SHIFT                                      4
#define       SAT_VSP_15_RX_LOCKED                                       (1<<4)
#define       SAT_VSP_15_CLOCK_DETECT_MASK                             0x000040
#define       SAT_VSP_15_CLOCK_DETECT_SHIFT                                   6
#define       SAT_VSP_15_CLOCK_DETECT_60MHZ                              (1<<6)
#define       SAT_VSP_15_CLOCK_DETECT_75MHZ                              (0<<6)

#define    SAT_VSP_16_REG                          (SAT_VSP_BASE + 0x40)
#define    SAT_VSP_17_REG                          (SAT_VSP_BASE + 0x44)
#define    SAT_VSP_18_REG                          (SAT_VSP_BASE + 0x48)
#define    SAT_VSP_19_REG                          (SAT_VSP_BASE + 0x4C)


#define ITC_DMA                   0x00000010
#define ITC_DMA_POS                     4
#define ITC_DMA_PIC                  (PIC_1)
#define DMA_BASE                           (HSX_BASE + 0x20000)
#define DMA_CHANNEL_BASE                   (DMA_BASE + 0x100)
#define DMA_INT_REG                        (DMA_BASE + 0x8)
/*******/
/* DMA */
/*******/

#define DMA_BASE                           (HSX_BASE + 0x20000)
#define DMA_CHANNEL_BASE                   (DMA_BASE + 0x100)
#define DMA_CHANNEL_BANK_SIZE              (0x100)
#define DMA_CHANNEL_CTRL_REG_OFFSET        (0x0)
#define DMA_CHANNEL_STATUS_REG_OFFSET      (0x4)
#define DMA_CHANNEL_INT_ENABLE_REG_OFFSET  (0x8)
#define DMA_CHANNEL_RECOVERY_REG_OFFSET    (0xC)
#define DMA_CHANNEL_SRC_ADDR_REG_OFFSET    (0x10)
#define DMA_CHANNEL_DST_ADDR_REG_OFFSET    (0x14)
#define DMA_CHANNEL_SRC_BASE_REG_OFFSET    (0x20)
#define DMA_CHANNEL_SRC_RUN_BUF_SIZE_REG_OFFSET (0x24)
#define DMA_CHANNEL_SRC_STRIDE_INT_REG_OFFSET   (0x28)
#define DMA_CHANNEL_SRC_NUM_STRIDES_REG_OFFSET  (0x2C)
#define DMA_CHANNEL_DEST_BASE_ADDR_REG_OFFSET   (0x30)
#define DMA_CHANNEL_DEST_RUN_BUF_SIZE_REG_OFFSET (0x34)
#define DMA_CHANNEL_DEST_STRIDE_INT_REG_OFFSET   (0x38)
#define DMA_CHANNEL_DEST_NUM_STRIDE_BUF_REG_OFFSET (0x3C)

/* General DMA registers and definitions */
#define DMA_CAPABILITY_REG                 (DMA_BASE + 0)
#define     DMA_CAP_CHANNELS_MASK          (0x00000007)
#define     DMA_CAP_CHANNELS_SHIFT         (0)
#define DMA_INT_REG                        (DMA_BASE + 0x8)
#define     DMA_INT_CH3_INT_MASK           (0x00000008)
#define     DMA_INT_CH3_INT_SHIFT          (3)
#define     DMA_INT_CH2_INT_MASK           (0x00000004)
#define     DMA_INT_CH2_INT_SHIFT          (2)
#define     DMA_INT_CH1_INT_MASK           (0x00000002)
#define     DMA_INT_CH1_INT_SHIFT          (1)
#define     DMA_INT_CH0_INT_MASK           (0x00000001)
#define     DMA_INT_CH0_INT_SHIFT          (0)
#define DMA_REQSTAT_REG                    (DMA_BASE + 0xC)
#define     DMA_REQSTAT_MASK               (0xFFFFFFFF)
#define     DMA_REQSTAT_SHIFT              (0)
/* DMA channel specific macros and definitions */
#define DMA_CTRL_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_CTRL_REG_OFFSET)
#define     DMA_CTRL_PACKET_SIZE_MASK      (0xFF000000)
#define     DMA_CTRL_PACKET_SIZE_SHIFT     (24)
#define     DMA_CTRL_DMA_REQUEST_MASK      (0x00FF0000)
#define     DMA_CTRL_DMA_REQUEST_SHIFT     (16)
/* Todo defines for which periferals */
#define     DMA_CTRL_INC_DEST_ADDR_MASK    (0x00000200)
#define     DMA_CTRL_INC_DEST_ADDR_SHIFT   (9)
#define         DMA_INC_DEST_ADDR          (DMA_CTRL_INC_DEST_ADDR_MASK)
#define     DMA_CTRL_INC_SRC_ADDR_MASK     (0x00000100)
#define     DMA_CTRL_INC_SRC_ADDR_SHIFT    (8)
#define         DMA_INC_SRC_ADDR           (DMA_CTRL_INC_SRC_ADDR_MASK)
#define     DMA_CTRL_DEST_XFER_SIZE_MASK   (0x000000C0)
#define     DMA_CTRL_DEST_XFER_SIZE_SHIFT  (6)
#define         DMA_XFER_SIZE_BYTE         (0)
#define         DMA_XFER_SIZE_HALF         (1)
#define         DMA_XFER_SIZE_WORD         (2)
#define     DMA_CTRL_SRC_XFER_SIZE_MASK    (0x00000030)
#define     DMA_CTRL_SRC_XFER_SIZE_SHIFT   (4)
#define     DMA_CTRL_XFER_MODE_MASK        (0x00000008)
#define     DMA_CTRL_XFER_MODE_SHIFT       (3)
#define         DMA_XFER_MODE_STOP         (1) /* not sure */
#define         DMA_XFER_MODE_NO_STOP      (0) /* not sure */
#define     DMA_CTRL_PREEMPT_MASK          (0x00000004)
#define     DMA_CTRL_PREEMPT_SHIFT         (2)
#define         DMA_PREEMPT_ENABLE         (1)
#define         DMA_PREEMPT_DISABLE        (0)
#define     DMA_CTRL_ENABLE_MASK           (0x00000001)
#define     DMA_CTRL_ENABLE_SHIFT          (0)
#define         DMA_ENABLE                 (1)
#define         DMA_DISABLE                (0)

#define DMA_STATUS_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_STATUS_REG_OFFSET)
#define     DMA_STATUS_REG_STATUS_MASK     (0x0000007F)
#define     DMA_STATUS_REG_STATUS_SHIFT    (0)
#define     DMA_STATUS_DMA_READ_ERROR      (1<<6)
#define     DMA_STATUS_DMA_WRITE_ERROR     (1<<5)
#define     DMA_STATUS_DMA_DEST_BUFF_INT   (1<<4)
#define     DMA_STATUS_DMA_SRC_BUFF_INT    (1<<3)
#define     DMA_STATUS_DMA_DEST_BUFF_FULL  (1<<2)
#define     DMA_STATUS_DMA_SRC_BUFF_EMPTY  (1<<1)
#define     DMA_STATUS_DMA_ACTIVE          (1<<0)
#define DMA_INT_ENABLE_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_INT_ENABLE_REG_OFFSET)
#define     DMA_INT_ENABLE_REG_INTS_MASK   (0x0000007F)
#define     DMA_INT_ENABLE_REG_INTS_SHIFT  (0)
#define     DMA_INT_ENABLE_DMA_READ_ERROR      (1<<6)
#define     DMA_INT_ENABLE_DMA_WRITE_ERROR     (1<<5)
#define     DMA_INT_ENABLE_DMA_DEST_BUFF_INT   (1<<4)
#define     DMA_INT_ENABLE_DMA_SRC_BUFF_INT    (1<<3)
#define     DMA_INT_ENABLE_DMA_DEST_BUFF_FULL  (1<<2)
#define     DMA_INT_ENABLE_DMA_SRC_BUFF_EMPTY  (1<<1)
#define     DMA_INT_ENABLE_DMA_ACTIVE          (1<<0)

#define DMA_RECOVERY_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_RECOVERY_REG_OFFSET)

#define DMA_SRC_ADDR_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_SRC_ADDR_REG_OFFSET)

#define DMA_DST_ADDR_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_DST_ADDR_REG_OFFSET)

#define DMA_SRC_BASE_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_SRC_BASE_REG_OFFSET)

#define DMA_SRC_RUN_BUF_SIZE_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_SRC_RUN_BUF_SIZE_REG_OFFSET)

#define DMA_SRC_STRIDE_INT_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_SRC_STRIDE_INT_REG_OFFSET)

#define DMA_SRC_NUM_STRIDES_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_SRC_NUM_STRIDES_REG_OFFSET)

#define DMA_DEST_BASE_ADDR_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_DEST_BASE_ADDR_REG_OFFSET)

#define DMA_DEST_RUN_BUF_SIZE_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_DEST_RUN_BUF_SIZE_REG_OFFSET)

#define DMA_DEST_STRIDE_INT_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_DEST_STRIDE_INT_REG_OFFSET)

#define DMA_DEST_NUM_STRIDE_BUF_REG_CH(ch) (DMA_CHANNEL_BASE + ((ch) * DMA_CHANNEL_BANK_SIZE) + DMA_CHANNEL_DEST_NUM_STRIDE_BUF_REG_OFFSET)

#endif   /* #ifdef _TRINITY_H_ */

/*******************************************************************************
 * Modifications:
 * $Log:
 *  5    Linux_SDK 1.4         10/10/07 2:55:31 PM ISTJaychander
 *       Thirunavukkarasu Added Definitions for SATA
 *  4    Linux_SDK 1.3         10/9/07 6:19:16 PM IST Sanjeev Kumar   Moved
 *       #defines above the log/version information.
 *  3    Linux_SDK 1.2         9/5/07 5:37:47 PM IST  Har Yash Bahadur Include
 *       definitions needed for building the SMSC driver.
 *  2    Linux_SDK 1.1         8/27/07 4:48:52 PM IST Harsh Kumar     Remove
 *       extra definition which is not required for kernel. This removes
 *       warnings while building the kernel.
 *  1    Linux_SDK 1.0         4/3/07 10:20:08 PM IST Vineet Seth     
 * $
 *
 ******************************************************************************/


