/******************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                    */
/*                        SOFTWARE FILE/MODULE HEADER                         */
/*                 Copyright Conexant Systems Inc. 2004-2007                  */
/*                                 Austin, TX                                 */
/*                            All Rights Reserved                             */
/******************************************************************************/
/*
 * Filename:        int_id.h
 *
 *
 * Description:     Interrupt Identifier definitions and associated macros.
 *
 *
 * Author:          Dave Wilson
 *
 ******************************************************************************/
/* $Id: intid.h,v 1.2, 2007-09-20 17:59:35Z, Miles Bintz$
 ******************************************************************************/

#ifndef _INTID_H_
#define _INTID_H_

/**********************************************************************/
/* Use this macro to generate the dwIntID parameter to any of the     */
/* KAL interrupt service functions. Parameter x is either 0 or 1 and  */
/* indicates the PIC in use, parameter y is the bit number associated */
/* with the requested interrupt.                                      */
/**********************************************************************/

#define PIC_0            0x00
#define PIC_1            0x01

/* since we're pecos (by virtue of the location of this file
   our ITC_TYPE can never be trinity 
#if(ITC_TYPE == TRINITY)
#define PIC_TIMERS       0x02
#else
*/
#define PIC_2            0x02                                /*Added for Pecos*/
#define PIC_TIMERS       0x02
#define PIC_3            0x03
/* #endif */
#define PIC_GPIO         0x04
    
    /* This is not a fool-proof macro.  GIGO... */
    #define IN_PIC_RANGE(x, y)   ( y == PIC_GPIO ? ((x > (PIC_GPIO*32)) && (x < 256)) : \
                                 ( (x > (y*32)) & (x < ((y+1)*32)) ) )
    #define CALC_INT_ID(x,y) ((x * 32) + (y))
    #define PIC_FROM_INT_ID(x) ( IN_PIC_RANGE(x, PIC_CENTRAL)   ? PIC_CENTRAL   : \
                                 IN_PIC_RANGE(x, PIC_EXPANSION) ? PIC_EXPANSION : \
                                 IN_PIC_RANGE(x, PIC_TIMERS)    ? PIC_TIMERS    : \
                                 IN_PIC_RANGE(x, PIC_GPIO)      ? PIC_GPIO      : \
                                 -1 /* holy shnikees! we're in trouble now! */    )
    #define INT_FROM_INT_ID(x) ( x - (32 * PIC_FROM_INT_ID(x)) )

/************************/
/* Useful Interrupt IDs */
/************************/

/* First interrupt status register */

#define INT_UART1         CALC_INT_ID(ITC_UART1_PIC,       ITC_UART1_POS)
#define INT_UART2         CALC_INT_ID(ITC_UART2_PIC,       ITC_UART2_POS)
#define INT_UART3         CALC_INT_ID(ITC_UART3_PIC,       ITC_UART3_POS)
#define INT_SMARTCARD0    CALC_INT_ID(ITC_SMARTCARD0_PIC,  ITC_SMARTCARD0_POS)
#define INT_SMARTCARD1    CALC_INT_ID(ITC_SMARTCARD1_PIC,  ITC_SMARTCARD1_POS)
#define INT_I2C0          CALC_INT_ID(ITC_I2C0_PIC,        ITC_I2C0_POS)
#define INT_I2C1          CALC_INT_ID(ITC_I2C1_PIC,        ITC_I2C1_POS)
#define INT_SYNCSERIAL    CALC_INT_ID(ITC_SYNCSERIAL_PIC,  ITC_SYNCSERIAL_POS)
#define INT_IR0           CALC_INT_ID(ITC_IR0_PIC,         ITC_IR0_POS)
#define INT_IR1           CALC_INT_ID(ITC_IR1_PIC,         ITC_IR1_POS)
#define INT_PULSETIMER0   CALC_INT_ID(ITC_PULSETIMER0_PIC, ITC_PULSETIMER0_POS)
#define INT_PULSETIMER1   CALC_INT_ID(ITC_PULSETIMER1_PIC, ITC_PULSETIMER1_POS)
#define INT_USB0          CALC_INT_ID(ITC_USB0_PIC,        ITC_USB0_POS)

#if ITC_TYPE == TRINITY
   #define INT_GPIO          CALC_INT_ID(ITC_GPIO_PIC,       ITC_GPIO_POS)
#else
   #define INT_GPIO          CALC_INT_ID(ITC_GPIO0_PIC,       ITC_GPIO0_POS)
#endif

#define INT_PWM           CALC_INT_ID(ITC_PWM_PIC,         ITC_PWM_POS)
#define INT_HSUART0       CALC_INT_ID(ITC_HSUART0_PIC,     ITC_HSUART0_POS)

#if !(ISTRINITYREVA)
#define INT_I2C2          CALC_INT_ID(ITC_I2C2_PIC,        ITC_I2C2_POS)
#endif
#if(ITC_TYPE == TRINITY)
#define INT_SMARTCARD2    CALC_INT_ID(ITC_SMARTCARD2_PIC,  ITC_SMARTCARD2_POS)
#define INT_USB1          CALC_INT_ID(ITC_USB1_PIC,        ITC_USB1_POS)
#endif
/* Second interrupt status register */

#define INT_DBGCOMMTX     CALC_INT_ID(ITC_DBGCOMMTX_PIC,   ITC_DBGCOMMTX_POS)
#define INT_DBGCOMMRX     CALC_INT_ID(ITC_DBGCOMMRX_PIC,   ITC_DBGCOMMRX_POS)
#define INT_CRYPTO        CALC_INT_ID(ITC_CRYPTO_PIC,      ITC_CRYPTO_POS)
#define INT_PCI           CALC_INT_ID(ITC_PCI_PIC,         ITC_PCI_POS)
#define INT_TIMERS        CALC_INT_ID(ITC_TIMER_PIC,       ITC_TIMER_POS)
#define INT_MC            CALC_INT_ID(ITC_MC_PIC,          ITC_MC_POS)
#define INT_SSPF          CALC_INT_ID(ITC_SSPF_PIC,        ITC_SSPF_POS)
#define INT_MCX           CALC_INT_ID(ITC_MCX_PIC,         ITC_MCX_POS)
#define INT_IMP           CALC_INT_ID(ITC_IMP_PIC,         ITC_IMP_POS)
#define INT_HDCP          CALC_INT_ID(ITC_HDCP_PIC,        ITC_HDCP_POS)
#define INT_TSS           CALC_INT_ID(ITC_TSS_PIC,         ITC_TSS_POS)
#define INT_GXA           CALC_INT_ID(ITC_GXA_PIC,         ITC_GXA_POS)
#define INT_DEINT         CALC_INT_ID(ITC_DEINT_PIC,       ITC_DEINT_POS)
#define INT_DRM0          CALC_INT_ID(ITC_DRM0_PIC,        ITC_DRM0_POS)
#define INT_DRM1          CALC_INT_ID(ITC_DRM1_PIC,        ITC_DRM1_POS)
#define INT_ICAM0         CALC_INT_ID(ITC_ICAM0_PIC,       ITC_ICAM0_POS)
#define INT_ICAM1         CALC_INT_ID(ITC_ICAM1_PIC,       ITC_ICAM1_POS)
#define INT_TSP0          CALC_INT_ID(ITC_TSP0_PIC,        ITC_TSP0_POS)
#define INT_TSP1          CALC_INT_ID(ITC_TSP1_PIC,        ITC_TSP1_POS)
#define INT_AMP           CALC_INT_ID(ITC_AMP_PIC,         ITC_AMP_POS)
#define INT_APP           CALC_INT_ID(ITC_APP_PIC,         ITC_APP_POS)
#define INT_PVC           CALC_INT_ID(ITC_PVC_PIC,         ITC_PVC_POS)



#if(ITC_TYPE == TRINITY)
#define INT_ATAPI     CALC_INT_ID(ITC_ATAPI_PIC, ITC_ATAPI_POS)
#define INT_DMA       CALC_INT_ID(ITC_DMA_PIC,   ITC_DMA_POS)
#define INT_DVT       CALC_INT_ID(ITC_DVT_PIC,   ITC_DVT_POS)
#define INT_AFE       CALC_INT_ID(ITC_AFE_PIC,   ITC_AFE_POS)
#define INT_DAA       CALC_INT_ID(ITC_DAA_PIC,   ITC_DAA_POS)
#define INT_SATA      CALC_INT_ID(ITC_SATA_PIC,  ITC_SATA_POS)
#define INT_SSP       CALC_INT_ID(ITC_SSP_PIC,   ITC_SSP_POS)
#define INT_TSP2      CALC_INT_ID(ITC_TSP2_PIC,  ITC_TSP2_POS)
#define INT_TSP3      CALC_INT_ID(ITC_TSP3_PIC,  ITC_TSP3_POS)
#else
#define INT_DMA0      CALC_INT_ID(ITC_DMA0_PIC,  ITC_DMA0_POS)
#define INT_DMA       INT_DMA0
#define INT_DMA1      CALC_INT_ID(ITC_DMA1_PIC,  ITC_DMA1_POS) 
#define INT_DVT0      CALC_INT_ID(ITC_DVT0_PIC,  ITC_DVT0_POS)
#define INT_DVT       INT_DVT0
#define INT_DVT1      CALC_INT_ID(ITC_DVT1_PIC,  ITC_DVT1_POS)
#define INT_SATA0     CALC_INT_ID(ITC_SATA0_PIC, ITC_SATA0_POS)
#define INT_SATA      INT_SATA0
#define INT_SATA1     CALC_INT_ID(ITC_SATA1_PIC, ITC_SATA1_POS)
#define INT_SSPI      CALC_INT_ID(ITC_SSPI_PIC,  ITC_SSPI_POS)
#define INT_SSP       INT_SSPI


/*To be discussed*/
#define INT_TSR0      CALC_INT_ID(ITC_TSR0_PIC,  ITC_TSR0_POS)
#define INT_TSR1      CALC_INT_ID(ITC_TSR1_PIC,  ITC_TSR1_POS)
#define INT_TSX       CALC_INT_ID(ITC_TSX_PIC,   ITC_TSX_POS)
#endif
/***************************/
/* Timer interrupt sources */
/***************************/
#if(ITC_TYPE == TRINITY)
#define INT_SYSTIMER      CALC_INT_ID(PIC_TIMERS, SYS_TIMER)
#define INT_WATCHDOG      CALC_INT_ID(PIC_TIMERS, WATCHDOG_TIMER)
#define INT_TICKTIMER     CALC_INT_ID(PIC_TIMERS, 2)
#define INT_RTC           CALC_INT_ID(PIC_TIMERS, TIMER_RTC)
#define INT_TIMER0        CALC_INT_ID(PIC_TIMERS, 0)
#define INT_TIMER1        CALC_INT_ID(PIC_TIMERS, 1)
#define INT_TIMER2        CALC_INT_ID(PIC_TIMERS, 2)
#define INT_TIMER3        CALC_INT_ID(PIC_TIMERS, 3)
#define INT_TIMER4        CALC_INT_ID(PIC_TIMERS, 4)
#define INT_TIMER5        CALC_INT_ID(PIC_TIMERS, 5)
#define INT_TIMER6        CALC_INT_ID(PIC_TIMERS, 6)
#define INT_TIMER7        CALC_INT_ID(PIC_TIMERS, 7)
#define INT_TIMER8        CALC_INT_ID(PIC_TIMERS, 8)
#define INT_TIMER9        CALC_INT_ID(PIC_TIMERS, 9)
#define INT_TIMER10       CALC_INT_ID(PIC_TIMERS, 10)
#define INT_TIMER11       CALC_INT_ID(PIC_TIMERS, 11)
#define INT_TIMER12       CALC_INT_ID(PIC_TIMERS, 12)
#define INT_TIMER13       CALC_INT_ID(PIC_TIMERS, 13)
#define INT_TIMER14       CALC_INT_ID(PIC_TIMERS, 14)
#define INT_TIMER15       CALC_INT_ID(PIC_TIMERS, 15)
#else /* Pecos timer id definitions */
#define INT_SYSTIMER      CALC_INT_ID(ITC_TIMER0_PIC, ITC_TIMER0_POS)
#define INT_WATCHDOG      CALC_INT_ID(ITC_TIMER1_PIC, ITC_TIMER1_POS)
#define INT_TICKTIMER     CALC_INT_ID(ITC_TIMER2_PIC, ITC_TIMER2_POS)
#define INT_RTC           CALC_INT_ID(ITC_TIMER3_PIC, ITC_TIMER3_POS)
#define INT_TIMER0        CALC_INT_ID(ITC_TIMER0_PIC, ITC_TIMER0_POS)
#define INT_TIMER1        CALC_INT_ID(ITC_TIMER1_PIC, ITC_TIMER1_POS)
#define INT_TIMER2        CALC_INT_ID(ITC_TIMER2_PIC, ITC_TIMER2_POS)
#define INT_TIMER3        CALC_INT_ID(ITC_TIMER3_PIC, ITC_TIMER3_POS)
#define INT_TIMER4        CALC_INT_ID(ITC_TIMER4_PIC, ITC_TIMER4_POS)
#define INT_TIMER5        CALC_INT_ID(ITC_TIMER5_PIC, ITC_TIMER5_POS)
#define INT_TIMER6        CALC_INT_ID(ITC_TIMER6_PIC, ITC_TIMER6_POS)
#define INT_TIMER7        CALC_INT_ID(ITC_TIMER7_PIC, ITC_TIMER7_POS)

#define INT_TIMER8        CALC_INT_ID(ITC_TIMER8_PIC, ITC_TIMER8_POS)
#define INT_TIMER9        CALC_INT_ID(ITC_TIMER9_PIC, ITC_TIMER9_POS)
#define INT_TIMER10       CALC_INT_ID(ITC_TIMER10_PIC, ITC_TIMER10_POS)
#define INT_TIMER11       CALC_INT_ID(ITC_TIMER11_PIC, ITC_TIMER11_POS)
#define INT_TIMER12       CALC_INT_ID(ITC_TIMER12_PIC, ITC_TIMER12_POS)
#define INT_TIMER13       CALC_INT_ID(ITC_TIMER13_PIC, ITC_TIMER13_POS)
#define INT_TIMER14       CALC_INT_ID(ITC_TIMER14_PIC, ITC_TIMER14_POS)
#define INT_TIMER15       CALC_INT_ID(ITC_TIMER15_PIC, ITC_TIMER15_POS)
#endif

/* Pecos Other Interrupt ID definitions */
#if(ITC_TYPE != TRINITY)
#define INT_UART4                 CALC_INT_ID(ITC_UART4_PIC,  ITC_UART4_POS)
#define INT_SCIF                  CALC_INT_ID(ITC_SCIF_PIC,   ITC_SCIF_POS)
#define INT_USB1                  CALC_INT_ID(ITC_USB1_PIC,   ITC_USB1_POS)
#define INT_USB2                  CALC_INT_ID(ITC_USB2_PIC,   ITC_USB2_POS)
#define INT_MCARD                 CALC_INT_ID(ITC_MCARD_PIC,   ITC_MCARD_POS)
#define INT_MSD0F                 CALC_INT_ID(ITC_MSDFIQ0_PIC,   ITC_MSDFIQ0_POS)
#define INT_MSD0I                 CALC_INT_ID(ITC_MSDIRQ0_PIC,   ITC_MSDIRQ0_POS)
#define INT_EMAC0                 CALC_INT_ID(ITC_EMAC0_PIC,   ITC_EMAC0_POS)
#define INT_EMAC1                 CALC_INT_ID(ITC_EMAC1_PIC,   ITC_EMAC1_POS)                          
#define INT_I2C3                  CALC_INT_ID(ITC_I2C3_PIC,   ITC_I2C3_POS)
#define INT_IMP0                  CALC_INT_ID(ITC_IMP0_PIC,   ITC_IMP0_POS)
#define INT_IMP1                  CALC_INT_ID(ITC_IMP1_PIC,   ITC_IMP1_POS)
#define INT_MSD1F                 CALC_INT_ID(ITC_MSDFIQ1_PIC,   ITC_MSDFIQ1_POS)
#define INT_MSD1I                 CALC_INT_ID(ITC_MSDIRQ1_PIC,   ITC_MSDIRQ1_POS)
#define INT_PVC1                  CALC_INT_ID(ITC_PVC1_PIC,   ITC_PVC1_POS)
#define INT_IPP                   CALC_INT_ID(ITC_IPP_PIC,   ITC_IPP_POS)
#define INT_A9ECT                 CALC_INT_ID(ITC_A9ECT_PIC,   ITC_A9ECT_POS)
#define INT_A11ECT                CALC_INT_ID(ITC_A11ECT_PIC,   ITC_A11ECT_POS)
#define INT_POSP0                 CALC_INT_ID(ITC_POSTP0_PIC,   ITC_POSTP0_POS)
#define INT_POSP1                 CALC_INT_ID(ITC_POSTP1_PIC,   ITC_POSTP1_POS)
#define INT_DENC                  CALC_INT_ID(ITC_DENC_PIC,   ITC_DENC_POS)
#define INT_HDMI                  CALC_INT_ID(ITC_HDMI_PIC,   ITC_HDMI_POS)
#define INT_JPEG                  CALC_INT_ID(ITC_JPEG_PIC, ITC_JPEG_POS)
#define INT_GPIO1                 CALC_INT_ID(ITC_GIPO1_PIC, ITC_GIPO1_POS)
#define INT_L2CC                  CALC_INT_ID(ITC_L2CC_PIC, ITC_L2CC_POS)
#define INT_SWARM9TO11_MSSGAVAIL  CALC_INT_ID(ITC_SWARM9TO11_MSSGAVAIL_PIC, ITC_SWARM9TO11_MSSGAVAIL_POS)
#define INT_SWARM9TO11_MSSGBFULL  CALC_INT_ID(ITC_SWARM9TO11_MSSGFULL_PIC, ITC_SWARM9TO11_MSSGBFULL_POS)
#define INT_SWARM11TO9            CALC_INT_ID(ITC_SWARM11TO9_PIC, ITC_SWARM11TO9_POS)
#endif
/**************************/
/* GPIO interrupt sources */
/**************************/
#define INT_GPIO_PIN(x)   CALC_INT_ID(PIC_GPIO, (x))

#endif /* INTID_H_ */

/*******************************************************************************
 * Modifications:
 * $Log:
 *  3    Linux_SDK 1.2         9/20/07 11:29:35 PM ISTMiles Bintz     CR 31881:
 *        Added support for the 4th interrupt bank in rev-b silicon
 *  2    Linux_SDK 1.1         3/27/07 5:42:58 PM IST Satpal Parmar   GPL
 *       header and Starteam footer addition.
 *  1    Linux_SDK 1.0         3/1/07 11:47:12 PM IST Vineet Seth     
 * $
 *
 ******************************************************************************/


