#ifndef _MEMMAP_H_
#define _MEMMAP_H_

/*TBD: Physical RAM to be decided for Presidio*/
#define PHYS_RAM_SIZE (512UL * 1024 * 1024)
#define KERNEL_RAM_SIZE (256UL * 1024 * 1024)
#define ARM9_RAM_SIZE (8UL * 1024 * 1024)
#define ARM9_SH_RAM (64UL * 1024)

/*
            Fixed Mappings
*/

   // Virtual Addresses 
#define FLASH_VADDR_BASE     0xF0000000  /* FC00.0000 - FCFF.FFFF */
#define ASX_VADDR_BASE       0xE0000000  /* E000.0000 - E07F.FFFF */
#define EXT_IO_VADDR_BASE    0xE1000000  /* E100.0000 - E17F.FFFF */
#define TEMPEST_VADDR_BASE   0xE4000000  /* E400.0000 - E407.FFFF */
#define PCI_MEM_VADDR_BASE   0x80000000  /* 8000.0000 - 9FFF.FFFF */
#define APP_VADDR_BASE       0xE8000000  /* 8000.0000 - 9FFF.FFFF */
#define SHARED_VADDR_BASE    0x4F800000  /*  Base of shared memory between ARM11 and ARM9 */

#define IO_BASE              (ASX_VADDR_BASE)

#define IO_ADDRESS(x)        (((x) & 0x00ffffff) | IO_BASE)

   // Mapping Range 
#define FLASH_SIZE           0x02000000
#define ASX_IO_SIZE          0x01000000 
#define EXT_IO_SIZE          0x01000000 /* EXT IO ecompases PCI IO and ISA IO */
#define TEMPEST_IO_SIZE      0x01000000
#define PCI_MEM_SIZE         0x20000000
#define PCI_IO_SIZE          0x00100000
#define APP_IO_SIZE          0x00040000
#define SHARED_MEM_SIZE      0x00800000

// Physical Addresses 
#define FLASH_PHYS_BASE      0xF0000000
#define ASX_PHYS_BASE        0xE0000000
#define EXT_IO_PHYS_BASE     0xE1000000 /* Include PCI IO and ISA IO */
#define TEMPEST_PHYS_BASE    0xE4000000
#define PCI_MEM_PHYS_BASE    0x80000000
#define APP_PHYS_BASE        0xE8000000
#define KAL_MEM_PHYS_BASE    0x00020000
#define SHARED_MEM_PHYS_BASE 0x0F800000  

#define KAL_MEM_SIZE         (PHYS_RAM_SIZE - KERNEL_RAM_SIZE - KAL_MEM_PHYS_BASE - ARM9_RAM_SIZE)

#define KAL_PHYSICAL_ADDRESS(ptr) ((u_int32)(ptr) & 0x07ffffff)
#endif
/****************************************************************************
 * Modifications:
 * $Log:
 *  8    Linux_SDK 1.7         8/17/2007 9:21:12 PM   Naveen Burmi    CR[s]
 *       30865 30866 : Now, total memory available on nocona is 256MB. A
 *       configurable parameter is provided in kernel configuration file(s) to
 *       configure the size of KAL memory area
 *  7    Linux_SDK 1.6         7/27/2007 11:51:39 PM  Naveen Burmi    CR[s]
 *       29260 30015 : Now Properly mapped the ARM9 and ARM11 shared memry area
 *       in order to make video working. Vide was stopped working after KAL
 *       becomes part of linux kernel.
 *  6    Linux_SDK 1.5         5/10/2007 10:06:14 PM  Yogesh Gaur     Relocated
 *       Linux Kernel above 256M.
 *  5    Linux_SDK 1.4         4/12/2007 8:26:15 PM   Naveen Burmi    code
 *       present under "#ifndef NEW_KAL_MEM" is removed.
 *  4    Linux_SDK 1.3         3/30/2007 9:52:30 PM   Naveen Burmi    "#if
 *       NEW_KAL_MEM" is replaced with "#ifdef NEW_KAL_MEM"
 *  3    Linux_SDK 1.2         3/30/2007 4:41:16 PM   Naveen Burmi    removed
 *       the #define entries for KAL
 *  2    Linux_SDK 1.1         3/27/2007 5:40:40 PM   Satpal Parmar   GPL
 *       header and starteam footer addition.
 *  1    Linux_SDK 1.0         3/1/2007 10:48:22 PM   Vineet Seth     
 * $
 *
 ****************************************************************************/


