/******************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                    */
/*                        SOFTWARE FILE/MODULE HEADER                         */
/*                    Copyright Conexant Systems Inc. 2006                    */
/*                                 Austin, TX                                 */
/*                            All Rights Reserved                             */
/******************************************************************************/
/*
 * Filename:        pecos.h
 *
 *
 * Description:     Public header file defining hardware-specific values (such
 *                  as register addresses, bit definitions, etc.) for Pecos.
 *
 *
 * Author:          
 *
 ******************************************************************************/
/* $Id: pecos.h,v 1.2.1.1, 2007-08-21 12:02:29Z, Jaychander Thirunavukkarasu$
 ******************************************************************************/

#ifndef _PECOS_H_
#define _PECOS_H_

#include <asm/arch/hwopts.h>
//RATH #include <asm/arch/hwdefault.h>


#define ISTRINITY      (0)
#define ISPECOS        (1)
#define ISTRINITYREVA  (0)
#define ISTRINITYREVB  (0)
#define ISTRINITYREVB1 (0)


#define ITC_TYPE     PECOS

/*********************************************
 * PLL_TYPE                                  *
 *********************************************/
#define PLL_TYPE     PECOS

#define ASX0_BASE               0xE0400000



/*********************************/
/* General Types used everywhere */
/*********************************/
#ifndef __ASSEMBLY__
   typedef unsigned long HW_DWORD;
   typedef unsigned short HW_WORD;
   typedef unsigned char HW_BYTE;
   typedef volatile HW_DWORD *LPREG; /*HW_DWORD defined Hwconfig.h*/
#endif /* __ASSEMBLY__ */




/****************************************************************************/
/*                           GPT                                            */
/****************************************************************************/
/* predefined timer assignments */
#define SYS_TIMER           0
#define WATCHDOG_TIMER      1
#define TIMER_RTC           3

#define TIM_BASE            (ASX0_BASE + 0x30000)
#define NUM_TIM_BANKS       16
#define TIM_BANK_SIZE       0x0010
#define TIM_BASE_UNIT(n)    (TIM_BASE + TIM_BANK_SIZE*(n))


#define TIM_VALUE_REG                                         (TIM_BASE + 0x00)
#define    TIM_INT_ENABLE_MASK                                    0x00000008
#define    TIM_ENABLE_MASK                                        0x00000001
#define TIMER_NUM_VALUE_REG(x)                      (TIM_VALUE_REG + ((x) << 4))
#define TIMER_NUM_MODE_REG(x)                       (TIM_MODE_REG  + ((x) << 4))
#define TIMER_NUM_LIMIT_REG(x)                      (TIM_LIMIT_REG + ((x) << 4))
#define TIM_MODE_REG                                          (TIM_BASE + 0x08)
#define TIM_LIMIT_REG                                         (TIM_BASE + 0x04)




/***********************************/
/* General Defines used everywhere */
/***********************************/
#define CHIP_CRYSTAL_FREQUENCY 60000000
#define XTAL_PRESCALE_FACTOR 1



/**************************************************/
/*  ARM1176  PIT                                  */
/**************************************************/

#define ARM11_PERIPHERAL_PORT_BASE     0xE8000000



/****************/
/* USB SPH Core */
/****************/

#define USB_SPH_BASE        ARM11_PERIPHERAL_PORT_BASE + 0x0
#define USB_SPH_BANK_SIZE                  0x1000

/* x = 0~2 */
#define USB_UNIT_BASE(x)              (USB_SPH_BASE + (USB_SPH_BANK_SIZE * (x)))



/**************/
/* ENET       */
/**************/

#define ENET_UNIT_BASE0           (0xE8003000)
#define ENET_UNIT_BASE1           (0xE8004000)
#define ENET_UNIT_BASE(x)         ((x) == 0 ? ENET_UNIT_BASE0 : ENET_UNIT_BASE1)


#define       ENET_STAT_TXPOLL       (1UL<<31)
#define ENET_ENABLE_REG(x)        (ENET_UNIT_BASE(x) + 0x008)
#define       ENET_XTRACTRL_TXBURST8      (1UL<<16)
#define       ENET_XTRACTRL_RXBURST8      (1UL<<17)
#define    ENET_XTRACTRL_TXTHRESH_MASK   0x000003FF  /* TX FIFO start threshold */
#define ENET_XTRACTRL_REG(x)           (ENET_UNIT_BASE(x) + 0x040)
#define       ENET_CONTROL_ENBFULL      (1UL<<10)
#define ENET_LAFL_REG(x)               (ENET_UNIT_BASE(x) + 0x02c)
#define       ENET_XTRACTRL_TXAUTOPAD_YES (1UL<<18)
#define ENET_LAFH_REG(x)               (ENET_UNIT_BASE(x) + 0x030)
#define ENET_CONTROL_REG(x)         (ENET_UNIT_BASE(x) + 0x00c)
#define       ENET_CONTROL_PROM         (1UL<<11)
#define ENET_MISS_REG(x)               (ENET_UNIT_BASE(x) + 0x018)
#define       ENET_XTRACTRL_BVCIFIX_ENA   (1UL<<19)
#define    ENET_MISS_MASK              0x000000FF  /* Missed packet counter   */
#define ENET_RXERR_REG(x)         (ENET_UNIT_BASE(x) + 0x014)
#define    ENET_RXERR_RXCRC_MASK       0x000000FF  /* CRC errors count */
#define    ENET_RXERR_RXFRAM_MASK      0x0000FF00  /* Frame errors count */
#define    ENET_RXERR_RXOFLOW_MASK     0x00FF0000  /* Overflow errors count */
#define    ENET_RXERR_RXFRAM_SHIFT            8
#define    ENET_RXERR_RXOFLOW_SHIFT          16
#define ENET_ID_REG(x)            (ENET_UNIT_BASE(x) + 0x000)
#define ENET_STAT_REG(x)          (ENET_UNIT_BASE(x) + 0x004)
#define       ENET_STAT_TXINT         (1UL<<0)
#define       ENET_XTRACTRL_XTRAENA       (1UL<<22)
#define       ENET_ID_VALUE         0x0007fd02
#define ENET_ADDRL_REG(x)              (ENET_UNIT_BASE(x) + 0x024)
#define       ENET_STAT_RXINT         (1UL<<1)
#define ENET_ADDRH_REG(x)              (ENET_UNIT_BASE(x) + 0x028)
#define ENET_STAT_ERR           (1UL<<2)
#define ENET_TXRINGPTR_REG(x)          (ENET_UNIT_BASE(x) + 0x01c)
#define ENET_RXRINGPTR_REG(x)          (ENET_UNIT_BASE(x) + 0x020)
#define ENET_POLLRATE_REG(x)      (ENET_UNIT_BASE(x) + 0x010)
#define    ENET_CONTROL_TXBDTLEN_SHIFT       16    /* bit shift  for TXBDTLEN */
#define       ENET_XTRACTRL_LOCKFIX_ENA   (1UL<<23)
#define ENET_RXPTRREAD_REG(x)          (ENET_UNIT_BASE(x) + 0x03c)
#define       ENET_CONTROL_ENABLE        (1UL<<0)
#define    ENET_CONTROL_RXBDTLEN_SHIFT       24    /* bit shift for RXBDTLEN  */
#define ENET_TXPTRREAD_REG(x)          (ENET_UNIT_BASE(x) + 0x038)
#define       ENET_CONTROL_TXRUN         (1UL<<3)
#define       ENET_CONTROL_RXRUN         (1UL<<4)
#define       ENET_STAT_MDIO         (1UL<<12)
#define       ENET_STAT_TXCH          (1UL<<3)
#define       ENET_STAT_MISSERR       (1UL<<4)
#define       ENET_STAT_RXCRC         (1UL<<8)
#define       ENET_STAT_RXFRAME       (1UL<<9)
#define       ENET_STAT_RXOFLOW      (1UL<<10)
#define       ENET_MDIO_SFD             (1UL<<30)
#define       ENET_MDIO_OP_READ         (2UL<<28)
#define       ENET_MDIO_TA              (1UL<<17)
#define    ENET_MDIO_PHY_SHIFT               23
#define    ENET_MDIO_PHY_MASK          0x0F800000
#define    ENET_MDIO_REG_SHIFT               18
#define    ENET_MDIO_REG_MASK          0x007C0000
#define       ENET_CONTROL_DISBDCST      (1UL<<8)
#define    ENET_CONTROL_TXBDTLEN_MASK  0x00FF0000  /* No. BDTs Tx ring 1-255  */
#define    ENET_CONTROL_RXBDTLEN_MASK  0xFF000000  /* No. BDTs Rx ring 1-255  */
#define    ENET_MDIO_DATA_MASK         0x0000FFFF
#define       ENET_MDIO_OP_WRITE        (1UL<<28)
#define ENET_MDIO_REG(x)               (ENET_UNIT_BASE(x) + 0x034)


/**************************************************************************/
/*                        UART & HSUART                                   */
/**************************************************************************/

#define SER_BASE                (ASX0_BASE + 0x10000)
#define HSUART_BASE             (HSX_BASE  + 0x40000)
#define SER_BANK_SIZE           0x1000
#define SER_BASE_UNIT(n)    ((n == 4) ? (HSUART_BASE) :  \
                              (SER_BASE+SER_BANK_SIZE*(n)))
#define SER_BASE_UART1          (SER_BASE_UNIT(0))
#define SER_BASE_UART2          (SER_BASE_UNIT(1))
#define SER_BASE_UART3          (SER_BASE_UNIT(2))



/****************************************************************************/
/*                         Page Table                                       */
/****************************************************************************/
#define HSX_BASE                0xE0000000




/**************/
/* ITC        */
/**************/
#define ITC_BANK_SIZE  0x20
#define NUM_ITC_BANKS  3

#define ITC_EINSTEIN   0
#define ITC_TESLA      1

#define ITC_BASE0      (ASX0_BASE + 0x00050000)
#define ITC_BASE1      (ASX4_BASE + 0x00050000)
#define ITC_BASE(itcNum)     ((itcNum) ? (ITC_BASE1) : (ITC_BASE0))


#define ITC_INTSTATSET_REG(itcNum,grpNum)     (ITC_BASE0 +  \
                                                (grpNum)*ITC_BANK_SIZE + 0x14)
#define ITC_INTENABLE_REG(itcNum,grpNum)      (ITC_BASE0 +   \
                                                (grpNum)*ITC_BANK_SIZE + 0x04)
#define ITC_INTRIRQ_REG(itcNum,grpNum)        (ITC_BASE0 +   \
                                                (grpNum)*ITC_BANK_SIZE + 0x08)
#define ITC_INTSTATCLR_REG(itcNum,grpNum)     (ITC_BASE0 +  \
                                                (grpNum)*ITC_BANK_SIZE + 0x10)
#define ITC_PCI_POS               5
#define ITC_GPIO0_PIC             (PIC_0)
#define ITC_GPIO0_POS             14
#define ITC_TIMER0                0x00000001
#define ITC_TIMER0_PIC            (PIC_2)
#define ITC_TIMER0_POS            0
#define ITC_INTDEST_REG(itcNum,grpNum)        (ITC_BASE0 +  \
                                                (grpNum)*ITC_BANK_SIZE + 0x00)
#define ITC_PCI_PIC               (PIC_1)
#define ITC_EMAC0_POS             24
#define ITC_EMAC1_POS             25
#define ITC_UART3_POS             2
#define ITC_UART2_POS             1
#define ITC_UART1_POS             0

#define ITC_USB0_POS              13
#define ITC_USB1_POS              17
#define ITC_USB2_POS              20



/****************************************************************************/
/*                         PCI                                              */
/****************************************************************************/

#define PCI_BASE        (HSX_BASE + 0x10000)

#define PCI_IO_BASE_ADDR        0xE1000000
#define ISA_BANK_SIZE 0x100000

#define PCI_DESC_BASE                                 (PCI_BASE + 0x00)
#define PCI_ROM_DESC_BASE                             (PCI_BASE + 0x00)

#define PCI_BASE_ENABLE_SIZE                                   0x10
#define PCI_REMAP_REG                                 (PCI_BASE + 0x58)
#define PCI_CFG_DATA_REG                              (PCI_BASE + 0x44)
#define     PCI_CFG_ADDR_FUNCTION_SHIFT                                   8
#define         PCI_NEEDS_ISA_REG                                    (1UL<<7)

#define PCI_ROM_DESC0_REG                             (PCI_BASE + 0x00)
#define PCI_ISAROM_DESC1_REG                          (PCI_BASE + 0x04)
#define     PCI_ROM_DESC_WIDTH_MASK                                0x00000030
#define     PCI_ROM_DESC_WIDTH_SHIFT                                      4
#define         PCI_ROM_DESC_WIDTH_32                                (0UL<<4)
#define         PCI_ROM_DESC_WIDTH_8                                 (1UL<<4)
#define         PCI_ROM_DESC_WIDTH_16                                (2UL<<4)
#define     PCI_ROM_DESC_CS_MASK                                   0x00000700
#define     PCI_ROM_DESC_CS_SHIFT                                         8
#define     PCI_ROM_DESC_BURST_ENABLE_MASK                         0x00000800
#define     PCI_ROM_DESC_BURST_ENABLE_SHIFT                              11
#define         PCI_ROM_DESC_BURST_DISABLE                          (0UL<<11)
#define         PCI_ROM_DESC_BURST_ENABLE                           (1UL<<11)
#define     PCI_ROM_DESC_BURST_WAIT_MASK                           0x0000F000
#define     PCI_ROM_DESC_BURST_WAIT_SHIFT                                12
#define     PCI_ROM_DESC_READ_WAIT_MASK                            0x007F0000
#define     PCI_ROM_DESC_READ_WAIT_SHIFT                                 16
#define     PCI_ROM_DESC_FLASH_PROGRAM_MASK                        0x00800000
#define     PCI_ROM_DESC_FLASH_PROGRAM_SHIFT                             23
#define         PCI_ROM_DESC_FLASH_NORMAL_MODE                      (0UL<<23)
#define         PCI_ROM_DESC_FLASH_PROGRAM_MODE                     (1UL<<23)
#define     PCI_ROM_DESC_WRITE_WAIT_MASK                           0x7F000000
#define     PCI_ROM_DESC_WRITE_WAIT_SHIFT                                24
#define     PCI_ROM_DESC_TYPE_MASK                                 0x80000000
#define     PCI_ROM_DESC_TYPE_SHIFT                                      31
#define         PCI_ROM_DESC_TYPE_ROM                               (0UL<<31)
#define         PCI_ROM_DESC_TYPE_ISA                               (1UL<<31)

#define PCI_ROM_DESC0_REG2                            (PCI_BASE + 0x80)

#define PCI_ISA_SECONDARY_DESC_ADDR(desc_num) (PCI_EXT_DESC_BASE + 4*(desc_num))
#define PCI_ISA_PRIMARY_DESC_ADDR(desc_num) (PCI_ISA_DESC_BASE + 4*(desc_num))
#define PCI_INTR_STATUS_REG                           (PCI_BASE + 0x54)
#define     PCI_INTR_INTA_ENABLE_MASK                              0x00000001
#define         PCI_ISA_DESC_TYPE_ISA                               (1UL<<31)
#define     PCI_ISA_DESC_EXT_WAIT_SHIFT                                  12
#define     PCI_ISA_DESC_IO_TYPE_SHIFT                                    6
#define     PCI_ISA_DESC_CS_SHIFT                                         8
#define     PCI_ISA_DESC_WIDTH_MASK                                0x00000030
#define     PCI_ISA_DESC_WIDTH_SHIFT                                      4
#define     PCI_ISA_DESC_INCR_MASK                                 0x00000800
#define     PCI_ISA_DESC_INCR_SHIFT                                      11
#define     PCI_ISA_DESC_SYNC_IO_ENABLE_MASK                       0x00004000
#define     PCI_ISA_DESC_SYNC_IO_ENABLE_SHIFT                            14
#define     PCI_ISA_DESC_WRITE_WAIT_MASK                           0x7F000000
#define     PCI_ISA_DESC_WRITE_WAIT_SHIFT                                24
#define     PCI_ISA_DESC_READ_WAIT_MASK                            0x007F0000
#define     PCI_ISA_DESC_READ_WAIT_SHIFT                                 16
#define     PCI_EXT_DESC_REG_SETUP_TIME_SHIFT                            28
#define     PCI_EXT_DESC_REG_ACCESS_TIME_SHIFT                           24
#define     PCI_EXT_DESC_CS_SETUP_TIME_SHIFT                             17
#define     PCI_EXT_DESC_CS_HOLD_TIME_SHIFT                              12
#define     PCI_EXT_DESC_CTRL_SETUP_TIME_SHIFT                            5
#define     PCI_EXT_DESC_ADDR_HOLD_TIME_SHIFT                             0
#define PCI_ROM_XOE_MASK_REG                          (PCI_BASE + 0x34)
#define PCI_ROM_MODE_REG                              (PCI_BASE + 0x30)
#define PCI_ISA_DESC_BASE                             (PCI_BASE + 0x00)
#define PCI_EXT_DESC_BASE                             (PCI_BASE + 0x80)
#define PCI_CFG_ADDR_REG                                       (PCI_BASE + 0x40)
#define     PCI_ROM_REQ0_SYNC_SHIFT                                      12
#define     PCI_ROM_REQ1_SYNC_SHIFT                                      14
#define     PCI_ROM_REQ2_SYNC_SHIFT                                      16
#define PCI_RESET_REG                                 (PCI_BASE + 0x48)
#define         PCI_RESET_DEASSERTED                                 (0UL<<0)
#define PCI_INTR_ENABLE_REG                           (PCI_BASE + 0x50)
#define PCI_CMD_STAT_OFF                                       0x01
#define     PCI_CMD_MASTER_ENABLE_MASK                             0x00000004
#define     PCI_CMD_MEM_ACCESS_ENABLE_MASK                         0x00000002
#define PCI_CLS_LT_HT_BIST_OFF                                 0x03




/******************************/
/* System Controller          */
/* Reset and Power Management */
/******************************/
#define RST_BASE (ASX0_BASE + 0)

#define RST_SOFTRESET_REG                             (RST_BASE + 0x1C)




/****************************************************************************/
/*                           GPIO                                           */
/****************************************************************************/
#define GPI_BASE             (ASX0_BASE + 0x00070000)
#define GPI_BANK_SIZE        0x40
#define NUM_GPI_BANKS        6
#define NUM_OF_GPIO          96
#define GPI_BASE_UNIT(n)     (GPI_BASE + GPI_BANK_SIZE*(n))

#define GPI_INT_STAT_REG                                    (GPI_BASE + 0x10)
#define GPI_INT_ENABLE0_REG                             (GPI_BASE + 0x14)
#define GPI_POS_EDGE_REG                               (GPI_BASE + 0x18)
#define GPI_NEG_EDGE_REG                               (GPI_BASE + 0x1C)
#define GPI_DRIVE_HIGH_REG                             (GPI_BASE + 0x04)
#define GPI_DRIVE_LOW_REG                              (GPI_BASE + 0x08)
#define GPI_DRIVE_OFF_REG                              (GPI_BASE + 0x0C)
#define    GPI_DRIVE_LOW_REG_BIT010_SHIFT         10
#define    GPI_DRIVE_HIGH_REG_BIT010_SHIFT        10
#define    GPI_DRIVE_LOW_REG_BIT031_SHIFT         31
#define    GPI_DRIVE_HIGH_REG_BIT031_SHIFT        31



/**************************************************/
/* System Registers PLLs and Clocking             */
/**************************************************/
#define SREGS_BASE          (ASX0_BASE + 0x40000)
#define PLL_BASE            (SREGS_BASE)

/* Pecos has two Smart Clock generators to derive 20 clocks MUXing 9 PLLs*/
#define PLL_NUM_SMART_CLK_GENS                                 2

#define PLL_NUM_AUD_CLKS                                       6

#define PLL_FENRUS_CTRL_REG                             (PLL_BASE + 0x48)
#define PLL_CONFIG0_REG                                     (PLL_BASE + 0x100)
#define PLL_PIN_GPIO_MUX0_REG                               (PLL_BASE + 0x180)
#define PLL_PIN_GPIO_MUX1_REG                               (PLL_BASE + 0x184)
#define PLL_PIN_GPIO_MUX2_REG                               (PLL_BASE + 0x188)
#define PLL_PIN_GPIO_MUX3_REG                               (PLL_BASE + 0x18C)
#define PLL_PIN_GPIO_MUX4_REG                               (PLL_BASE + 0x190)
#define PLL_PIN_GPIO_MUX5_REG                               (PLL_BASE + 0x194)
#define SREG_2ND_PIN_MUX0_REG                               (PLL_BASE + 0x1C0)
#define SREG_2ND_PIN_MUX1_REG                               (PLL_BASE + 0x1C4)
#define SREG_2ND_PIN_MUX2_REG                                (PLL_BASE + 0x1C8)
#define SREG_2ND_PIN_MUX3_REG                               (PLL_BASE + 0x1CC)
#define SREG_2ND_PIN_MUX4_REG                               (PLL_BASE + 0x1D0)
#define SREG_2ND_PIN_MUX5_REG                               (PLL_BASE + 0x1D4)
#define PLL_PIN_ALT_FUNC_REG                                (PLL_BASE + 0x110)
#define DIV_MUX_CTRL1_REG                                   (PLL_BASE + 0x54)
#define     MPG0PLL          0x0
#define     DIV_MUX_CTRL1_ARM926_PLL_SEL_MASK                  0x00000F00
#define     MPG1PLL          0x1
#define     HDPLL            0x2
#define     AUDPLL           0x3
#define     DIV_MUX_CTRL1_ARM926_PLL_DIV_MASK                  0x000000FF
#define PLL_MEM_CTRL2_REG                                   (PLL_BASE + 0x40)
#define     PLL_MEM_CTRL2_PLL_N_VALUE_MASK                     0x000001FF
#define     PLL_MEM_CTRL2_PLL_FRAC_VALUE_MASK                  0xFFFFF000
#define PLL_MEM_CTRL1_REG                                   (PLL_BASE + 0x3C)
#define     PLL_MEM_CTRL1_PLL_POST_DIV_MASK                    0x00000700
#define     PLL_MEM_CTRL1_PLL_PRE_DIV_MASK                     0x00000070
#define     PLL_MEM_CTRL2_PLL_FRAC_VALUE_SHIFT                 12
#define PLL_MPG0_INTFRAC_REG                                (PLL_BASE + 0x0)
#define     PLL_MPG0_INTFRAC_INT_MASK                          0x7E000000
#define     PLL_MPG0_INTFRAC_FRAC_MASK                         0x01FFFFFF
#define PLL_MPG0_CTRL_REG                                   (PLL_BASE + 0x04)
#define     PLL_MPG0_CTRL_PRE_DIV_MASK                         0x000F0000
#define     PLL_MPG0_INTFRAC_FRAC_SHIFT                        0
#define     PLL_FENRUS_CTRL_LOOP_DIV_MASK                  0x3F000000
#define DIV_MUX_CTRL10_REG                                  (PLL_BASE + 0x78)
#define     DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK                   0x000000FF
#define     PLL0             0x5
#define     PLL1             0x6
#define     PLL2             0x7
#define     PLL_PIN_ALT_FUNC_PCI_REQGNT2_MASK                 0x40000000
#define     PLL_PIN_ALT_FUNC_PCI_REQGNT1_MASK                 0x20000000
#define     PLL_PIN_ALT_FUNC_IO_RW_MASK                       0x10000000
#define         PLL_PIN_ALT_FUNC_PCI_REQGNT2_PIO_64_63_5      (1UL<<30)
#define         PLL_PIN_ALT_FUNC_PCI_REQGNT1_PIO_7_6          (1UL<<29)
#define         PLL_PIN_ALT_FUNC_IO_RW_NORMAL                 (0UL<<28)

/**************************************************/
/* SATA System Registers PLLs                     */
/**************************************************/

#define SATA_CLK_CTRL_REG                                   (PLL_BASE + 0x4c)
#define     SATA_CLK_CTRL_SATA1_TX_BYPASS_DIS_MASK             0x00000001
#define     SATA_CLK_CTRL_SATA1_TX_BYPASS_DIS_SHIFT            0
#define     SATA_CLK_CTRL_SATA1_RX_BYPASS_DIS_MASK             0x00000002
#define     SATA_CLK_CTRL_SATA1_RX_BYPASS_DIS_SHIFT            1
#define     SATA_CLK_CTRL_SATA2_TX_BYPASS_DIS_MASK             0x00000004
#define     SATA_CLK_CTRL_SATA2_TX_BYPASS_DIS_SHIFT            2
#define     SATA_CLK_CTRL_SATA2_RX_BYPASS_DIS_MASK             0x00000008
#define     SATA_CLK_CTRL_SATA2_RX_BYPASS_DIS_SHIFT            3

/********************************************
 * SATA_TYPE                                *
 ********************************************/
#define SATA_TYPE    PECOS
#define SAT_BASE                 (HSX_BASE + 0x200000)
#define SAT_DMA_CONFIG_BASE      (SAT_BASE)
#define SAT_BASE_CH0_START       (SAT_BASE + 0x100)
#define SAT_BASE_CH1_OFFSET      (0x100000)

/*****************************/
/* SATA Register Definitions */
/*****************************/

/* tx_ch_mode rx_ch_mode */
#define SAT_DMA_CH_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x00)
#define SAT_DMA_CH_MODE_RX_REG                  (SAT_DMA_CH_MODE_TX_REG + 0x80)
#define     SAT_DMA_CH_MODE_GO_MASK                                     0x8000
#define     SAT_DMA_CH_MODE_GO_SHIFT                                        15
#define     SAT_DMA_CH_MODE_GO                                        (1UL<<15)
#define     SAT_DMA_CH_MODE_ABORT_MASK                                  0x4000
#define     SAT_DMA_CH_MODE_ABORT_SHIFT                                     14
#define     SAT_DMA_CH_MODE_ABORT                                     (1UL<<14)
#define     SAT_DMA_CH_MODE_HALT_MASK                                   0x2000
#define     SAT_DMA_CH_MODE_HALT_SHIFT                                      13
#define     SAT_DMA_CH_MODE_HALT                                      (1UL<<13)
#define     SAT_DMA_CH_MODE_INT_ENABLE_MASK                             0x0080
#define     SAT_DMA_CH_MODE_INT_ENABLE_SHIFT                                 7
#define     SAT_DMA_CH_MODE_INT_ENABLE_ENABLED                         (1UL<<7)
#define     SAT_DMA_CH_MODE_INT_ENABLE_DISABLED                        (0UL<<7)
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_MASK                       0x0040
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_SHIFT                           6
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_ENABLED                   (1UL<<6)
#define     SAT_DMA_CH_MODE_ABORT_INT_ENABLE_DISABLED                  (0UL<<6)
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_MASK                        0x0020
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_SHIFT                            5
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_ENABLED                    (1UL<<5)
#define     SAT_DMA_CH_MODE_HALT_INT_ENABLE_DISABLED                   (0UL<<5)
#define     SAT_DMA_CH_MODE_MODE_MASK                                   0x0008
#define     SAT_DMA_CH_MODE_MODE_SHIFT                                       3
#define     SAT_DMA_CH_MODE_MODE_ONLY_BD0                              (0UL<<3)
#define     SAT_DMA_CH_MODE_MODE_BD0_AND_BD1                           (1UL<<3)
#define     SAT_DMA_CH_MODE_BURST_SIZE_MASK                             0x0007
#define     SAT_DMA_CH_MODE_BURST_SIZE_SHIFT                                 0
#define     SAT_DMA_CH_MODE_BURST_4_BEAT                               (0UL<<0)
#define     SAT_DMA_CH_MODE_BURST_8_BEAT                               (1UL<<0)
#define     SAT_DMA_CH_MODE_BURST_16_BEAT                              (2UL<<0)
#define     SAT_DMA_CH_MODE_BURST_24_BEAT                              (3UL<<0)
#define     SAT_DMA_CH_MODE_BURST_32_BEAT                              (4UL<<0)
#define     SAT_DMA_CH_MODE_BURST_64_BEAT                              (5UL<<0)
#define     SAT_DMA_CH_MODE_BURST_128_BEAT                             (6UL<<0)
#define     SAT_DMA_CH_MODE_BURST_256_BEAT                             (7UL<<0)

/* tx_b0_mode, tx_b0_al, tx_b0_ah, tx_b0_size */
/* rx_b0_mode, rx_b0_al, rx_b0_ah, rx_b0_size */
#define SAT_DMA_B0_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x08)
#define SAT_DMA_B0_MODE_RX_REG                  (SAT_DMA_B0_MODE_TX_REG + 0x80)
#define     SAT_DMA_B0_MODE_EOP_MASK                                    0x0002
#define     SAT_DMA_B0_MODE_EOP_SHIFT                                        1
#define     SAT_DMA_B0_MODE_EOP                                        (1UL<<1)
#define     SAT_DMA_B0_MODE_ENABLE_MASK                                 0x0001
#define     SAT_DMA_B0_MODE_ENABLE_SHIFT                                     0
#define     SAT_DMA_B0_MODE_ENABLED                                    (1UL<<0)
#define     SAT_DMA_B0_MODE_INUSE                                      (1UL<<0)
#define     SAT_DMA_B0_MODE_DISABLED                                   (0UL<<0)
#define     SAT_DMA_B0_MODE_FREE                                       (0UL<<0)
#define SAT_DMA_B0_ADR_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x10)
#define SAT_DMA_B0_ADR_RX_REG                    (SAT_DMA_B0_ADR_TX_REG + 0x80)
#define SAT_DMA_B0_ADL_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x10)
#define SAT_DMA_B0_ADL_RX_REG                    (SAT_DMA_B0_ADL_TX_REG + 0x80)
#define SAT_DMA_B0_ADH_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x12)
#define SAT_DMA_B0_ADH_RX_REG                    (SAT_DMA_B0_ADH_TX_REG + 0x80)
#define SAT_DMA_B0_SIZE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x18)
#define SAT_DMA_B0_SIZE_RX_REG                  (SAT_DMA_B0_SIZE_TX_REG + 0x80)

/* tx_b1_mode, tx_b1_al, tx_b1_ah, tx_b1_size */
/* rx_b1_mode, rx_b1_al, rx_b1_ah, rx_b1_size */
#define SAT_DMA_B1_MODE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x20)
#define SAT_DMA_B1_MODE_RX_REG                  (SAT_DMA_B1_MODE_TX_REG + 0x80)
#define     SAT_DMA_B1_MODE_EOP_MASK                                    0x0002
#define     SAT_DMA_B1_MODE_EOP_SHIFT                                        1
#define     SAT_DMA_B1_MODE_EOP                                        (1UL<<1)
#define     SAT_DMA_B1_MODE_ENABLE_MASK                                 0x0001
#define     SAT_DMA_B1_MODE_ENABLE_SHIFT                                     0
#define     SAT_DMA_B1_MODE_ENABLED                                    (1UL<<0)
#define     SAT_DMA_B1_MODE_INUSE                                      (1UL<<0)
#define     SAT_DMA_B1_MODE_DISABLED                                   (0UL<<0)
#define     SAT_DMA_B1_MODE_FREE                                       (0UL<<0)
#define SAT_DMA_B1_ADR_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x28)
#define SAT_DMA_B1_ADR_RX_REG                    (SAT_DMA_B1_ADR_TX_REG + 0x80)
#define SAT_DMA_B1_ADL_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x28)
#define SAT_DMA_B1_ADL_RX_REG                    (SAT_DMA_B1_ADL_TX_REG + 0x80)
#define SAT_DMA_B1_ADH_TX_REG                      (SAT_DMA_CONFIG_BASE + 0x2A)
#define SAT_DMA_B1_ADH_RX_REG                    (SAT_DMA_B1_ADH_TX_REG + 0x80)
#define SAT_DMA_B1_SIZE_TX_REG                     (SAT_DMA_CONFIG_BASE + 0x30)
#define SAT_DMA_B1_SIZE_RX_REG                  (SAT_DMA_B1_SIZE_TX_REG + 0x80)

/* tx_ch_sts rx_ch_sts */
#define SAT_DMA_CH_STATUS_TX_REG                   (SAT_DMA_CONFIG_BASE + 0x38)
#define SAT_DMA_CH_STATUS_RX_REG              (SAT_DMA_CH_STATUS_TX_REG + 0x80)
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_MASK                        0x0040
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_SHIFT                            6
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_STILLACTIVE                (0UL<<6)
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_ABORTED                    (1UL<<6)
#define     SAT_DMA_CH_STATUS_ABORT_INT_ACK_CLEAR                      (1UL<<6)
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_MASK                         0x0020
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_SHIFT                             5
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_STILLACTIVE                 (0UL<<5)
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_HALTED                      (1UL<<5)
#define     SAT_DMA_CH_STATUS_HALT_INT_ACK_CLEAR                       (1UL<<5)
#define     SAT_DMA_CH_STATUS_B1_EOP_DONE_MASK                          0x0010
#define     SAT_DMA_CH_STATUS_B1_EOP_DONE_SHIFT                              4
#define     SAT_DMA_CH_STATUS_B1_EOP_DONE                              (1UL<<4)
#define     SAT_DMA_CH_STATUS_B1_EOP_CLEAR                             (1UL<<4)
#define     SAT_DMA_CH_STATUS_B1_DONE_MASK                              0x0008
#define     SAT_DMA_CH_STATUS_B1_DONE_SHIFT                                  3
#define     SAT_DMA_CH_STATUS_B1_DONE                                  (1UL<<3)
#define     SAT_DMA_CH_STATUS_B1_CLEAR                                 (1UL<<3)
#define     SAT_DMA_CH_STATUS_B1_INUSE                                 (0UL<<3)
#define     SAT_DMA_CH_STATUS_B0_EOP_DONE_MASK                          0x0004
#define     SAT_DMA_CH_STATUS_B0_EOP_DONE_SHIFT                              2
#define     SAT_DMA_CH_STATUS_B0_EOP_DONE                              (1UL<<2)
#define     SAT_DMA_CH_STATUS_B0_EOP_CLEAR                             (1UL<<2)
#define     SAT_DMA_CH_STATUS_B0_DONE_MASK                              0x0002
#define     SAT_DMA_CH_STATUS_B0_DONE_SHIFT                                  1
#define     SAT_DMA_CH_STATUS_B0_DONE                                  (1UL<<1)
#define     SAT_DMA_CH_STATUS_B0_CLEAR                                 (1UL<<1)
#define     SAT_DMA_CH_STATUS_B0_INUSE                                 (0UL<<1)
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_MASK                        0x0001
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_SHIFT                            0
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_B0                         (0UL<<0)
#define     SAT_DMA_CH_STATUS_ACTIVE_BUFFER_B1                         (1UL<<0)

/* SATA Channel 0 Shadow Block Register 1: CH0SBREG1 */
#define SAT_CH0_SB1_REG                            (SAT_BASE_CH0_START + 0x000)
#define SAT_CH0_SB_DATA_REG                                    SAT_CH0_SB1_REG
#define    SAT_CH0_SB_DATA_MASK                                     0x0000FFFF
#define    SAT_CH0_SB_DATA_SHIFT                                             0

/* SATA Channel 0 Shadow Block Register 2: CH0SBREG2 */
#define SAT_CH0_SB2_REG                            (SAT_BASE_CH0_START + 0x008)
#define SAT_CH0_SB_FEATURES_REG                                SAT_CH0_SB2_REG
#define    SAT_CH0_SB_FEATURES_MASK                                 0x000000FF
#define    SAT_CH0_SB_FEATURES_SHIFT                                         0
#define SAT_CH0_SB_ERROR_REG                                   SAT_CH0_SB2_REG
#define    SAT_CH0_SB_ERROR_MASK                                    0x000000FF
#define    SAT_CH0_SB_ERROR_SHIFT                                            0

/* SATA Channel 0 Shadow Block Register 3: CH0SBREG3 */
#define SAT_CH0_SB3_REG                            (SAT_BASE_CH0_START + 0x010)
#define SAT_CH0_SB_SECTOR_COUNT_REG                            SAT_CH0_SB3_REG
#define    SAT_CH0_SB_SECTOR_COUNT_MASK                             0x000000FF
#define    SAT_CH0_SB_SECTOR_COUNT_SHIFT                                     0

/* SATA Channel 0 Shadow Block Register 4: CH0SBREG4 */
#define SAT_CH0_SB4_REG                            (SAT_BASE_CH0_START + 0x018)
#define SAT_CH0_SB_SECTOR_NUM_REG                              SAT_CH0_SB4_REG
#define    SAT_CH0_SB_SECTOR_NUM_MASK                               0x0000FF00
#define    SAT_CH0_SB_SECTOR_NUM_SHIFT                                       8
#define SAT_CH0_SB_LBA_LOW_REG                                 SAT_CH0_SB4_REG
#define    SAT_CH0_SB_LBA_LOW_MASK                                  0x000000FF
#define    SAT_CH0_SB_LBA_LOW_SHIFT                                          0

/* SATA Channel 0 Shadow Block Register 5: CH0SBREG5 */
#define SAT_CH0_SB5_REG                            (SAT_BASE_CH0_START + 0x020)
#define SAT_CH0_SB_CYL_LOW_REG                                 SAT_CH0_SB5_REG
#define    SAT_CH0_SB_CYL_LOW_MASK                                  0x000000FF
#define    SAT_CH0_SB_CYL_LOW_SHIFT                                          0
#define SAT_CH0_SB_LBA_MID_REG                                 SAT_CH0_SB5_REG
#define    SAT_CH0_SB_LBA_MID_MASK                                  0x0000FF00
#define    SAT_CH0_SB_LBA_MID_SHIFT                                          8

/* SATA Channel 0 Shadow Block Register 6: CH0SBREG6 */
#define SAT_CH0_SB6_REG                            (SAT_BASE_CH0_START + 0x028)
#define SAT_CH0_SB_CYL_HIGH_REG                                SAT_CH0_SB6_REG
#define    SAT_CH0_SB_CYL_HIGH_MASK                                 0x0000FF00
#define    SAT_CH0_SB_CYL_HIGH_SHIFT                                         8
#define SAT_CH0_SB_LBA_HIGH_REG                                SAT_CH0_SB6_REG
#define  SAT_CH0_SB_LBA_HIGH_MASK                                   0x000000FF
#define  SAT_CH0_SB_LBA_HIGH_SHIFT                                           0

/* SATA Channel 0 Shadow Block Register 7: CH0SBREG7 */
#define SAT_CH0_SB7_REG                            (SAT_BASE_CH0_START + 0x030)
#define SAT_CH0_SB_DEV_HEAD_REG                                SAT_CH0_SB7_REG
#define    SAT_CH0_SB_DEV_HEAD_MASK                                 0x000000FF
#define    SAT_CH0_SB_DEV_HEAD_SHIFT                                         0
#define SAT_CH0_SB_DEVICE_REG                                  SAT_CH0_SB7_REG
#define    SAT_CH0_SB_DEVICE_MASK                                   0x000000FF
#define    SAT_CH0_SB_DEVICE_SHIFT                                           0

/* SATA Channel 0 Shadow Block Register 8: CH0SBREG8 */
#define SAT_CH0_SB8_REG                            (SAT_BASE_CH0_START + 0x038)
#define SAT_CH0_SB_COMMAND_REG                                 SAT_CH0_SB8_REG
#define    SAT_CH0_SB_COMMAND_MASK                                  0x000000FF
#define    SAT_CH0_SB_COMMAND_SHIFT                                          0
#define SAT_CH0_SB_STATUS_REG                                  SAT_CH0_SB8_REG
#define    SAT_CH0_SB_STATUS_MASK                                   0x000000FF
#define    SAT_CH0_SB_STATUS_SHIFT                                           0

/* SATA Channel 0 Shadow Block Register 9: CH0SBREG9 */
#define SAT_CH0_SB9_REG                            (SAT_BASE_CH0_START + 0x040)
#define SAT_CH0_SB_ASTATUS_REG                                 SAT_CH0_SB9_REG
#define    SAT_CH0_SB_ASTATUS_MASK                                  0x000000FF
#define    SAT_CH0_SB_ASTATUS_SHIFT                                          0
#define SAT_CH0_SB_DEVICE_CONTROL_REG                          SAT_CH0_SB9_REG
#define    SAT_CH0_SB_DEVICE_CONTROL_MASK                           0x000000FF
#define    SAT_CH0_SB_DEVICE_CONTROL_SHIFT                                   0

/* SATA Channel 0 DMA Request FIFO Water Mark Register: CH0DMAREQWMREG */
#define SAT_CH0_DMA_REQWM_REG                      (SAT_BASE_CH0_START + 0x048)
#define     SAT_CH0_DMA_REQWM_MASK                                  0x000001FF
#define     SAT_CH0_DMA_REQWM_SHIFT                                          0
#define     SAT_CH0_DMA_REQWM_DEFAULT                                    0x0FF

/* SATA Channel 0 FIS Transmission FIFO Water Mark Register: CH0FISTXWMREG */
#define SAT_CH0_FIS_TXWM_REG                       (SAT_BASE_CH0_START + 0x050)
#define     SAT_CH0_FIS_TXWM_MASK                                   0x000001FF
#define     SAT_CH0_FIS_TXWM_SHIFT                                           0
#define     SAT_CH0_FIS_TXWM_DEFAULT                                     0x0FF

/* SATA Channel 0 PIO Status Update FIFO Water Mark Register: CH0PIOSTSWMREG */
#define SAT_CH0_PIO_STSWM_REG                      (SAT_BASE_CH0_START + 0x058)
#define     SAT_CH0_PIO_STSWM_MASK                                  0x000001FF
#define     SAT_CH0_PIO_STSWM_SHIFT                                          0
#define     SAT_CH0_PIO_STSWM_DEFAULT                                    0x0FF

/* SATA Bist FIS Register 1: BISTFISREG1 */
#define SAT_CH0_FIS_BIST1_REG                      (SAT_BASE_CH0_START + 0x060)
#define     SAT_CH0_FIS_BIST1_FE_VENDSPCLB_MASK                         0x0001
#define     SAT_CH0_FIS_BIST1_FE_VENDSPCLB_SHIFT                             0
#define        SAT_CH0_FIS_BIST1_FE_VENDSPCLB_DIS                      (0UL<<0)
#define        SAT_CH0_FIS_BIST1_FE_VENDSPCLB_ENA                      (1UL<<0)
#define     SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_MASK                      0x0004
#define     SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_SHIFT                          2
#define        SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_DIS                   (0UL<<2)
#define        SAT_CH0_FIS_BIST1_FE_TXPRIMITIVES_ENA                   (1UL<<2)
#define     SAT_CH0_FIS_BIST1_FE_ANALOGLB_MASK                          0x0008
#define     SAT_CH0_FIS_BIST1_FE_ANALOGLB_SHIFT                              3
#define        SAT_CH0_FIS_BIST1_FE_ANALOGLB_DIS                       (0UL<<3)
#define        SAT_CH0_FIS_BIST1_FE_ANALOGLB_ENA                       (1UL<<3)
#define     SAT_CH0_FIS_BIST1_FE_RETIMEDLB_MASK                         0x0010
#define     SAT_CH0_FIS_BIST1_FE_RETIMEDLB_SHIFT                             4
#define        SAT_CH0_FIS_BIST1_FE_RETIMEDLB_DIS                      (0UL<<4)
#define        SAT_CH0_FIS_BIST1_FE_RETIMEDLB_ENA                      (1UL<<4)
#define     SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_MASK                    0x0020
#define     SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_SHIFT                        5
#define        SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_DIS                 (0UL<<5)
#define        SAT_CH0_FIS_BIST1_FE_SCRAMBLEBYPASS_ENA                 (1UL<<5)
#define     SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_MASK                       0x0040
#define     SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_SHIFT                           6
#define        SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_DIS                    (0UL<<6)
#define        SAT_CH0_FIS_BIST1_FE_ALIGNBYPASS_ENA                    (1UL<<6)
#define     SAT_CH0_FIS_BIST1_FE_TX_MASK                                0x0080
#define     SAT_CH0_FIS_BIST1_FE_TX_SHIFT                                    7
#define        SAT_CH0_FIS_BIST1_FE_TX_NO                              (0UL<<7)
#define        SAT_CH0_FIS_BIST1_FE_TX_YES                             (1UL<<7)

/* SATA BIST FIS Register 2: BISTFISREG2 */
#define SAT_CH0_FIS_BIST2_REG                      (SAT_BASE_CH0_START + 0x068)
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_MASK                           0xFFFF
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_SHIFT                               0
#define     SAT_CH0_FIS_BIST2_DWORD0_LSW_DEFAULT                        0x0000

/* SATA BIST FIS Register 3: BISTFISREG3 */
#define SAT_CH0_FIS_BIST3_REG                      (SAT_BASE_CH0_START + 0x070)
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_MASK                            0xFFFF
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_SHIFT                                0
#define     SAT_CH0_FIS_BIST3_DWORD0_MSW_DEFAULT                         0x0000

/* SATA BIST FIS Register 4: BISTFISREG4 */
#define SAT_CH0_FIS_BIST4_REG                      (SAT_BASE_CH0_START + 0x078)
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_MASK                           0xFFFF
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_SHIFT                               0
#define     SAT_CH0_FIS_BIST4_DWORD1_LSW_DEFAULT                        0x0000

/* SATA BIST FIS Register 5: BISTFISREG5 */
#define SAT_CH0_FIS_BIST5_REG                      (SAT_BASE_CH0_START + 0x080)
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_MASK                           0xFFFF
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_SHIFT                               0
#define     SAT_CH0_FIS_BIST5_DWORD1_MSW_DEFAULT                        0x0000

/* SATA DMA SETUP FIS Register 1: DMASETUPREG1 */
#define SAT_CH0_DMA_SETUP1_REG                     (SAT_BASE_CH0_START + 0x088)
#define     SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_MASK                       0x0040
#define     SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_SHIFT                           6
#define         SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_ENABLE                (1UL<<6)
#define         SAT_CH0_DMA_SETUP1_FPDMA_ENDINTR_DISABLE               (0UL<<6)
#define     SAT_CH0_DMA_SETUP1_FPDMA_DIR_MASK                           0x0020
#define     SAT_CH0_DMA_SETUP1_FPDMA_DIR_SHIFT                               5
#define         SAT_CH0_DMA_SETUP1_FPDMA_DIR_HOST2DEV                  (1UL<<5)
#define         SAT_CH0_DMA_SETUP1_FPDMA_DIR_DEV2HOST                  (0UL<<5)

/* SATA DMA SETUP FIS Register 2: DMASETUPREG2 */
#define SAT_CH0_DMA_SETUP2_REG                     (SAT_BASE_CH0_START + 0x090)
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_MASK                        0xFFFF
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_TAG_MASK                    0x001F
#define     SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_SHIFT                            0
#define         SAT_CH0_DMA_SETUP2_ADDRESS0_LSW_DEFAULT                 0x0000


/* SATA DMA SETUP FIS Register 3: DMASETUPREG3 */
#define SAT_CH0_DMA_SETUP3_REG                     (SAT_BASE_CH0_START + 0x098)
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_MASK                        0xFFFF
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_SHIFT                            0
#define     SAT_CH0_DMA_SETUP3_ADDRESS0_MSW_DEFAULT                     0x0000

/* SATA DMA SETUP FIS Register 4: DMASETUPREG4 */
#define SAT_CH0_DMA_SETUP4_REG                     (SAT_BASE_CH0_START + 0x0A0)
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_MASK                        0xFFFF
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_SHIFT                            0
#define     SAT_CH0_DMA_SETUP4_ADDRESS1_LSW_DEFAULT                     0x0000

/* SATA DMA SETUP FIS Register 5: DMASETUPREG5 */
#define SAT_CH0_DMA_SETUP5_REG                     (SAT_BASE_CH0_START + 0x0A8)
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_MASK                        0xFFFF
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_SHIFT                            0
#define     SAT_CH0_DMA_SETUP5_ADDRESS1_MSW_DEFAULT                     0x0000

/* SATA DMA SETUP FIS Register 6: DMASETUPREG6 */
#define SAT_CH0_DMA_SETUP6_REG                     (SAT_BASE_CH0_START + 0x0B0)
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_MASK                          0xFFFF
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_SHIFT                              0
#define     SAT_CH0_DMA_SETUP6_OFFSET_LSW_DEFAULT                       0x0000

/* SATA DMA SETUP FIS Register 7: DMASETUPREG7 */
#define SAT_CH0_DMA_SETUP7_REG                     (SAT_BASE_CH0_START + 0x0B8)
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_MASK                          0xFFFF
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_SHIFT                              0
#define     SAT_CH0_DMA_SETUP7_OFFSET_MSW_DEFAULT                       0x0000

/* SATA DMA SETUP FIS Register 8: DMASETUPREG8 */
#define SAT_CH0_DMA_SETUP8_REG                     (SAT_BASE_CH0_START + 0x0C0)
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_MASK                          0xFFFF
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_SHIFT                              0
#define     SAT_CH0_DMA_SETUP8_XFRCNT_LSW_DEFAULT                       0x0000

/* SATA DMA SETUP FIS Register 9: DMASETUPREG9 */
#define SAT_CH0_DMA_SETUP9_REG                     (SAT_BASE_CH0_START + 0x0C8)
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_MASK                          0xFFFF
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_SHIFT                              0
#define     SAT_CH0_DMA_SETUP9_XFRCNT_MSW_DEFAULT                       0x0000

/* SATA Transmit FIS Request Register: TXFISREQREG */
#define SAT_CH0_FIS_TXREQ_REG                      (SAT_BASE_CH0_START + 0x0D0)
#define     SAT_CH0_FIS_TXREQ_DMASETUP_MASK                             0x0002
#define     SAT_CH0_FIS_TXREQ_DMASETUP_SHIFT                                 1
#define        SAT_CH0_FIS_TXREQ_DMASETUP_                             (1UL<<1)
#define     SAT_CH0_FIS_TXREQ_BIST_MASK                                 0x0001
#define     SAT_CH0_FIS_TXREQ_BIST_SHIFT                                     0
#define        SAT_CH0_FIS_TXREQ_BIST_ENA                              (1UL<<0)

/* SATA BIST Mode Control Register: BISTMODECTRLREG */
#define SAT_CH0_BIST_CONTROL_REG                   (SAT_BASE_CH0_START + 0x0D8)
#define     SAT_CH0_BIST_CONTROL_HOSTACCESS_MASK                        0x0001
#define     SAT_CH0_BIST_CONTROL_HOSTACCESS_SHIFT                            0
#define        SAT_CH0_BIST_CONTROL_HOSTACCESS_NO                      (0UL<<0)
#define        SAT_CH0_BIST_CONTROL_HOSTACCESS_YES                     (1UL<<0)
#define     SAT_CH0_BIST_CONTROL_LOADING_MASK                           0x0002
#define     SAT_CH0_BIST_CONTROL_LOADING_SHIFT                               1
#define        SAT_CH0_BIST_CONTROL_LOADING_NO                         (0UL<<1)
#define        SAT_CH0_BIST_CONTROL_LOADING_YES                        (1UL<<1)
#define     SAT_CH0_BIST_CONTROL_VALIDMODE_MASK                         0x0004
#define     SAT_CH0_BIST_CONTROL_VALIDMODE_SHIFT                             2
#define         SAT_CH0_BIST_CONTROL_VALIDMODE                         (1UL<<2)
#define     SAT_CH0_BIST_CONTROL_INVALIDMODE_MASK                       0x0008
#define     SAT_CH0_BIST_CONTROL_INVALIDMODE_SHIFT                           3
#define         SAT_CH0_BIST_CONTROL_INVALIDMODE                       (1UL<<3)
#define     SAT_CH0_BIST_CONTROL_FISRCVD_MASK                           0x0010
#define     SAT_CH0_BIST_CONTROL_FISRCVD_SHIFT                               4
#define         SAT_CH0_BIST_CONTROL_FISRCVD_CLEAR                     (1UL<<4)

/* SATA FPDMA Control Register: FPDMACTRLREG */
#define SAT_CH0_FPDMA_CONTROL_REG                  (SAT_BASE_CH0_START + 0x0E0)
#define     SAT_CH0_FPDMA_CONTROL_HOSTACCESS_MASK                       0x0001
#define     SAT_CH0_FPDMA_CONTROL_HOSTACCESS_SHIFT                           0
#define        SAT_CH0_FPDMA_CONTROL_HOSTACCESS_ENA                    (0UL<<0)
#define     SAT_CH0_FPDMA_CONTROL_LOADING_MASK                          0x0002
#define     SAT_CH0_FPDMA_CONTROL_LOADING_SHIFT                              1
#define        SAT_CH0_FPDMA_CONTROL_LOADING_NO                        (0UL<<1)
#define        SAT_CH0_FPDMA_CONTROL_LOADING_YES                       (1UL<<1)
#define     SAT_CH0_FPDMA_CONTROL_VALIDDMA_MASK                         0x0004
#define     SAT_CH0_FPDMA_CONTROL_VALIDDMA_SHIFT                             2
#define         SAT_CH0_FPDMA_CONTROL_VALIDDMA                         (1UL<<2)
#define     SAT_CH0_FPDMA_CONTROL_INVALIDDMA_MASK                       0x0008
#define     SAT_CH0_FPDMA_CONTROL_INVALIDDMA_SHIFT                           3
#define         SAT_CH0_FPDMA_CONTROL_INVALIDDMA                       (1UL<<3)
#define     SAT_CH0_FPDMA_CONTROL_DMARCVD_MASK                          0x0010
#define     SAT_CH0_FPDMA_CONTROL_DMARCVD_SHIFT                              4
#define         SAT_CH0_FPDMA_CONTROL_DMARCVD_CLEAR                    (1UL<<4)

/* SATA FIS Transmission Retry Control Register: FISTXRTRYREG */
#define SAT_CH0_FIS_TX_RETRY_REG                   (SAT_BASE_CH0_START + 0x0E8)
#define     SAT_CH0_FIS_TX_RETRY_ENABLE_MASK                            0x0001
#define     SAT_CH0_FIS_TX_RETRY_ENABLE_SHIFT                                0
#define         SAT_CH0_FIS_TX_RETRY_ENABLED                             (1<<0)
#define         SAT_CH0_FIS_TX_RETRY_DISABLED                            (0<<0)
#define     SAT_CH0_FIS_TX_RETRY_STATUS_MASK                            0x0002
#define     SAT_CH0_FIS_TX_RETRY_STATUS_SHIFT                                1
#define        SAT_CH0_FIS_TX_RETRY_STATUS_YES                         (1UL<<1)

/* SATA Data Transfer Control Register: DATTSFRCTRLREG */
#define SAT_CH0_DATA_XFER_CONTROL_REG              (SAT_BASE_CH0_START + 0x0F0)
#define     SAT_CH0_DATA_XFER_CONTROL_ABORTRX_MASK                      0x0001
#define     SAT_CH0_DATA_XFER_CONTROL_ABORTRX_SHIFT                          0
#define         SAT_CH0_DATA_XFER_CONTROL_ABORTRX                      (1UL<<0)

/* SATA FIFO Control Register: FIFCTRLREG */
#define SAT_CH0_FIFO_CONTROL_REG                   (SAT_BASE_CH0_START + 0x0F8)
#define     SAT_CH0_FIFO_CONTROL_FLUSHTX_MASK                           0x0001
#define     SAT_CH0_FIFO_CONTROL_FLUSHTX_SHIFT                               0
#define         SAT_CH0_FIFO_CONTROL_FLUSHTX                           (1UL<<0)
#define     SAT_CH0_FIFO_CONTROL_FLUSHRX_MASK                           0x0002
#define     SAT_CH0_FIFO_CONTROL_FLUSHRX_SHIFT                               1
#define         SAT_CH0_FIFO_CONTROL_FLUSHRX                           (1UL<<1)

/* SATA FIFO Status Register: FIFOSTATREG */
#define SAT_CH0_FIFO_STATUS_REG                    (SAT_BASE_CH0_START + 0x100)
#define     SAT_CH0_FIFO_STATUS_TXFIFOFULL_MASK                         0x8000
#define     SAT_CH0_FIFO_STATUS_TXFIFOFULL_SHIFT                            15
#define         SAT_CH0_FIFO_STATUS_TXFIFOFULL                        (1UL<<15)
#define     SAT_CH0_FIFO_STATUS_TXFIFOCOUNT_MASK                        0x3F00
#define     SAT_CH0_FIFO_STATUS_TXFIFOCOUNT_SHIFT                            8
#define     SAT_CH0_FIFO_STATUS_RXFIFOEMPTY_MASK                          0x80
#define     SAT_CH0_FIFO_STATUS_RXFIFOEMPTY_SHIFT                            7
#define         SAT_CH0_FIFO_STATUS_RXFIFOEMPTY                        (1UL<<7)
#define     SAT_CH0_FIFO_STATUS_RXFIFOCOUNT_MASK                          0x3F
#define     SAT_CH0_FIFO_STATUS_RXFIFOCOUNT_SHIFT                            0

/* SATA HOLD Configuration Register: HOLDCFGREG */
/* This register is for Rev C and later devices */
#define SAT_CH0_HOLD_CONFIG_REG                    (SAT_BASE_CH0_START + 0x108)
#define     SAT_CH0_HOLD_CONFIG_THRESHOLD_MASK                            0xFF
#define     SAT_CH0_HOLD_CONFIG_THRESHOLD_SHIFT                              0

/* SATA Serial ATA Interrupt Enable Register: SIENBREG1 */
#define SAT_CH0_INT_ENABLE_REG                     (SAT_BASE_CH0_START + 0x110)
#define     SAT_CH0_INT_ENABLE_PHYRDYCHNG_MASK                          0x0001
#define     SAT_CH0_INT_ENABLE_PHYRDYCHNG_SHIFT                              0
#define        SAT_CH0_INT_ENABLE_PHYRDYCHNG                           (1UL<<0)
#define        SAT_CH0_INT_DISABLE_PHYRDYCHNG                          (0UL<<0)
#define     SAT_CH0_INT_ENABLE_PHYINTERR_MASK                           0x0002
#define     SAT_CH0_INT_ENABLE_PHYINTERR_SHIFT                               1
#define        SAT_CH0_INT_ENABLE_PHYINTERR                            (1UL<<1)
#define        SAT_CH0_INT_DISABLE_PHYINTERR                           (0UL<<1)
#define     SAT_CH0_INT_ENABLE_COMWAKDET_MASK                           0x0004
#define     SAT_CH0_INT_ENABLE_COMWAKDET_SHIFT                               2
#define        SAT_CH0_INT_ENABLE_COMWAKDET                            (1UL<<2)
#define     SAT_CH0_INT_ENABLE_CRCERRRCVD_MASK                          0x0008
#define     SAT_CH0_INT_ENABLE_CRCERRRCVD_SHIFT                              3
#define        SAT_CH0_INT_ENABLE_CRCERRRCVD                           (1UL<<3)
#define     SAT_CH0_INT_ENABLE_HSKERRRCVD_MASK                          0x0010
#define     SAT_CH0_INT_ENABLE_HSKERRRCVD_SHIFT                              4
#define        SAT_CH0_INT_ENABLE_HSKERRRCVD                           (1UL<<4)
#define     SAT_CH0_INT_ENABLE_LNKSEQERR_MASK                           0x0020
#define     SAT_CH0_INT_ENABLE_LNKSEQERR_SHIFT                               5
#define        SAT_CH0_INT_ENABLE_LNKSEQERR                            (1UL<<5)
#define     SAT_CH0_INT_ENABLE_TRNSPTSEQERR_MASK                        0x0040
#define     SAT_CH0_INT_ENABLE_TRNSPTSEQERR_SHIFT                            6
#define        SAT_CH0_INT_ENABLE_TRNSPTSEQERR                         (1UL<<6)
#define     SAT_CH0_INT_ENABLE_UNRECFISRCVD_MASK                        0x0080
#define     SAT_CH0_INT_ENABLE_UNRECFISRCVD_SHIFT                            7
#define        SAT_CH0_INT_ENABLE_UNRECFISRCVD                         (1UL<<7)
#define     SAT_CH0_INT_ENABLE_PMODECHANGE_MASK                         0x0100
#define     SAT_CH0_INT_ENABLE_PMODECHANGE_SHIFT                             8
#define        SAT_CH0_INT_ENABLE_PMODECHANGE                          (1UL<<8)
#define     SAT_CH0_INT_ENABLE_BISTRCVD_MASK                            0x0200
#define     SAT_CH0_INT_ENABLE_BISTRCVD_SHIFT                                9
#define        SAT_CH0_INT_ENABLE_BISTRCVD                             (1UL<<9)
#define     SAT_CH0_INT_ENABLE_FPDMARCVD_MASK                           0x0400
#define     SAT_CH0_INT_ENABLE_FPDMARCVD_SHIFT                              10
#define        SAT_CH0_INT_ENABLE_FPDMARCVD                           (1UL<<10)
#define     SAT_CH0_INT_ENABLE_FISTXRETRY_MASK                          0x0800
#define     SAT_CH0_INT_ENABLE_FISTXRETRY_SHIFT                             11
#define        SAT_CH0_INT_ENABLE_FISTXRETRY                          (1UL<<11)

/* SATA Port 0 Serial ATA Power Mode Status and Control Register: P0PMREG */
#define SAT_CH0_POWERMODE_REG                      (SAT_BASE_CH0_START + 0x118)
#define     SAT_CH0_POWERMODE_SLUMBERSTATUS_MASK                        0x0001
#define     SAT_CH0_POWERMODE_SLUMBERSTATUS_SHIFT                            0
#define         SAT_CH0_POWERMODE_SLUMBERSTATUS_CLEAR                  (1UL<<0)
#define     SAT_CH0_POWERMODE_PARTIALSTATUS_MASK                        0x0002
#define     SAT_CH0_POWERMODE_PARTIALSTATUS_SHIFT                            1
#define         SAT_CH0_POWERMODE_PARTIALSTATUS_CLEAR                  (1UL<<1)
#define     SAT_CH0_POWERMODE_SLUMBERREQ_MASK                           0x0004
#define     SAT_CH0_POWERMODE_SLUMBERREQ_SHIFT                               2
#define        SAT_CH0_POWERMODE_SLUMBERREQ                            (1UL<<2)
#define     SAT_CH0_POWERMODE_PARTIALREQ_MASK                           0x0008
#define     SAT_CH0_POWERMODE_PARTIALREQ_SHIFT                               3
#define        SAT_CH0_POWERMODE_PARTIALREQ                            (1UL<<3)
#define     SAT_CH0_POWERMODE_WAKEUPREQ_MASK                            0x0010
#define     SAT_CH0_POWERMODE_WAKEUPREQ_SHIFT                                4
#define        SAT_CH0_POWERMODE_WAKEUPREQ                             (1UL<<4)
#define     SAT_CH0_POWERMODE_CHANGESTATUS_MASK                         0x0020
#define     SAT_CH0_POWERMODE_CHANGESTATUS_SHIFT                             5
#define         SAT_CH0_POWERMODE_CHANGESTATUS_CLEAR                   (1UL<<5)

/* SATA Port 0 Serial ATA Interface Status Register: P0SSTATUSREG */
#define SAT_CH0_SCR0_SSTATUS_REG                   (SAT_BASE_CH0_START + 0x120)
#define     SAT_CH0_SCR0_SSTATUS_DET_MASK                               0x000F
#define     SAT_CH0_SCR0_SSTATUS_DET_SHIFT                                   0
#define         SAT_CH0_SCR0_SSTATUS_DET_NOTPRESENT                    (0UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_NOTESTABLISHED                (0UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICENOPHY                   (1UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICEANDPHY                  (2UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_DEVICEANDPHY1                 (3UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_PHYOFFLINE                    (4UL<<0)
#define         SAT_CH0_SCR0_SSTATUS_DET_PHYINBISTLB                   (4UL<<0)
#define     SAT_CH0_SCR0_SSTATUS_SPD_MASK                               0x00F0
#define     SAT_CH0_SCR0_SSTATUS_SPD_SHIFT                                   4
#define         SAT_CH0_SCR0_SSTATUS_SPD_NOTPRESENT                    (0UL<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_NOTESTABLISHED                (0UL<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_GEN1                          (1UL<<4)
#define         SAT_CH0_SCR0_SSTATUS_SPD_GEN2                          (2UL<<4)
#define     SAT_CH0_SCR0_SSTATUS_IPM_MASK                               0x0F00
#define     SAT_CH0_SCR0_SSTATUS_IPM_SHIFT                                   8
#define         SAT_CH0_SCR0_SSTATUS_IPM_NOTPRESENT                    (0UL<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_NOTESTABLISHED                (0UL<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_ACTIVE                        (1UL<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_PARTIALPM                     (2UL<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_SLUMBERPM                     (4UL<<8)
#define         SAT_CH0_SCR0_SSTATUS_IPM_SLUMBERPM1                    (6UL<<8)

/* SATA Port 0 Serial ATA Interface Error Register: P0SERRORREG */
#define SAT_CH0_SCR1_SERROR_DIAG_REG               (SAT_BASE_CH0_START + 0x128)
#define     SAT_CH0_SCR1_SERROR_DIAG_N_PHYRDY_MASK                    (1UL<<16)
#define     SAT_CH0_SCR1_SERROR_DIAG_N_PHYRDY_SHIFT                         16
#define     SAT_CH0_SCR1_SERROR_DIAG_I_PHYINT_MASK                    (1UL<<17)
#define     SAT_CH0_SCR1_SERROR_DIAG_I_PHYINT_SHIFT                         17
#define     SAT_CH0_SCR1_SERROR_DIAG_W_COMMWAKE_MASK                  (1UL<<18)
#define     SAT_CH0_SCR1_SERROR_DIAG_W_COMMWAKE_SHIFT                       18
#define     SAT_CH0_SCR1_SERROR_DIAG_B_DECODE_MASK                    (1UL<<19)
#define     SAT_CH0_SCR1_SERROR_DIAG_B_DECODE_SHIFT                         19
#define     SAT_CH0_SCR1_SERROR_DIAG_D_DISPARITY_MASK                 (1UL<<20)
#define     SAT_CH0_SCR1_SERROR_DIAG_D_DISPARITY_SHIFT                      20
#define     SAT_CH0_SCR1_SERROR_DIAG_C_CRC_MASK                       (1UL<<21)
#define     SAT_CH0_SCR1_SERROR_DIAG_C_CRC_SHIFT                            21
#define     SAT_CH0_SCR1_SERROR_DIAG_H_HANDSHAKE_MASK                 (1UL<<22)
#define     SAT_CH0_SCR1_SERROR_DIAG_H_HANDSHAKE_SHIFT                      22
#define     SAT_CH0_SCR1_SERROR_DIAG_S_LINKSEQ_MASK                   (1UL<<23)
#define     SAT_CH0_SCR1_SERROR_DIAG_S_LINKSEQ_SHIFT                        23
#define     SAT_CH0_SCR1_SERROR_DIAG_T_TRNSPTSEQ_MASK                 (1UL<<24)
#define     SAT_CH0_SCR1_SERROR_DIAG_T_TRNSPTSEQ_SHIFT                      24
#define     SAT_CH0_SCR1_SERROR_DIAG_F_UNRECFIS_MASK                  (1UL<<25)
#define     SAT_CH0_SCR1_SERROR_DIAG_F_UNRECFIS_SHIFT                       25

/* SATA Port 0 Serial ATA Interface Control Register: P0SSICREG */
#define SAT_CH0_SCR2_SCONTROL_REG                  (SAT_BASE_CH0_START + 0x130)
#define     SAT_CH0_SCR2_SCONTROL_DET_MASK                              0x000F
#define     SAT_CH0_SCR2_SCONTROL_DET_SHIFT                                  0
#define         SAT_CH0_SCR2_SCONTROL_DET_NODETECT                     (0UL<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_REINIT                       (1UL<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_RESET                        (1UL<<0)
#define         SAT_CH0_SCR2_SCONTROL_DET_DISABLE                      (4UL<<0)
#define     SAT_CH0_SCR2_SCONTROL_SPD_MASK                              0x00F0
#define     SAT_CH0_SCR2_SCONTROL_SPD_SHIFT                                  4
#define         SAT_CH0_SCR2_SCONTROL_SPD_NORESTRICT                   (0UL<<4)
#define         SAT_CH0_SCR2_SCONTROL_SPD_RESTRICT_GEN1                (1UL<<4)
#define         SAT_CH0_SCR2_SCONTROL_SPD_RESTRICT_GEN2                (2UL<<4)
#define     SAT_CH0_SCR2_SCONTROL_IPM_MASK                              0x0F00
#define     SAT_CH0_SCR2_SCONTROL_IPM_SHIFT                                  8
#define         SAT_CH0_SCR2_SCONTROL_IPM_NORESTRICT                   (0UL<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIAL                    (1UL<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOSLUMBER                    (2UL<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIALSLUMBER             (4UL<<8)
#define         SAT_CH0_SCR2_SCONTROL_IPM_NOPARTIALSLUMBER1            (3UL<<8)

/* SATA Port 0 Serial ATA Interface SActive Register: P0SACTIVEREG */
#define SAT_CH0_SCR3_SACTIVE_REG                   (SAT_BASE_CH0_START + 0x138)
#define     SAT_CH0_SCR3_SACTIVE_CMD_ACTIVE                                  1
#define     SAT_CH0_SCR3_SACTIVE_CMD_COMPLETE                                0

/* SATA Port 0 Serial ATA Interface SNotification Register:P0SNOTIFICATIONREG */
#define SAT_CH0_SCR4_SNOTIFICATION_REG             (SAT_BASE_CH0_START + 0x140)

/* SATA Reserved Registers 3 through 15 here */
#define SAT_CH0_SCR5_RESERVED_REG                  (SAT_BASE_CH0_START + 0x150)
#define SAT_CH0_SCR6_RESERVED_REG                  (SAT_BASE_CH0_START + 0x158)
#define SAT_CH0_SCR7_RESERVED_REG                  (SAT_BASE_CH0_START + 0x160)
#define SAT_CH0_SCR8_RESERVED_REG                  (SAT_BASE_CH0_START + 0x168)
#define SAT_CH0_SCR9_RESERVED_REG                  (SAT_BASE_CH0_START + 0x170)
#define SAT_CH0_SCR10_RESERVED_REG                 (SAT_BASE_CH0_START + 0x178)
#define SAT_CH0_SCR11_RESERVED_REG                 (SAT_BASE_CH0_START + 0x180)
#define SAT_CH0_SCR12_RESERVED_REG                 (SAT_BASE_CH0_START + 0x188)
#define SAT_CH0_SCR13_RESERVED_REG                 (SAT_BASE_CH0_START + 0x190)
#define SAT_CH0_SCR14_RESERVED_REG                 (SAT_BASE_CH0_START + 0x198)
#define SAT_CH0_SCR15_RESERVED_REG                 (SAT_BASE_CH0_START + 0x1A0)

/* Enhancement Enable Register: ENHANCEENBREG */
#define SAT_CH0_ENHANCE_ENABLE_REG                 (SAT_BASE_CH0_START + 0x200)
#define     SAT_CH0_ENHANCE_ASYNC_NOTIFY_ENABLE_MASK                    0x0001
#define     SAT_CH0_ENHANCE_ASYNC_NOTIFY_ENABLE_SHIFT                        0
#define         SAT_CH0_ENHANCE_ASYNC_NOTIFY_ENABLE                    (1UL<<0)
#define         SAT_CH0_ENHANCE_ASYNC_NOTIFY_DISABLE                   (0UL<<0)
#define     SAT_CH0_ENHANCE_AUTO_ACTIVATE_ENABLE_MASK                   0x0002
#define     SAT_CH0_ENHANCE_AUTO_ACTIVATE_ENABLE_SHIFT                       1
#define         SAT_CH0_ENHANCE_AUTO_ACTIVATE_ENABLE                   (1UL<<1)
#define         SAT_CH0_ENHANCE_AUTO_ACTIVATE_DISABLE                  (0UL<<1)
#define     SAT_CH0_ENHANCE_PIO_THROUGH_DMA_ENABLE_MASK                 0x0004
#define     SAT_CH0_ENHANCE_PIO_THROUGH_DMA_ENABLE_SHIFT                     2
#define         SAT_CH0_ENHANCE_PIO_THROUGH_DMA_ENABLE                 (1UL<<2)
#define         SAT_CH0_ENHANCE_PIO_THROUGH_DMA_DISABLE                (0UL<<2)
#define     SAT_CH0_ENHANCE_SHADOW_BLOCK_32BIT_ENABLE_MASK              0x0010
#define     SAT_CH0_ENHANCE_SHADOW_BLOCK_32BIT_ENABLE_SHIFT                  4
#define         SAT_CH0_ENHANCE_SHADOW_BLOCK_32BIT_ENABLE              (1UL<<4)
#define         SAT_CH0_ENHANCE_SHADOW_BLOCK_32BIT_DISABLE             (0UL<<4)
#define     SAT_CH0_ENHANCE_FORCE_FIFO_FLUSH_ENABLE_MASK                0x0100
#define     SAT_CH0_ENHANCE_FORCE_FIFO_FLUSH_ENABLE_SHIFT                    8
#define         SAT_CH0_ENHANCE_FORCE_FIFO_FLUSH_ENABLE                (1UL<<8)
#define         SAT_CH0_ENHANCE_FORCE_FIFO_FLUSH_DISABLE               (0UL<<8)
#define     SAT_CH0_ENHANCE_SIGNAL_LEVEL_VALID_ERROR_ENABLE_MASK        0x0200
#define     SAT_CH0_ENHANCE_SIGNAL_LEVEL_VALID_ERROR_ENABLE_SHIFT            9
#define         SAT_CH0_ENHANCE_SIGNAL_LEVEL_VALID_ERROR_ENABLE        (1UL<<9)
#define         SAT_CH0_ENHANCE_SIGNAL_LEVEL_VALID_ERROR_DISABLE       (0UL<<9)
#define     SAT_CH0_ENHANCE_AUTO_ACTIVATE_BIT_ERROR_ENABLE_MASK         0x0400
#define     SAT_CH0_ENHANCE_AUTO_ACTIVATE_BIT_ERROR_ENABLE_SHIFT            10
#define         SAT_CH0_ENHANCE_AUTO_ACTIVATE_BIT_ERROR_ENABLE        (1UL<<10)
#define         SAT_CH0_ENHANCE_AUTO_ACTIVATE_BIT_ERROR_DISABLE       (0UL<<10)
#define     SAT_CH0_ENHANCE_ASYNC_NOTIFY_ERROR_ENABLE_MASK              0x0800
#define     SAT_CH0_ENHANCE_ASYNC_NOTIFY_ERROR_ENABLE_SHIFT                 11
#define         SAT_CH0_ENHANCE_ASYNC_NOTIFY_ERROR_ENABLE             (1UL<<11)
#define         SAT_CH0_ENHANCE_ASYNC_NOTIFY_ERROR_DISABLE            (0UL<<11)
#define     SAT_CH0_ENHANCE_DMA_RECEIVE_UNDERFLOW_ENABLE_MASK           0x1000
#define     SAT_CH0_ENHANCE_DMA_RECEIVE_UNDERFLOW_ENABLE_SHIFT              12
#define         SAT_CH0_ENHANCE_DMA_RECEIVE_UNDERFLOW_ENABLE          (1UL<<12)
#define         SAT_CH0_ENHANCE_DMA_RECEIVE_UNDERFLOW_DISABLE         (0UL<<12)
#define     SAT_CH0_ENHANCE_DMA_RECEIVE_OVERFLOW_ENABLE_MASK            0x2000
#define     SAT_CH0_ENHANCE_DMA_RECEIVE_OVERFLOW_ENABLE_SHIFT               13
#define         SAT_CH0_ENHANCE_DMA_RECEIVE_OVERFLOW_ENABLE           (1UL<<13)
#define         SAT_CH0_ENHANCE_DMA_RECEIVE_OVERFLOW_DISABLE          (0UL<<13)
#define     SAT_CH0_ENHANCE_PIO_RECEIVE_UNDERFLOW_ENABLE_MASK           0x4000
#define     SAT_CH0_ENHANCE_PIO_RECEIVE_UNDERFLOW_ENABLE_SHIFT              14
#define         SAT_CH0_ENHANCE_PIO_RECEIVE_UNDERFLOW_ENABLE          (1UL<<14)
#define         SAT_CH0_ENHANCE_PIO_RECEIVE_UNDERFLOW_DISABLE         (0UL<<14)
#define     SAT_CH0_ENHANCE_PIO_RECEIVE_OVERFLOW_ENABLE_MASK            0x8000
#define     SAT_CH0_ENHANCE_PIO_RECEIVE_OVERFLOW_ENABLE_SHIFT               15
#define         SAT_CH0_ENHANCE_PIO_RECEIVE_OVERFLOW_ENABLE           (1UL<<15)
#define         SAT_CH0_ENHANCE_PIO_RECEIVE_OVERFLOW_DISABLE          (0UL<<15)
#define     SAT_CH0_ENHANCE_PIO_TRF_COUNT_ERROR_ENABLE_MASK            0x10000
#define     SAT_CH0_ENHANCE_PIO_TRF_COUNT_ERROR_ENABLE_SHIFT                16
#define         SAT_CH0_ENHANCE_PIO_TRF_COUNT_ERROR_ENABLE            (1UL<<16)
#define         SAT_CH0_ENHANCE_PIO_TRF_COUNT_ERROR_DISABLE           (0UL<<16)
#define     SAT_CH0_ENHANCE_SACTIVE_OVERRIDE_ENABLE_MASK               0x20000
#define     SAT_CH0_ENHANCE_SACTIVE_OVERRIDE_ENABLE_SHIFT                   17
#define         SAT_CH0_ENHANCE_SACTIVE_OVERRIDE_ENABLE               (1UL<<17)
#define         SAT_CH0_ENHANCE_SACTIVE_OVERRIDE_DISABLE              (0UL<<17)
#define     SAT_CH0_ENHANCE_DMA_SETUP_32BIT_ENABLE_MASK                0x40000
#define     SAT_CH0_ENHANCE_DMA_SETUP_32BIT_ENABLE_SHIFT                    18
#define         SAT_CH0_ENHANCE_DMA_SETUP_32BIT_ENABLE                (1UL<<18)
#define         SAT_CH0_ENHANCE_DMA_SETUP_32BIT_DISABLE               (0UL<<18)
#define     SAT_CH0_ENHANCE_HSX_MASTER_ERROR_ENABLE_MASK               0x80000
#define     SAT_CH0_ENHANCE_HSX_MASTER_ERROR_ENABLE_SHIFT                   19
#define         SAT_CH0_ENHANCE_HSX_MASTER_ERROR_ENABLE               (1UL<<19)
#define         SAT_CH0_ENHANCE_HSX_MASTER_ERROR_DISABLE              (0UL<<19)

/* Interrupt Bit Register: INTBITREG */
#define SAT_CH0_INTERRUPT_BIT_REG                  (SAT_BASE_CH0_START + 0x210)
#define     SAT_CH0_INTERRUPT_REG_DTH_FIS_INT_MASK                      0x0001
#define     SAT_CH0_INTERRUPT_REG_DTH_FIS_INT_SHIFT                          0
#define     SAT_CH0_INTERRUPT_SDB_FIS_INT_MASK                          0x0002
#define     SAT_CH0_INTERRUPT_SDB_FIS_INT_SHIFT                              1
#define     SAT_CH0_INTERRUPT_PIO_SETUP_FIS_INT_MASK                    0x0004
#define     SAT_CH0_INTERRUPT_PIO_SETUP_FIS_INT_SHIFT                        2
#define     SAT_CH0_INTERRUPT_DMA_SETUP_FIS_INT_MASK                    0x0008
#define     SAT_CH0_INTERRUPT_DMA_SETUP_FIS_INT_SHIFT                        3

/* Logical Block Address Low Register: LBALOWREG */
#define SAT_CH0_LBA_LOW_REG                        (SAT_BASE_CH0_START + 0x218)

/* Logical Block Address High Register: LBAHIGHREG */
#define SAT_CH0_LBA_HIGH_REG                       (SAT_BASE_CH0_START + 0x220)
#define     SAT_CH0_LBA_HIGH_REG_MASK                                   0xFFFF
#define     SAT_CH0_LBA_HIGH_REG_SHIFT                                       0

/* RX Data FIS Header Register: RXDATAHDRREG */
#define SAT_CH0_RX_DATA_HDR_REG                    (SAT_BASE_CH0_START + 0x238)

/* Internal Error Register: INTERRREG */
#define SAT_CH0_INTERNAL_ERROR_REG                 (SAT_BASE_CH0_START + 0x250)
#define     SAT_CH0_INTERNAL_SIGNAL_LEVEL_VALID_ERROR_MASK              0x0001
#define     SAT_CH0_INTERNAL_SIGNAL_LEVEL_VALID_ERROR_SHIFT                  0
#define     SAT_CH0_INTERNAL_AUTO_ACTIVATE_ERROR_MASK                   0x0002
#define     SAT_CH0_INTERNAL_AUTO_ACTIVATE_ERROR_SHIFT                       1
#define     SAT_CH0_INTERNAL_ASYNC_NOTIFICATION_ERROR_MASK              0x0004
#define     SAT_CH0_INTERNAL_ASYNC_NOTIFICATION_ERROR_SHIFT                  2
#define     SAT_CH0_INTERNAL_DMA_RECEIVE_UNDERFLOW_ERROR_MASK           0x0008
#define     SAT_CH0_INTERNAL_DMA_RECEIVE_UNDERFLOW_ERROR_SHIFT               3
#define     SAT_CH0_INTERNAL_DMA_RECEIVE_OVERFLOW_ERROR_MASK            0x0010
#define     SAT_CH0_INTERNAL_DMA_RECEIVE_OVERFLOW_ERROR_SHIFT                4
#define     SAT_CH0_INTERNAL_PIO_RECEIVE_UNDERFLOW_ERROR_MASK           0x0020
#define     SAT_CH0_INTERNAL_PIO_RECEIVE_UNDERFLOW_ERROR_SHIFT               5
#define     SAT_CH0_INTERNAL_PIO_RECEIVE_OVERFLOW_ERROR_MASK            0x0040
#define     SAT_CH0_INTERNAL_PIO_RECEIVE_OVERFLOW_ERROR_SHIFT                6
#define     SAT_CH0_INTERNAL_PIO_TRF_COUNT_ERROR_MASK                   0x0080
#define     SAT_CH0_INTERNAL_PIO_TRF_COUNT_ERROR_SHIFT                       7
#define     SAT_CH0_INTERNAL_HSX_MASTER_ERROR_MASK                      0x0100   
#define     SAT_CH0_INTERNAL_HSX_MASTER_ERROR_SHIFT                          8

/* PIO Transfer Count Register: PIOTSFRCNTREG */
#define SAT_CH0_PIO_TRANSFER_COUNT_REG             (SAT_BASE_CH0_START + 0x2D8)
#define     SAT_CH0_PIO_TRANSFER_COUNT_MASK                             0xFFFF
#define     SAT_CH0_PIO_TRANSFER_COUNT_SHIFT                                 0

/* State Machine Read Register: SMREADREG */
#define SAT_CH0_STATE_MACHINE_READ_REG             (SAT_BASE_CH0_START + 0x2E0)
#define     SAT_CH0_LINK_STATE_MACHINE_READ_MASK                      (31UL<<0)
#define     SAT_CH0_LINK_STATE_MACHINE_READ_SHIFT                            0
#define     SAT_CH0_TRANSPORT_STATE_MACHINE_READ_MASK                 (31UL<<7)
#define     SAT_CH0_TRANSPORT_STATE_MACHINE_READ_SHIFT                       7
#define     SAT_CH0_LINK_TRAINING_STATE_MACHINE_READ_MASK            (31UL<<15)
#define     SAT_CH0_LINK_TRAINING_STATE_MACHINE_READ_SHIFT                  15
#define     SAT_CH0_DMA_STATE_MACHINE_READ_MASK                      (31UL<<23)
#define     SAT_CH0_DMA_STATE_MACHINE_READ_SHIFT                            23

/* Features Read Register: FEATREADREG */
#define SAT_CH0_FEATURE_READ_REG                   (SAT_BASE_CH0_START + 0x2E8)
#define     SAT_CH0_FEATURE_READ_MASK                                   0xFFFF
#define     SAT_CH0_FEATURE_READ_SHIFT                                       0

/* Command Read Register: CMDREADREG */
#define SAT_CH0_COMMAND_READ_REG                   (SAT_BASE_CH0_START + 0x2F0)
#define     SAT_CH0_COMMAND_READ_MASK                                   0x00FF
#define     SAT_CH0_COMMAND_READ_SHIFT                                       0

/* Device Control Read Register: DEVCTRLREADREG */
#define SAT_CH0_DEVICE_CTRL_READ_REG               (SAT_BASE_CH0_START + 0x2F8)
#define     SAT_CH0_DEVICE_CTRL_READ_MASK                               0x00FF
#define     SAT_CH0_DEVICE_CTRL_READ_SHIFT                                   0

/* DIB Block and HSX Slave Retry feature Register: SAT_DIB_HSX_FEATURE */
#define SAT_DIB_HSX_FEATURE                        (SAT_BASE_CH0_START + 0x5F0)
#define     SAT_HSX_SLAVE_RETRY_MASK                                    0x0001
#define     SAT_HSX_SLAVE_RETRY_SHIFT                                        0
#define        SAT_HSX_SLAVE_RETRY_ENABLE                              (1UL<<0)
#define        SAT_HSX_SLAVE_RETRY_DISABLE                             (0UL<<0)
#define     SAT_DIB_CONT_PRIMITIVE_BLK_MASK                             0x0002
#define     SAT_DIB_CONT_PRIMITIVE_BLK_SHIFT                                 1
#define        SAT_DIB_CONT_PRIMITIVE_BLK_ENABLE                       (1UL<<1)
#define        SAT_DIB_CONT_PRIMITIVE_BLK_DISABLE                      (0UL<<2)
#define     SAT_DIB_PRIMITIVE_BLK_MASK                                  0x0004
#define     SAT_DIB_PRIMITIVE_BLK_SHIFT                                      2
#define        SAT_DIB_PRIMITIVE_BLK_ENABLE                            (1UL<<2)
#define        SAT_DIB_PRIMITIVE_BLK_DISABLE                           (0UL<<2)

/**************************************************/
/* SATA2 NCQ Hardware Assist Register Definitions */
/**************************************************/

/* SATA NCQ Feature Register */
#define SAT_NCQ_FEATURES                           (SAT_BASE_CH0_START + 0x300)
#define     SAT_NCQ_FEATURES_HW_ASSIST_MASK                            (0x0001)
#define     SAT_NCQ_FEATURES_HW_ASSIST_SHIFT                                 0
#define        SAT_NCQ_FEATURES_HW_ASSIST_ENABLE                       (1UL<<0)
#define        SAT_NCQ_FEATURES_HW_ASSIST_DISABLE                      (0UL<<0)
#define     SAT_NCQ_FEATURES_CMD_REG_WRITTEN_DEASSERT_MASK             (0x0002)
#define     SAT_NCQ_FEATURES_CMD_REG_WRITTEN_DEASSERT_SHIFT                  1
#define        SAT_NCQ_FEATURES_LOAD_ON_CMD_REG_DEASSERT_ENABLE        (1UL<<1)
#define        SAT_NCQ_FEATURES_LOAD_ON_CMD_REG_DEASSERT_DISABLE       (0UL<<1)
#define     SAT_NCQ_FEATURES_LOAD_ON_BUSY_STS_DEASSERT_MASK            (0x0004)
#define     SAT_NCQ_FEATURES_LOAD_ON_BUSY_STS_DEASSERT_SHIFT                 2
#define        SAT_NCQ_FEATURES_LOAD_ON_BUSY_STS_DEASSERT_ENABLE       (1UL<<2)
#define        SAT_NCQ_FEATURES_LOAD_ON_BUSY_STS_DEASSERT_DISABLE      (0UL<<2)
#define     SAT_NCQ_FEATURES_RESET_HW_ASSIST_MASK                      (0x0008)
#define     SAT_NCQ_FEATURES_RESET_HW_ASSIST_SHIFT                           3
#define     SAT_NCQ_FEATURES_CHECK_BUFF_OFFSET_VALID_MASK              (0x0010)
#define     SAT_NCQ_FEATURES_CHECK_BUFF_OFFSET_VALID_SHIFT                   4
#define        SAT_NCQ_FEATURES_CHECK_BUF_OFFSET_VALID_ENABLE          (1UL<<4)
#define        SAT_NCQ_FEATURES_CHECK_BUF_OFFSET_VALID_DISABLE         (0UL<<4)
#define     SAT_NCQ_FEATURES_STORE_ONE_EXTRA_PRD_ENTRY_MASK            (0x0020)
#define     SAT_NCQ_FEATURES_STORE_ONE_EXTRA_PRD_ENTRY_SHIFT                 5
#define        SAT_NCQ_FEATURES_STORE_ONE_EXTRA_PRD_ENTRY_ENABLE       (1UL<<5)
#define        SAT_NCQ_FEATURES_STORE_ONE_EXTRA_PRD_ENTRY_DISABLE      (0UL<<5)
#define     SAT_NCQ_FEATURES_DMA_TRAN_COMPLETE_INT_MASK                (0x0040)
#define     SAT_NCQ_FEATURES_DMA_TRAN_COMPLETE_INT_SHIFT                     6
#define        SAT_NCQ_FEATURES_DMA_TRAN_COMPLETE_INT_ENABLE           (1UL<<6)
#define        SAT_NCQ_FEATURES_DMA_TRAN_COMPLETE_INT_DISABLE          (0UL<<6)
#define     SAT_NCQ_FEATURES_TABLE_COMPLETE_INT_MASK                   (0x0080)
#define     SAT_NCQ_FEATURES_TABLE_COMPLETE_INT_SHIFT                        7
#define        SAT_NCQ_FEATURES_TABLE_COMPLETE_INT_ENABLE              (1UL<<7)
#define        SAT_NCQ_FEATURES_TABLE_COMPLETE_INT_DISABLE             (0UL<<7)

/* SATA NCQ Status Register */
#define SAT_NCQ_STATUS                             (SAT_BASE_CH0_START + 0x308)
#define     SAT_NCQ_STATUS_CMD_REG_WRITTEN_PULSE_RCVD_MASK             (0x0001)
#define     SAT_NCQ_STATUS_CMD_REG_WRITTEN_PULSE_RCVD_SHIFT                  0
#define     SAT_NCQ_STATUS_BUSY_STS_CLR_PULSE_RCVD_MASK                (0x0002)
#define     SAT_NCQ_STATUS_BUSY_STS_CLR_PULSE_RCVD_SHIFT                     1
#define     SAT_NCQ_STATUS_TABLE_HAS_UNFINISHED_ENTRY_MASK             (0x0004)
#define     SAT_NCQ_STATUS_TABLE_HAS_UNFINISHED_ENTRY_SHIFT                  2
#define     SAT_NCQ_STATUS_HW_STATE_MACHINE_COMPLETED_MASK             (0x0008)
#define     SAT_NCQ_STATUS_HW_STATE_MACHINE_COMPLETED_SHIFT                  3
#define     SAT_NCQ_STATUS_HW_STATE_MACHINE_ERR_REPORTED_MASK          (0x0010)
#define     SAT_NCQ_STATUS_HW_STATE_MACHINE_ERR_REPORTED_SHIFT               4
#define     SAT_NCQ_STATUS_DMA_TRAN_COMPLETE_INT_MASK                  (0x0040)
#define     SAT_NCQ_STATUS_DMA_TRAN_COMPLETE_INT_SHIFT                       6
#define     SAT_NCQ_STATUS_TABLE_COMPLETE_INT_MASK                     (0x0080)
#define     SAT_NCQ_STATUS_TABLE_COMPLETE_INT_SHIFT                          7

/* SATA NCQ Tag Register */
#define SAT_NCQ_TAG                                (SAT_BASE_CH0_START + 0x310)
#define     SAT_NCQ_TAG_VALUE_MASK                                     (0x001F)
#define     SAT_NCQ_TAG_VALUE_SHIFT                                          0
#define     SAT_NCQ_TAG_ENHANCED_PRD_MODE_MASK                         (0x0020)
#define     SAT_NCQ_TAG_ENHANCED_PRD_MODE_SHIFT                              5
#define        SAT_NCQ_TAG_ENHANCED_PRD_MODE_ENABLE                    (1UL<<5)
#define        SAT_NCQ_TAG_ENHANCED_PRD_MODE_DISABLE                   (0UL<<5)
#define     SAT_NCQ_TAG_ADDRESS_MODE_MASK                              (0x0040)
#define     SAT_NCQ_TAG_ADDRESS_MODE_SHIFT                                   6
#define        SAT_NCQ_TAG_ADDRESS_PRD_MODE                            (1UL<<6)
#define        SAT_NCQ_TAG_ADDRESS_ABSOLUTE_MODE                       (0UL<<6)
#define     SAT_NCQ_TAG_DIRECTION_MASK                                 (0x0080)
#define     SAT_NCQ_TAG_DIRECTION_SHIFT                                      7
#define        SAT_NCQ_TAG_DIRECTION_DEVICE_TO_HOST                    (1UL<<7)
#define        SAT_NCQ_TAG_DIRECTION_HOST_TO_DEVICE                    (0UL<<7)

/* SATA NCQ Count Register */
#define SAT_NCQ_CNT                                (SAT_BASE_CH0_START + 0x318)

/* SATA NCQ Memory/Memory Table Base Address Register */
#define SAT_NCQ_MEM                                (SAT_BASE_CH0_START + 0x320)

/* SATA NCQ Holding Register for Tag values Register */
#define SAT_TAG_HOLDING                            (SAT_BASE_CH0_START + 0x328)

/* SATA NCQ Holding Register for Count values Register */
#define SAT_CNT_HOLDING                            (SAT_BASE_CH0_START + 0x330)

/* SATA NCQ Holding Register for Memory address values Register */
#define SAT_MEM_HOLDING                            (SAT_BASE_CH0_START + 0x338)

/* SATA NCQ Table Count Field Register(s) */
#define SAT_NCQ_TAG00_CNT                          (SAT_BASE_CH0_START + 0x340)
#define SAT_NCQ_TAG01_CNT                          (SAT_BASE_CH0_START + 0x348)
#define SAT_NCQ_TAG02_CNT                          (SAT_BASE_CH0_START + 0x350)
#define SAT_NCQ_TAG03_CNT                          (SAT_BASE_CH0_START + 0x358)
#define SAT_NCQ_TAG04_CNT                          (SAT_BASE_CH0_START + 0x360)
#define SAT_NCQ_TAG05_CNT                          (SAT_BASE_CH0_START + 0x368)
#define SAT_NCQ_TAG06_CNT                          (SAT_BASE_CH0_START + 0x370)
#define SAT_NCQ_TAG07_CNT                          (SAT_BASE_CH0_START + 0x378)
#define SAT_NCQ_TAG08_CNT                          (SAT_BASE_CH0_START + 0x380)
#define SAT_NCQ_TAG09_CNT                          (SAT_BASE_CH0_START + 0x388)
#define SAT_NCQ_TAG10_CNT                          (SAT_BASE_CH0_START + 0x390)
#define SAT_NCQ_TAG11_CNT                          (SAT_BASE_CH0_START + 0x398)
#define SAT_NCQ_TAG12_CNT                          (SAT_BASE_CH0_START + 0x3a0)
#define SAT_NCQ_TAG13_CNT                          (SAT_BASE_CH0_START + 0x3a8)
#define SAT_NCQ_TAG14_CNT                          (SAT_BASE_CH0_START + 0x3b0)
#define SAT_NCQ_TAG15_CNT                          (SAT_BASE_CH0_START + 0x3b8)
#define SAT_NCQ_TAG16_CNT                          (SAT_BASE_CH0_START + 0x3c0)
#define SAT_NCQ_TAG17_CNT                          (SAT_BASE_CH0_START + 0x3c8)
#define SAT_NCQ_TAG18_CNT                          (SAT_BASE_CH0_START + 0x3d0)
#define SAT_NCQ_TAG19_CNT                          (SAT_BASE_CH0_START + 0x3d8)
#define SAT_NCQ_TAG20_CNT                          (SAT_BASE_CH0_START + 0x3e0)
#define SAT_NCQ_TAG21_CNT                          (SAT_BASE_CH0_START + 0x3e8)
#define SAT_NCQ_TAG22_CNT                          (SAT_BASE_CH0_START + 0x3f0)
#define SAT_NCQ_TAG23_CNT                          (SAT_BASE_CH0_START + 0x3f8)
#define SAT_NCQ_TAG24_CNT                          (SAT_BASE_CH0_START + 0x400)
#define SAT_NCQ_TAG25_CNT                          (SAT_BASE_CH0_START + 0x408)
#define SAT_NCQ_TAG26_CNT                          (SAT_BASE_CH0_START + 0x410)
#define SAT_NCQ_TAG27_CNT                          (SAT_BASE_CH0_START + 0x418)
#define SAT_NCQ_TAG28_CNT                          (SAT_BASE_CH0_START + 0x420)
#define SAT_NCQ_TAG29_CNT                          (SAT_BASE_CH0_START + 0x428)
#define SAT_NCQ_TAG30_CNT                          (SAT_BASE_CH0_START + 0x430)
#define SAT_NCQ_TAG31_CNT                          (SAT_BASE_CH0_START + 0x438)

/* SATA NCQ Table Memory/Memory Table Base Address Field Register(s) */
#define SAT_NCQ_TAG00_MEM                          (SAT_BASE_CH0_START + 0x440)
#define SAT_NCQ_TAG01_MEM                          (SAT_BASE_CH0_START + 0x448)
#define SAT_NCQ_TAG02_MEM                          (SAT_BASE_CH0_START + 0x450)
#define SAT_NCQ_TAG03_MEM                          (SAT_BASE_CH0_START + 0x458)
#define SAT_NCQ_TAG04_MEM                          (SAT_BASE_CH0_START + 0x460)
#define SAT_NCQ_TAG05_MEM                          (SAT_BASE_CH0_START + 0x468)
#define SAT_NCQ_TAG06_MEM                          (SAT_BASE_CH0_START + 0x470)
#define SAT_NCQ_TAG07_MEM                          (SAT_BASE_CH0_START + 0x478)
#define SAT_NCQ_TAG08_MEM                          (SAT_BASE_CH0_START + 0x480)
#define SAT_NCQ_TAG09_MEM                          (SAT_BASE_CH0_START + 0x488)
#define SAT_NCQ_TAG10_MEM                          (SAT_BASE_CH0_START + 0x490)
#define SAT_NCQ_TAG11_MEM                          (SAT_BASE_CH0_START + 0x498)
#define SAT_NCQ_TAG12_MEM                          (SAT_BASE_CH0_START + 0x4a0)
#define SAT_NCQ_TAG13_MEM                          (SAT_BASE_CH0_START + 0x4a8)
#define SAT_NCQ_TAG14_MEM                          (SAT_BASE_CH0_START + 0x4b0)
#define SAT_NCQ_TAG15_MEM                          (SAT_BASE_CH0_START + 0x4b8)
#define SAT_NCQ_TAG16_MEM                          (SAT_BASE_CH0_START + 0x4c0)
#define SAT_NCQ_TAG17_MEM                          (SAT_BASE_CH0_START + 0x4c8)
#define SAT_NCQ_TAG18_MEM                          (SAT_BASE_CH0_START + 0x4d0)
#define SAT_NCQ_TAG19_MEM                          (SAT_BASE_CH0_START + 0x4d8)
#define SAT_NCQ_TAG20_MEM                          (SAT_BASE_CH0_START + 0x4e0)
#define SAT_NCQ_TAG21_MEM                          (SAT_BASE_CH0_START + 0x4e8)
#define SAT_NCQ_TAG22_MEM                          (SAT_BASE_CH0_START + 0x4f0)
#define SAT_NCQ_TAG23_MEM                          (SAT_BASE_CH0_START + 0x4f8)
#define SAT_NCQ_TAG24_MEM                          (SAT_BASE_CH0_START + 0x500)
#define SAT_NCQ_TAG25_MEM                          (SAT_BASE_CH0_START + 0x508)
#define SAT_NCQ_TAG26_MEM                          (SAT_BASE_CH0_START + 0x510)
#define SAT_NCQ_TAG27_MEM                          (SAT_BASE_CH0_START + 0x518)
#define SAT_NCQ_TAG28_MEM                          (SAT_BASE_CH0_START + 0x520)
#define SAT_NCQ_TAG29_MEM                          (SAT_BASE_CH0_START + 0x528)
#define SAT_NCQ_TAG30_MEM                          (SAT_BASE_CH0_START + 0x530)
#define SAT_NCQ_TAG31_MEM                          (SAT_BASE_CH0_START + 0x538)

/* SATA NCQ Table Tag Associted Properties Field Register(s) */
#define SAT_NCQ_TAG00_TAG01_PROP                   (SAT_BASE_CH0_START + 0x540)
#define SAT_NCQ_TAG02_TAG03_PROP                   (SAT_BASE_CH0_START + 0x548)
#define SAT_NCQ_TAG04_TAG05_PROP                   (SAT_BASE_CH0_START + 0x550)
#define SAT_NCQ_TAG06_TAG07_PROP                   (SAT_BASE_CH0_START + 0x558)
#define SAT_NCQ_TAG08_TAG09_PROP                   (SAT_BASE_CH0_START + 0x560)
#define SAT_NCQ_TAG10_TAG11_PROP                   (SAT_BASE_CH0_START + 0x568)
#define SAT_NCQ_TAG12_TAG13_PROP                   (SAT_BASE_CH0_START + 0x570)
#define SAT_NCQ_TAG14_TAG15_PROP                   (SAT_BASE_CH0_START + 0x578)
#define SAT_NCQ_TAG16_TAG17_PROP                   (SAT_BASE_CH0_START + 0x580)
#define SAT_NCQ_TAG18_TAG19_PROP                   (SAT_BASE_CH0_START + 0x588)
#define SAT_NCQ_TAG20_TAG21_PROP                   (SAT_BASE_CH0_START + 0x590)
#define SAT_NCQ_TAG22_TAG23_PROP                   (SAT_BASE_CH0_START + 0x598)
#define SAT_NCQ_TAG24_TAG25_PROP                   (SAT_BASE_CH0_START + 0x5a0)
#define SAT_NCQ_TAG26_TAG27_PROP                   (SAT_BASE_CH0_START + 0x5a8)
#define SAT_NCQ_TAG28_TAG29_PROP                   (SAT_BASE_CH0_START + 0x5b0)
#define SAT_NCQ_TAG30_TAG31_PROP                   (SAT_BASE_CH0_START + 0x5b8)

/* SATA NCQ DMA Features (for data transfers from host to device) Register */
#define SAT_NCQ_DMA_TX_FEATURES                    (SAT_BASE_CH0_START + 0x5c0)
#define     SAT_NCQ_DMA_TX_CH_INT_MASK                                 (0x0001)
#define     SAT_NCQ_DMA_TX_CH_INT_SHIFT                                      0
#define        SAT_NCQ_DMA_TX_CH_INT_ENABLE                            (1UL<<0)
#define        SAT_NCQ_DMA_TX_CH_INT_DISABLE                           (0UL<<0)
#define     SAT_NCQ_DMA_TX_CH_ABORT_INT_MASK                           (0x0020)
#define     SAT_NCQ_DMA_TX_CH_ABORT_INT_SHIFT                                1
#define        SAT_NCQ_DMA_TX_CH_ABORT_INT_ENABLE                      (1UL<<1)
#define        SAT_NCQ_DMA_TX_CH_ABORT_INT_DISABLE                     (0UL<<1)
#define     SAT_NCQ_DMA_TX_CH_HALT_INT_MASK                            (0x0040)
#define     SAT_NCQ_DMA_TX_CH_HALT_INT_SHIFT                                 2
#define        SAT_NCQ_DMA_TX_CH_HALT_INT_ENABLE                       (1UL<<2)
#define        SAT_NCQ_DMA_TX_CH_HALT_INT_DISABLE                      (0UL<<2)
#define     SAT_NCQ_DMA_TX_BURST_SIZE_MASK                             (0x0070)
#define     SAT_NCQ_DMA_TX_BURST_SIZE_SHIFT                                  4

/* SATA NCQ DMA Features (for data transfers from device to host) Register */
#define SAT_NCQ_DMA_RX_FEATURES                    (SAT_BASE_CH0_START + 0x5c8)
#define     SAT_NCQ_DMA_RX_CH_INT_MASK                                 (0x0001)
#define     SAT_NCQ_DMA_RX_CH_INT_SHIFT                                      0
#define        SAT_NCQ_DMA_RX_CH_INT_ENABLE                            (1UL<<0)
#define        SAT_NCQ_DMA_RX_CH_INT_DISABLE                           (0UL<<0)
#define     SAT_NCQ_DMA_RX_CH_ABORT_INT_MASK                           (0x0020)
#define     SAT_NCQ_DMA_RX_CH_ABORT_INT_SHIFT                                1
#define        SAT_NCQ_DMA_RX_CH_ABORT_INT_ENABLE                      (1UL<<1)
#define        SAT_NCQ_DMA_RX_CH_ABORT_INT_DISABLE                     (0UL<<1)
#define     SAT_NCQ_DMA_RX_CH_HALT_INT_MASK                            (0x0040)
#define     SAT_NCQ_DMA_RX_CH_HALT_INT_SHIFT                                 2
#define        SAT_NCQ_DMA_RX_CH_HALT_INT_ENABLE                       (1UL<<2)
#define        SAT_NCQ_DMA_RX_CH_HALT_INT_DISABLE                      (0UL<<2)
#define     SAT_NCQ_DMA_RX_BURST_SIZE_MASK                             (0x0070)
#define     SAT_NCQ_DMA_RX_BURST_SIZE_SHIFT                                  4

/* SATA NCQ Hardware Errors Register */
#define SAT_NCQ_HW_ERRORS                          (SAT_BASE_CH0_START + 0x5d0)

/* SATA NCQ Hardware Error TAG Register */
#define SAT_NCQ_HW_ERROR_TAG                       (SAT_BASE_CH0_START + 0x5d8)

/* SATA NCQ Absolute Transfer Size Information Table Register */
#define SAT_NCQ_ABS_TXFR_SIZE_INFO_TABLE           (SAT_BASE_CH0_START + 0x5e0)

/* SATA NCQ HSX Burst Size Register */
#define SAT_NCQ_HSX_BURST_SIZE                     (SAT_BASE_CH0_START + 0x5e8)

/*************************************/
/* SATA PHY I2C Register Definitions */
/*************************************/
#define SAT_PHY_I2C_BUS          (4)
#define SAT_PHY_I2C_SLAVE_ADDR   (0x38)

/* SATA PHY I2C Master Register Definitions */
#define SAT_PHY_I2C_BASE                                       I2C_BASE_UNIT(3)
#define  SAT_PHY_I2C_SLAVE_ADDR                                          (0x38)
#define  SAT_PHY_I2C_MODE_REG                       (SAT_PHY_I2C_BASE | 0x0000)
#define  SAT_PHY_I2C_CTRL_REG                       (SAT_PHY_I2C_BASE | 0x0004)
#define  SAT_PHY_I2C_STAT_REG                       (SAT_PHY_I2C_BASE | 0x0008)
#define  SAT_PHY_I2C_RDATA_REG                      (SAT_PHY_I2C_BASE | 0x000C)

/* SATA PHY CMU Register Definitions */
#define SAT_PHY_CMU_BASE                                               (0x0000)
#define  SAT_PHY_CMU_00_REG                           (SAT_PHY_CMU_BASE | 0x00)
#define  SAT_PHY_CMU_01_REG                           (SAT_PHY_CMU_BASE | 0x04)
#define  SAT_PHY_CMU_02_REG                           (SAT_PHY_CMU_BASE | 0x08)
#define  SAT_PHY_CMU_03_REG                           (SAT_PHY_CMU_BASE | 0x0c)
#define  SAT_PHY_CMU_04_REG                           (SAT_PHY_CMU_BASE | 0x10)
#define  SAT_PHY_CMU_05_REG                           (SAT_PHY_CMU_BASE | 0x14)
#define  SAT_PHY_CMU_06_REG                           (SAT_PHY_CMU_BASE | 0x18)
#define  SAT_PHY_CMU_07_REG                           (SAT_PHY_CMU_BASE | 0x1c)
#define  SAT_PHY_CMU_08_REG                           (SAT_PHY_CMU_BASE | 0x20)

/* SATA PHY CHANNEL0 Register Definitions */
#define SAT_PHY_CH0_BASE                                               (0x0100)
#define  SAT_PHY_CH0_00_REG                           (SAT_PHY_CH0_BASE | 0x00)
#define  SAT_PHY_CH0_01_REG                           (SAT_PHY_CH0_BASE | 0x04)
#define  SAT_PHY_CH0_02_REG                           (SAT_PHY_CH0_BASE | 0x08)
#define  SAT_PHY_CH0_03_REG                           (SAT_PHY_CH0_BASE | 0x0c)
#define  SAT_PHY_CH0_04_REG                           (SAT_PHY_CH0_BASE | 0x10)
#define  SAT_PHY_CH0_05_REG                           (SAT_PHY_CH0_BASE | 0x14)
#define  SAT_PHY_CH0_06_REG                           (SAT_PHY_CH0_BASE | 0x18)
#define  SAT_PHY_CH0_07_REG                           (SAT_PHY_CH0_BASE | 0x1c)
#define  SAT_PHY_CH0_08_REG                           (SAT_PHY_CH0_BASE | 0x20)
#define  SAT_PHY_CH0_09_REG                           (SAT_PHY_CH0_BASE | 0x24)

/* SATA PHY CHANNEL1 Register Definitions */
#define SAT_PHY_CH1_BASE                                               (0x0200)
#define  SAT_PHY_CH1_00_REG                           (SAT_PHY_CH1_BASE | 0x00)
#define  SAT_PHY_CH1_01_REG                           (SAT_PHY_CH1_BASE | 0x04)
#define  SAT_PHY_CH1_02_REG                           (SAT_PHY_CH1_BASE | 0x08)
#define  SAT_PHY_CH1_03_REG                           (SAT_PHY_CH1_BASE | 0x0c)
#define  SAT_PHY_CH1_04_REG                           (SAT_PHY_CH1_BASE | 0x10)
#define  SAT_PHY_CH1_05_REG                           (SAT_PHY_CH1_BASE | 0x14)
#define  SAT_PHY_CH1_06_REG                           (SAT_PHY_CH1_BASE | 0x18)
#define  SAT_PHY_CH1_07_REG                           (SAT_PHY_CH1_BASE | 0x1c)
#define  SAT_PHY_CH1_08_REG                           (SAT_PHY_CH1_BASE | 0x20)
#define  SAT_PHY_CH1_09_REG                           (SAT_PHY_CH1_BASE | 0x24)

/* SATA PHY CHANNEL2 Register Definitions */
#define SAT_PHY_CH2_BASE                                               (0x0300)
#define  SAT_PHY_CH2_00_REG                           (SAT_PHY_CH2_BASE | 0x00)
#define  SAT_PHY_CH2_01_REG                           (SAT_PHY_CH2_BASE | 0x04)
#define  SAT_PHY_CH2_02_REG                           (SAT_PHY_CH2_BASE | 0x08)
#define  SAT_PHY_CH2_03_REG                           (SAT_PHY_CH2_BASE | 0x0c)
#define  SAT_PHY_CH2_04_REG                           (SAT_PHY_CH2_BASE | 0x10)
#define  SAT_PHY_CH2_05_REG                           (SAT_PHY_CH2_BASE | 0x14)
#define  SAT_PHY_CH2_06_REG                           (SAT_PHY_CH2_BASE | 0x18)
#define  SAT_PHY_CH2_07_REG                           (SAT_PHY_CH2_BASE | 0x1c)
#define  SAT_PHY_CH2_08_REG                           (SAT_PHY_CH2_BASE | 0x20)
#define  SAT_PHY_CH2_09_REG                           (SAT_PHY_CH2_BASE | 0x24)

/* SATA PHY CHANNEL3 Register Definitions */
#define SAT_PHY_CH3_BASE                                               (0x0400)
#define  SAT_PHY_CH3_00_REG                           (SAT_PHY_CH3_BASE | 0x00)
#define  SAT_PHY_CH3_01_REG                           (SAT_PHY_CH3_BASE | 0x04)
#define  SAT_PHY_CH3_02_REG                           (SAT_PHY_CH3_BASE | 0x08)
#define  SAT_PHY_CH3_03_REG                           (SAT_PHY_CH3_BASE | 0x0c)
#define  SAT_PHY_CH3_04_REG                           (SAT_PHY_CH3_BASE | 0x10)
#define  SAT_PHY_CH3_05_REG                           (SAT_PHY_CH3_BASE | 0x14)
#define  SAT_PHY_CH3_06_REG                           (SAT_PHY_CH3_BASE | 0x18)
#define  SAT_PHY_CH3_07_REG                           (SAT_PHY_CH3_BASE | 0x1c)
#define  SAT_PHY_CH3_08_REG                           (SAT_PHY_CH3_BASE | 0x20)
#define  SAT_PHY_CH3_09_REG                           (SAT_PHY_CH3_BASE | 0x24)

#define    ITC_SATA1_MASK                                 0x00000008
#define    ITC_SATA1_SHIFT                                       3
#define       ITC_SATA1_IRQGEN_ENA_IRQPEND_FIQPEND          (0UL<<3)
#define       ITC_SATA1_FIQGEN_DIS_NOIRQPE_NOFIQPE          (1UL<<3)

#define    ITC_SATA0_MASK                                 0x00000400
#define    ITC_SATA0_SHIFT                                      10
#define       ITC_SATA0_IRQGEN_ENA_IRQPEND_FIQPEND         (0UL<<10)
#define       ITC_SATA0_FIQGEN_DIS_NOIRQPE_NOFIQPE         (1UL<<10)

#define ITC_SATA             ITC_SATA0  /* For backwards compatibility */

#define       ITC_INTSTATCLR_SATA1_WRITE_CLEAR              (1UL<<3)                       
#define       ITC_INTSTATCLR_SATA1_WRITE_NOEFFECT           (0UL<<3)
#define       ITC_INTSTATCLR_SATA1_READ_PEND                (1UL<<3)
#define       ITC_INTSTATCLR_SATA1_READ_NOPEND              (0UL<<3)

#define       ITC_INTSTATCLR_SATA0_WRITE_CLEAR             (1UL<<10)                       
#define       ITC_INTSTATCLR_SATA0_WRITE_NOEFFECT          (0UL<<10)
#define       ITC_INTSTATCLR_SATA0_READ_PEND               (1UL<<10)
#define       ITC_INTSTATCLR_SATA0_READ_NOPEND             (0UL<<10)

#define       ITC_INTSTATSET_SATA1_WRITE_SET                 (1UL<<3)                       
#define       ITC_INTSTATSET_SATA1_WRITE_NOEFFECT            (0UL<<3)
#define       ITC_INTSTATSET_SATA1_READ_PEND                 (1UL<<3)
#define       ITC_INTSTATSET_SATA1_READ_NOPEND               (0UL<<3)

#define       ITC_INTSTATSET_SATA0_WRITE_SET                (1UL<<10)                       
#define       ITC_INTSTATSET_SATA0_WRITE_NOEFFECT           (0UL<<10)
#define       ITC_INTSTATSET_SATA0_READ_PEND                (1UL<<10)
#define       ITC_INTSTATSET_SATA0_READ_NOPEND              (0UL<<10)

#define ITC_SATA1                 0x00000008
#define ITC_SATA1_POS             3
#define ITC_SATA1_PIC             (PIC_1)

#define ITC_SATA0                 0x00000400
#define ITC_SATA0_POS             10
#define ITC_SATA_POS              ITC_SATA0_POS /* For backward compatibility */
#define ITC_SATA0_PIC             (PIC_1)
#define ITC_SATA_PIC              ITC_SATA0_PIC /* For backward compatibility */
#define ITC_SATA                  ITC_SATA0  /* For backwards compatibility */
#define PLL_LOCK_STAT_REG                                   (PLL_BASE + 0x124)
#define     PLL_LOCK_STAT_TIMER15_MASK                          0x80000000
#define     PLL_LOCK_STAT_TIMER15_SHIFT                         31
#define     PLL_LOCK_STAT_TIMER14_MASK                          0x40000000
#define     PLL_LOCK_STAT_TIMER14_SHIFT                         30
#define     PLL_LOCK_STAT_TIMER13_MASK                          0x20000000
#define     PLL_LOCK_STAT_TIMER13_SHIFT                         29
#define     PLL_LOCK_STAT_TIMER12_MASK                          0x10000000
#define     PLL_LOCK_STAT_TIMER12_SHIFT                         28
#define     PLL_LOCK_STAT_TIMER11_MASK                          0x08000000
#define     PLL_LOCK_STAT_TIMER11_SHIFT                         27
#define     PLL_LOCK_STAT_TIMER10_MASK                          0x04000000
#define     PLL_LOCK_STAT_TIMER10_SHIFT                         26
#define     PLL_LOCK_STAT_TIMER9_MASK                           0x02000000
#define     PLL_LOCK_STAT_TIMER9_SHIFT                          25
#define     PLL_LOCK_STAT_TIMER8_MASK                           0x01000000
#define     PLL_LOCK_STAT_TIMER8_SHIFT                          24
#define     PLL_LOCK_STAT_TIMER7_MASK                           0x00800000
#define     PLL_LOCK_STAT_TIMER7_SHIFT                          23
#define     PLL_LOCK_STAT_TIMER6_MASK                           0x00400000
#define     PLL_LOCK_STAT_TIMER6_SHIFT                          22
#define     PLL_LOCK_STAT_TIMER5_MASK                           0x00200000
#define     PLL_LOCK_STAT_TIMER5_SHIFT                          21
#define     PLL_LOCK_STAT_TIMER4_MASK                           0x00100000
#define     PLL_LOCK_STAT_TIMER4_SHIFT                          20
#define     PLL_LOCK_STAT_TIMER3_MASK                           0x00080000
#define     PLL_LOCK_STAT_TIMER3_SHIFT                          19
#define     PLL_LOCK_STAT_TIMER2_MASK                           0x00040000
#define     PLL_LOCK_STAT_TIMER2_SHIFT                          18
#define     PLL_LOCK_STAT_TIMER1_MASK                           0x00020000
#define     PLL_LOCK_STAT_TIMER1_SHIFT                          17
#define     PLL_LOCK_STAT_TIMER0_MASK                           0x00010000
#define     PLL_LOCK_STAT_TIMER0_SHIFT                          16
#define     PLL_LOCK_STAT_GLOBAL_RESET_MASK                     0x00002000
#define     PLL_LOCK_STAT_GLOBAL_RESET_SHIFT                    13
#define     PLL_LOCK_STAT_GLOBAL_RESET_WRITE_ENABLE             (0UL<<13)
#define     PLL_LOCK_STAT_GLOBAL_RESET_WRITE_DISABLE            (1UL<<13)
#define     PLL_LOCK_STAT_SMART_RESET_MASK                      0x00001000
#define     PLL_LOCK_STAT_SMART_RESET_SHIFT                     12
#define     PLL_LOCK_STAT_STOP_MODE_MASK                        0x00000800
#define     PLL_LOCK_STAT_STOP_MODE_SHIFT                       11
#define     PLL_LOCK_STAT_SUSPEND_MODE_MASK                     0x00000400
#define     PLL_LOCK_STAT_SUSPEND_MODE_SHIFT                    10
#define     PLL_LOCK_STAT_PLL_CTRL_MASK                         0x00000200
#define     PLL_LOCK_STAT_PLL_CTRL_SHIFT                        9
#define     PLL_LOCK_STAT_FENRUS_CTRL_MASK                      0x00000100
#define     PLL_LOCK_STAT_FENRUS_CTRL_SHIFT                     8
#define     PLL_LOCK_STAT_FENRUS_CTRL_ENABLE                    (0UL<<8)
#define     PLL_LOCK_STAT_FENRUS_CTRL_DISABLE                   (1UL<<8)
#define     PLL_LOCK_STAT_PLL2_DIV_MASK                         0x00000080
#define     PLL_LOCK_STAT_PLL2_DIV_SHIFT                        7
#define     PLL_LOCK_STAT_PLL1_DIV_MASK                         0x00000040
#define     PLL_LOCK_STAT_PLL1_DIV_SHIFT                        6
#define     PLL_LOCK_STAT_PLL0_DIV_MASK                         0x00000020
#define     PLL_LOCK_STAT_PLL0_DIV_SHIFT                        5
#define     PLL_LOCK_STAT_MEM_CTRL_MASK                         0x00000010
#define     PLL_LOCK_STAT_MEM_CTRL_SHIFT                        4
#define     PLL_LOCK_STAT_AUD_DIV_MASK                          0x00000008
#define     PLL_LOCK_STAT_AUD_DIV_SHIFT                         3
#define     PLL_LOCK_STAT_HD_DIV_MASK                           0x00000004
#define     PLL_LOCK_STAT_HD_DIV_SHIFT                          2
#define     PLL_LOCK_STAT_MPG1_DIV_MASK                         0x00000002
#define     PLL_LOCK_STAT_MPG1_DIV_SHIFT                        1
#define     PLL_LOCK_STAT_MPG0_DIV_MASK                         0x00000001
#define     PLL_LOCK_STAT_MPG0_DIV_SHIFT                        0

#define PLL_LOCK_CMD_REG                                    (PLL_BASE + 0x120)
#define     PLL_LOCK_CMD_MASK                                  0x000000FF
#define     PLL_LOCK_CMD_SHIFT                                 0
#define         PLL_UNLOCK_CMD1                                0xF8
#define         PLL_UNLOCK_CMD2                                0x2B
#define         PLL_LOCK_CMD                                   0x00

#define PLL_UNLOCK()              *(LPREG)PLL_LOCK_CMD_REG = PLL_UNLOCK_CMD1; \
                                  *(LPREG)PLL_LOCK_CMD_REG = PLL_UNLOCK_CMD2;
#define PLL_LOCK()                *(LPREG)PLL_LOCK_CMD_REG = PLL_LOCK_CMD;

#define PIO_LED_HARD_DISK (4+GPIO_DEVICE_ID_ISA_1+GPIO_PIN_IS_OUTPUT+GPIO_NEGATIVE_POLARITY)

#endif


/****************************************************************************
* Modifications:
* $Log:
*  5    stb_linux_kernel_2.6.18.2_1.061.2.1.1     8/21/2007 5:32:29 PM  
*       Jaychander Thirunavukkarasu To add IIC bus number and slave address
*       definitions for SATA PHY
*  4    stb_linux_kernel_2.6.18.2_1.061.2.1.0     8/9/2007 8:31:56 PM   
*       RadhaKumar Pulyala MTD flash write problem fixed in Bhavani JFFS2/MTD
*       support.
*  3    Linux_SDK 1.2         6/28/2007 12:45:22 PM  Jaychander
*       Thirunavukkarasu Move SATA definitions from sreg_pecos.h, sata_pecos.h
*       to new pecos.h and remove the dependencies on the native sata driver
*       header files
*  2    Linux_SDK 1.1         6/13/2007 7:27:56 PM   Debashish Rath  single
*       pecos.h file instead of old pecos.h, modules directory and firmware
*       directory
*  1    Linux_SDK 1.0         3/1/2007 10:48:25 PM   Vineet Seth     
* $
*
****************************************************************************/


