/******************************************************************************
 *  linux/include/asm-arm/arch-pecos/time.h
 *
 *  Copyright (C) 2007 Conexant Systems Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Verison 2 as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
 *****************************************************************************/
/*$Id$
 *****************************************************************************/

#include <asm/system.h>
#include <asm/irq.h>
#include <asm-arm/io.h>
#include <linux/interrupt.h>


/* Clock ticks at 54 Mhz, giving maximum 79.5sec count */
#define uSEC_1 54
#define TIMER_INTERVAL	((uSEC_1 * 1000000) / HZ)
#if 0
/*
 * IRQ handler for the system timer (Timer 0)
 */
extern struct s_latency latency;
extern unsigned long (*gettimeoffset)(void);
extern void idleAddCount(void);

static unsigned long pecos_gettimeoffset(void)
{
   volatile unsigned int *timer_count_reg;
   volatile unsigned int *timer_irq_stat;
   unsigned long ticks;

   timer_irq_stat = (int*)ASX_TO_VIRT( TIM_INT_STATUS_REG );
   timer_count_reg = (volatile unsigned int *)ASX_TO_VIRT( TIM_VALUE_REG );

   /* Since the timer is resetting itself to 0 after an interrupt
      the TIM_VALUE_REG, by definition, is the number of clocks
      since the last interrupt -- which is exactly what this 
      function is supposed to return -- only this fn returns
      it in uSecs
    */
   ticks = *timer_count_reg;

   /* Since we're called with interrupts disabled, the timer could have
      generated an interrupt -- which would mean the timer reset itself
      to zero.  THUS, if an interrupt is pending, we need to add that many
      ticks to our value
    */
   if (*timer_irq_stat & 0x01)
   {
      ticks += TIMER_INTERVAL;
   }

   return (ticks) / (uSEC_1);
}


static irqreturn_t  pecos_sys_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
   volatile unsigned int *timer_count_reg;

//  printk("Inside SYS TIMER INTERRUPT\n");
   /* the status is cleared because the interrupt mechanism
    * calls mask_and_ack_irq which in turn writes the mode 
    * reg which has the effect of clearing the status.
    * if we're here, it happened... 
    */
   
   /* read the time counter
    * register.  We've set it up so when the timer reaches the 
    * limit, it interrupts us and resets it to 0.  Thus,
    * the delta from zero could be considered general system 
    * latency...
    */
   timer_count_reg = (volatile unsigned int *)ASX_TO_VIRT( TIM_VALUE_REG );

   latency.delta[latency.count++] =  *timer_count_reg;
   if (latency.count > LATENCY_COUNT) latency.count = 0;   

   /* Whew!  The meat of the timer! */
   do_timer( regs );
   
   raise_softirq(TIMER_SOFTIRQ);
   
   idleAddCount();

   return IRQ_HANDLED;
   
}

static struct irqaction trinity_sys_timer = {
	.name = "Pecos System Timer",
	.flags = SA_INTERRUPT,
	.handler = pecos_sys_timer_interrupt
};

/*
 * Set up timer system timer interrupt
 */
static inline void setup_timer( void )
{
    volatile unsigned int *p;

    /* keep this interrupt atomic */
	/*
    timer_irq.handler = bronco_sys_timer_interrupt;
    timer_irq.flags = SA_INTERRUPT;
  */  /*block noida ..check out which irq+timer struct to be updated */
	
    /* Make sure Interrupt generation is off */
    p = (volatile unsigned int *)ASX_TO_VIRT( TIM_MODE_REG ); 
    __raw_writel( 0L, p );
    
    /* Clear Counter value to zero */
    p = (volatile unsigned int *)ASX_TO_VIRT( TIM_VALUE_REG ); 
    __raw_writel( 0L, p );
 
    /* Program Limit to 10ms */
    p = (volatile unsigned int *)ASX_TO_VIRT( TIM_LIMIT_REG );
    __raw_writel( TIMER_INTERVAL, p );

    /* Clear Base value for Timer*/
    p = (volatile unsigned int *)ASX_TO_VIRT( TIM_BASE_REG );
    __raw_writel( 0L, p );

   /* 
    * Make irqs happen for the system timer
    */
   
    /* Enable Timer IRQs in main PIC */
    setup_irq(INT_SYSTIMER, &trinity_sys_timer);
   
    /* Enable irq generation by timer 0 */
    p = (volatile unsigned int *)ASX_TO_VIRT( TIM_MODE_REG ); 
    __raw_writel( 9L, p );
    
}
#endif

/****************************************************************************
 * Modifications:
 * $Log$
 *
 ****************************************************************************/

